program ConvertKnmiAscToAMD
  !  #[ Documentation
  !     a program convert meteo files in KNMI ASCII format
  !     to Earth Explorer format, suitable for input for the L2BP software
  !
  !     written by: J. de Kloe (KNMI)
  !     created:    02-Aug-2006
  !
  !     Modifications:
  !     02-Aug-2006  J. de Kloe  first try, based on the code previously 
  !                              available in: TestWriteAMDdata.F90
  !     19-Oct-2007  P. Poli     split AMD_DataType in two parts:
  !                              AMD_HDR_DataType and AMD_BRC_DataType
  !     17-Jan-2008  J. de Kloe  removed a last few 64bit incompatibilities
  !     22-Feb-2008  J. de Kloe  added FillNadir/FillOffNadir flags
  !  #]
  !  #[ Modules used

  ! define a module to read ascii met data into a profile_type
  USE AuxiliaryModule, only: InitAuxiliaryModule, &
       profile_type, GetAuxiliaryDataKnmi, &
       RawProfile_to_AMD_DataStructure, &
       DeallocAuxiliaryData
       

  USE EE_CFI_Datatypes, only: IntAus, IntAl

  ! define a module to hold the AMD_DataStructure
  USE AMD_DataStructure, only: AMD_HDR_DataType, AMD_BRC_DataType, &
                               Init_AMD_HDR_DataStructure, &
                               Init_AMD_BRC_DataStructure, &
                               Delete_AMD_HDR_DataStructure,&
                               Delete_AMD_BRC_DataStructure,&
                               Print_AMD_HDR_DataStructure, &
                               Print_AMD_BRC_DataStructure, &
                               Handle_AMDHeaders
  USE AMD_Geoloc_ADS,   only: Init_AMD_Geoloc_ADS
  USE AMD_Met_MDS,      only: Init_AMD_Met_MDS
  USE WriteAMDdata,     only: InitWriteAMDfileModule, WriteAMDfile

  ! support modules
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options, &
                          request_program_to_stop, error_allocate
  !  #]
  !  #[ local variables and parameters

  implicit none

  integer            :: error_flag, num_profiles, num_profiles_to_skip
  integer            :: nr_of_arguments, current_argument_nr
  integer            :: num_knmi_profiles, num_profiles_to_convert
  integer            :: i, prof_start, prof_end, p, AllocStatus
  integer            :: nlevels_nadir, nlevels_offnadir
  integer            :: num_profiles_nadir, num_profiles_offnadir
  character(len=256) :: current_argument, txt_count
  character(len=256) :: knmi_file ! filename to read aux data from
  TYPE(profile_type), dimension(:), pointer :: profiles
  TYPE(AMD_HDR_DataType)                            :: AMD_Headers
  TYPE(AMD_BRC_DataType), dimension(:), allocatable :: AMD
  logical            :: fill_off_nadir_flag
  logical            :: FillNadir, FillOffNadir
  !  #]
  !  #[ program code

  print *,"starting ConvertKnmiAscToAMD"

  !*** initialisations ***
  error_flag = no_error
  nullify(profiles)
  FillNadir    = .true.
  FillOffNadir = .true.

  call InitWriteAMDfileModule(error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitWriteAMDfileModule()")

  call InitAuxiliaryModule(error_flag)
  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitAuxiliaryModule")

  ! set defaults to be used when no command line option is used to change it
  knmi_file    = "knmi_file.asc"
  num_profiles = 1
  num_profiles_to_skip = 0

  ! handle commandline arguments
  nr_of_arguments = iargc_aeolus()
  current_argument_nr = 0
  argloop: DO WHILE (current_argument_nr .lt. nr_of_arguments)
     !  #[ process the given commandline arguments
     current_argument_nr = current_argument_nr+1
     call getarg_aeolus(current_argument_nr,current_argument)
     SELECT CASE (trim(current_argument))
     CASE("-f","--file")
        !  #[
        current_argument_nr = current_argument_nr+1
        IF (current_argument_nr .gt. nr_of_arguments) THEN
           print *,"============================================"
           print *,"filename is missing after "//&
                trim(current_argument)//" option !!!"
           print *,"============================================"
           goto 999
        END IF
        call getarg_aeolus(current_argument_nr,knmi_file)
        print *,'setting input file to: '//trim(knmi_file)
         !  #]
     CASE("-np","--numprofiles")
        !  #[
        current_argument_nr = current_argument_nr+1
        IF (current_argument_nr .gt. nr_of_arguments) THEN
           print *,"============================================"
           print *,"count is missing after "//&
                trim(current_argument)//" option !!!"
           print *,"============================================"
           goto 999
        END IF
        call getarg_aeolus(current_argument_nr,txt_count)
        read(txt_count,"(i6)",err=998) num_profiles

        print *,'setting num_profiles to: ',num_profiles
         !  #]
     CASE("-ns","--numprofilestoskip")
        !  #[
        current_argument_nr = current_argument_nr+1
        IF (current_argument_nr .gt. nr_of_arguments) THEN
           print *,"============================================"
           print *,"count is missing after "//&
                trim(current_argument)//" option !!!"
           print *,"============================================"
           goto 999
        END IF
        call getarg_aeolus(current_argument_nr,txt_count)
        read(txt_count,"(i6)",err=998) num_profiles_to_skip

        print *,'setting num_profiles_to_skip to: ',num_profiles_to_skip
         !  #]
     CASE("-on","--onlynadir")
        !  #[
        print *,'Only filling nadir profiles'
        FillNadir    = .true.
        FillOffNadir = .false.
         !  #]
     CASE("-oo","--onlyoffnadir")
        !  #[
        print *,'Only filling offnadir profiles'
        FillNadir    = .false.
        FillOffNadir = .true.
         !  #]
     CASE("-fb","--fillboth")
        !  #[
        print *,'Filling both nadir and offnadir profiles'
        FillNadir    = .true.
        FillOffNadir = .true.
         !  #]
     CASE("-h","--help")
        !  #[
        CALL Print_Usage
        call program_abort(request_program_to_stop,"ConvertKnmiAscToAMD")
        !  #]
     CASE DEFAULT
        !  #[
        print *,"============================================"
        print *,"unrecognised commandline option: ",trim(current_argument)
        IF (current_argument(1:1) .ne. '-') THEN
           print *,"This argument does not start with: -"
           print *,"Maybe you forgot the - character to indicate that this"
           print *,"is an option, or maybe you supplied the wrong number"
           print *,"of items to an option."
        END IF
        print *,"============================================"
        goto 999
        !  #]
     END SELECT
     !  #]
  END DO argloop

  ! some sanity checks
  if (num_profiles .lt. 1) then
     print *,"illegal value for num_profiles: ",num_profiles
     call program_abort(error_cmdline_options,"ConvertKnmiAscToAMD")
  end if
  if (num_profiles_to_skip .lt. 0) then
     print *,"illegal value for num_profiles_to_skip: ",num_profiles_to_skip
     call program_abort(error_cmdline_options,"ConvertKnmiAscToAMD")
  end if

  print *,"-------------------------------------"
  print *,"using input file:  ",trim(knmi_file)
  print *,"trying to convert: ",num_profiles," profiles"
  print *,"skipping:          ",num_profiles_to_skip,&
       " before starting the conversion"
  print *,"-------------------------------------"

  call GetAuxiliaryDataKnmi(knmi_file,                   & ! input
                            num_knmi_profiles, profiles, & ! output
                            error_flag)
  IF (error_flag .ne. no_error) THEN
     print *,"GetAuxiliaryDataKnmi failed"
     call program_abort(error_flag,"GetAuxiliaryDataKnmi")
  END IF

  print *,"input file contained: ",num_knmi_profiles," profiles"

  prof_start = num_profiles_to_skip+1
  prof_end   = num_profiles_to_skip+num_profiles
  if (prof_start .gt. num_knmi_profiles) then
     print *,"ERROR: the input file contains not enough profiles."
     print *,"when you skip ",num_profiles_to_skip,&
          " profiles, none are left"
     print *,"please change the input settings..."
     call program_abort(request_program_to_stop,"ConvertKnmiAscToAmd")
  end if
  if (prof_end   .gt. num_knmi_profiles) prof_end = num_knmi_profiles

  num_profiles_to_convert = prof_end-prof_start+1

  print *,"Converting ",num_profiles_to_convert," profiles"
  print *,"numbered ",prof_start," upto ",prof_end

  allocate(AMD(num_profiles_to_convert),stat=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_allocate,"ConvertKnmiAscToAmd")
  
  do i=1,num_profiles_to_convert
     p=prof_start+i-1
     print *,"ConvertKnmiAscToAmd: "//&
          "copying KNMI profile ",p," to AMD profile ",i

     call Init_AMD_BRC_DataStructure(AMD(i), profiles(1)%nlevels, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Init_AMD_BRC_DataStructure")

     ! note that the amd_geoloc_ads has to be initialised for both
     ! nadir and off-nadir, even if you just use one of them.
     ! The not used one will just contain missing values.
     call Init_AMD_Geoloc_ADS(AMD(i)%Geoloc_ADS_off_nadir, error_flag)
     if (error_flag .ne. no_error) &
        call Program_Abort(error_flag,"Init_AMD_Geoloc_ADS")

     call Init_AMD_Geoloc_ADS(AMD(i)%Geoloc_ADS_nadir,  error_flag)
     if (error_flag .ne. no_error) &
        call Program_Abort(error_flag,"Init_AMD_Geoloc_ADS")

     ! note that the amd_met_ads has to be initialised for both
     ! nadir and off-nadir, even if you just use one of them.
     ! The not used one will just contain missing values for the
     ! surface elements, and a nullified pointer to the list of layers.

     nlevels_nadir = 0
     if (FillNadir) nlevels_nadir = profiles(1)%nlevels

     call Init_AMD_Met_MDS(AMD(i)%Met_MDS_nadir, &
                           nlevels_nadir, error_flag)
     if (error_flag .ne. no_error) &
          call Program_Abort(error_flag,"Init_AMD_Met_MDS")

     nlevels_offnadir = 0
     if (FillOffNadir) nlevels_offnadir = profiles(1)%nlevels

     call Init_AMD_Met_MDS(AMD(i)%Met_MDS_off_nadir, &
                           nlevels_offnadir, error_flag)
     if (error_flag .ne. no_error) &
          call Program_Abort(error_flag,"Init_AMD_Met_MDS")

     !call Print_AMD_DataStructure(AMD(i), error_flag)

     ! Transfer from raw profiles to AMD_DataStructure
     if (FillOffNadir) then
        fill_off_nadir_flag=.true.
        call RawProfile_to_AMD_DataStructure(fill_off_nadir_flag,profiles(p),&
                                             AMD(i),error_flag)
     end if
     if (FillNadir) then
        fill_off_nadir_flag=.false.
        call RawProfile_to_AMD_DataStructure(fill_off_nadir_flag,profiles(p),&
                                             AMD(i),error_flag)
     end if

     ! Additional prints for debugging

     !print *,"Post-conversion result:"
     !call Print_AMD_BRC_DataStructure(AMD(i), error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Print_AMD_BRC_DataStructure")
     !call Print_AMD_HDR_DataStructure(AMD(i), error_flag)
     !IF (error_flag .ne. no_error) &
     !     call program_abort(error_flag,"Print_AMD_HDR_DataStructure")

  end do

  ! Handle Headers
  print *,'ConvertKnmiAscToAmd: initializing AMD_Headers'
  call Init_AMD_HDR_DataStructure(AMD_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_AMD_HDR_DataStructure")

  call InitWriteAMDfileModule(error_flag)
  IF (error_flag .ne. no_error) &
       & call program_abort(error_flag,"InitWriteAMDfile")

  num_profiles_nadir = 0
  if (FillNadir) num_profiles_nadir = num_profiles_to_convert
  num_profiles_offnadir = 0
  if (FillOffNadir) num_profiles_offnadir = num_profiles_to_convert

  call Handle_AMDHeaders(AMD_Headers, AMD(1), &
       error_flag, &
       int(profiles(1)%nlevels,IntAus),    & ! Num_of_Model_Layers
       int(num_profiles_offnadir,IntAl),   & ! Num_Records_in_DS1 (offnadir)
       int(num_profiles_nadir,IntAl),      & ! Num_Records_in_DS2 (nadir)
       int(num_profiles_to_convert,IntAl), & ! Num_Avail_L1B_Obs
       0_IntAl,                            & ! Num_Missing_L1B_Obs
       int(num_profiles_to_convert,IntAl), & ! Num_Computed_Locations
       switch_init=.true.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Handle_AMDHeaders")
  print *,'ConvertKnmiAscToAmd: initialized AMD_Headers'

  ! Write AMD DataStructure to EE file
  print *,'ConvertKnmiAscToAmd: calling WriteAMDfile'
  call WriteAMDfile(num_profiles_to_convert, AMD_Headers, AMD, error_flag)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"WriteAMDfile")

  ! clean the used memory
  DO i=1,size(AMD)
     call Delete_AMD_BRC_DataStructure(AMD(i))
  END DO
  deallocate(AMD)
  call Delete_AMD_HDR_DataStructure(AMD_Headers)

  call DeallocAuxiliaryData(profiles) 

  goto 111

998 print *,"ERROR: could not convert count: "//trim(txt_count)
  print *,"given as commandline argument for option "//trim(current_argument)
  print *,"to an integer ..."
  call program_abort(error_cmdline_options,"ConvertKnmiAscToAMD")

  ! error handler for commandline handling problems
999 CALL Print_Usage
  !call program_abort(error_cmdline_options,"ConvertKnmiAscToAMD")

111 continue
  print *,"done"
  ! the end
  !  #]
contains
  subroutine Print_Usage
    !  #[
    print *,""
    print *,"Possible commandline options for the ConvertKnmiAscToAMD"
    print *,"conversion tool are:"
    print *,""
    print *,"ConvertKnmiAscToAMD"
    print *,"   [-f/--file <KnmiAsciiFileName.asc> (default:knmi_file.asc)]"
    print *,"   [-np/--numprofiles       <count> ]"
    print *,"   [-ns/--numprofilestoskip <count> ]"
    print *,"   [-on/--onlynadir   ]"
    print *,"   [-oo/--onlyoffnadir]"
    print *,"   [-fb/--fillboth    ]"
    print *,"   [-h/--help         ]"
  end subroutine Print_Usage
    !  #]
end program ConvertKnmiAscToAMD
