program TestReadAMDdata
  !  #[ Documentation
  !     a test program for the ReadAMDdata module
  !
  !     written by: David Tan (ECMWF)
  !     created:    07-Mar-2006
  !
  !     Modifications:
  !     07-Mar-2006 D. Tan     Original version
  !     07-Apr-2006 D. Tan     Add conversion AMD_DataStructure_to_RawProfile
  !     12-May-2006 D. Tan     Null use of ParKind1 module
  !     07-Aug-2006 J. de Kloe added proper initialisation and deallocation
  !                            for the raw_profiles array
  !     17-Aug-2006 P. Poli    Removed obsolete variables
  !     19-May-2007 D. Tan     Migrate to Copy_AMD_to_RawProfile
  !     19-Oct-2007 P. Poli    split AMD_DataType in two parts:
  !                            AMD_HDR_DataType and AMD_BRC_DataType
  !     16-Jan-2008 J. de Kloe phase out integer kind i_
  !     17-Jan-2008 J. de Kloe removed a last few 64bit incompatibilities
  !
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE AMD_DataStructure, only: AMD_HDR_DataType, &
                               AMD_BRC_DataType, &
                               Init_AMD_HDR_DataStructure, &
                               Init_AMD_BRC_DataStructure, &
                               Delete_AMD_HDR_DataStructure, &
                               Delete_AMD_BRC_DataStructure, &
                               Print_AMD_HDR_DataStructure, &
                               Print_AMD_BRC_DataStructure
  USE ReadAMDdata, only: ReadAMDfile, InitReadAMDfileModule
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus

  USE AuxiliaryModule, only: profile_type, Init_Profile,&
       TestPrintAuxiliaryData, &
       InitAuxiliaryModule,    Copy_AMD_to_RawProfile, Delete_Profile
  USE EE_CFI_Datatypes, only: IntAus, GetIntAus
  !  #]
  !  #[ variables
  implicit none
  integer                :: error_flag, nr_of_arguments
  character(len=256)     :: filename ! without extension
  TYPE(AMD_HDR_DataType) :: AMD_Headers
  TYPE(AMD_BRC_DataType) :: AMD

  TYPE(profile_type) :: raw_profile

  ! local variables
  integer         :: N_model_levels = 0
  integer(IntAus) :: n_model_layers
  logical         :: from_off_nadir_flag

  !  #]
  !  #[ program code
  error_flag = no_error

  call Init_AMD_HDR_DataStructure(AMD_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_AMD_HDR_DataStructure")  

  call Init_AMD_BRC_DataStructure(AMD, N_model_levels, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_AMD_BRC_DataStructure")  

  call InitReadAMDfileModule(error_flag,ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitReadAMDfileModule")

  call InitAuxiliaryModule(error_flag)!,ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitAuxiliaryModule")

  ! read the filename without extension as 1st commandline parameter:
  nr_of_arguments = iargc_aeolus()
  IF (nr_of_arguments .ne. 1) THEN
     print *,"Usage: TestReadAMDdata <filename[without extension]>"
     call program_abort(error_cmdline_options,"TestReadAMDdata")
  END IF

  call getarg_aeolus(1,filename)

  ! read all datablocks
  call ReadAMDfile(AMD_Headers, filename, error_flag, &
       AMD = AMD, &
       ReadHeaders=.true., &
       ReadGeoloc_offnadir=.true., ReadGeoloc_nadir=.true., & 
       ReadMet_offnadir=.true.   , ReadMet_nadir=.true.     )
!                   ReadAll=.true., ReadPCD=.false.)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"ReadAMDfile")

  ! Print the AMD_BRC_DataStructure before conversion
  call Print_AMD_BRC_DataStructure(AMD, error_flag)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"Print_AMD_DataStructure")

    !-------------------------
    ! Initialize raw_profile
    !-------------------------

  ! Determine the number of model layers
  call GetIntAus(AMD_Headers%SPH%Num_of_Model_Layers, n_model_layers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Get_n_model_layers")
  ! print *, "n_model_layers =", n_model_layers

  call Init_Profile(raw_profile,int(n_model_layers),error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Profile")

  ! Convert one BRC to the RawProfile datastructure
!  iBRC = 1
  from_off_nadir_flag = .true.
  call Copy_AMD_to_RawProfile(from_off_nadir_flag, AMD, &
                              raw_profile, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Copy_AMD_to_RawProfile")

  ! print the result after conversion
  ! now done within Copy_AMD_to_RawProfile
  ! old: print *,"Result of conversion to RawProfile:"
  ! new: print *, 'Conversion from AMD_DataStructure to RawProfile ...'
  !call TestPrintAuxiliaryData(raw_profile)

  call Delete_Profile(raw_profile)
  call Delete_AMD_BRC_DataStructure(AMD)
  call Delete_AMD_HDR_DataStructure(AMD_Headers)
  !  #]
end program TestReadAMDdata
