program TestL1B_ee2bufr
  !---------------------------------
  !  #[ Documentation:
  ! - program to test L1B_ee2bufr
  !   print decoded BUFR subsets for comparison
  !
  ! - written by: M. Rohn (ECMWF)
  !
  ! - created    : 13-Dec-2005
  !
  ! Modifications:
  !
  !    14-Dec-2006 D. Tan     REALACC(6) for data after section 3
  !    16-Jan-2008 J. de Kloe phase out integer kind i_
  !  #]
  !---------------------------------
  ! --- declarations
  !  #[ Modules used
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus

  USE L1B_BufrUtil, only: r8_, rvind, &
                          jsec0, jsec1, jsec2, jsec3, jsec4, kelem, kvals, &
                          ksec0, ksec1, ksec2, ksec3, ksec4, &
                          ktdlst_size,ktdlst, &
                          key, &
                          kbufr, &
                          cvals, cvals, &
                          cnames, cunits, &
                          values, &
                          NELEMENTS_L1B_HEAD, &
                          NELEMENTS_L1B_OBS, &
                          NELEMENTS_L1B_MEAS

  !  #]
  !  #[ variables
  implicit none

  ! constants
  integer, parameter :: jbyte      = 2048000, &
                        jsup       = 9

  real(r8_), parameter :: eps      = 10.E-8

  ! local variables
  integer ::             ksup(JSUP)
  integer  ::            ktdexp(kelem)

  integer :: error_flag, &
             i, istring, ivalue, &
             no_element_start, &
             no_element_end, &
             imeasurement, &
             nr_of_arguments, &
             iunit_bufr, &
             imessage, &
             isubset, &
             kbufr_size_byte, &
             kbufr_size_word, &
             ktdexp_size 


  character(len=256) :: filename, &      ! without extension
                        filename_bufr, & ! with    extension 
                        carg

  !  #]
  ! --- main
  !  #[ 1.0 init
  nr_of_arguments = iargc_aeolus()
  IF (nr_of_arguments .ne. 3) THEN
     print *,"Usage: TestL1B_ee2bufr <filename[without extension]> <no-message> <no-subset>"
     call program_abort(error_cmdline_options,"TestL1B_ee2bufr")
  END IF

  call getarg_aeolus(1,filename)

  call getarg_aeolus(2,carg)
  read (carg, '(I4)', iostat=error_flag) imessage

  call getarg_aeolus(3,carg)
  read (carg, '(I4)', iostat=error_flag) isubset
  !  #]
  !  #[ 2.0 open file
  filename_bufr = trim( filename )//".BUFR"

  call pbopen(iunit_bufr,filename_bufr,'R',error_flag)
  if(error_flag.ne.0) then
    print*,'Open error on ', filename_bufr
    stop 2
!    call exit(2)
  end if
  !  #]
  !  #[ 3.0 read requested message
  do i=1,imessage
    call pbbufr(iunit_bufr, kbufr, JBYTE, kbufr_size_byte, error_flag)
    if(error_flag.lt.0) then
      print*,'Read error on ', filename_bufr
      stop 2
!      call exit(2)
    end if
  enddo
  !  #]
  !  #[ 4.0 decode message
  kbufr_size_word = kbufr_size_byte / 4 + 1

  ! 4.1 sections
  call bus012(kbufr_size_word, kbufr, ksup, ksec0, ksec1, ksec2, error_flag)
  if(error_flag.ne.0) then
    print*,'error bus012 ', error_flag
    stop 2
!    call exit(2)
  end if

  ! 4.2 message
  call bufrex(kbufr_size_word, kbufr, ksup, ksec0, ksec1, ksec2, ksec3, ksec4, &
              kelem, cnames, cunits, kvals, values, cvals, & 
              error_flag)
  if(error_flag.ne.0) then
    print*,'error bufrex ', error_flag
    stop 2
!    call exit(2)
  end if

  ! 4.3 descriptor list
  call busel(ktdlst_size, ktdlst, ktdexp_size, ktdexp, error_flag) 
  if(error_flag.ne.0) then
    print*,'error busel ', error_flag
    stop 2
!    call exit(2)
  end if
  !  #]
  !  #[ 5.0 print 

  ! 5.1 section 0
  call buprs0(ksec0)

  ! 5.2 section 1
  call buprs1(ksec1)

  ! 5.3 section 2
  call buukey(ksec1, ksec2, key, ksup, error_flag)

  call buprs2(ksup, key)

  ! 5.4 section 3
  !call buprs3(ksec3, ktdlst_size, ktdlst, ktdexp_size, ktdexp, kvals, cnames)


  ! 5.5 print data

  if ( isubset .gt.ksup(6) ) then
    print*,' Number of subsets encoded : ', ksup(6)
    print*,' Requested subset too large: ', isubset
    stop 2
!    call exit(2)
  endif

  write(*, *)
  write(*, *) "REALACC(6)"

  ! 5.5.1 header + obs scale
  no_element_start = 1
  no_element_end   = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS

  ! the '/' character is not valid in Fortran90 formats to indicate
  ! a newline (it is an extension to the language and thus not portable).
  ! use an extra write statement, or the newline constant defined
  ! in the Compiler_Features module to achive this.
  !write(*, ('/," --- PRINT HEADER + OBS SCALE elements",/') )
  write(*, *)
  write(*, *) "--- PRINT HEADER + OBS SCALE elements"
  write(*, *)

  do i=no_element_start,no_element_end

    ivalue = (isubset-1)*kelem + i

    if ( cunits(i)(1:8) == 'CCITTIA5' ) then 
      istring = int(values(ivalue) )/1000
      write(*, 102 ) i, cnames(i), cunits(i), cvals(istring)
    else
      if (values(ivalue) >= rvind-eps .and. values(i) <= rvind+eps) then
        write(*, 101 ) i, cnames(i), cunits(i)
      else
        write(*, 100 ) i, cnames(i), cunits(i), values(ivalue)
      end if
    endif

  end do

  ! 5.5.2 meas scale:
  ! first
  imeasurement = 1 
 
  no_element_start = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS + 1 + (imeasurement-1) * NELEMENTS_L1B_MEAS
  no_element_end   = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS + 1 + (imeasurement  ) * NELEMENTS_L1B_MEAS

  ! the '/' character is not valid in Fortran90 formats to indicate
  ! a newline (it is an extension to the language and thus not portable).
  ! use an extra write statement, or the newline constant defined
  ! in the Compiler_Features module to achive this.
  !write(*, ('/," --- PRINT MEAS SCALE elements for MEAS No ",i6,/') ) imeasurement
  write(*, *)
  write(*, "(' --- PRINT MEAS SCALE elements for MEAS No ',i6)" ) imeasurement
  write(*, *)

  do i=no_element_start,no_element_end

    ivalue = (isubset-1)*kelem + i

    if ( cunits(i)(1:4) .EQ. 'CCIT' ) then 
      istring = ( int(values(ivalue))-32 )/1000
      write(*, 102 ) i, cnames(i), cunits(i), cvals(istring)
    else
      if (values(ivalue) >= rvind-eps .and. values(i) <= rvind+eps) then
        write(*, 101 ) i, cnames(i), cunits(i)
      else
        write(*, 100 ) i, cnames(i), cunits(i), values(ivalue)
      end if
    endif

  end do
  ! last
  i = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS + 1
  imeasurement = int(values(i))
 
  no_element_start = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS + 1 + (imeasurement-1) * NELEMENTS_L1B_MEAS
  no_element_end   = NELEMENTS_L1B_HEAD + NELEMENTS_L1B_OBS + 1 + (imeasurement  ) * NELEMENTS_L1B_MEAS

  ! the '/' character is not valid in Fortran90 formats to indicate
  ! a newline (it is an extension to the language and thus not portable).
  ! use an extra write statement, or the newline constant defined
  ! in the Compiler_Features module to achive this.
  !write(*, ('/," --- PRINT MEAS SCALE elements for MEAS No ",i6,/') ) imeasurement
  write(*, *)
  write(*, "(' --- PRINT MEAS SCALE elements for MEAS No ',i6)" ) imeasurement
  write(*, *)

  do i=no_element_start,no_element_end

    ivalue = (isubset-1)*kelem + i

    if ( cunits(i)(1:4) .EQ. 'CCIT' ) then 
      istring = ( int(values(ivalue))-32 )/1000
      write(*, 102 ) i, cnames(i), cunits(i), cvals(istring)
    else
      if (values(ivalue) >= rvind-eps .and. values(i) <= rvind+eps) then
        write(*, 101 ) i, cnames(i), cunits(i)
      else
        write(*, 100 ) i, cnames(i), cunits(i), values(ivalue)
      end if
    endif

  end do

  write(*, *) "ENDREALACC"
  !  #]
  !  #[ 6.0 close file
  call pbclose(iunit_bufr,error_flag)
  if(error_flag.ne.0) then
    print*,'Close error on ', filename_bufr
    stop 2
!    call exit(2)
  end if
  !  #]
  !  #[ 7.0 format definition
100 format(i6, " ", a64, " ", a20, " ", e20.10 )
101 format(i6, " ", a64, " ", a20, " ", " MISSING " )
102 format(i6, " ", a64, " ", a20, " ", a64 )
  !  #]
end program TestL1B_ee2bufr
