program TestAMD_DataStructure
  !  #[ Description
  !     a simple test program to test the AMD_DataStructure module
  !     created:     11-07-2005, by: D. Tan, ECMWF
  !     based on:    TestAMD_DataStructure by Jos de Kloe, KNMI.
  !
  !     Modifications
  !      31-Jan-2006 D. Tan      Original version
  !      12-May-2006 D. Tan      Null use of ParKind1 module
  !      19-Oct-2007 P. Poli     split AMD_DataType in two parts:
  !                              AMD_HDR_DataType and AMD_BRC_DataType
  !      16-Jan-2008 J. de Kloe  phase out integer kind i_
  !
  !  #]
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE AMD_Datastructure
  USE ErrorHandler, only: no_error, program_abort

  implicit none

  TYPE(AMD_HDR_DataType) :: AMD_Headers
  TYPE(AMD_BRC_DataType) :: AMD
  integer                :: error_flag
  integer, parameter     :: N_model_levels         = 60

  print *,"test AMD_HDR_DataStructure"

  call Init_AMD_HDR_DataStructure(AMD_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_AMD_HDR_DataStructure")
  print *,"structure is initialised"

  call Delete_AMD_HDR_DataStructure(AMD_Headers)
  print *,"structure is deleted"

  print *,"test AMD_BRC_DataStructure"

  call Init_AMD_BRC_DataStructure(AMD, N_model_levels, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_AMD_BRC_DataStructure")
  print *,"structure is initialised"

  call Delete_AMD_BRC_DataStructure(AMD)
  print *,"structure is deleted"

end program TestAMD_DataStructure
