program TestAMD_Met_MDS
  !  #[ Description
  !     a simple test program to test the AMD_Met_MDS module
  !     created:     01-Feb-2006, by: D. Tan, ECMWF
  !     based on:    Test_L2B_Rayleigh_MDS by Jos de Kloe, KNMI.
  !
  !     Modifications
  !      01-Feb-2006 D. Tan     Original version
  !      02-Feb-2006 D. Tan     Change IntAs to IntAus for Num_Model_Layers.
  !      12-May-2006 D. Tan     Null use of ParKind1 module
  !      16-Jan-2008 J. de Kloe phase out integer kind i_
  !      17-Jan-2008 J. de Kloe removed a last few 64bit incompatibilities
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE AMD_Met_MDS, only: AMD_Met_MDS_type, Init_AMD_Met_MDS, &
                              Print_AMD_Met_MDS, Delete_AMD_Met_MDS,&
                              GetByteSize_Met_Layer, &
                              GetByteSize_AMD_Met_MDS
  USE EE_CFI_Datatypes, only: IntAs, IntAl, IntAus
  USE ErrorHandler, only: no_error  
  !  #]
  !  #[ variables
  implicit none
  TYPE(AMD_Met_MDS_type), pointer :: test_AMD_Met_MDS
  integer :: error_flag
  integer(IntAl) :: c_bytesize
  !  #]
  !  #[ program code
  nullify(test_AMD_Met_MDS)

  print *,"first try, before allocating this MDS"
  print *,"(this should trigger an error...)"
  call Init_AMD_Met_MDS(test_AMD_Met_MDS, 0, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_AMD_Met_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_AMD_Met_MDS(test_AMD_Met_MDS, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_AMD_Met_MDS():"
     print *,"error_flag = ",error_flag
  endif

  allocate(test_AMD_Met_MDS)

  print *,"next try, after allocating this MDS"

  call Init_AMD_Met_MDS(test_AMD_Met_MDS, 60, error_flag)
!                             91, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_AMD_Mie_Met_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_AMD_Met_MDS(test_AMD_Met_MDS, error_flag, &
                              PrintAll=.false.)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_AMD_Met_MDS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"reporting size:"

  c_bytesize = GetByteSize_Met_Layer()
  print *,"c_bytesize for AMD_Met_Layer = ",c_bytesize

  c_bytesize = GetByteSize_AMD_Met_MDS(test_AMD_Met_MDS)
  print *,"c_bytesize for AMD_Met_MDS = ",c_bytesize

  print *,"done"

  call Delete_AMD_Met_MDS(test_AMD_Met_MDS)
  deallocate(test_AMD_Met_MDS)
  !  #]
end program TestAMD_Met_MDS
