program TestL2BC_DataStructure
  !  #[ Description
  !     a simple test program to test the L2BC_DataStructure module
  !     created:     11-07-2005, by: J. de Kloe, KNMI
  !
  !     Modifications:
  !      12-May-2006  D. Tan       Null use of ParKind1 module
  !      20-Oct-2006  D. Tan       Generalize to L2BC_DataStructure
  !      25-Oct-2006  D. Tan       introduce L2C_flag
  !      04-Sep-2007  P. Poli      extra calls to test BRC and HDR 
  !                                parts of L2BC datastructure, 
  !                                change L2BC_DataType to 
  !                                L2BC_HDR_DataType and L2BC_BRC_DataType
  !      16-Jan-2008 J. de Kloe    phase out integer kind i_ 
  !
  !  #]
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BC_Datastructure
  USE L2BC_DS_Settings, only:   Set_Default_L2BCds_Settings, &
                                Get_Cmdline_Options_L2BCds,  &
                                Print_L2BCds_Settings
  USE EE_CFI_Datatypes, only: IntAuc
  USE ErrorHandler, only: no_error, program_abort
  USE Logging, only: logmsg,log_error
  USE AeolusConstants, only: N_range_gates

  implicit none

  TYPE(L2BC_BRC_DataType) :: L2BC
  TYPE(L2BC_HDR_DataType) :: L2BC_Headers
  logical             :: L2C_flag
  integer             :: error_flag
  integer, parameter  :: N_meas        = 67
!  integer, parameter :: N_Obs         = 4
  integer(IntAuc), parameter :: N_Obs_Mie     = 4
  integer(IntAuc), parameter :: N_Obs_Rayleigh= 4

  print *,"test L2BC_DataStructure"

  !*** command line settings ***

  call Set_Default_L2BCds_Settings(L2C_flag)

  call Get_Cmdline_Options_L2BCds(L2C_flag, error_flag)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_error,"aborted Get_Cmdline_Options_L2BCds")
     call program_abort(error_flag,"Get_Cmdline_Options_L2BCds")
  END IF

  ! print the settings defined in Set_L2BCds_Settings
  ! and any changes added to them by the command line options
  call Print_L2BCds_Settings(L2C_flag)

  !*** initialisations ***

  call Init_L2BC_BRC_DataStructure(L2BC, N_meas, N_Obs_Mie, N_Obs_Rayleigh, &
         int(N_range_gates), error_flag, L2C_flag=L2C_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2BC_BRC_DataStructure")
  print *,"BRC structure is initialised"

  call Delete_L2BC_BRC_DataStructure(L2BC, L2C_flag=L2C_flag)
  print *,"BRC structure is deleted"

  !*** initialisations ***

  call Init_L2BC_HDR_DataStructure(L2BC_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2BC_HDR_DataStructure")
  print *,"HDR structure is initialised"

  call Delete_L2BC_HDR_DataStructure(L2BC_Headers)
  print *,"HDR structure is deleted"

end program TestL2BC_DataStructure
