program TestL2BC_SPH
  !  #[ Documentation
  !
  ! A program to test the L2BC_SPH module.
  !
  ! Written by J. de Kloe (KNMI)
  !
  ! History:
  !    26-Oct-2006  D. Tan       Introduce L2C_flag
  !    24-Apr-2007  J. de Kloe   phase out the SummaryVars code
  !    16-Jan-2008  J. de Kloe   phase out integer kind i_ 
  !
  !  #] 
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BC_SPH, only: L2BC_SPH_type, Init_L2BC_SPH, Print_L2BC_SPH, &
                      GetByteSize_L2B_SPH_KVT
  USE ErrorHandler, only: no_error, program_abort
  USE Logging, only: logmsg,log_error
  USE L2BC_DS_Settings, only: Set_Default_L2BCds_Settings, &
                              Get_Cmdline_Options_L2BCds,  &
                              Print_L2BCds_Settings
  !  #]
  !  #[ variables
  implicit none
  TYPE(L2BC_SPH_type), pointer :: test_L2BC_SPH
  integer :: error_flag, size
  logical :: L2C_flag
  !  #]
  !  #[ program code

  print *,"Test_L2BC_SPH"

  !*** command line settings ***

  call Set_Default_L2BCds_Settings(L2C_flag)

  call Get_Cmdline_Options_L2BCds(L2C_flag, error_flag)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_error,"aborted Get_Cmdline_Options_L2BCds")
     call program_abort(error_flag,"Get_Cmdline_Options_L2BCds")
  END IF

  ! print the settings defined in Set_L2BCds_Settings
  ! and any changes added to them by the command line options
  call Print_L2BCds_Settings(L2C_flag)

  !*** initialisations ***

  nullify(test_L2BC_SPH)

  ! suppress the error handling for this first call, 
  ! (it provokes an error for testing purposes, which would stop this
  !  test program)
  call Init_L2BC_SPH(test_L2BC_SPH, error_flag)
  !if (error_flag .ne. no_error) call program_abort(error_flag,"Init_L2BC_SPH")
  call print_L2BC_SPH(test_L2BC_SPH, L2C_flag,error_flag)
  if (error_flag .ne. no_error) call program_abort(error_flag,"print_L2BC_SPH")

  allocate(test_L2BC_SPH)

  call Init_L2BC_SPH(test_L2BC_SPH, error_flag)
  if (error_flag .ne. no_error) call program_abort(error_flag,"Init_L2BC_SPH")
  call print_L2BC_SPH(test_L2BC_SPH, L2C_flag,error_flag)
  if (error_flag .ne. no_error) call program_abort(error_flag,"print_L2BC_SPH")

  size = GetByteSize_L2B_SPH_KVT()
  print *,"GetByteSize_L2B_SPH_KVT()                = ",size
  size = GetByteSize_L2B_SPH_KVT(L2C_flag=.true.)
  print *,"GetByteSize_L2B_SPH_KVT(L2C_flag=.true.) = ",size

  deallocate(test_L2BC_SPH)

  !  #]
end program TestL2BC_SPH
