program TestL2B_Geoloc_ADS
  !  #[ documentation
  !
  !     Modifications:
  !     14-Nov-2006 J. de Kloe  added tests for the new AsciiWrite routines
  !                             and filled the test of L2B_Geoloc_ADS_type 
  !                             with some numbers to allow proper testing
  !     01-Dec-2006 D. Tan      Change some fill values
  !     15-mar-2007 J. de Kloe  removed use of SummaryVars
  !     23-Apr-2007 J. de Kloe  phase out fillwithzero code
  !     09-Jan-2008 J. de Kloe  removed all old style print routines
  !                             and changed AsciiWrite calls into new style
  !                             print calls
  !     16-Jan-2008 J. de Kloe  phase out integer kind i_ 
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2B_Geoloc_ADS, only: Init_L2B_Geoloc_ADS_module, &
       Geolocation_Bin_type, Init_Geoloc_Height_Bin, &
       Print_Geoloc_Height_Bin, GetByteSize_Geoloc_Height_Bin, &
       Write_Geoloc_Height_Bin, Read_Geoloc_Height_Bin, &
       Geolocation_type, Init_Geolocation, Print_Geolocation, &
       Delete_Geolocation, GetByteSize_Geolocation, Write_Geolocation, &
       Read_Geolocation, L2B_Geoloc_ADS_type, Init_L2B_Geoloc_ADS, &
       Delete_L2B_Geoloc_ADS, Print_L2B_Geoloc_ADS, &
       Write_L2B_Geoloc_ADS, Read_L2B_Geoloc_ADS, &
       GetByteSize_L2B_Geoloc_ADS
  USE DirectBinaryIO, only: Init_DirectBinaryIO_module, &
       OpenDirectBinaryFile, CloseDirectBinaryFile, BigEndian
  USE EE_CFI_Datatypes, only: IntAuc, IntAs, IntAl, FAdoxy, SetDateTime
  USE LunManager, only: fileunit_stdout
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, VerbosityReport
  USE ErrorHandler, only: no_error,  error_opening_file, program_abort
  !  #]
  !  #[ variables and parameters
  implicit none

  TYPE(Geolocation_Bin_type)  :: Test_Bin
  TYPE(Geolocation_type)      :: Test_Geolocation
  integer(IntAl)              :: c_bytesize
  character(len=*), parameter :: filename1 = "testfile1_L2B_Geoloc.bin"
  character(len=*), parameter :: filename2 = "testfile2_L2B_Geoloc.bin"
  character(len=*), parameter :: filename3 = "testfile3_L2B_Geoloc.bin"
  integer                     :: o, r, fileunit, error_flag
  integer(IntAs), parameter   :: N_Range_Gates = 2
  integer(IntAuc)             :: N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write

  TYPE(L2B_Geoloc_ADS_type), pointer :: test_L2B_Geoloc_ADS
  !  #]
  !  #[ initialise 
  error_flag = no_error

  ! this automatically selects the proper byteswapping to be used if needed
  call Init_DirectBinaryIO_module(error_flag,FileByteOrder=BigEndian)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_DirectBinaryIO_module")

  call Init_L2B_Geoloc_ADS_module(ChoosenVerbosity=VerbosityReport)
  !  #]
  !  #[ Tests for the Geolocation_Bin_type
  print *,"==="
  print *,"Testing type: Geolocation_Bin_type"
  call Init_Geoloc_Height_Bin(Test_Bin)
  Test_Bin%Latitude_Start  = 123
  Test_Bin%Longitude_Start = 456
  Test_Bin%Altitude_Bottom = -789
  Test_Bin%LOS_Azimuth     = 123._FAdoxy
  print *,"Print routine start"
  call Print_Geoloc_Height_Bin(Test_Bin, &
                               prefix="Test_Bin", error_flag = error_flag)
  print *,"Print routine end"
  ! size test
  c_bytesize = GetByteSize_Geoloc_Height_Bin()
  print *,"GetByteSize_Geoloc_Height_Bin() = ",c_bytesize
  ! writing test
  call OpenDirectBinaryFile(filename1,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Write_Geoloc_Height_Bin(fileunit,Test_Bin, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Geoloc_Height_Bin")
  call CloseDirectBinaryFile(fileunit)
  ! reading tests
  call OpenDirectBinaryFile(filename1,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  Test_Bin%Latitude_Start  = -111
  Test_Bin%Longitude_Start = -111
  Test_Bin%Altitude_Bottom = -111
  Test_Bin%LOS_Azimuth     =  999._FAdoxy
  call Read_Geoloc_Height_Bin(fileunit,Test_Bin, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Geoloc_Height_Bin")
  print *,"1st reading result for: Geoloc_Height_Bin"
  print *,"Print routine start"
  call Print_Geoloc_Height_Bin(Test_Bin, &
                               prefix="Test_Bin", error_flag = error_flag)
  print *,"Print routine end"

  call CloseDirectBinaryFile(fileunit)  

  print *,"Done testing type: Geolocation_Bin_type"
  !  #]
  !  #[ Tests for the Geolocation_type
  print *,"==="
  print *,"Testing type: Geolocation_type"
  call Init_Geolocation(Test_Geolocation,N_Range_Gates,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Geolocation")
  DO r=1,size(Test_Geolocation%Height_Bin)
     Test_Geolocation%Height_Bin(r)%Latitude_Start  = int( 123 + r,IntAl)
     Test_Geolocation%Height_Bin(r)%Longitude_Start = int( 456 + r,IntAl)
     Test_Geolocation%Height_Bin(r)%Altitude_Bottom = int(-789 + r,IntAl)
     Test_Geolocation%Height_Bin(r)%LOS_Azimuth     = int(123._FAdoxy + r,IntAl)
  END DO
  Test_Geolocation%Lat_of_DEM_Intersection = 33
  Test_Geolocation%Lon_of_DEM_Intersection = 44
  Test_Geolocation%Alt_of_DEM_Intersection = 55
  print *,"Print routine start"
  call Print_Geolocation(Test_Geolocation, &
       prefix="Test_Geolocation", PrintAll=.false., error_flag=error_flag)
  print *,"Print routine end"

  ! size test
  c_bytesize = GetByteSize_Geolocation(Test_Geolocation)
  print *,"GetByteSize_Geolocation() = ",c_bytesize
  ! writing test
  call OpenDirectBinaryFile(filename2,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Write_Geolocation(fileunit, Test_Geolocation, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Geolocation")
  call CloseDirectBinaryFile(fileunit)
  ! reading test
  call OpenDirectBinaryFile(filename2,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_Geolocation(fileunit, Test_Geolocation, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Geolocation")
  print *,"Print routine start"
  call Print_Geolocation(Test_Geolocation, &
       prefix="Test_Geolocation", PrintAll=.false., error_flag=error_flag)
  print *,"Print routine end"

  call CloseDirectBinaryFile(fileunit)

  call Delete_Geolocation(Test_Geolocation)

  print *,"Done testing type: Geolocation_type"

  !  #]
  !  #[ Tests for the L2B_Geoloc_ADS_type
  print *,"==="
  print *,"Testing type: L2B_Geoloc_ADS_type"

  nullify(test_L2B_Geoloc_ADS)
  N_Obs_Mie_to_write      = 5_IntAuc
  N_Obs_Rayleigh_to_write = 5_IntAuc

  print *,"first try, before allocating this MDS"
  print *,"(this should trigger an error...)"
  call Init_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS, &
                           0_IntAs, 0_IntAs, 0_IntAs, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2B_Geoloc_ADS():"
     print *,"error_flag = ",error_flag
     ! ignore this error, since we are testing
     error_flag = no_error
  endif

  print *,"GetByteSize_L2B_Geoloc_ADS() = ",&
       GetByteSize_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS,&
                                  N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write)

  print *,"Print routine start"
  call Print_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS, &
       prefix="test_L2B_Geoloc_ADS",PrintAll=.false., error_flag=error_flag)
  print *,"Print routine end"

  allocate(test_L2B_Geoloc_ADS)

  print *,"next try, after allocating this ADS"

  call Init_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS, &
                           5_IntAs, 5_IntAs, 24_IntAs, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L2B_Geoloc_ADS")

  print *,"Setting N_Obs_Mie_Actual to 2:"
  test_L2B_Geoloc_ADS%N_Obs_Mie_Actual = 2_IntAs

  print *,"Setting N_Obs_Rayleigh_Actual to 3:"
  test_L2B_Geoloc_ADS%N_Obs_Rayleigh_Actual = 3_IntAs

  print *,"GetByteSize_L2B_Geoloc_ADS() = ",&
       GetByteSize_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS,&
                                  N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write)


  ! for a proper test populate the datastruct with some numbers:
  call SetDateTime(test_L2B_Geoloc_ADS%Start_of_Obs_DateTime,&
                   2006,12,31,23,59,29.999_FAdoxy)
  DO o=1,test_L2B_Geoloc_ADS%N_Obs_Mie_Actual
     DO r=1,size(test_L2B_Geoloc_ADS%Mie_Geolocation(o)%Height_Bin)
        test_L2B_Geoloc_ADS%Mie_Geolocation(o)%Height_Bin(r)%&
             Latitude_Start  = int( 123 + r,IntAl)
        test_L2B_Geoloc_ADS%Mie_Geolocation(o)%Height_Bin(r)%&
             Longitude_Start = int( 456 + r,IntAl)
        test_L2B_Geoloc_ADS%Mie_Geolocation(o)%Height_Bin(r)%&
             Altitude_Bottom = int(-789 + r,IntAl)
        test_L2B_Geoloc_ADS%Mie_Geolocation(o)%Height_Bin(r)%&
             LOS_Azimuth     = int( 123._FAdoxy + r,IntAl)
     END DO
     test_L2B_Geoloc_ADS%Mie_Geolocation(o)%&
          Lat_of_DEM_Intersection = int(33+100*o,IntAl)
     test_L2B_Geoloc_ADS%Mie_Geolocation(o)%&
          Lon_of_DEM_Intersection = int(44+100*o,IntAl)
     test_L2B_Geoloc_ADS%Mie_Geolocation(o)%&
          Alt_of_DEM_Intersection = int(55+100*o,IntAl)
  END DO
  DO o=1,test_L2B_Geoloc_ADS%N_Obs_Rayleigh_Actual
     DO r=1,size(test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin)
        test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin(r)%&
             Latitude_Start  = int( 234 + r,IntAl)
        test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin(r)%&
             Longitude_Start = int( 567 + r,IntAl)
        test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin(r)%&
             Altitude_Bottom = int(-654 + r,IntAl)
        test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin(r)%&
             LOS_Azimuth     = int( 137._FAdoxy + r,IntAl)
     END DO
     test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%&
          Lat_of_DEM_Intersection = int(66+100*o,IntAl)
     test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%&
          Lon_of_DEM_Intersection = int(77+100*o,IntAl)
     test_L2B_Geoloc_ADS%Rayleigh_Geolocation(o)%&
          Alt_of_DEM_Intersection = int(88+100*o,IntAl)
  END DO
  test_L2B_Geoloc_ADS%WGS84_to_Geoid_Altitude = 1234_IntAl

  call OpenDirectBinaryFile(filename3,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_L2B_Geoloc_ADS(fileunit,test_L2B_Geoloc_ADS, &
        N_Obs_Mie_to_write,N_Obs_Rayleigh_to_write,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_L2B_Geoloc_ADS")
  call CloseDirectBinaryFile(fileunit)

  print *,"Print routine start"
  call Print_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS, &
       prefix="test_L2B_Geoloc_ADS",PrintAll=.false., error_flag=error_flag)
  print *,"Print routine end"

  call OpenDirectBinaryFile(filename3,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Read_L2B_Geoloc_ADS(fileunit,test_L2B_Geoloc_ADS,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_L2B_Geoloc_ADS")
  call CloseDirectBinaryFile(fileunit)

  print *,"Print routine start"
  call Print_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS, &
!       prefix="test_L2B_Geoloc_ADS",PrintAll=.true., error_flag=error_flag)
       prefix="test_L2B_Geoloc_ADS",PrintAll=.false., error_flag=error_flag)
  print *,"Print routine end"

  print *,"Done testing type: L2B_Geoloc_ADS_type"

  call Delete_L2B_Geoloc_ADS(test_L2B_Geoloc_ADS)
  deallocate(test_L2B_Geoloc_ADS)
  print *,"==="
  !  #]
  print *,"done"
end program TestL2B_Geoloc_ADS
