program TestL2B_PCD_ADS
  !  #[ documentation
  !
  !     Modifications:
  !     09-Mar-2007 J. de Kloe completely reorganised
  !     16-Jan-2008 J. de Kloe phase out integer kind i_ 
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2B_PCD_ADS
  USE DirectBinaryIO, only: Init_DirectBinaryIO_module, &
       OpenDirectBinaryFile, CloseDirectBinaryFile, BigEndian
  USE EE_CFI_Datatypes, only: IntAs, IntAl
  ! support modules
  USE ErrorHandler, only: no_error, error_opening_file, program_abort
  USE c_support, only: get_filesize
  !  #]
  !  #[ variables

  implicit none
  TYPE(L1B_Input_Screening_Type)    :: test_L1B_Input_Screening
  TYPE(L2B_AMD_Screening_Type)      :: test_L2B_AMD_Screening
  TYPE(Mie_Classification_QC_Type)  :: test_Mie_Classification_QC
  TYPE(Rayl_Classification_QC_Type) :: test_Rayl_Classification_QC
  TYPE(Mie_Processing_QC_Type)      :: test_Mie_Processing_QC
  TYPE(Rayleigh_Processing_QC_Type) :: test_Rayleigh_Processing_QC
  TYPE(L2B_PCD_ADS_type),   pointer :: test_L2B_PCD_ADS
  integer         :: fileunit, error_flag, s_actual
  integer(IntAs)  :: N_Meas_to_write
  integer(IntAs)  :: N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write
  integer(IntAs)  :: N_Range_Gates0, N_Meas0
  integer(IntAs)  :: N_Obs_Mie_Max0, N_Obs_Rayleigh_Max0
  integer(IntAl)  :: s
  character(len=*), parameter :: filename_test = "testfile_L2B_pcd.bin"

  !  #]
  print *,"testing l2b_pcd_ads module"
  !  #[ initialise
  print *,"initialise ..."

  error_flag = no_error

  N_Range_Gates0          = 24_IntAs
  N_Meas0                 = 67_IntAs
  N_Meas_to_write         = 50_IntAs
  N_Obs_Mie_Max0          =  5_IntAs
  N_Obs_Mie_to_write      =  3_IntAs
  N_Obs_Rayleigh_Max0     =  5_IntAs
  N_Obs_Rayleigh_to_write =  2_IntAs

  ! this automatically selects the proper byteswapping to be used if needed
  call Init_DirectBinaryIO_module(error_flag,FileByteOrder=BigEndian)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_DirectBinaryIO_module")
  !  #]
  !  #[ Tests for L1B_Input_Screening_Type
  print *,"==="
  print *,"Testing type: L1B_Input_Screening_Type"
  ! init
  print *,"=== test init"
  call Init_L1B_Input_Screening(test_L1B_Input_Screening, &
                                N_Meas0        = N_Meas0, &
                                N_Range_Gates0 = N_Range_Gates0, &
                                error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L1B_Input_Screening")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_L1B_Inp_Screening(fileunit, &
                               test_L1B_Input_Screening, &
                               N_Meas_to_write=N_Meas_to_write, &
                               error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_L1B_Inp_Screening")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_L1B_Input_Screening(test_L1B_Input_Screening)

  ! init again 
  print *,"=== test init"
  call Init_L1B_Input_Screening(test_L1B_Input_Screening, &
                                N_Meas0        = N_Meas_to_write, &
                                N_Range_Gates0 = N_Range_Gates0, &
                                error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L1B_Input_Screening")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_L1B_Inp_Screening(fileunit, test_L1B_Input_Screening, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_L1B_Inp_Screening")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_L1B_Input_Screening(test_L1B_Input_Screening)
  print *,"=== test AsciiWrite"
  call AsciiWrite_L1B_Input_Screening(test_L1B_Input_Screening,&
         fileunit_stdout,&
         prefix="test_L1B_Input_Screening",PrintAll=.false.,&
         error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"AsciiWrite_L1B_Input_Screening")
  ! size test
  print *,"=== test size"
  s = GetByteSize_L1B_Mie_MeasBin_Scr()
  print *,"GetByteSize_L1B_Mie_MeasBin_Scr() = ",s
  s = GetByteSize_L1B_RaylMeasBin_Scr()
  print *,"GetByteSize_L1B_RaylMeasBin_Scr() = ",s
  s = GetByteSize_Mie_Meas_Screening(test_L1B_Input_Screening%Mie_Meas(1))
  print *,"GetByteSize_Mie_Meas_Screening() = ",s
  s = GetByteSize_Rayl_Meas_Screening(test_L1B_Input_Screening%Rayl_Meas(1))
  print *,"GetByteSize_Rayl_Meas_Screening() = ",s
  s = GetByteSize_L1B_MRC_Screening()
  print *,"GetByteSize_L1B_MRC_Screening() = ",s
  s = GetByteSize_L1B_RRC_Screening()
  print *,"GetByteSize_L1B_RRC_Screening() = ",s
  s = GetByteSize_L1B_ZWC_Screening()
  print *,"GetByteSize_L1B_ZWC_Screening() = ",s
  s = GetByteSize_L1B_RBC_Screening()
  print *,"GetByteSize_L1B_RBC_Screening() = ",s
  s = GetByteSize_L1B_Input_Screening(test_L1B_Input_Screening, &
                               N_Meas_to_write=int(N_Meas0,IntAuc))
  print *,"GetByteSize_L1B_Input_Screening( ",N_Meas0," ) = ",s
  s = GetByteSize_L1B_Input_Screening(test_L1B_Input_Screening, &
                               N_Meas_to_write=int(N_Meas_to_write,IntAuc))
  print *,"GetByteSize_L1B_Input_Screening( ",N_Meas_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_L1B_Input_Screening(test_L1B_Input_Screening)

  print *,"Done testing type: L1B_Mie_Meas_Bin_Scr_Type"
  !  #]
  !  #[ Tests for L2B_AMD_Screening_Type
  print *,"==="
  print *,"Testing type: L2B_AMD_Screening_Type"
  ! init
  print *,"=== test init"
  call Init_L2B_AMD_Screening(test_L2B_AMD_Screening, &
                              N_Meas0 = N_Meas0, &
                              error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L2B_AMD_Screening")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_L2B_AMD_Screening(fileunit, &
                               test_L2B_AMD_Screening, &
                               N_Meas_to_write=N_Meas_to_write, &
                               error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_L1B_Inp_Screening")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_L2B_AMD_Screening(test_L2B_AMD_Screening)

  ! init again 
  print *,"=== test init"
  call Init_L2B_AMD_Screening(test_L2B_AMD_Screening, &
                              N_Meas0=N_Meas_to_write, &
                              error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L2B_AMD_Screening")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_L2B_AMD_Screening(fileunit, test_L2B_AMD_Screening, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_L2B_AMD_Screening")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_L2B_AMD_Screening(test_L2B_AMD_Screening)
  print *,"=== test AsciiWrite"
  call AsciiWrite_L2B_AMD_Screening(test_L2B_AMD_Screening,&
                                    fileunit_stdout,&
                                    prefix="test_L2B_AMD_Screening",&
                                    PrintAll=.false.,&
                                    error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"AsciiWrite_L2B_AMD_Screening")
  ! size test
  print *,"=== test size"
  s = GetByteSize_L2B_AMD_Colloc()
  print *,"GetByteSize_L2B_AMD_Colloc() = ",s
  s = GetByteSize_L2B_AMD_Screening(test_L2B_AMD_Screening, &
                               N_Meas_to_write=int(N_Meas0,IntAuc))
  print *,"GetByteSize_L2B_AMD_Screening( ",N_Meas0," ) = ",s
  s = GetByteSize_L2B_AMD_Screening(test_L2B_AMD_Screening, &
                               N_Meas_to_write=int(N_Meas_to_write,IntAuc))
  print *,"GetByteSize_L2B_AMD_Screening( ",N_Meas_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_L2B_AMD_Screening(test_L2B_AMD_Screening)

  print *,"Done testing type: L2B_AMD_Screening_Type"
  !  #]
  !  #[ Tests for Mie_Classification_QC_Type
  print *,"==="
  print *,"Testing type: Mie_Classification_QC_Type"
  ! init
  print *,"=== test init"
  call Init_Mie_Classification_QC(test_Mie_Classification_QC, &
                                  N_Range_Gates0 = N_Range_Gates0, &
                                  N_Meas0        = N_Meas0, &
                                  N_Obs_Mie_Max0 = N_Obs_Mie_Max0, &
                                  error_flag     = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Mie_Classification_QC")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_Mie_Classification_QC(fileunit, &
                                   test_Mie_Classification_QC, &
                                   N_Meas_to_write    = N_Meas_to_write, &
                                   N_Obs_Mie_to_write = N_Obs_Mie_to_write, &
                                   error_flag         = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Mie_Classification_QC")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_Mie_Classification_QC(test_Mie_Classification_QC)

  ! init again 
  print *,"=== test init"
  call Init_Mie_Classification_QC(test_Mie_Classification_QC, &
                                  N_Range_Gates0 = N_Range_Gates0, &
                                  N_Meas0        = N_Meas_to_write, &
                                  N_Obs_Mie_Max0 = N_Obs_Mie_to_write, &
                                  error_flag     = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Mie_Classification_QC")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_Mie_Classification_QC(fileunit, test_Mie_Classification_QC,&
                                  error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Mie_Classification")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_Mie_Classification_QC(test_Mie_Classification_QC)
  print *,"=== test AsciiWrite"
  call AsciiWrite_Mie_Classific_QC(test_Mie_Classification_QC, &
                                   fileunit_stdout,&
                                   prefix="test_Mie_Classification_QC",&
                                   PrintAll=.false.,&
                                   error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"AsciiWrite_Mie_Classific_QC")
  ! size test
  print *,"=== test size"
  s = GetByteSize_L2B_Mie_Bin_Class()
  print *,"GetByteSize_L2B_Mie_Bin_Class() = ",s
  s = GetByteSize_L2B_Mie_Meas_Class(&
       test_Mie_Classification_QC%L2B_Mie_Meas_Classific(1))
  print *,"GetByteSize_L2B_Mie_Meas_Class() = ",s
  s = GetByteSize_L2B_Mie_Obs_Class(&
       test_Mie_Classification_QC%L2B_Mie_Obs_Classific(1))
  print *,"GetByteSize_L2B_Mie_Obs_Class() = ",s
  s = GetByteSize_Mie_Classific_QC(test_Mie_Classification_QC, &
                        N_Meas_to_write    = int(N_Meas0, IntAuc), &
                        N_Obs_Mie_to_write = int(N_Obs_Mie_Max0, IntAuc))
  print *,"GetByteSize_Mie_Classific_QC( ",N_Meas0,N_Obs_Mie_Max0," ) = ",s
  s = GetByteSize_Mie_Classific_QC(test_Mie_Classification_QC, &
                        N_Meas_to_write    = int(N_Meas_to_write, IntAuc), &
                        N_Obs_Mie_to_write = int(N_Obs_Mie_to_write, IntAuc))
  print *,"GetByteSize_Mie_Classific_QC( ",&
                        N_Meas_to_write,N_Obs_Mie_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_Mie_Classification_QC(test_Mie_Classification_QC)

  print *,"Done testing type: Mie_Classification_QC_Type"
  !  #]
  !  #[ Tests for Rayl_Classification_QC_Type
  print *,"==="
  print *,"Testing type: Rayl_Classification_QC_Type"
  ! init
  print *,"=== test init"
  call Init_Rayl_Classification_QC(test_Rayl_Classification_QC, &
                        N_Range_Gates0      = N_Range_Gates0, &
                        N_Meas0             = N_Meas0, &
                        N_Obs_Rayleigh_Max0 = N_Obs_Rayleigh_Max0, &
                        error_flag          = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Rayl_Classification_QC")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_Rayl_Classification_QC(fileunit, &
                        test_Rayl_Classification_QC, &
                        N_Meas_to_write         = N_Meas_to_write, &
                        N_Obs_Rayleigh_to_write = N_Obs_Rayleigh_to_write,&
                        error_flag              = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Rayl_Classification_QC")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_Rayl_Classification_QC(test_Rayl_Classification_QC)

  ! init again 
  print *,"=== test init"
  call Init_Rayl_Classification_QC(test_Rayl_Classification_QC, &
                        N_Range_Gates0      = N_Range_Gates0, &
                        N_Meas0             = N_Meas_to_write, &
                        N_Obs_Rayleigh_Max0 = N_Obs_Rayleigh_to_write, &
                        error_flag          = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Rayl_Classification_QC")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_Rayl_Classification_QC(fileunit, test_Rayl_Classification_QC,&
                                   error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Rayl_Classification")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_Rayl_Classification_QC(test_Rayl_Classification_QC)
  print *,"=== test AsciiWrite"
  call AsciiWrite_Rayl_Classific_QC(test_Rayl_Classification_QC, &
                                    fileunit_stdout,&
                                    prefix="test_Rayl_Classification_QC",&
                                    PrintAll=.false.,&
                                    error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"AsciiWrite_Rayl_Classific_QC")
  ! size test
  print *,"=== test size"
  s = GetByteSize_L2B_Rayl_Bin_Class()
  print *,"GetByteSize_L2B_Rayl_Bin_Class() = ",s
  s = GetByteSize_L2B_Rayl_Meas_Class(&
       test_Rayl_Classification_QC%L2B_Rayl_Meas_Classific(1))
  print *,"GetByteSize_L2B_Rayl_Meas_Class() = ",s
  s = GetByteSize_L2B_Rayl_Obs_Class(&
       test_Rayl_Classification_QC%L2B_Rayl_Obs_Classific(1))
  print *,"GetByteSize_L2B_Rayl_Obs_Class() = ",s
  s = GetByteSize_Rayl_Classific_QC(test_Rayl_Classification_QC, &
               N_Meas_to_write         = int(N_Meas0, IntAuc), &
               N_Obs_Rayleigh_to_write = int(N_Obs_Rayleigh_Max0, IntAuc))
  print *,"GetByteSize_Rayl_Classific_QC( ",&
       N_Meas0,N_Obs_Rayleigh_Max0," ) = ",s
  s = GetByteSize_Rayl_Classific_QC(test_Rayl_Classification_QC, &
               N_Meas_to_write         = int(N_Meas_to_write, IntAuc), &
               N_Obs_Rayleigh_to_write = int(N_Obs_Rayleigh_to_write, IntAuc))
  print *,"GetByteSize_Rayl_Classific_QC( ",&
                        N_Meas_to_write,N_Obs_Rayleigh_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_Rayl_Classification_QC(test_Rayl_Classification_QC)

  print *,"Done testing type: Rayl_Classification_QC_Type"
  !  #]
  !  #[ Tests for Mie_Processing_QC_Type

  print *,"==="
  print *,"Testing type: Mie_Processing_QC_Type"
  ! init
  print *,"=== test init"
  call Init_Mie_Processing_QC(test_Mie_Processing_QC, &
                              N_Obs_Mie_Max0 = N_Obs_Mie_Max0, &
                              N_Range_Gates0 = N_Range_Gates0, &
                              error_flag     = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Mie_Processing_QC")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_Mie_Processing_QC(fileunit, test_Mie_Processing_QC, &
                               N_Obs_Mie_to_write = N_Obs_Mie_to_write,&
                               error_flag         = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Mie_Processing_QC")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_Mie_Processing_QC(test_Mie_Processing_QC)

  ! init again 
  print *,"=== test init"
  call Init_Mie_Processing_QC(test_Mie_Processing_QC, &
                              N_Obs_Mie_Max0 = N_Obs_Mie_to_write, &
                              N_Range_Gates0 = N_Range_Gates0, &
                              error_flag     = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Mie_Processing_QC")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_Mie_Processing_QC(fileunit, test_Mie_Processing_QC,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Mie_Processing_QC")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_Mie_Processing_QC(test_Mie_Processing_QC)
  print *,"=== test AsciiWrite"
  call AsciiWrite_Mie_Processing_QC(test_Mie_Processing_QC, &
                                    fileunit_stdout,&
                                    prefix="test_Mie_Processing_QC",&
                                    PrintAll=.false.,&
                                    error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"AsciiWrite_Mie_Processing_QC")
  ! size test
  print *,"=== test size"
  s = GetByteSize_Obs_Mie_Bin_QC()
  print *,"GetByteSize_Obs_Mie_Bin_QC() = ",s
  s = GetByteSize_Obs_Mie_QC(test_Mie_Processing_QC%Obs_Mie_QC(1))
  print *,"GetByteSize_Obs_Mie_QC() = ",s
  s = GetByteSize_Mie_Processing_QC(test_Mie_Processing_QC,&
               N_Obs_Mie_to_write = int(N_Obs_Mie_Max0, IntAuc))
  print *,"GetByteSize_Mie_Processing_QC(",N_Obs_Mie_Max0," ) = ",s
  s = GetByteSize_Mie_Processing_QC(test_Mie_Processing_QC,&
                                    N_Obs_Mie_to_write = N_Obs_Mie_to_write)
  print *,"GetByteSize_Mie_Processing_QC(",N_Obs_Mie_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_Mie_Processing_QC(test_Mie_Processing_QC)

  print *,"Done testing type: Mie_Processing_QC_Type"

  !  #]
  !  #[ Tests for Rayleigh_Processing_QC_Type
  print *,"==="
  print *,"Testing type: Rayleigh_Processing_QC_Type"
  ! init
  print *,"=== test init"
  call Init_Rayleigh_Processing_QC(test_Rayleigh_Processing_QC, &
                                   N_Obs_Rayleigh_Max0 = N_Obs_Rayleigh_Max0, &
                                   N_Range_Gates0      = N_Range_Gates0, &
                                   error_flag          = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Rayleigh_Processing_QC")
  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_Rayleigh_Processing_QC(fileunit, test_Rayleigh_Processing_QC, &
                       N_Obs_Rayleigh_to_write = N_Obs_Rayleigh_to_write,&
                       error_flag              = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_Rayleigh_Processing_QC")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_Rayleigh_Processing_QC(test_Rayleigh_Processing_QC)

  ! init again 
  print *,"=== test init"
  call Init_Rayleigh_Processing_QC(test_Rayleigh_Processing_QC, &
                           N_Obs_Rayleigh_Max0 = N_Obs_Rayleigh_to_write, &
                           N_Range_Gates0      = N_Range_Gates0, &
                           error_flag          = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_Rayleigh_Processing_QC")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_Rayleigh_Processing_QC(fileunit, &
                                   test_Rayleigh_Processing_QC,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_Rayleigh_Processing_QC")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_Rayleigh_Processing_QC(test_Rayleigh_Processing_QC)
  print *,"=== test AsciiWrite"
  call AsciiWrite_Rayl_Processing_QC(test_Rayleigh_Processing_QC, &
                                     fileunit_stdout,&
                                     prefix="test_Rayleigh_Processing_QC",&
                                     PrintAll=.false.,&
                                     error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,&
                          "AsciiWrite_Rayl_Processing_QC")
  ! size test
  print *,"=== test size"
  s = GetByteSize_Obs_Rayleigh_Bin_QC()
  print *,"GetByteSize_Obs_Rayleigh_Bin_QC() = ",s
  s = GetByteSize_Obs_Rayleigh_QC(&
                test_Rayleigh_Processing_QC%Obs_Rayleigh_QC(1))
  print *,"GetByteSize_Obs_Rayleigh_QC() = ",s
  s = GetByteSize_Rayl_Processing_QC(test_Rayleigh_Processing_QC,&
               N_Obs_Rayleigh_to_write = int(N_Obs_Rayleigh_Max0, IntAuc))
  print *,"GetByteSize_Rayleigh_Processing_QC(",N_Obs_Rayleigh_Max0," ) = ",s
  s = GetByteSize_Rayl_Processing_QC(test_Rayleigh_Processing_QC,&
                           N_Obs_Rayleigh_to_write = N_Obs_Rayleigh_to_write)
  print *,"GetByteSize_Rayleigh_Processing_QC(",&
                     N_Obs_Rayleigh_to_write," ) = ",s

  print *,"=== test delete"
  call Delete_Rayleigh_Processing_QC(test_Rayleigh_Processing_QC)

  print *,"Done testing type: Rayleigh_Processing_QC_Type"
  !  #]
  !  #[ Tests for L2B_PCD_ADS_type
  print *,"==="
  print *,"Testing type: L2B_PCD_ADS_type"
  ! init

  print *,"=== test init nulpointer"
  nullify(test_L2B_PCD_ADS)

  print *,"first try, before allocating this ADS"
  print *,"(this should trigger an error...)"
  call Init_L2B_PCD_ADS(test_L2B_PCD_ADS, &
                        N_Meas0, N_Obs_Mie_Max0, N_Obs_Rayleigh_Max0, &
                        N_Range_Gates0, error_flag)
  ! this error was provoked, so ignore it
  error_flag = no_error
  !if (error_flag .ne. no_error) &
  !     call program_abort(error_opening_file,"Init_L2B_PCD_ADS")

  print *,"=== test printing nulpointer"
  print *,"printing ProdConfData_ADS:"
  call print_L2B_PCD_ADS(test_L2B_PCD_ADS)
  if (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"print_L2B_PCD_ADS")

  allocate(test_L2B_PCD_ADS)
  print *,"next try, after allocating this ADS"

  print *,"=== test init"
  call Init_L2B_PCD_ADS(test_L2B_PCD_ADS, &
                        N_Meas0, N_Obs_Mie_Max0, N_Obs_Rayleigh_Max0, &
                        N_Range_Gates0, error_flag)
  if (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L2B_PCD_ADS")

  ! writing test
  print *,"=== test write"
  call OpenDirectBinaryFile(filename_test,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call Write_L2B_PCD_ADS(fileunit,test_L2B_PCD_ADS,N_Meas_to_write,&
                  N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Write_L2B_PCD_ADS")
  call CloseDirectBinaryFile(fileunit)
  print *,"=== test delete"
  call Delete_L2B_PCD_ADS(test_L2B_PCD_ADS)

  ! init again 
  print *,"=== test init"
  call Init_L2B_PCD_ADS(test_L2B_PCD_ADS, &
                        N_Meas_to_write, N_Obs_Mie_to_write, &
                        N_Obs_Rayleigh_to_write, &
                        N_Range_Gates0, error_flag)
  if (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Init_L2B_PCD_ADS")

  ! reading test
  print *,"=== test read"
  call OpenDirectBinaryFile(filename_test,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  call Read_L2B_PCD_ADS(fileunit,test_L2B_PCD_ADS,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"Read_L2B_PCD_ADS")
  call CloseDirectBinaryFile(fileunit)
  ! print test
!  print *,"=== test print"
!  call Print_L2B_PCD_ADS(test_L2B_PCD_ADS)
  print *,"=== test AsciiWrite"
  call AsciiWrite_L2B_PCD_ADS(test_L2B_PCD_ADS, &
                              fileunit_stdout,&
                              prefix="test_L2B_PCD_ADS",&
                              PrintAll=.false.,&
                              error_flag=error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,&
                          "AsciiWrite_L2B_PCD_ADS")
  ! size test
  print *,"=== test size"
  s = GetByteSize_L2B_PCD_ADS(test_L2B_PCD_ADS, &
          N_Meas0,  N_Obs_Mie_Max0, N_Obs_Rayleigh_Max0)
  print *,"GetByteSize_L2B_PCD_ADS( ",&
       N_Meas0,  N_Obs_Mie_Max0, N_Obs_Rayleigh_Max0," ) = ",s

  print *,"=== test size"
  s = GetByteSize_L2B_PCD_ADS(test_L2B_PCD_ADS, &
          N_Meas_to_write, N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write)
  print *,"GetByteSize_L2B_PCD_ADS( ",N_Meas_to_write,&
       N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write," ) = ",s

  s_actual = get_filesize(filename_test)
  print *,"Actual filesize for test file: ",s_actual

  if (s .ne. s_actual) then
     print *,"ERROR in TestL2B_PCD_ADS():"
     print *,"   GetByteSize_L2B_PCD_ADS() result is not consistent"
     print *,"   with the actual file written by Write_L2B_PCD_ADS() !!!"
     print *,"   GetByteSize_L2B_PCD_ADS() reports: ",s
     print *,"   Write_L2B_PCD_ADS()       reports: ",s_actual
     print *,"   This is a programming error, please fix it before"
     print *,"   releasing the current software branch ..."
     stop 1
  end if

  print *,"=== test delete"
  call Delete_L2B_PCD_ADS(test_L2B_PCD_ADS)
  deallocate(test_L2B_PCD_ADS)

  print *,"Done testing type: Rayleigh_Processing_QC_Type"
  !  #]
  print *,"test ended succesfull"

end program TestL2B_PCD_ADS
