program Test_L2B_Proc_Settings
  !  #[ Documentation
  !
  ! a little test program for L2B_Proc_Settings
  !
  ! Written by J. de Kloe (KNMI)
  ! Created:     09-11-2005
  !
  ! Modifications:
  !      12-May-2006  D. Tan      Null use of ParKind1 module
  !      09-Aug-2006  J. de Kloe  interface changes: added parameter
  !                               L2BP_Settings and split get_from_commandline 
  !      16-Aug-2006  D. Tan      Allow a particular error code as an exception
  !      03-Oct-2007  J. de kloe  corrected an error string 
  !      16-Jan-2008  J. de Kloe  phase out integer kind i_ 
  !
  !  #] 
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2B_Proc_Settings, only: L2BP_Settings_type, &
                               Init_L2B_Proc_Settings, &
                               Delete_L2B_Proc_Settings, &
                               Get_JobOrderName_from_CmdLine, &
                               Get_Additional_Cmdline_Options, &
                               Set_Derived_L2B_Proc_Settings, &
                               Print_L2B_Proc_Settings
  USE L2B_AuxPar_DataStructure, only: Init_L2B_AuxPar_Data, &
                                      Delete_L2B_AuxPar!, Print_L2B_AuxPar
  USE ErrorHandler, only: no_error, program_abort, error_allocate, &
                          request_program_to_stop
  !  #]
  !  #[ Variables
  implicit none
  integer                  :: error_flag, AllocStatus
  TYPE(L2BP_Settings_type) :: L2BP_Settings
  character(len=256)       :: filename_JobOrder
  !  #]
  !  #[ program code
  call Init_L2B_Proc_Settings(L2BP_Settings,error_flag)
  IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Init_L2B_Proc_Settings")

  ! allocate this struct (otherwise running the Set_Derived_L2B_Proc_Settings
  ! routine is not possible, since that tries to read some fields from it.
  allocate(L2BP_Settings%L2B_AuxPar%Data,STAT=AllocStatus)
  IF (AllocStatus .ne. 0) THEN
     error_flag = error_allocate
     call program_abort(error_flag,&
          "Allocation problem in Init_L2B_Proc_Settings")
  END IF

  call Init_L2B_AuxPar_Data(L2BP_Settings%L2B_AuxPar%Data, error_flag)
  IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Init_L2B_AuxPar_Data")


  !call Print_L2B_AuxPar(L2BP_Settings%L2B_AuxPar, &
  !                      "L2BP_Settings%L2B_AuxPar", error_flag)

  call Get_JobOrderName_from_CmdLine(filename_JobOrder,error_flag)
  IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Get_JobOrderName_from_CmdLine")

  ! It is expected that the input arguments to 
  ! this test program include -h, so that the following call
  ! will print the command line options, in which case error_flag
  ! will return with a value request_program_to_stop.
  !
  call Get_Additional_Cmdline_Options(L2BP_Settings,error_flag)
  !
  ! On return, the usual test should be
  ! IF (error_flag .ne.  no_error) &
  !     call program_abort(error_flag,"Get_Additional_Cmdline_Options")
  !
  ! However, as an exception in this test program, a return value
  ! of request_program_to_stop is allowed for two reasons.
  ! The first is to allow testing of subsequent subroutine calls.
  ! The second relates to a portability problem: the pgf90 compiler
  ! differs from other compilers in that a program_abort
  ! results in "FORTRAN STOP" appearing in stderr.
  !
  IF (error_flag .ne.  request_program_to_stop) THEN
     IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Get_Additional_Cmdline_Options")
  ENDIF

  call Set_Derived_L2B_Proc_Settings(L2BP_Settings,error_flag)
  IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Set_Derived_L2B_Proc_Settings")

  call Print_L2B_Proc_Settings(L2BP_Settings,"L2BP_Settings",error_flag)
  IF (error_flag .ne.  no_error) &
       call program_abort(error_flag,"Print_L2B_Proc_Settings")

  call Delete_L2B_Proc_Settings(L2BP_Settings)
  !  #]
end program Test_L2B_Proc_Settings
