program TestL2B_Rayleigh_MDS
  !  #[ Description
  !     a simple test program to test the l2b_rayleigh_mds module
  !     created by: J. de Kloe, KNMI 
  !
  !     Modifications: 
  !       16-Jan-2008 J. de Kloe phase out integer kind i_ 
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2B_Rayleigh_MDS, only: L2B_Rayleigh_MDS_type, Init_L2B_Rayleigh_MDS, &
                              Print_L2B_Rayleigh_MDS, Delete_L2B_Rayleigh_MDS,&
                              Get_ByteSize_Rayl_Height_Bin, &
                              Get_ByteSize_Rayleigh_Profile, &
                              Get_ByteSize_L2B_Rayleigh_MDS
  USE EE_CFI_Datatypes, only: IntAs, IntAl, IntAuc
  USE ErrorHandler, only: no_error  

  !  #]
  !  #[ variables
  implicit none
  TYPE(L2B_Rayleigh_MDS_type), pointer :: test_L2B_Rayleigh_MDS
  integer :: error_flag

  integer(IntAl) :: c_bytesize
  !  #]
  !  #[ program code
  nullify(test_L2B_Rayleigh_MDS)

  print *,"first try, before allocating this MDS"
  print *,"(this should trigger an error...)"
  call Init_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS, &
                             0_IntAs, 0_IntAs, 0_IntAs, &
                             error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2B_Rayleigh_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2B_Rayleigh_MDS():"
     print *,"error_flag = ",error_flag
  endif

  allocate(test_L2B_Rayleigh_MDS)

  print *,"next try, after allocating this MDS"

  call Init_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS, &
                             67_IntAs, 5_IntAs, 24_IntAs, error_flag)
!                             77_IntAs, 5_IntAs, 24_IntAs, error_flag)
!                             27_IntAs, 5_IntAs, 18_IntAs, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2B_Rayleigh_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS, error_flag, &
                              PrintAll=.true.)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2B_Rayleigh_MDS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"Setting N-Obs_Actual to 1:"
  test_L2B_Rayleigh_MDS%N_Obs_Rayleigh_Actual=1_IntAs
  
  call print_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS, error_flag, &
                              PrintAll=.true.)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2B_Rayleigh_MDS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"reporting size:"

  c_bytesize = Get_ByteSize_Rayl_Height_Bin()
      ! test_L2B_Rayleigh_MDS%Rayleigh_Profile(1)%Height_Bin(1))
  print *,"c_bytesize for Rayleigh_Height_Bin = ",c_bytesize

  c_bytesize = Get_ByteSize_Rayleigh_Profile(&
                        test_L2B_Rayleigh_MDS%Rayleigh_Profile(1))
  print *,"c_bytesize for Rayleigh_Profile = ",c_bytesize

  c_bytesize = Get_ByteSize_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS,&
                                             67_IntAuc,5_IntAuc)
  print *,"c_bytesize for Rayleigh_MDS = ",c_bytesize

  print *,"done"

  call Delete_L2B_Rayleigh_MDS(test_L2B_Rayleigh_MDS)
  deallocate(test_L2B_Rayleigh_MDS)
  !  #]
end program TestL2B_Rayleigh_MDS
