program TestL2C_AssimPCD_ADS
  !  #[ Description
  !     a simple test program to test the L2C_AssimPCD_ADS module
  !     created:     17-Oct-2006, by: D. Tan, ECMWF
  !     based on:    Test_L2B_PCD_ADS.F90 by J. de Kloe, KNMI
  !
  !     Modifications:
  !       17-Oct-2006 D. Tan     Original version
  !       16-Jan-2008 J. de Kloe phase out integer kind i_
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2C_AssimPCD_ADS
  USE EE_CFI_Datatypes, only: IntAs, IntAl
  USE ErrorHandler, only: no_error  
  !  #]
  !  #[ variables
  implicit none
  TYPE(L2C_AssimPCD_ADS_type), pointer :: test_L2C_AssimPCD_ADS
  integer        :: error_flag
  integer(IntAl) :: s
  character(len=20)  :: prefix
  logical  :: PrintAll
  !  #]
  !  #[ program code
  print *,"testing L2C_AssimPCD_ADS"

  print *,"initialise ..."

  prefix="L2C_AssimPCD_ADS"
  PrintAll=.false.

  nullify(test_L2C_AssimPCD_ADS)

  print *,"first try, before allocating this ADS"
  print *,"(this should trigger an error...)"
  call Init_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS, &
                        0_IntAs, 0_IntAs, 0_IntAs, 0_IntAs, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2C_AssimPCD_ADS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"printing AssimProdConfData_ADS:"
  call print_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS, &
                              prefix, PrintAll, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2C_AssimPCD_ADS():"
     print *,"error_flag = ",error_flag
  endif

  allocate(test_L2C_AssimPCD_ADS)
  print *,"next try, after allocating this ADS"

  call Init_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS, &
                        67_IntAs, 5_IntAs, 5_IntAs, 24_IntAs, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2C_AssimPCD_ADS():"
     print *,"error_flag = ",error_flag
  endif

  call print_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS, &
                              prefix, PrintAll, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2C_AssimPCD_ADS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"testing sizes of the datastructure"
  print *,"------------------------------------------"
  print *,"==> sizes inside L2B_Input_Screening:"
  s = GetByteSize_L2B_Input_Screening()
  print *,"GetByteSize_L2B_Input_Screening() = ",s
  print *,"------------------------------------------"
  print *,"==> sizes in Assim_Model_PCD, "//&
          "common to [Mie,Rayleigh]_Quality_Params:"
  s = GetByteSize_HLOS_Obs_Errors()
  print *,"GetByteSize_HLOS_Obs_Errors()         = ",s
  s = GetByteSize_Assim_Model_PCD()
  print *,"GetByteSize_Assim_Model_PCD()         = ",s
  print *,"------------------------------------------"
  print *,"==> sizes inside Mie_Quality_Params:"
  s = GetByteSize_L2B_Mie_Obs_Scr()
  print *,"GetByteSize_L2B_Mie_Obs_Scr()         = ",s
  s = GetByteSize_L2C_Mie_Bin_QC()
  print *,"GetByteSize_L2C_Mie_Bin_QC()          = ",s
  s = GetByteSize_L2C_Mie_Qual_Params(&
       test_L2C_AssimPCD_ADS%L2C_Mie_Quality_Params(1))
  print *,"GetByteSize_L2C_Mie_Qual_Params()     = ",s
  print *,"------------------------------------------"
  print *,"==> sizes inside Rayleigh_Quality_Params:"
  s = GetByteSize_L2B_Rayl_Obs_Scr()
  print *,"GetByteSize_L2B_Rayl_Obs_Scr()        = ",s
  s = GetByteSize_L2C_Rayleigh_Bin_QC()
  print *,"GetByteSize_L2C_Rayleigh_Bin_QC()     = ",s
  s = GetByteSize_L2C_Ray_Qual_Params(&
       test_L2C_AssimPCD_ADS%L2C_Rayleigh_Quality_Params(1))
  print *,"GetByteSize_L2C_Ray_Qual_Params()     = ",s
  print *,"------------------------------------------"
  s = GetByteSize_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS)
  print *,"GetByteSize_L2C_AssimPCD_ADS()    = ",s
  print *,"------------------------------------------"

  print *,"deallocating datastructure"

  call Delete_L2C_AssimPCD_ADS(test_L2C_AssimPCD_ADS)
  deallocate(test_L2C_AssimPCD_ADS)

  print *,"test ended succesfully"
  !  #]
end program TestL2C_AssimPCD_ADS
