program TestL2C_RayleighVec_MDS
  !  #[ Documentation
  !
  !     Test program for the L2C_RayleighVec_MDS module
  !
  !     Written by David Tan, ECMWF
  !     Based on Test_L2B_Mie_MDS.F90 by  J. de Kloe, KNMI 
  !                              and P. Poli,    Meteo-France
  !     Modifications:
  !      04-Jul-2006 David Tan  Original version
  !      05-Mar-2007 J. de Kloe added tests for the new GetBytesize functions
  !      16-Jan-2008 J. de Kloe phase out integer kind i_
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2C_RayleighVec_MDS, only: L2C_RayleighVec_MDS_type, &
                                 Init_L2C_RayleighVec_MDS, &
                                 Print_L2C_RayleighVec_MDS, &
                                 Delete_L2C_RayleighVec_MDS, &
                                 Get_ByteSize_L2B_RaylVec_H_Bin, &
                                 Get_ByteSize_L2B_RaylVec_Prof, &
                                 Get_ByteSize_L2C_RaylVec_MDS
  USE EE_CFI_Datatypes, only: IntAs, IntAl, IntAuc
  USE ErrorHandler, only: no_error  
  !  #]
  !  #[ variables
  implicit none
  TYPE(L2C_RayleighVec_MDS_type), pointer :: test_L2C_RayleighVec_MDS
  integer        :: error_flag
  integer(IntAl) :: c_bytesize
  !  #]
  !  #[ program code
  nullify(test_L2C_RayleighVec_MDS)

  print *,"first try, before allocating this MDS"
  print *,"(this should trigger an error...)"
  call Init_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS, &
                             0_IntAs, 0_IntAs, 0_IntAs, &
                             error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2C_RayleighVec_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2C_RayleighVec_MDS():"
     print *,"error_flag = ",error_flag
  endif

  allocate(test_L2C_RayleighVec_MDS)

  print *,"next try, after allocating this MDS"

  call Init_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS, &
                             67_IntAs, 5_IntAs, 24_IntAs, error_flag)
!                             77_IntAs, 5_IntAs, 24_IntAs, error_flag)
!                             27_IntAs, 5_IntAs, 18_IntAs, error_flag)
  if (error_flag .ne. no_error) then
     print *,"error reported by Init_L2C_RayleighVec_MDS():"
     print *,"error_flag = ",error_flag
  endif

  call print_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS, error_flag, &
                              PrintAll=.true.)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2C_RayleighVec_MDS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"Setting N-Obs_Actual to 1:"
  test_L2C_RayleighVec_MDS%N_Obs_Rayleigh_Actual=1_IntAs
  
  call print_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS, error_flag, &
                              PrintAll=.true.)
  if (error_flag .ne. no_error) then
     print *,"error reported by Print_L2C_RayleighVec_MDS():"
     print *,"error_flag = ",error_flag
  endif

  print *,"reporting size:"

  c_bytesize = Get_ByteSize_L2B_RaylVec_H_Bin()
  print *,"c_bytesize for L2C_Height_Bin_VecWind_type = ",c_bytesize

  c_bytesize = Get_ByteSize_L2B_RaylVec_Prof(&
                       test_L2C_RayleighVec_MDS%RayleighVec_Profile(1))

  print *,"c_bytesize for RayleighVec_Profile_type    = ",c_bytesize

  c_bytesize = Get_ByteSize_L2C_RaylVec_MDS(test_L2C_RayleighVec_MDS,&
                                            N_Rayleigh_Obs_to_write=5_IntAuc)
  print *,"c_bytesize for L2C_RayleighVec_MDS         = ",c_bytesize

  print *,"done"

  call Delete_L2C_RayleighVec_MDS(test_L2C_RayleighVec_MDS)
  deallocate(test_L2C_RayleighVec_MDS)
  !  #]
end program TestL2C_RayleighVec_MDS
