program Test_RBC_DataStructure
  !  #[ Description
  !     a simple test program to test the RBC_DataStructure module
  !     created:     15-02-2006, by: J. de Kloe
  !
  !     Modifications:
  !      12-May-2006 D. Tan     Null use of ParKind1 module
  !      16-Jan-2008 J. de Kloe phase out integer kind i_
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE RBC_DataStructure, only: RBC_DataType, Init_RBC_DataStructure,&
       Print_RBC_DataStructure, Delete_RBC_DataStructure, &
       GetByteSize_RBC_DataStruct_KVT, &
       Init_Spec_Grid_TF, Spec_Grid_TF_type, Delete_Spec_Grid_TF, &
       GetByteSize_Spec_Grid_TF, Print_Spec_Grid_TF, &
       Spec_Grid_PTF_type, Init_Spec_Grid_PTF, GetByteSize_Spec_Grid_PTF, &
       Delete_Spec_Grid_PTF, Print_Spec_Grid_PTF, &
       Fcalib_TR_type, GetByteSize_Fcalib_TR, Init_Fcalib_TR, &
       Delete_Fcalib_TR, Print_Fcalib_TR, &
       Init_Fcalib_PTR, Print_Fcalib_PTR, &
       Fcalib_PTR_type, Delete_Fcalib_PTR, GetByteSize_Fcalib_PTR, &
       Init_Nab_TFd, Nab_TFd_type, Delete_Nab_TFd, &
       GetByteSize_Nab_TFd, Print_Nab_TFd, &
       Nab_PTFd_type, Init_Nab_PTFd, GetByteSize_Nab_PTFd, &
       Delete_Nab_PTFd, Print_Nab_PTFd, &
       RBC_DS_type, Init_RBC_DS, Print_RBC_DS, Delete_RBC_DS, &
       GetByteSize_RBC_DS
  USE EE_CFI_Datatypes, only: IntAl, IntAs
  USE ErrorHandler, only: no_error, program_abort, error_allocate
  !  #]
  !  #[ variables
  implicit none

  TYPE(Spec_Grid_TF_type)  :: Spec_Grid_TF
  TYPE(Spec_Grid_PTF_type) :: Spec_Grid_PTF
  TYPE(Fcalib_TR_type)     :: Fcalib_TR
  TYPE(Fcalib_PTR_type)    :: Fcalib_PTR
  TYPE(Nab_TFd_type)       :: Nab_TFd
  TYPE(Nab_PTFd_type)      :: Nab_PTFd
  TYPE(RBC_DS_type)        :: RBC_DS
  TYPE(RBC_DataType)       :: RBC
  integer                  :: error_flag, AllocStatus
  integer(IntAs)           :: Num_F, Num_T, Num_RR, Num_P, Num_fd, Num_FP
  integer(IntAl)           :: c_bytesize
  !  #]
  !  #[ code to test the Spec_Grid_TF_type
  print *,"--------------------------"
  print *,"testing: Spec_Grid_TF_type"

  Num_F = 937
  call Init_Spec_Grid_TF(Spec_Grid_TF, Num_F, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Spec_Grid_TF")

  call Print_Spec_Grid_TF(Spec_Grid_TF,"Spec_Grid_TF",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Spec_Grid_TF")

  c_bytesize = GetByteSize_Spec_Grid_TF(Spec_Grid_TF)
  print *,"GetByteSize_Spec_Grid_TF(Spec_Grid_TF) = ",c_bytesize

  call Delete_Spec_Grid_TF(Spec_Grid_TF)
  !  #]
  !  #[ code to test the Spec_Grid_PTF_type
  print *,"---------------------------"
  print *,"testing: Spec_Grid_PTF_type"

  Num_T = 161
  call Init_Spec_Grid_PTF(Spec_Grid_PTF, Num_T, Num_F, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Spec_Grid_PTF")

  call Print_Spec_Grid_PTF(Spec_Grid_PTF,"Spec_Grid_PTF",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Spec_Grid_PTF")

  c_bytesize = GetByteSize_Spec_Grid_PTF(Spec_Grid_PTF)
  print *,"GetByteSize_Spec_Grid_PTF(Spec_Grid_PTF) = ",c_bytesize

  call Delete_Spec_Grid_PTF(Spec_Grid_PTF)
  !  #]
  !  #[ code to test the Fcalib_TR_type
  print *,"--------------------------"
  print *,"testing: Fcalib_TR_type"

  Num_RR = 101
  call Init_Fcalib_TR(Fcalib_TR, Num_RR, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Fcalib_TR")

  call Print_Fcalib_TR(Fcalib_TR,"Fcalib_TR",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Fcalib_TR")

  c_bytesize = GetByteSize_Fcalib_TR(Fcalib_TR)
  print *,"GetByteSize_Fcalib_TR(Fcalib_TR) = ",c_bytesize

  call Delete_Fcalib_TR(Fcalib_TR)
  !  #]
  !  #[ code to test the Fcalib_TR_type
  print *,"--------------------------"
  print *,"testing: Fcalib_PTR_type"

  call Init_Fcalib_PTR(Fcalib_PTR, Num_T, Num_RR, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Fcalib_PTR")

  call Print_Fcalib_PTR(Fcalib_PTR,"Fcalib_PTR",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Fcalib_PTR")

  c_bytesize = GetByteSize_Fcalib_PTR(Fcalib_PTR)
  print *,"GetByteSize_Fcalib_PTR(Fcalib_PTR) = ",c_bytesize

  call Delete_Fcalib_PTR(Fcalib_PTR)
  !  #]
  !  #[ code to test the Nab_TFd_type
  print *,"--------------------------"
  print *,"testing: Nab_TFd_type"

  Num_fd = 61
  call Init_Nab_TFd(Nab_TFd, Num_fd, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Nab_TFd")

  call Print_Nab_TFd(Nab_TFd,"Nab_TFd",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Nab_TFd")

  c_bytesize = GetByteSize_Nab_TFd(Nab_TFd)
  print *,"GetByteSize_Nab_TFd(Nab_TFd) = ",c_bytesize

  call Delete_Nab_TFd(Nab_TFd)
  !  #]
  !  #[ code to test the Nab_PTFd_type
  print *,"---------------------------"
  print *,"testing: Nab_PTFd_type"

  Num_fd = 61
  Num_T = 161
  call Init_Nab_PTFd(Nab_PTFd, Num_T, Num_fd, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Nab_PTFd")

  call Print_Nab_PTFd(Nab_PTFd,"Nab_PTFd",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_Nab_PTFd")

  c_bytesize = GetByteSize_Nab_PTFd(Nab_PTFd)
  print *,"GetByteSize_Nab_PTFd(Nab_PTFd) = ",c_bytesize

  call Delete_Nab_PTFd(Nab_PTFd)
  !  #]
  !  #[ code to test the RBC_DS_type
  print *,"--------------------------"
  print *,"testing: RBC_DS_type"

  Num_P  = 23
  Num_fd = 61
  Num_FP = 877
  call Init_RBC_DS(RBC_DS, Num_P, Num_T, Num_F, Num_FP, Num_fd, Num_RR, &
                   error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DS")

  ! WARNING: PrintAll=.true. will generate about 274 MB ascii output !!!
  call Print_RBC_DS(RBC_DS,"RBC_DS",error_flag,PrintAll=.false.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_RBC_DS")

  c_bytesize = GetByteSize_RBC_DS(RBC_DS)
  print *,"GetByteSize_RBC_DS(RBC_DS) = ",c_bytesize

  call Delete_RBC_DS(RBC_DS)
  !  #]
  !  #[ code to test the RBC_DataStructure
  print *,"testing: RBC_DataStructure"
  call Init_RBC_DataStructure(RBC, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DataStructure")
  print *,"structure is initialised"

  allocate(RBC%RBC_DS,STAT=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_allocate,"Test_RBC_DataStructure")

  call Init_RBC_DS(RBC%RBC_DS, Num_P, Num_T, Num_F, Num_FP, Num_fd, Num_RR, &
                   error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DS")

  call Print_RBC_DataStructure(RBC, "RBC", error_flag,PrintAll=.false.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Print_RBC_DataStructure")

  c_bytesize = GetByteSize_RBC_DataStruct_KVT(RBC)
  print *,"GetByteSize_RBC_DataStruct_KVT(RBC) = ",c_bytesize

  call Delete_RBC_DataStructure(RBC)
  print *,"structure is deleted"
  !  #]
end program Test_RBC_DataStructure
