program Test_Working_DataStructure
  !  #[ Description
  !     a simple test program to test the Working_DataStructure module
  !     created:     15-Aug-2006, by: J. de Kloe, KNMI
  !
  !     Modifications:
  !       16-Jan-2008 J. de Kloe phase out integer kind i_
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Working_Datastructure, only: WorkingDataType, InitWorkingData, &
       DeleteWorkingData, PrintMieRangeGate, PrintMieMeasurement, &
       PrintRayleighRangeGate, PrintRayleighMeasurement, PrintWorkingData
  USE EE_CFI_Datatypes, only: IntAs
  USE ErrorHandler, only: no_error, program_abort
  !  #]
  !  #[ variables and parameters
  implicit none

  TYPE(WorkingDataType)     :: WorkingData
  integer                   :: error_flag
  integer(IntAs), parameter :: N_Meas             = 67_IntAs
  integer(IntAs), parameter :: N_Obs_Mie_Max      =  5_IntAs
  integer(IntAs), parameter :: N_Obs_Rayleigh_Max =  3_IntAs
  integer(IntAs), parameter :: N_Range_Gates      = 24_IntAs
  !  #]
  !  #[ program code
  print *,"--------------------------"
  print *,"test Working_DataStructure"
  print *,"--------------------------"

  call InitWorkingData(WorkingData, N_range_gates, N_Meas, &
                       N_Obs_Mie_Max, N_Obs_Rayleigh_Max, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitWorkingData")
  print *,"structure is initialised"

  print *,"--------------------------"
  print *,"testing PrintMieRangeGate:"
  print *,"--------------------------"
  call PrintMieRangeGate(WorkingData%MieMeas(1)%Rangegate(1),&
                        "WorkingData%MieMeas(1)%Rangegate(1)",error_flag)

  print *,"-------------------------------"
  print *,"testing PrintRayleighRangeGate:"
  print *,"-------------------------------"
  call PrintRayleighRangeGate(WorkingData%RayleighMeas(1)%Rangegate(1),&
                        "WorkingData%RayleighMeas(1)%Rangegate(1)",error_flag)


  print *,"----------------------------"
  print *,"testing PrintMieMeasurement:"
  print *,"----------------------------"
  call PrintMieMeasurement(WorkingData%MieMeas(1),&
                          "WorkingData%MieMeas(1)",.false.,error_flag)

  print *,"---------------------------------"
  print *,"testing PrintRayleighMeasurement:"
  print *,"---------------------------------"
  call PrintRayleighMeasurement(WorkingData%RayleighMeas(1),&
                          "WorkingData%RayleighMeas(1)",.false.,error_flag)

  print *,"---------------------------------"
  print *,"testing PrintRayleighMeasurement:"
  print *,"---------------------------------"
  call PrintWorkingData(WorkingData,"WorkingData",.false.,error_flag)

  print *,"---------------------------------"
  print *,"deleting datastructure"
  print *,"---------------------------------"
  call DeleteWorkingData(WorkingData)

  print *,"test ended succesfull"
  !  #]
end program Test_Working_DataStructure
