module L2B_AuxPar_SPH
  !  #[ Documentation
  !
  ! this module defines the Specific Product Header for the ee_cfi fileformat
  ! for the L2B processor Auxiliary Parameter inputfile
  ! The SPH format may differ for different file definitions, therefore
  ! we use a private definition for this filetype only
  !
  ! for a description, see 
  ! TN2.1, sec.6.3.1, table 64
  !
  ! Created 05-04-2006 by J. de Kloe
  !
  ! History:
  !      12-May-2006  D. Tan       Null use of ParKind1 module
  !      09-Mar-2007  D. Tan       Remove un-needed RBC code
  !      07-Jun-2007  J. de Kloe   adapted to changed xml interface
  !      16-Jan-2008  J. de Kloe   phase out integer kind i_ 
  !
  !  #] 
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE XML_module, only: xml_node_type, &
                    max_str_len, xml_node_get_string, xml_set_fixed_header, &
                    check_length_xml_item, check_string_length, &
                    xml_tree_set_string_node_value, xml_tree_add_child, &
                    ee_vh_name, ee_sph_name, xml_add_SPH_item, &
                    xml_tree_add_and_set_attribute
  USE EE_CFI_Datatypes, only: IntAs,  maxlen_IntAs,  SetIntAs,  &
                              IntAus, maxlen_IntAus, SetIntAus, &
                              IntAl,  &
                              IntAul, maxlen_IntAul, SetIntAul, &
                              IntAd, &
                              Remove_Plus_Sign
  USE DataSetDescriptor, only: DSD_type, Init_DSD, &!AsciiWrite_DSD, &
                               Read_DSD_from_KVT, Get_DSD_from_xml_tree
!                               Add_DSD_to_xml_SPH
  ! support modules
  USE Logging, only: logmsg,log_debug,log_info,log_progress,log_warn,log_error
  USE ErrorHandler, only: no_error, error_allocate, error_programming, &
                          error_reading_file, error_writing_file
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  USE LunManager, only: fileunit_stdout
  USE Compiler_Features, only: newline
  !  #]
  !  #[ Type definitions and parameters
  implicit none

  public

  integer, parameter :: maxlen_SPH_Descriptor   = 28

  TYPE L2B_AuxPar_SPH_type
     !  #[
     character(len=maxlen_SPH_Descriptor) :: SPH_Descriptor
     ! Spare 1

! note that it is totally useless to have a DSD in this SPH
! because there is no binary dataset associated with it ...

     TYPE(DSD_type), pointer :: AuxPar_DSD
     !
  END TYPE L2B_AuxPar_SPH_type
    !  #]

  ! the default verbosity setting is Err (as silent as possible)
  ! (this switch is not yet set by an init routine, and can be set 
  !  to VerbosityReport manually for debugging purposes)
  integer, save :: LocalVerbosity = VerbosityError
!  integer, save :: LocalVerbosity = VerbosityReport
  
  !  #]
  !  #[ Module interface
  ! all is public for the moment, unless specified explicitely otherwise
  !  #]
contains
  !---------------------------------
  subroutine Init_L2B_AuxPar_SPH(L2B_AuxPar_SPH,error_flag)
    !  #[
    TYPE(L2B_AuxPar_SPH_type), pointer     :: L2B_AuxPar_SPH
    integer,                   intent(out) :: error_flag
    
    error_flag = no_error
    IF (.not. associated(L2B_AuxPar_SPH)) THEN
       call logmsg(log_error,"ERROR in Init_L2B_AuxPar_SPH(): "//&
            "The pointer to L2B_AuxPar_SPH seems not associated !!")
       error_flag = error_programming
       return
    END IF

    ! see the remark in MainProductHeader.F90 for the reason why I use
    ! the form x(:)=' ' to initialise the strings

    L2B_AuxPar_SPH%SPH_Descriptor(:) = ' '
    ! Spare 1
    nullify(L2B_AuxPar_SPH%AuxPar_DSD)

  end subroutine Init_L2B_AuxPar_SPH
    !  #]
  subroutine Print_L2B_AuxPar_SPH(L2B_AuxPar_SPH,error_flag,prefix)
    !  #[
    TYPE(L2B_AuxPar_SPH_type), pointer     :: L2B_AuxPar_SPH
    character(len=*),          intent(in)  :: prefix
    integer,                   intent(out) :: error_flag

    error_flag = no_error
    call AsciiWrite_L2B_AuxPar_SPH(L2B_AuxPar_SPH,fileunit_stdout,prefix,&
                                   error_flag)
    IF (error_flag .ne. no_error) return

  end subroutine Print_L2B_AuxPar_SPH
    !  #]
  subroutine AsciiWrite_L2B_AuxPar_SPH(L2B_AuxPar_SPH,fileunit,prefix,&
                                       error_flag)
    !  #[
    TYPE(L2B_AuxPar_SPH_type), pointer     :: L2B_AuxPar_SPH
    integer,                   intent(in)  :: fileunit
    character(len=*),          intent(in)  :: prefix
    integer,                   intent(out) :: error_flag

    ! local variable
    character(len=256) :: txt

    error_flag = no_error

    IF (.not. associated(L2B_AuxPar_SPH)) THEN
       write(fileunit,"(a)",err=999) trim(prefix)//" [seems not associated]"
       return
    END IF

    txt = L2B_AuxPar_SPH%SPH_Descriptor
    IF (len_trim(txt).eq.0) txt = "[empty]"
    write(fileunit,"(a)",err=999) &
         trim(prefix)//"%SPH_Descriptor = "//trim(txt)

    ! Spare 1

    write(fileunit,"(a)",err=999) &
         "call to AsciiWrite_DSD has been temporarily disabled until the"
    write(fileunit,"(a)",err=999) &
         "branch is merged with a branch holding the AsciiWrite routines ..."
    
!    call AsciiWrite_DSD(L2B_AuxPar_SPH%AuxPar_DSD,fileunit,&
!         trim(prefix)//"%AuxPar_DSD",error_flag)
!    IF (error_flag .ne. no_error) return

    return

999 call logmsg(log_error,"ERROR in AsciiWrite_L2B_AuxPar_SPH "//&
                          "error while writing to outputfile")
    error_flag = error_writing_file
    return

  end subroutine AsciiWrite_L2B_AuxPar_SPH
    !  #]
  subroutine Delete_L2B_AuxPar_SPH(L2B_AuxPar_SPH)
    !  #[
    TYPE(L2B_AuxPar_SPH_type), pointer :: L2B_AuxPar_SPH

    IF (.not. associated(L2B_AuxPar_SPH)) return ! nothing to do

    IF (associated(L2B_AuxPar_SPH%AuxPar_DSD)) &
         deallocate(L2B_AuxPar_SPH%AuxPar_DSD)

  end subroutine Delete_L2B_AuxPar_SPH
    !  #]
  subroutine Get_L2B_AuxPar_SPH_from_xmltree(xml_tree, path_in,&
                                             L2B_AuxPar_SPH, error_flag)
    !  #[
    TYPE(xml_node_type),        pointer     :: xml_tree       ! input
    character(len=*),           intent(in)  :: path_in        ! input
    TYPE(L2B_AuxPar_SPH_type),  pointer     :: L2B_AuxPar_SPH ! result
    integer,                    intent(out) :: error_flag

    ! local variables
    character(len=max_str_len) :: path
    character(len=max_str_len) :: text
    integer                    :: AllocStatus

    ! init
    error_flag = no_error

    IF (.not. associated(L2B_AuxPar_SPH)) THEN
       call logmsg(log_error,"ERROR in Get_L2B_AuxPar_SPH_from_xml_tree():"//&
            "the pointer L2B_AuxPar_SPH seems not associated !!!")
       error_flag = error_programming
       return
    END IF

    IF (.not. associated(xml_tree)) THEN
       call logmsg(log_error,"ERROR in Get_L2B_AuxPar_SPH_from_xml_tree():"//&
            "the pointer xml_tree seems not associated !!!")
       error_flag = error_programming
       return
    END IF

    !---------------------------------
    ! copying the L2B_AuxPar_SPH XML elements
    !---------------------------------

    path  =trim(path_in)//"/"//ee_vh_name//"/"//ee_sph_name//"/Sph_Descriptor"
    call xml_node_get_string(xml_tree,path,text,error_flag)
    IF (error_flag .ne. no_error) return
    L2B_AuxPar_SPH%SPH_Descriptor = &
         check_length_xml_item(text,path,maxlen_SPH_Descriptor,&
                               "Get_L2B_AuxPar_SPH_from_xml_tree")

    ! Spare_1: no need to read this one

    allocate(L2B_AuxPar_SPH%AuxPar_DSD,STAT=AllocStatus)
    IF (AllocStatus .ne. 0) THEN
       call logmsg(log_error,&
            "Allocation problem in Get_L2B_AuxPar_SPH_from_xml_tree():")
       error_flag = error_allocate
       return
    END IF

    call Init_DSD(L2B_AuxPar_SPH%AuxPar_DSD,error_flag)
    IF (error_flag .ne. no_error) return

    ! get the DSD from the xml_tree structure
    call Get_DSD_from_xml_tree(xml_tree, path_in, &
                               L2B_AuxPar_SPH%AuxPar_DSD, 1, &
                               "Level_2B_Proc_Params", "G", &
                               error_flag)
    IF (error_flag .ne. no_error) return

  end subroutine Get_L2B_AuxPar_SPH_from_xmltree
    !  #]
  !---------------------------------
end module L2B_AUXPAR_SPH
