module L2B_parameters
  !  #[ Documentation
  ! 
  ! this module defines a number of parameters used by the 
  ! classification and observation constructing code of the
  ! L2BP. Especially the not_yet_classified, undefined_obs_type
  ! values are needed for initialisation of the datastructures.
  ! Therefore this parameter list is needed in this DataStructures
  ! directory.
  !
  ! Written by J. de Kloe (KNMI)
  ! Created:     07-11-2005
  !
  ! Modifications:
  !      12-May-2006 D. Tan     Null use of ParKind1 module
  !      02-Mar-2007 J. de Kloe small type change for Obs_Type
  !      16-Jan-2008 J. de Kloe phase out integer kind i_ 
  !
  !  #] 
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE EE_CFI_Datatypes, only: IntAuc
  !  #]
  !  #[ Parameter definitions
  implicit none

  ! possible results of the classification algoritm
  ! to be assigned to each range-bin of each measurement  
  ! in the L2B%Rayleigh_MDS and L2B%Mie_MDS
  integer, parameter :: not_yet_classified         = 0
  integer, parameter :: classified_as_cloud        = 1
  integer, parameter :: classified_as_no_cloud     = 2
  integer, parameter :: nr_of_classes              = 2
!  integer(i_), parameter :: classified_as_dust         = 3
! add?: classified_as_below_clouds
! add?: classified_as_precipitation
! add?: classified_as_aerosol_layer
  integer, parameter :: no_classification_possible = 9
  ! due to zero or too low Rayleigh signal

  ! possible observation types
  integer, parameter :: Obs_Type_undefined           = 0
  integer, parameter :: Obs_Type_cloudy_returns      = 1
  integer, parameter :: Obs_Type_clear_returns       = 2
  integer, parameter :: nr_of_Obs_Types              = 2

  ! REMARK: the measurements may also be split in 2 sets if there
  ! is for example a strong wind shear (or other phenomenon that 
  ! would justify it). So multiple observations with the same
  ! observation type are possible. 
  !  #]
contains
  !-----------------------
  function GetClassTypeText(Class_Type) result(txt)
    !  #[
    integer            :: Class_Type ! input 
    character(len=256) :: txt        ! result

    select case (Class_Type)
    case(not_yet_classified)
       txt = "not_yet_classified"
    case(classified_as_cloud)
       txt = "classified_as_cloud"
    case(classified_as_no_cloud)
       txt = "classified_as_no_cloud"
    case(no_classification_possible)
       txt = "no_classification_possible"
    case default 
       txt = "unknown obs. type (programming error !!!)"
    end select

  end function GetClassTypeText
    !  #]
  function GetObsTypeText(Obs_Type) result(txt)
    !  #[
    integer(IntAuc)    :: Obs_Type ! input 
    character(len=256) :: txt      ! result

    select case (Obs_Type)
    case(Obs_Type_undefined)
       txt = "undefined_obs_type"
    case(Obs_Type_cloudy_returns)
       txt = "Obs_Type_cloudy_returns"
    case(Obs_Type_clear_returns)
       txt = "Obs_Type_clear_returns"
    case default 
       txt = "unknown obs. type (programming error !!!)"
    end select

  end function GetObsTypeText
    !  #]
  !-----------------------
end module L2B_parameters
