program TestBuffering
  !  #[ Documentation
  !     a testprogram for the Buffering module
  !     written by: J. de Kloe
  !
  !     Modifications:
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     18-Oct-2007  J. de Kloe   added a nullify statement
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_ 
  !
  !  #]
  !  #[ Timing results for reading
  !     -results on KNMI Suse Linux (bhw034)
  !          compiler: g95     gfortran pgf90   ifort
  !          Buffered: 11.16 s  1.17 s  11.70 s  2.41 s
  !          Direct:   76.50 s 48.10 s  17.58 s 67.56 s
  !     -results on KNMI Sun (bsf15k)
  !          compiler Sun ONE Studio 8, f90 compiler
  !          Buffered:   1.18 s
  !          Direct:   211.98 s [!!!]
  !     -results on ECMWF (ecgate)
  !          compiler ibm xlf90 
  !          Buffered:  8.88 s
  !          Direct:   23.36 s
  !     -results on my Fedore Linux laptop at home (JK)
  !          compiler:  g95     gfortran
  !          Buffered:  11.13 s  1.38 s 
  !          Direct:   155.94 s 80.95 s 
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Buffering
  USE DirectBinaryIO, only: OpenDirectBinaryFile, CloseDirectBinaryFile, &
                            Init_DirectBinaryIO_module, LittleEndian, &
                            read_int4_from_bin_file, read_int1_from_bin_file,&
                            write_int4_to_bin_file,  write_int1_to_bin_file
  USE ErrorHandler, only: no_error, program_abort
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  USE DateTimeMod, only: get_current_date_time, time2string_sep_ms, &
                         date2string_sep,date_type, time_type, &
                         convert_time_to_real
  USE numerics, only: r8_, i4_, i1_
  !  #]
  !  #[ variables and parameters
  implicit none

  TYPE(BufferedFile_type), pointer :: BufferedFile 
  TYPE(date_type)   :: date
  TYPE(time_type)   :: time
  character(len=50) :: filename, txt
  character(len=50) :: filename_out1, filename_out2
  character(len=50) :: filename_out3, filename_out4
  character(len=1), dimension(:), pointer :: data
  integer           :: error_flag, AllocStatus, filesize
  integer           :: startpos, endpos, word, fileunit, i, ios
  integer           :: BufferSize 
  integer(i1_)      :: int1_val
  integer(i4_)      :: int4_val
  real(r8_)         :: seconds_start, seconds_end
  !  #]

  nullify(BufferedFile)

  !filename = "TENTI_SPECTRUM.dat"
  filename = "CALIB_GRID.dat"
  !filename = "Makefile.aeolus"
  filename_out1 = "testfile.localcopy1"
  filename_out2 = "testfile.localcopy2"
  filename_out3 = "testfile.localcopy3"
  filename_out4 = "testfile.localcopy4"
  filesize = get_filesize(filename)
  BufferSize = 128 ! 8192 ! 500
  allocate(data(filesize),stat=AllocStatus)
  IF (AllocStatus .ne. 0) call program_abort(error_flag,"TestBuffering")

  !  #[ buffered reading test
  call InitBuffering!(ChoosenVerbosity=VerbosityReport)
  allocate(BufferedFile,stat=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_flag,"TestBuffering")

  call OpenBufferedFile(BufferedFile,filename,"read",error_flag, &
                        BufferSize=BufferSize)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"OpenBufferedFile")

  call get_current_date_time(date,time)
  !txt = date2string_sep(date,':')
  !print *,"today is:   ",txt

  ! temporary hacks for debugging
  !BufferedFile%filesize = 150
  !BufferedFile%filesize = BufferedFile%filesize + 4

  print *,"Reading buffered data from binary file: ",trim(filename)

  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  ! this do loop and its 6M calls of the subroutine 
  ! costs 3 seconds for the big file
  DO word=1,BufferedFile%filesize/4
     startpos = (word-1)*4+1
     endpos   = (word-1)*4+4
     call ReadBytesBuffered(BufferedFile,startpos,endpos,&
                            data(startpos:endpos),error_flag)
     !call ReadNextBytesBuffered(BufferedFile,4,&
     !                           data(startpos:endpos),error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"ReadBytesBuffered")
  END DO

  !print *,"startpos = ",startpos," endpos = ",endpos
  IF (endpos .ne. BufferedFile%filesize) THEN
     !print *,"WARNING: byte seems not a multiple of 4 bytes !"
     !print *,"WARNING: endpos = ",endpos
     !print *,"WARNING:BufferedFile%filesize = ",BufferedFile%filesize
     startpos = endpos+1
     endpos   = BufferedFile%filesize
     !print *,"Reading last ",endpos-startpos+1,"bytes..."
     call ReadBytesBuffered(BufferedFile,startpos,endpos,&
                            data(startpos:endpos),error_flag)
  END IF

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"Done with buffered data test"
  print *,"BufferedFile%count_reads   = ",BufferedFile%count_reads
  print *,"elapsed time while reading = ",seconds_end-seconds_start," [s]"

  ! write the read data to an unformatted file, to allow
  ! using the diff command to inspect the result:
  open(unit=7,file=filename_out1,form="unformatted",access="direct",&
       status="replace",recl=BufferedFile%filesize)
  write(unit=7,rec=1) data(1:BufferedFile%filesize)
  close(unit=7)

  call CloseBufferedFile(BufferedFile, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"CloseBufferedFile")
  deallocate(BufferedFile)
  !  #]
  !  #[ byte-by-byte reading test
  ! open the same file again for byte-by-byte reading using DirectBinaryIO
  ! to be able to compare the run time
  print *,"DirectBinaryIO: reading data from binary file: ",trim(filename)

  call Init_DirectBinaryIO_module(error_flag, FileByteOrder=LittleEndian)
  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  DO word=1,filesize/4
     call read_int4_from_bin_file(fileunit,int4_val,error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_reading_file,"read_int1_from_bin_file")
  END DO

  word=filesize/4
  startpos = (word-1)*4+1
  endpos   = (word-1)*4+4
  !print *,"startpos = ",startpos," endpos = ",endpos
  IF (endpos .ne. filesize) THEN
     !print *,"WARNING: byte seems not a multiple of 4 bytes !"
     !print *,"WARNING: endpos = ",endpos
     !print *,"WARNING:BufferedFile%filesize = ",filesize
     startpos = endpos+1
     endpos   = filesize
     !print *,"Reading last ",endpos-startpos+1,"bytes..."
     DO i=startpos,endpos
        call read_int1_from_bin_file(fileunit,int1_val,error_flag)
     END DO
  END IF

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"DirectBinaryIO: done with reading test"
  print *,"elapsed time while reading = ",seconds_end-seconds_start," [s]"

  call CloseDirectBinaryFile(fileunit)
  !  #]
  !  #[ single-read test
  print *,"DirectBinaryIO: reading data in 1 block from file: ",trim(filename)

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  fileunit=get_lun()
  open(unit=fileunit,file=filename,status="old",&
       action="read",access="direct",recl=filesize,form="UNFORMATTED",ERR=998)
  read(fileunit,rec=1,iostat=ios) data
  IF (ios .ne. 0) THEN
     print *,"ERROR: could not read data from file: "//trim(filename)
     call program_abort(error_reading_file,"TestBuffering")     
  END IF
  close(unit=fileunit)

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"DirectBinaryIO: done with reading test"
  print *,"elapsed time while reading = ",seconds_end-seconds_start," [s]"
  !  #]
  !  #[ single-write test
  print *,"DirectBinaryIO: writing data in 1 block from file: ",trim(filename)

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  fileunit=get_lun()
  open(unit=fileunit,file=filename_out2,status="replace",&
       action="write",access="direct",recl=filesize,form="UNFORMATTED",ERR=998)
  write(fileunit,rec=1) data
  close(unit=fileunit)

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"DirectBinaryIO: done with writing test"
  print *,"elapsed time while writing = ",seconds_end-seconds_start," [s]"
  !  #]
  !  #[ byte-by-byte writing test
  ! write a file for byte-by-byte writing using DirectBinaryIO
  ! to be able to compare the run time
  print *,"DirectBinaryIO: writing data to binary file: ",trim(filename)

  call OpenDirectBinaryFile(filename_out3,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  int4_val=123456
  DO word=1,filesize/4
     startpos = (word-1)*4+1
     endpos   = (word-1)*4+4
     int4_val = transfer(data(startpos:endpos),int4_val)
     call write_int4_to_bin_file(fileunit,int4_val,error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_writing_file,"write_int4_to_bin_file")
  END DO

  word=filesize/4
  startpos = (word-1)*4+1
  endpos   = (word-1)*4+4
  !print *,"startpos = ",startpos," endpos = ",endpos
  IF (endpos .ne. filesize) THEN
     !print *,"WARNING: byte seems not a multiple of 4 bytes !"
     !print *,"WARNING: endpos = ",endpos
     !print *,"WARNING: filesize = ",filesize
     startpos = endpos+1
     endpos   = filesize
     !print *,"Writing last ",endpos-startpos+1,"bytes..."
     DO i=startpos,endpos
        int1_val = transfer(data(i:i),int1_val)
        call write_int1_to_bin_file(fileunit,int1_val,error_flag)
     END DO
  END IF

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"DirectBinaryIO: done with writing test"
  print *,"elapsed time while writing = ",seconds_end-seconds_start," [s]"

  call CloseDirectBinaryFile(fileunit)
  !  #]
  !  #[ buffered writing test
  allocate(BufferedFile,stat=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_flag,"TestBuffering")

  call OpenBufferedFile(BufferedFile,filename_out4,"write",&
                        error_flag, BufferSize=BufferSize)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"OpenBufferedFile")

  call get_current_date_time(date,time)
  !txt = date2string_sep(date,':')
  !print *,"today is:   ",txt

  print *,"Writing buffered data from binary file: ",trim(filename)

  txt = time2string_sep_ms(time,':')
  print *,"start time: ",txt
  seconds_start = convert_time_to_real(time)

  DO word=1,filesize/4
     startpos = (word-1)*4+1
     endpos   = (word-1)*4+4
     call WriteBytesBuffered(BufferedFile,startpos,endpos,&
                             data(startpos:endpos),error_flag)
     !call ReadNextBytesBuffered(BufferedFile,4,&
     !                           data(startpos:endpos),error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"WriteBytesBuffered")
  END DO

  IF (endpos .ne. filesize) THEN
     startpos = endpos+1
     endpos   = filesize
     !print *,"Writing last ",endpos-startpos+1,"bytes..."
     call WriteBytesBuffered(BufferedFile,startpos,endpos,&
                             data(startpos:endpos),error_flag)
  END IF

  call CloseBufferedFile(BufferedFile, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"CloseBufferedFile")

  call get_current_date_time(date,time)
  txt = time2string_sep_ms(time,':')
  print *,"end time: ",txt
  seconds_end = convert_time_to_real(time)

  print *,"Done with buffered data test"
  print *,"BufferedFile%count_writes  = ",BufferedFile%count_writes
  print *,"elapsed time while writing = ",seconds_end-seconds_start," [s]"

  deallocate(BufferedFile)
  !  #]

  deallocate(data)
  stop

998 call program_abort(error_flag,"opening file")

end program TestBuffering
