program TestDirectBinaryIO
  !  #[ Description
  !     a test program to test the DirectBinaryIO module
  !     split 12-07-2005 from the KVT_module by J. de Kloe, KNMI
  !
  !     Modifications:
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_ 
  !
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  ! the module to be tested
  USE DirectBinaryIO

  ! support modules
  USE Numerics, only: i1_, i2_, i4_, i8_, r4_, r8_,&
                      missing_indicator_integer_i1_, &
                      missing_indicator_integer_i2_, &
                      missing_indicator_integer_i4_, &
                      missing_indicator_integer_i8_, &
                      missing_indicator_real_r4,&
                      missing_indicator_real_r8
  USE LunManager, only: get_lun, free_lun
  USE ErrorHandler, only: no_error, error_reading_file,&
                          error_writing_file, error_opening_file, &
                          error_programming, error_allocate, program_abort
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  !  #]
  !  #[ Variable declarations
  implicit none
  character(len=256)         :: filename
  integer                    :: fileunit
  integer                    :: error_flag
  integer(i1_)               :: int1_val
  integer(i2_)               :: uns_int1_val
  integer(i2_)               :: int2_val
  integer(i4_)               :: uns_int2_val
  integer(i4_)               :: int4_val
  integer(i8_)               :: uns_int4_val
  integer(i4_), dimension(3) :: int4_array
  real(r4_)                  :: real4_val
  real(r8_)                  :: real8_val
  real(r4_),    dimension(3) :: real4_array
  real(r8_),    dimension(3) :: real8_array

  ! the following arrays are used for reading "SomeNumbers.bin"
  integer(i2_), dimension(5) :: uns_int1_arr
  integer(i4_), dimension(5) :: uns_int2_arr
  integer(i8_), dimension(5) :: uns_int4_arr
  integer(i1_), dimension(5) :: int1_arr
  integer(i2_), dimension(5) :: int2_arr
  integer(i4_), dimension(5) :: int4_arr
  integer(i8_), dimension(5) :: int8_arr
  real(r4_),    dimension(5) :: real4_arr
  real(r8_),    dimension(5) :: real8_arr

  !  #]
  !  #[ the program code
  ! this automatically selects the proper byteswapping to be used if needed
  call Init_DirectBinaryIO_module(error_flag,FileByteOrder=BigEndian)
                              !,ChoosenVerbosity=VerbosityReport)

  filename = "test_file.bin"
  !  #[ test writing to binary file
  print *,"------"
  print *,"writing data to binary file: ",trim(filename)  
  call OpenDirectBinaryFile(filename,"write",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
 
  call write_int1_to_bin_file(fileunit,-123_i1_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_int1_to_bin_file")

  call write_uns_int1_to_bin_file(fileunit,177_i2_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_uns_int1_to_bin_file")

  call write_int2_to_bin_file(fileunit,-27345_i2_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_int2_to_bin_file")

  call write_uns_int2_to_bin_file(fileunit,57345_i4_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_uns_int2_to_bin_file")

  call write_int4_to_bin_file(fileunit,-2111333444_i4_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_int4_to_bin_file")

  call write_uns_int4_to_bin_file(fileunit,4222111222_i8_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_uns_int4_to_bin_file")

  call write_int4array_to_bin_file(fileunit,(/4_i4_,6_i4_,8_i4_/),error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_int4array_to_bin_file")

  call write_real4_to_bin_file(fileunit,1.234_r4_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_real4_to_bin_file")

  call write_real8_to_bin_file(fileunit,1.23456789111222_r8_,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_real8_to_bin_file")

  call write_real4array_to_bin_file(fileunit,(/1.2_r4_,2.3_r4_,3.4_r4_/),&
                                    error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_real4array_to_bin_file")

  call write_real8array_to_bin_file(fileunit,(/1.2_r8_,2.3_r8_,3.4_r8_/),&
                                    error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_real8array_to_bin_file")

  call write_zero_bytes_bin_file(fileunit,100,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_zero_bytes_bin_file")
  
  call CloseDirectBinaryFile(fileunit)
  print *,"done with writing test"  
  !  #]
  filename = "test_file.bin"
  !  #[ test reading from the generated binary file
  print *,"------"
  print *,"reading data from the generated binary file: ",trim(filename)
  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")
  
  call read_int1_from_bin_file(fileunit,int1_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int1_from_bin_file")
  print *,"read int1 value = ",int1_val

  call read_uns_int1_from_bin_file(fileunit,uns_int1_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int1_from_bin_file")
  print *,"read uns_int1 value = ",uns_int1_val

  call read_int2_from_bin_file(fileunit,int2_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int2_from_bin_file")
  print *,"read int2 value = ",int2_val

  call read_uns_int2_from_bin_file(fileunit,uns_int2_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int2_from_bin_file")
  print *,"read uns_int2 value = ",uns_int2_val

  call read_int4_from_bin_file(fileunit,int4_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int4_from_bin_file")
  print *,"read int4 value = ",int4_val

  call read_uns_int4_from_bin_file(fileunit,uns_int4_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int4_from_bin_file")
  print *,"read uns_int4 value = ",uns_int4_val

  int4_array(:)=-1
  call read_int4array_from_bin_file(fileunit,int4_array,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int4array_from_bin_file")
  print *,"read array of int4 values: ",int4_array

  call read_real4_from_bin_file(fileunit,real4_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real4_from_bin_file")
  print *,"REALACC(6) read real4 value = ",real4_val," ENDREALACC"

  call read_real8_from_bin_file(fileunit,real8_val,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real8_from_bin_file")
  print *,"REALACC(6) read real8 value = ",real8_val," ENDREALACC"

  real4_array(:)=-1.
  call read_real4array_from_bin_file(fileunit,real4_array,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real4array_from_bin_file")
  print *,"REALACC(6) read array of real4 values: ",real4_array," ENDREALACC"

  real8_array(:)=-1.
  call read_real8array_from_bin_file(fileunit,real8_array,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real8array_from_bin_file")
  print *,"REALACC(6) read array of real8 values: ",real8_array," ENDREALACC"

  call CloseDirectBinaryFile(fileunit)
  print *,"done with reading test"  
  !  #]
  filename = "SomeNumbers.bin"
  !  #[ test reading from another binary file
  print *,"------"
  print *,"reading data from a binary file written by c code: ",trim(filename)

  ! this disables any byteswapping that might have been active
  call Init_DirectBinaryIO_module(error_flag,FileByteOrder=DefaultByteOrder)

  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_opening_file,"OpenDirectBinaryFile")

  uns_int1_arr(:)=-1
  call read_uns_int1arr_from_bin_file(fileunit,uns_int1_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int1arr_from_bin_file")
  print *,"read array of unsigned int1 values: ",uns_int1_arr

  int1_arr(:) = missing_indicator_integer_i1_
  call read_int1array_from_bin_file(fileunit,int1_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int1array_from_bin_file")
  print *,"read array of signed int1 values: ",int1_arr

  uns_int2_arr(:)=-1
  call read_uns_int2arr_from_bin_file(fileunit,uns_int2_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int2arr_from_bin_file")
  print *,"read array of unsigned int2 values: ",uns_int2_arr

  int2_arr(:) = missing_indicator_integer_i2_
  call read_int2array_from_bin_file(fileunit,int2_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int2array_from_bin_file")
  print *,"read array of signed int2 values: ",int2_arr

  uns_int4_arr(:)=-1
  call read_uns_int4arr_from_bin_file(fileunit,uns_int4_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_uns_int4arr_from_bin_file")
  print *,"read array of unsigned int4 values: ",uns_int4_arr

  int4_arr(:) = missing_indicator_integer_i4_
  call read_int4array_from_bin_file(fileunit,int4_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int4array_from_bin_file")
  print *,"read array of signed int4 values: ",int4_arr

  int8_arr(:) = missing_indicator_integer_i8_
  call read_int8array_from_bin_file(fileunit,int8_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_int8array_from_bin_file")
  print *,"read array of signed int8 values: ",int8_arr

  real4_arr(:) = missing_indicator_real_r4
  call read_real4array_from_bin_file(fileunit,real4_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real4array_from_bin_file")
  print *,"REALACC(6)"
  print *,"read array of real4 values = ",real4_arr(:)
  print *,"ENDREALACC"

  real8_arr(:) = missing_indicator_real_r8
  call read_real8array_from_bin_file(fileunit,real8_arr,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_real8array_from_bin_file")
  print *,"REALACC(12)"
  print *,"read array of real8 values = ",real8_arr(:)
  print *,"ENDREALACC"

  call CloseDirectBinaryFile(fileunit)
  print *,"done with reading test"  
  !  #]

  !  #]
end program TestDirectBinaryIO
