program TestUnformattedIO

  ! Written by: J. de Kloe, KNMI
  !
  ! Modifications:
  !     16-Jan-2008  J. de Kloe phase out integer kind i_ 

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE DirectBinaryIO, only: OpenDirectBinaryFile, CloseDirectBinaryFile, &
                            Init_DirectBinaryIO_module, LittleEndian, BigEndian
  USE UnformattedIO, only: read_unf_int4arr
  USE ErrorHandler, only: no_error, program_abort, error_opening_file
  USE VerbosityControl, only: VerbosityReport
  USE Numerics, only: i4_

  implicit none
  
  integer      :: error_flag,fileunit,headtailsize,filepos
  integer(i4_) :: a,b,c
  integer(i4_), dimension(3) :: arr
  character(len=256) :: filename

  error_flag = no_error

  ! write a binary file, with native format for this platform
  ! by comparing with the provided example files
  ! uf.4b.be.bin, uf.4b.le.bin, uf.8b.le.bin
  ! it should be very easy to determine if we are on a platform 
  ! using 4 or 8 byte headtailsize, and whether the byte order
  ! is big or little endian.
  a=23
  b=161
  c=937 
  print *,"a,b,c = ",a,b,c
  filename = "uf.bin"
  open(unit=11,FILE=filename,FORM='UNFORMATTED',err=999)
  write(unit=11,err=997) a,b,c
  close(unit=11)

! this read crashed with a runtime error on the sun
!  print *,"-----------------------------------"
!  filename = "uf.4b.le.bin"
!  print *,"testing file: ",trim(filename)
!  open(unit=11,FILE=filename,FORM='UNFORMATTED',status='OLD',err=999)
!  read(unit=11,iostat=ios) a,b,c
!  if (ios .ne. 0) goto 998
!  close(unit=11)
!  print *,"result for reading g95 unformatted binary file"
!  print *,"a,b,c = ",a,b,c
!
!  print *,"-----------------------------------"
!  filename = "uf.8b.le.bin"
!  print *,"testing file: ",trim(filename)
!  open(unit=11,FILE=filename,FORM='UNFORMATTED',status='OLD',err=999)
!  read(unit=11,iostat=ios) a,b,c
!  if (ios .ne. 0) goto 998
!  close(unit=11)
!  print *,"result for reading gfortran unformatted binary file"
!  print *,"a,b,c = ",a,b,c

  print *,"-----------------------------------"
  print *,"initialise"
  call Init_DirectBinaryIO_module(error_flag, &
       FileByteOrder=LittleEndian,&
       ChoosenVerbosity=VerbosityReport)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"Init_DirectBinaryIO_module")

  print *,"-----------------------------------"
  filename = "uf.4b.le.bin"
  print *,"testing file: ",trim(filename)
  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"OpenDirectBinaryFile")

  inquire(unit=fileunit,nextrec=filepos)
  print *,"filepos=",filepos

  headtailsize=4
  call read_unf_int4arr(fileunit,headtailsize,3,arr,error_flag)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"read_unf_intarr")

  print *,"arr = ",arr

  call CloseDirectBinaryFile(fileunit)

  print *,"-----------------------------------"
  print *,"Little endian tests"
  filename = "uf.8b.le.bin"
  print *,"testing file: ",trim(filename)
  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"OpenDirectBinaryFile")

  headtailsize=4 ! 8
  call read_unf_int4arr(fileunit,headtailsize,3,arr,error_flag)
  IF (error_flag .eq. error_opening_file) goto 999

  IF (error_flag .ne. no_error) THEN

     print *,"==> retrying with headtailsize=8 ...."
     error_flag = no_error
     ! headtailsize of 4 mgiht be wrong, so try again with 8
     ! the filepos is shifted by 4 bytes now, so reopen the file
     ! to correct for this.
     call CloseDirectBinaryFile(fileunit)
     
     call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
     if (error_flag .ne. no_error) &
          call program_abort(error_flag,"OpenDirectBinaryFile")

     headtailsize=8
     call read_unf_int4arr(fileunit,headtailsize,3,arr,error_flag)
     if (error_flag .ne. no_error) &
          call program_abort(error_flag,"read_unf_int4arr")
  
  END IF

  print *,"arr = ",arr

  call CloseDirectBinaryFile(fileunit)
  print *,"-----------------------------------"
  print *,"initialise again"
  call Init_DirectBinaryIO_module(error_flag, &
       FileByteOrder=BigEndian,&
       ChoosenVerbosity=VerbosityReport)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"Init_DirectBinaryIO_module")

  print *,"-----------------------------------"
  print *,"Big endian tests"
  filename = "uf.4b.be.bin"
  print *,"testing file: ",trim(filename)
  call OpenDirectBinaryFile(filename,"read",fileunit,error_flag)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"OpenDirectBinaryFile")

  inquire(unit=fileunit,nextrec=filepos)
  print *,"filepos=",filepos

  headtailsize=4
  call read_unf_int4arr(fileunit,headtailsize,3,arr,error_flag)
  if (error_flag .ne. no_error) &
        call program_abort(error_flag,"read_unf_intarr")

  print *,"arr = ",arr

  call CloseDirectBinaryFile(fileunit)
  print *,"-----------------------------------"
  print *,"done"

  stop

997 print *,"ERROR in TestUnformattedIO():"
    print *,"could not write file: ",trim(filename)
    stop

!998 print *,"ERROR in TestUnformattedIO():"
!    print *,"could not read file: ",trim(filename)
!    stop

999 print *,"ERROR in TestUnformattedIO():"
    print *,"could not open file: ",trim(filename)
    stop

end program TestUnformattedIO
