program Test_HLOS_Retrieval
  !  #[ Description:
  !     A test program for the HLOS_retrieval module
  !   
  !     created:     31-Aug-2006, by: D. Tan, ECMWF
  !     based on:    ../classification/TestClassification.F90 by J. de Kloe
  !
  !     Modifications:
  !     31-Aug-2006 D. Tan      Original version
  !     06-Oct-2006 J. de Kloe  added use of WorkingDataType
  !     23-Oct-2006 D. Tan      Use of WorkingDataType for old L2B-working
  !     24-Apr-2007 J. de Kloe  phase out the SummaryVars code
  !     28-Aug-2007 D. Tan      Remove reference to obsolete routine
  !     03-Sep-2007 P. Poli     Modified calls to Init_ and Delete_
  !     04-Sep-2007 P. Poli     change L1B_DataType to L1B_BRC_DataType,
  !                             change L2BC_DataType to L2BC_BRC_DataType
  !     05-Oct-2007 J. de Kloe  added use of the new Meas_Selection_Weighting 
  !                             module
  !     26-Oct-2007 J. de Kloe  removed 2 return statements in this 
  !                             main program
  !     16-Jan-2008 J. de Kloe  phase out integer kind i_
  !
  !  #]
  !  #[ Modules used:
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE LiteDataModule, only: Fill_WorkingData_With_Lite_Data
  USE OpticalProperties, only: InitOpticalProperties,&
                               CloseOpticalProperties,&
                               Calc_Extinction, Calc_BackscatterRatio,&
                               Copy_Extinction_Rayl_to_Mie, &
                               Copy_Backsc_Ratio_Rayl_to_Mie, &
                               PrintBackscatterRatio, &
                               Match_Mie_And_Rayleigh_Bins
  USE L2B_Proc_Settings, only: Scat_Ratio_from_L1B, Scat_Ratio_Dont_Use, &
                               Scat_Ratio_from_MieRayl,&
                               PartExt_Method_Undefined, &
                               PartExt_Method_Old, PartExt_Method_Iterative, &
                               MolExt_Method_Undefined,&
                               MolExt_Method_Parametrised, MolExt_Method_Proper
  USE L2B_AuxPar_DataStructure, only: L2B_AuxPar_Type
  USE Classification, only: InitClassification, &
                            Classify_Mie_Meas_No_Clouds, &
                            Classify_Rayl_Meas_No_Clouds, &
                            Classify_Mie_Meas_Backsc_Ratio, &
                            Classify_Rayl_Meas_Backsc_Ratio
  USE Select_And_Weigh_Measurements, only: Select_And_Weigh_Rayleigh_Meas,&
                                           Select_And_Weigh_Mie_Meas
  USE Construct_Obs, only:  Construct_Rayleigh_Obs, Construct_Mie_Obs, &
                            Print_Constructed_Obs, &
                            Print_Constr_Obs_Matched
  ! a routine to define the globally used L1B data struct
  USE L1B_DataStructure, only: L1B_BRC_DataType, &
                               Init_L1B_BRC_DataStructure, &
                               Delete_L1B_BRC_DataStructure
  ! a routine to define the globally used L2B data struct
  USE L2BC_DataStructure, only: L2BC_BRC_DataType, Init_L2BC_BRC_DataStructure, &
                                Delete_L2BC_BRC_DataStructure
  USE L2B_PCD_ADS, only: L1B_Mie_Bin_OK, L1B_Rayl_Bin_OK
  USE Meas_Map, only:  Print_Meas_Map
  USE Working_Datastructure, only: WorkingDataType,InitWorkingData, &
                                   DeleteWorkingData
  USE EE_CFI_Datatypes, only: IntAs, IntAuc
  ! support modules
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, VerbosityReport
  USE numerics, only: r8_
  USE ErrorHandler, only: no_error, error_allocate, error_cmdline_options, &
                          program_abort
  USE ProfileInterpolate, only: profile_type, init_profile, delete_profile, &
                                interp_type_linear, extrap_type_linear
  USE HLOS_retrieval, only: Init_HLOS_Retrieval, &
                            Process_Mie_Observations, &
                            Process_Rayleigh_Observations
  !  #]
  !  #[ Parameter/Variable/Type declarations

  implicit none

  ! variables
  character(len=256) :: filename
!  real(r8_)          :: Threshold_BackscatterRatio ! never used
  integer            :: error_flag
  integer            :: nr_BRCs_available
  integer            :: read_which_BRC
!  integer            :: AllocStatus ! not used
  integer            :: N_meas,r,m
  integer            :: nr_of_arguments, current_argument_nr
  character(len=256) :: current_argument
  logical            :: use_noclouds
  integer(IntAuc)    :: ScatRatio_Method, ScatRatio_Method2

  ! define how the measurements are organised
  ! (in 67 profiles consisting of 24 range gates each)
  integer, parameter :: N_meas_max    = 67 ! max used nr of meas.
  integer, parameter :: N_range_gates = 24 ! current used nr of gates
  integer(IntAuc), parameter :: N_Obs_Mie_Max = 4      ! current used nr of obs
  integer(IntAuc), parameter :: N_Obs_Rayleigh_Max = 4 ! current used nr of obs

  ! a struct to hold all necessary data for the processor
  TYPE(L1B_BRC_DataType)     :: L1B
  TYPE(L2BC_BRC_DataType)    :: L2B
  TYPE(WorkingDataType)      :: WorkingData
  TYPE(profile_type)         :: BackscRatio_Thresh_Profile

  ! a struct to hold parameter settings for the processor
  ! in particular the Mie core algorithm parameters
!  TYPE(L2B_AuxPar_Type)      :: L2B_AuxPar ! input
  !  #]
  !  #[ program code
  print *,"running testprogram: Test_HLOS_Retrieval"

  ! init
  N_meas = N_meas_max
  error_flag = no_error
  use_noclouds = .false.

  ! set defaults
  ScatRatio_Method  = Scat_Ratio_from_MieRayl
  ScatRatio_Method2 = Scat_Ratio_Dont_Use

  ! read commandline arguments
  nr_of_arguments = iargc_aeolus()
  current_argument_nr = 0  
  DO WHILE (current_argument_nr .lt. nr_of_arguments)
     !  #[ process the given commandline arguments
     current_argument_nr = current_argument_nr+1
     call getarg_aeolus(current_argument_nr,current_argument)
     SELECT CASE (trim(current_argument))
     CASE ("-30") 
        print *,"using only 30 measurements"
        N_meas = 30
     CASE ("-nocloud")
        print *,"using no_cloud (dummy) classification"
        use_noclouds = .true.
     CASE DEFAULT
        print *,"commandline argument not recognised: ",trim(current_argument)
        error_flag = error_cmdline_options
        call program_abort(error_flag,"Test_HLOS_Retrieval")
     END SELECT
     !  #]
  END DO

  ! initialise the modules used by the program
  call InitOpticalProperties(PartExt_Method = PartExt_Method_Old, &
                             error_flag     = error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitOpticalProperties")

  ! initialise the data structures used by the program
  call Init_L1B_BRC_DataStructure(L1B, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L1B_BRC_DataStructure")

  !call InitClassification(ChoosenVerbosity=VerbosityError)

  ! initialise the hlos_retrieval module
  call Init_HLOS_Retrieval(error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_HLOS_Retrieval")

  filename = "backscatterdata.dat"

  ! define a dummy threshold profile with value 2.0 everywhere
  ! to get the same results as the old 
  ! Threshold_BackscatterRatio = 2.0 setting.
  call init_profile(BackscRatio_Thresh_Profile,&
       (/0._r8_,30.e3_r8_/),(/2._r8_,2._r8_/),2, &
       "BackscRatio_Thresh_Profile", error_flag, &
       interpolation_type      = interp_type_linear,&
       extrapolation_type_down = extrap_type_linear,&
       extrapolation_type_up   = extrap_type_linear)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"init_profile")

  ! loop over all BRCs in the file
  read_which_BRC = 1
  brc_loop: DO

     ! WorkingData must be initialized/deleted every BRC so that
     ! working values from a previous BRC are not retained 
     call InitWorkingData(WorkingData, &
                 int(N_range_gates,IntAs), int(N_Meas,IntAs), &
                 int(N_Obs_Mie_Max,IntAs), int(N_Obs_Rayleigh_Max,IntAs),&
                 error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"InitWorkingData")

     call Fill_WorkingData_With_Lite_Data(WorkingData, &
          filename, read_which_BRC, nr_BRCs_available, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Fill_WorkingData_With_Lite_Data")

     call Init_L2BC_BRC_DataStructure(L2B, N_meas, &
          N_Obs_Mie_Max, N_Obs_Rayleigh_Max, N_range_gates, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Init_L2BC_BRC_DataStructure")

     ! Set all measurement range-bins to OK
     DO m=1,size(L2B%PCD_ADS%L1B_Input_Screening%Mie_Meas)
       DO r=1,size(L2B%PCD_ADS%L1B_Input_Screening%Mie_Meas(m)%Bin_Screening)
         L2B%PCD_ADS%L1B_Input_Screening%Mie_Meas(m)%Bin_Screening(r)%Bin_QC=&
         L1B_Mie_Bin_OK
       ENDDO
     ENDDO
     DO m=1,size(L2B%PCD_ADS%L1B_Input_Screening%Rayl_Meas)
       DO r=1,size(L2B%PCD_ADS%L1B_Input_Screening%Rayl_Meas(m)%Bin_Screening)
         L2B%PCD_ADS%L1B_Input_Screening%Rayl_Meas(m)%Bin_Screening(r)%Bin_QC=&
         L1B_Rayl_Bin_OK
       ENDDO
     ENDDO

     call Match_Mie_And_Rayleigh_Bins(WorkingData, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Match_Mie_And_Rayleigh_Bins")

     call Calc_Extinction(L1B, WorkingData,                           &
                          MolExt_Method       = MolExt_Method_Proper, &
                          PartExt_Method      = PartExt_Method_Old,   &
                          error_flag          = error_flag,           &
                          calibr_on_upper_bin = .true.,               &
                          use_lite_test_data  = .true.                )
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Calc_Extinction")

     call Copy_Extinction_Rayl_to_Mie(WorkingData)

     call Calc_BackscatterRatio(L1B, WorkingData, error_flag,      &
                                Method         = ScatRatio_Method, &
                                FallbackMethod = ScatRatio_Method2 )
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Calc_BackscatterRatio")

     call Copy_Backsc_Ratio_Rayl_to_Mie(WorkingData)

     IF (use_noclouds) THEN
        ! switch to no-clouds for all rangebins as test     
        call Classify_Mie_Meas_No_Clouds(WorkingData)
        call Classify_Rayl_Meas_No_Clouds(WorkingData)
     ELSE
        call Classify_Mie_Meas_Backsc_Ratio(WorkingData, &
                                            BackscRatio_Thresh_Profile, &
                                            error_flag)
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"Classify_Mie_Meas_Backsc_Ratio")
        call Classify_Rayl_Meas_Backsc_Ratio(WorkingData, &
                                             BackscRatio_Thresh_Profile, &
                                             error_flag)
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"Classify_Rayl_Meas_Backsc_Ratio")
     END IF

     call Select_And_Weigh_Rayleigh_Meas(WorkingData, L2B, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Select_And_Weigh_Rayleigh_Meas")

     call Select_And_Weigh_Mie_Meas(WorkingData, L2B, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Select_And_Weigh_Mie_Meas")

     call Construct_Rayleigh_Obs(WorkingData, L2B, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Construct_Rayleigh_Obs")

     call Construct_Mie_Obs(WorkingData, L2B, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Construct_Mie_Obs")

     write(6,'(A)')'WARNING: testing of hlos retrieval'//&
                   ' not fully implemented'
!     TODO: Implement testing of hlos retrieval:
!           1) Load or set L1B fields needed for HLOS retrieval
!           2) Activate calls to Process_*_Observations
!
!     call Process_Rayleigh_Observations(L1B, L2B, error_flag)
!     IF (error_flag .ne. no_error) &
!          call program_abort(error_flag,"Process_Rayleigh_Observations")
!     call Process_Mie_Observations(L1B, L2B_AuxPar, L2B, error_flag)
!     IF (error_flag .ne. no_error) &
!          call program_abort(error_flag,"Process_Mie_Observations")

     ! print classification counts for this BRC
     !call Print_Constructed_Obs(L2B)
     ! improved version printing the same numbers
     call Print_Constr_Obs_Matched(WorkingData,L2B)

     ! print BackscatterRatio, Extinction and classification
     ! result for a single Measurement
     call PrintBackscatterRatio(WorkingData,Meas_Nr=1,PrintClass=.true.)

     ! print the results for a single Measurement as test
     !call PrintClassificationResult(L1B,WorkingData,L2B,Meas_Nr=1)

     ! print the Map describing which measurement contributed
     ! to which observation for the Rayleigh channel
     print *,"Map of Rayleigh L1B measurements used:"
     call Print_Meas_Map(L2B%Rayleigh_MDS%Map_of_L1B_Measurements_Used)
     
     ! delete the L2B datastrcuture, it will be allocated again for
     ! the next BRC after reading the next BRC from the L1B file.
     call Delete_L2BC_BRC_DataStructure(L2B)

     ! delete the WorkingData structure, it will be allocated again for
     ! the next BRC after reading the next BRC from the L1B file.
     call DeleteWorkingData(WorkingData)

     ! as test, just loop once
     ! exit brc_loop

     read_which_BRC =  read_which_BRC + 1
     IF (read_which_BRC .gt. nr_BRCs_available) exit brc_loop
  END DO brc_loop

  print *,"=================================="
  print *,"nr of processed BRC's was: ",nr_BRCs_available
  print *,"=================================="

  ! free allocated memory when done
  call Delete_L1B_BRC_DataStructure(L1B)
  call DeleteWorkingData(WorkingData)
  call delete_profile(BackscRatio_Thresh_Profile)
  call CloseOpticalProperties

  !  #]
end program Test_HLOS_Retrieval
