program Test_Screening_L1B_Data
  !  #[ Documentation
  !     a test program for the screening_l1b_data module
  !     written by: J. de Kloe (KNMI)
  !     based on: ../L1B_file_handling/TestReadL1Bdata.F90
  !
  !     Modifications:
  !        08-Jun-2006  J. de Kloe  initial version
  !        28-Aug-2007  D. Tan      remove reference to obsolete routine
  !        04-Sep-2007  P. Poli     introduced L1B_Headers, change
  !                                 L1B_DataType to L1B_BRC_DataType and
  !                                 L1B_HDR_DataType
  !        16-Jan-2008  J. de Kloe  phase out integer kind i_
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L1B_DataStructure, only: L1B_HDR_DataType, &
                               L1B_BRC_DataType, &
                               Init_L1B_HDR_DataStructure, &
                               Init_L1B_BRC_DataStructure, &
                               Delete_L1B_HDR_DataStructure, &
                               Delete_L1B_BRC_DataStructure
  USE L2B_PCD_ADS, only: L1B_Input_Screening_type, Init_L1B_Input_Screening,&
                         Delete_L1B_Input_Screening

  USE L2B_AuxPar_DataStructure, only: L2B_AuxPar_Type, Init_L2B_AuxPar, &
                                      Delete_L2B_AuxPar
  USE Working_DataStructure, only: WorkingDataType,&
                                   InitWorkingData, DeleteWorkingData
  USE Read_L2B_AuxPar_data, only: InitRead_L2B_AuxPar_fileModule, &
                                  Read_L2B_AuxPar_file

  USE ReadL1Bdata, only: ReadL1Bfile, InitReadL1BfileModule
  USE screening_l1b_data, only: screen_l1b_data
  USE EE_CFI_DataTypes, only: IntAs

  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus
  !  #]
  !  #[ variables
  implicit none

  integer            :: error_flag, nr_of_arguments
  character(len=256) :: L1B_Filename        ! without extension
  character(len=256) :: L2B_AuxPar_Filename ! with .EEF extension
  TYPE(L1B_HDR_DataType) :: L1B_Headers
  TYPE(L1B_BRC_DataType) :: L1B
  TYPE(L1B_Input_Screening_type) :: L1B_Screening
  TYPE(L2B_AuxPar_Type)          :: L2B_AuxPar
  TYPE(WorkingDataType)          :: WorkingData
  integer :: N_meas
  integer :: N_range_gates
  integer :: N_Obs_Mie_Max, N_Obs_Rayleigh_Max
  !  #]
  !  #[ program code

  ! read the filename without extension as 1st commandline parameter:
  nr_of_arguments = iargc_aeolus()
  IF (nr_of_arguments .ne. 2) THEN
     print *,"Usage: TestReadL1Bdata <L1B_Filename[without extension]> "
     print *,"                       <L2B_AuxPar_Filename[with extension]>"
     call program_abort(error_cmdline_options,"TestReadL1Bdata")
  END IF

  ! get the L1B filename given on the commandline (without extension)
  call getarg_aeolus(1,L1B_Filename)

  ! get the L2B_AuxPar filename given on the commandline (with extension)
  call getarg_aeolus(2,L2B_AuxPar_Filename)
  
  call Init_L1B_HDR_DataStructure(L1B_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L1B_HDR_DataStructure")  

  call Init_L1B_BRC_DataStructure(L1B, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L1B_BRC_DataStructure")  

  call InitReadL1BfileModule(error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitReadL1BfileModule")

  ! read all datablocks for DSR=1
  call ReadL1Bfile(L1B_Headers,L1B_Filename,error_flag, &
       L1B=L1B,ReadAll=.true.,which_DSR=1)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"ReadL1Bfile")

  N_meas             = L1B_Headers%SPH%N_Meas_Max
  N_range_gates      = 24
  N_Obs_Mie_Max      = 4
  N_Obs_Rayleigh_Max = 4

!  print *,"initialising working arrays..."
  call InitWorkingData(WorkingData, &
                 int(N_range_gates,IntAs), int(N_Meas,IntAs), &
                 int(N_Obs_Mie_Max,IntAs), int(N_Obs_Rayleigh_Max,IntAs),&
                 error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitWorkingData")
!  print *,"done initialising working arrays"

  print *,"N_meas = ",N_meas
!  print *,"N_meas = ",L1B%nr_of_measurements

  ! init L2B_AuxPars module and datastruct
  call InitRead_L2B_AuxPar_fileModule(error_flag)
!                ,ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitRead_L2B_AuxPar_fileModule")

  call Init_L2B_AuxPar(L2B_AuxPar,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2B_AuxPar")

  ! read L2B_AuxPars
  call Read_L2B_AuxPar_file(L2B_AuxPar, L2B_AuxPar_Filename, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Read_L2B_AuxPar_file")

  call Init_L1B_Input_Screening(L1B_Screening, int(N_Meas,IntAs),&
                                int(N_range_gates,IntAs),error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L1B_Input_Screening")
  
  ! change this threshold to force a threshold passed message
  ! (for testing purposes)
  L2B_AuxPar%Data%Screening_Params%L1B_Screening_Params%&
       L1B_Obs_Screening_Params%L1B_Laser_Freq_Unlocked_Thresh = -1

  call Screen_L1B_Data(L1B, WorkingData, L1B_Screening, &
                       L2B_AuxPar%Data%Screening_Params%L1B_Screening_Params,&
                       error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Screen_L1B_Data")
  
  call DeleteWorkingData(WorkingData)
  call Delete_L1B_Input_Screening(L1B_Screening)
  call Delete_L2B_AuxPar(L2B_AuxPar)
  call Delete_L1B_BRC_DataStructure(L1B)
  call Delete_L1B_HDR_DataStructure(L1B_Headers)
  !  #]
end program Test_Screening_L1B_Data
