program TestKVT_module
  !  #[ Description
  !     a test program to test the KVT_module
  !     created:     23-06-2005 by J. de Kloe, KNMI
  !
  !     Modifications:
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_
  !
  !  #]
  !  #[ Modules used
  ! the module to be tested
  USE KVT_module, only: Init_KVT_module, &
                        write_kvt_string, write_spare_kvt_string, &
                        read_kvt_string, read_spare_kvt_string
  ! support modules
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE LunManager, only: get_lun, free_lun
  USE ErrorHandler, only: no_error, error_reading_file,&
                          error_writing_file, error_opening_file, &
                          program_abort
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  !  #]
  !  #[ Variable declarations
  implicit none
  character(len=256)        :: filename
  integer                   :: fileunit
  integer                   :: error_flag
  character(len=256)        :: expected_name
  character(len=256)        :: value
  character(len=256)        :: phys_unit
  !  #]
  !  #[ the program code
!  call Init_KVT_module(ChoosenVerbosity=VerbosityReport)
  call Init_KVT_module()
  filename = "kvt_test_file.bin"
  !  #[ test writing KVT items to binary file
  print *,"writing KVT data to binary file: ",trim(filename)  
  fileunit = get_lun()
  IF (fileunit .lt. 0) call program_abort(error_opening_file,"get_lun")
  
  open(unit=fileunit,file=filename,status="replace",&
       action="write",access="direct",recl=1,form="UNFORMATTED",ERR=999)
  
  call write_kvt_string(fileunit,"testitem1","dummy value",15,error_flag,&
                        use_quotes=.true.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_kvt_string")

  call write_kvt_string(fileunit,"testitem2","15",2,error_flag,&
                        phys_unit="bytes")
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_kvt_string")

  call write_spare_kvt_string(fileunit,25,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_spare_kvt_string")

  call write_kvt_string(fileunit,"testitem3",&
       "value deliberately made too long",20,error_flag,use_quotes=.true.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"write_kvt_string")

  close(unit=fileunit)
  call free_lun(fileunit)

  print *,"done with writing test"  
  !  #]
  filename = "kvt_test_file.bin"
  !  #[ test reading KVT items from binary file
  print *,"reading KVT data from binary file: ",trim(filename)
  
  fileunit = get_lun()
  IF (fileunit .lt. 0) call program_abort(error_opening_file,"get_lun")
  
  open(unit=fileunit,file=filename,status="old",&
       action="read",access="direct",recl=1,form="UNFORMATTED",ERR=998)

  expected_name = "testitem1"
  call read_kvt_string(fileunit,expected_name,value,error_flag,&
                       expected_value_length=15,expect_quotes=.true.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_kvt_string")
  print *,"reading result: name = [",trim(expected_name),&
       "] value = [",trim(value),"]"

  expected_name = "testitem2"
  phys_unit = "bytes"
  call read_kvt_string(fileunit,expected_name,value,error_flag,&
                       expected_value_length=2,expected_phys_unit=phys_unit)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_kvt_string")
  print *,"reading result: name = [",trim(expected_name),&
       "] value = [",trim(value),"] phys_unit = [",trim(phys_unit),"]"

  call read_spare_kvt_string(fileunit,25,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_writing_file,"read_spare_kvt_string")
  print *,"successfully read spare item of size : 25"

  expected_name = "testitem3"
  call read_kvt_string(fileunit,expected_name,value,error_flag,&
                       expected_value_length=20,expect_quotes=.true.)
  IF (error_flag .ne. no_error) &
       call program_abort(error_reading_file,"read_kvt_string")
  print *,"reading result: name = [",trim(expected_name),&
       "] value = [",trim(value),"]"

  close(unit=fileunit)
  call free_lun(fileunit)

  print *,"done with reading test"  
  !  #]
  stop

998 print *,"ERROR could not open outputfile: ",trim(filename)
    call program_abort(error_opening_file,"open")

999 print *,"ERROR could not open inputfile: ",trim(filename)
    call program_abort(error_opening_file,"open")
  !  #]
end program TestKVT_module
