program TestReadL2BCdata
  !  #[ Documentation
  !     a test program for the ReadL2BCdata module
  !     written by: J. de Kloe (KNMI)
  !     created:     02-09-2005
  !
  !     Modifications:
  !     12-May-2006 D. Tan      Null use of ParKind1 module
  !     17-Nov-2006 J. de Kloe  removed fixed dimensions N_Obs_Mie etc
  !                             since these should be read from the
  !                             input file before use
  !     17-Nov-2006 J. de Kloe  added a switch to detect whether a L2B or
  !                             L2C file should be read 
  !     21-Nov-2006 J. de Kloe  added inspection of executable name
  !                             to switch between TestReadL2Bdata and
  !                             Write_L2B_Text_Product functionality
  !     23-Feb-2007 J. de Kloe  Expanded the commandline handling, to enable
  !                             selecting datasets and BRC's 
  !     15-Mar-2007 J. de Kloe  added reading of L2B PCD and all L2C datasets
  !                             added AsciiWrite for L2B PCD (not yet for the
  !                             L2C datasets)
  !     19-Mar-2007 J. de Kloe  fixed, to make sure the headers are printed
  !                             by the Write_L2BC_Text_Product tool, even
  !                             when Num_BRC=0
  !     19-Mar-2007 J. de Kloe  added AsciiWrite for all L2C datasets
  !     03-Sep-2007 P. Poli     introduced L2BC_Headers, change L2BC_DataType 
  !                             to L2BC_BRC_DataType and L2BC_HDR_DataType
  !     10-Oct-2007 J. de Kloe  adapted to changed AsciiWrite routines
  !     16-Jan-2008 J. de Kloe  phase out integer kind i_
  !  #]
  !  #[ Modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BC_DataStructure, only: L2BC_HDR_DataType, &
                                L2BC_BRC_DataType, &
                                Init_L2BC_HDR_DataStructure, &
                                Init_L2BC_BRC_DataStructure, &
                                Delete_L2BC_HDR_DataStructure, &
                                Delete_L2BC_BRC_DataStructure, &
                                Empty_L2BC_BinaryDatasets, &
                                AsciiWrite_L2BC_HDR_DataStruct, &
                                AsciiWrite_L2BC_BRC_DataStruct, &
                                AsciiWrite_L2BC_hlos_Data
!                                Print_L2BC_BRC_DataStructure
  USE ReadL2BCdata, only: ReadL2BCfile, InitReadL2BCfileModule
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport
  USE StringTools, only: Split_path_and_filename
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options, &
                          error_writing_file, error_opening_file, &
                          request_program_to_stop
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus
  USE EE_CFI_Datatypes, only: IntAs, IntAuc
  USE LunManager, only: get_lun, free_lun, fileunit_stdout
  !  #]
  !  #[ variables
  implicit none
  integer             :: error_flag, nr_of_arguments
  integer             :: current_argument_nr, dsr
  integer             :: Num_BRC, iBRC, BRC_Start, BRC_Stop
  integer             :: fileunit
  character(len=256)  :: path_and_filename ! without extension
  character(len=256)  :: path
  character(len=256)  :: current_argument
  character(len=256)  :: ds_name
  character(len=256)  :: dsr_text
  character(len=256)  :: filename ! without extension
  character(len=256)  :: filename_txt
  character(len=256)  :: executable_name, arg0, exe_path
  character(len=5)    :: bool_text
  character(len=4)    :: extension
  character(len=4)    :: str_cntr
  character(len=2)    :: L2BC_Key
  logical             :: L2C_flag
  logical             :: do_Write_L2BC_Text_Product
  logical             :: UseGeoloc, UsePCD, UseMie, UseRayleigh
  logical             :: UseAssimPCD, UseMieVecWind, UseRaylVecWind
  logical             :: PrintHeaders
  logical             :: first_ds_switch
  logical             :: PrintAll
  logical             :: hlos_only
  TYPE(L2BC_HDR_DataType) :: L2BC_Headers
  TYPE(L2BC_BRC_DataType) :: L2BC
  !  #]
  !  #[ program code

  !  #[ initialisations
  error_flag = no_error
  ! the default is to read all dsr's
  dsr = -1
  ! the default is to read all datasets
  first_ds_switch = .true.
  UseGeoloc    = .true.
  UsePCD       = .true.
  UseMie       = .true.
  UseRayleigh  = .true.
  UseAssimPCD    = .true. ! L2C only
  UseMieVecWind  = .true. ! L2C only
  UseRaylVecWind = .true. ! L2C only
  PrintHeaders = .true.
  PrintAll     = .true. ! print complete arrays
  hlos_only    = .false.
  !  #]
  !  #[ commandline handling

  call getarg_aeolus(0,arg0)
  call Split_path_and_filename(arg0,exe_path,executable_name)
  !print *,"arg0 = ",trim(arg0)
  !print *,"executable_name = ",trim(executable_name)
  do_Write_L2BC_Text_Product = .false.
  select case (trim(executable_name))
  case("TestReadL2BCdata",&
       "TestReadL2Bdata", &
       "TestReadL2Cdata")
     do_Write_L2BC_Text_Product = .false.
  case("Write_L2BC_Text_Product",&
       "Write_L2B_Text_Product", &
       "Write_L2C_Text_Product")
     do_Write_L2BC_Text_Product = .true.
  case default
     print *,"Please don't rename this tool, since it uses its name"
     print *,"to decide what it should do !"
     print *,"current name seems to be: ",trim(arg0)
     print *,"allowed names are:"
     print *,"   TestReadL2BCdata"
     print *,"   TestReadL2Bdata"
     print *,"   TestReadL2Cdata"
     print *,"   Write_L2BC_Text_Product"
     print *,"   Write_L2B_Text_Product"
     print *,"   Write_L2C_Text_Product"
     call program_abort(error_cmdline_options,trim(arg0))     
  end select

  ! read the filename without extension as 1st commandline parameter:
  nr_of_arguments = iargc_aeolus()
  IF (nr_of_arguments .lt. 1) THEN
     call Prnt_Usage_Reader_Cmdl_Options
     call program_abort(error_cmdline_options,trim(executable_name))
  END IF

  ! get the filename (this commandline argument is always present)
  call getarg_aeolus(1,path_and_filename)

  ! the first option might also be "-help" so take care of that possibility
  IF (trim(path_and_filename) .eq. "-help") THEN
     print *,"handling -help option"
     call Print_Extended_Help
     call program_abort(request_program_to_stop,trim(executable_name))
  END IF

  ! see if we have a L2B or L2C file by cutting a few characters
  ! from the filename and inspecting them.
  ! example filenames:
  !   AE_TEST_ALD_U_T_2B_20010312T123456_20010312T123700_0001
  !   AE_TEST_ALD_U_T_2C_20010312T123456_20010312T123700_0001
  !   123456789012345678
  call Split_path_and_filename(path_and_filename,path,filename)
  L2BC_Key = filename(17:18)
  !print *,"L2BC_Key = ",L2BC_Key

  select case(L2BC_Key)
  case("2B")
     L2C_flag = .false.
     UseAssimPCD    = .false.
     UseMieVecWind  = .false.
     UseRaylVecWind = .false.
  case("2C")
     L2C_flag = .true.
  case default
     print *,"ERROR in "//trim(executable_name)//&
          ": inputfile name not recognised"
     print *,"could not extract 2B/2C string from it...."
     call program_abort(error_flag,trim(executable_name))
  end select

  ! extra sanity check
  extension = filename(len_trim(filename)-3:len_trim(filename))
  IF ((extension .eq. ".DBL") .or. (extension .eq. ".HDR")) THEN
     print *,"ERROR in "//trim(executable_name)//&
          ": could not load file"
     print *,"filename seems to end with an extension: "//extension
     print *,"file: "//trim(filename)
     print *,"but this tool requires the filename to be specified"
     print *,"without extension ....."
     call Prnt_Usage_Reader_Cmdl_Options
     call program_abort(error_cmdline_options,trim(executable_name))     
  END IF

  ! handle the optional commandline arguments
  current_argument_nr = 1
  IF (nr_of_arguments .gt. 1) THEN
     argloop: DO WHILE (current_argument_nr .lt. nr_of_arguments)
        !  #[ process the given commandline arguments

        !     but skip the first one, which was handled above
        current_argument_nr = current_argument_nr+1
        call getarg_aeolus(current_argument_nr,current_argument)

        SELECT CASE (trim(current_argument))
        CASE ("-dsr")
           !  #[
           print *,"handling -dsr option"
           current_argument_nr = current_argument_nr+1
           IF (current_argument_nr .gt. nr_of_arguments) THEN
              print *,"dataset record number is missing after -dsr option !!!"
              call Prnt_Usage_Reader_Cmdl_Options
              call program_abort(error_cmdline_options,trim(executable_name))
           END IF
           call getarg_aeolus(current_argument_nr,dsr_text)
           print *,"using dsr: ",trim(dsr_text)
           read(dsr_text,"(i8)",err=999) dsr
           print *,"using dsr: ",dsr
           !  #]
        CASE ("-ds")
           !  #[
           print *,"handling -ds option"
           current_argument_nr = current_argument_nr+1
           IF (current_argument_nr .gt. nr_of_arguments) THEN
              print *,"dataset name is missing after -ds option !!!"
              call Prnt_Usage_Reader_Cmdl_Options
              call program_abort(error_cmdline_options,trim(executable_name))
           END IF
           call getarg_aeolus(current_argument_nr,ds_name)
           print *,"using dsr: ",trim(ds_name)
           ! when this option is set, no datasets are read, except the 
           ! selected one(s), so set all these switches to false.
           ! However, to enable selecting 2 or more, this should only
           ! be done when the first "-ds" switch is handled
           IF (first_ds_switch) THEN
              print *,"switching all datasets off"
              UseGeoloc       = .false.
              UsePCD          = .false.
              UseMie          = .false.
              UseRayleigh     = .false.
              UseAssimPCD     = .false.
              UseMieVecWind   = .false.
              UseRaylVecWind  = .false.
              first_ds_switch = .false.
           END IF

           SELECT CASE (trim(ds_name))
              CASE("geoloc")
                 print *,"switching geoloc dataset on"
                 UseGeoloc   = .true.
              CASE("pcd") 
                 print *,"switching pcd dataset on"
                 UsePCD      = .true.
              CASE("mie")
                 print *,"switching mie dataset on"
                 UseMie      = .true.
              CASE("rayleigh")
                 print *,"switching rayleigh dataset on"
                 UseRayleigh = .true.
              CASE("assimpcd")
                 IF (L2C_flag) THEN
                    print *,"switching assimpcd dataset on"
                    UseAssimPCD = .true.
                 ELSE
                    call Prnt_Usage_Reader_Cmdl_Options
                    print *,"switch -ds assimpcd is only usefull "//&
                         "when reading a L2C file"
                    print *,"but this seems an L2B file !"
                    call program_abort(error_cmdline_options,&
                         trim(executable_name))
                 END IF
              CASE("mievecwind")
                 IF (L2C_flag) THEN
                    print *,"switching mievecwind dataset on"
                    UseMieVecWind = .true.
                 ELSE
                    call Prnt_Usage_Reader_Cmdl_Options
                    print *,"switch -ds mievecwind is only usefull "//&
                         "when reading a L2C file"
                    print *,"but this seems an L2B file !"
                    call program_abort(error_cmdline_options,&
                         trim(executable_name))
                 END IF
              CASE("raylvecwind")
                 IF (L2C_flag) THEN
                    print *,"switching raylvecwind dataset on"
                    UseRaylVecWind = .true.
                 ELSE
                    call Prnt_Usage_Reader_Cmdl_Options
                    print *,"switch -ds raylvecwind is only usefull "//&
                         "when reading a L2C file"
                    print *,"but this seems an L2B file !"
                    call program_abort(error_cmdline_options,&
                         trim(executable_name))
                 END IF
              CASE DEFAULT
                 call Prnt_Usage_Reader_Cmdl_Options
                 print *,"unrecognised switch after the -ds option: ",&
                      trim(ds_name)
                 call program_abort(error_cmdline_options,&
                      trim(executable_name))
           END SELECT
           !  #]
        CASE ("-noheaders")
           !  #[
           print *,"not printing headers"
           PrintHeaders = .false.
           !  #]
        CASE ("-printall")
           !  #[
           print *,"handling -printall option"
           current_argument_nr = current_argument_nr+1
           IF (current_argument_nr .gt. nr_of_arguments) THEN
              print *,"boolean switch is missing after -printall option !!!"
              call Prnt_Usage_Reader_Cmdl_Options
              call program_abort(error_cmdline_options,trim(executable_name))
           END IF
           call getarg_aeolus(current_argument_nr,bool_text)
           SELECT CASE (trim(bool_text))
              CASE("true")
                 print *,"switching to PrintAll=.true. (printing full arrays)"
                 PrintAll = .true.
              CASE("false")
                 print *,"switching to PrintAll=.false. "//&
                      "(printing only first and last elements of arrays)"
                 PrintAll = .false.
              CASE DEFAULT
                 call Prnt_Usage_Reader_Cmdl_Options
                 print *,"unrecognised switch after the -ds option: ",&
                      trim(ds_name)
                 call program_abort(error_cmdline_options,&
                      trim(executable_name))
           END SELECT
           !  #]
        CASE ("-onlyhlos")
           !  #[
           print *,"printing hlos results only"
           hlos_only    = .true.
           !  #]
        CASE ("-help")
           !  #[
           print *,"handling -help option"
           call Print_Extended_Help
           call program_abort(request_program_to_stop,&
                              trim(executable_name))
           !  #]
        CASE DEFAULT
           !  #[
           call Prnt_Usage_Reader_Cmdl_Options
           print *,"unrecognised commandline option: ",trim(current_argument)
           IF (current_argument(1:1) .ne. '-') THEN
              print *,"This argument does not start with: -"
              print *,"Maybe you forgot the - character to indicate that this"
              print *,"is an option, or maybe you supplied the wrong number"
              print *,"of items to an option."
           END IF
           print *,"============================================"
           call program_abort(error_cmdline_options,trim(executable_name))
           !  #]
        END SELECT
        !  #]
     END DO argloop
  END IF

  !  #] end of commandline handling

  ! initialise the module and the datastructure
  call Init_L2BC_BRC_DataStructure(L2BC, 0, 0_IntAuc, 0_IntAuc, 0, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2BC_BRC_DataStructure")  

  call Init_L2BC_HDR_DataStructure(L2BC_Headers, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2BC_HDR_DataStructure")

  IF (do_Write_L2BC_Text_Product) THEN
     call InitReadL2BCfileModule(error_flag,ChoosenVerbosity=VerbosityError)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"InitReadL2BCfileModule")
  ELSE
     call InitReadL2BCfileModule(error_flag,ChoosenVerbosity=VerbosityReport)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"InitReadL2BCfileModule")
  END IF

  ! read the headers only, to retrieve the nr of BRC's in this file
  call ReadL2BCfile(L2BC_Headers, filename, error_flag, Which_DSR=1, &
                    L2C_flag=L2C_flag, ReadHeaders=.true.)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"ReadL2BCfile")

  IF (do_Write_L2BC_Text_Product) THEN
     !filename_txt = trim(filename)//".TXT"
     filename_txt = trim(path_and_filename)//".TXT"
     fileunit = get_lun()
     IF (fileunit .lt. 0) THEN
        ! no fileunits available, so report an error
        print *,"Sorry, no free fileunits available..."
        call program_abort(error_opening_file,trim(executable_name))
     END IF
     open(unit=fileunit,file=filename_txt,status="replace",&
          action="write",form="FORMATTED",ERR=997)
  ELSE
     fileunit=Fileunit_stdout
  END IF

  IF (do_Write_L2BC_Text_Product .and. PrintHeaders .and. .not. hlos_only) THEN
     ! headers need only to be printed once, since they are
     ! the same for all BRC's, and they need to be printed, even if 
     ! Num_BRC=0, so print them outside the BRC loop
     call AsciiWrite_L2BC_HDR_DataStruct(L2BC_Headers, fileunit, &
                   prefix     = "L2BC", &
                   error_flag = error_flag,&
                   L2C_flag   = L2C_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"AsciiWrite_L2BC_HDR_DataStruct")
  END IF

  ! Set the number of BRCs
  Num_BRC = L2BC_Headers%SPH%Geolocation_DSD%nr_of_dataset_records

  BRC_Start = 1
  BRC_Stop  = Num_BRC

  IF ((dsr .lt. -1) .or. (dsr .gt. Num_BRC)) THEN
     print *,"ERROR: datasetrecord has an invalid value!"
     print *,"dsr = ",dsr
     print *,"but: Num_BRC is only: ",Num_BRC
     call program_abort(error_cmdline_options,trim(executable_name))
  END IF

  IF (dsr .ne. -1) THEN
     ! a specific dsr was requested 
     BRC_Start = dsr
     BRC_Stop  = dsr
  END IF

  print *,"Num_BRC, BRC_Start, BRC_Stop = ",Num_BRC, BRC_Start, BRC_Stop

  brc_loop: DO iBRC=BRC_Start,BRC_Stop
     !  #[

     ! read all datablocks
     call ReadL2BCfile(L2BC_Headers, filename, error_flag,     &
                       L2BC            = L2BC,         &
                       Which_DSR       = iBRC,         &
                       L2C_flag        = L2C_flag,     &
                       ReadGeoloc      = UseGeoloc,    &
                       ReadPCD         = UsePCD,       &
                       ReadMie         = UseMie,       &
                       ReadRayleigh    = UseRayleigh,  &
                       ReadAssimPCD    = UseAssimPCD,  &
                       ReadMieVecWind  = UseMieVecWind,&
                       ReadRaylVecWind = UseRaylVecWind )
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"ReadL2BCfile")

     IF (do_Write_L2BC_Text_Product .and. .not. hlos_only) THEN
        write(str_cntr,"(i4.4)",err=998) iBRC
        call AsciiWrite_L2BC_BRC_DataStruct(L2BC, fileunit, &
                   prefix="L2BC("//trim(str_cntr)//")", &
                   PrintAll      = PrintAll,  &
                   error_flag    = error_flag,&
                   WriteGeoloc   = UseGeoloc, &
                   WritePCD      = UsePCD,    &
                   WriteMie      = UseMie,    &
                   WriteRayleigh = UseRayleigh, &
                   L2C_flag      = L2C_flag,  &
                   WriteAssimPCD    = UseAssimPCD,   &
                   WriteMieVecWind  = UseMieVecWind, & 
                   WriteRaylVecWind = UseRaylVecWind )
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"AsciiWrite_L2BC_BRC_DataStruct")
     END IF

     IF (do_Write_L2BC_Text_Product .and. hlos_only) THEN
        call AsciiWrite_L2BC_hlos_Data(L2BC, fileunit, error_flag, &
                                       WriteMie      = UseMie,      &
                                       WriteRayleigh = UseRayleigh, &
                                       L2C_flag      = L2C_flag)
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"AsciiWrite_L2BC_hlos_Data")
     END IF

     call Empty_L2BC_BinaryDatasets(L2BC)

     !  #]
  END DO brc_loop

  IF (do_Write_L2BC_Text_Product) THEN
     close(unit=fileunit)
     call free_lun(fileunit)
     print *,"Conversion to ASCII was successfull"
  ELSE
     print *,"File validation was successfull"
  END IF

  call Delete_L2BC_BRC_DataStructure(L2BC,L2C_flag)
  call Delete_L2BC_HDR_DataStructure(L2BC_Headers)
  goto 111
  !  #] end of the main program
  !  #[ exception/error handlers
997 print *,"ERROR  in: "//trim(executable_name)
    print *,"could not open outputfile: ",trim(filename_txt)
    call program_abort(error_opening_file,trim(executable_name))

998 print *,"conversion error in: "//trim(executable_name)
    print *,"could not convert the counter iBRC = ",iBRC," to a string .."
    call program_abort(error_writing_file,trim(executable_name))    

999 print *,"ERROR: could not convert datasetrecord into an integer number!"
  print *,"dsr_text = "//trim(dsr_text)
  call program_abort(error_cmdline_options,trim(executable_name))

111 continue 
  !  #] end of the error exception handler
  !--------------------
contains ! a helper subroutine
  !--------------------
  subroutine Prnt_Usage_Reader_Cmdl_Options
    !  #[ 
    print *,"============================================"
    print *,""
    print *,"Usage: "
    print *,trim(executable_name)//" <filename[without extension]>"
    print *,"       [-ds <geoloc|pcd|mie|rayleigh>]         (L2B and L2C)"
    print *,"       [-ds <assimpcd|mievecwind|raylvecwind>] (L2C only)"
    print *,"       [-dsr <which_dataset_record>]"
    print *,"       [-noheaders]"
    print *,"       [-printall <true|false>]"
    print *,"       [-onlyhlos]"
    print *,"       [-help]"
    print *,""
    print *,"============================================"

  end subroutine Prnt_Usage_Reader_Cmdl_Options
    !  #]
  subroutine Print_Extended_Help
    !  #[ 
    print *,"============================================"
    print *,""
    print *,"Usage of the L2B/L2C reading tool: "
    print *,""
    print *,"This tool may be used to validate ADM-Aeolus L2B or L2C files"
    print *,"or to convert them to a human readably ascii format."
    print *,"For these different uses, the tool may be renamed to the"
    print *,"following names:"
    print *,""
    print *,"   TestReadL2Bdata"
    print *,"   TestReadL2Cdata"
    print *,"   TestReadL2BCdata"
    print *,"   Write_L2B_Text_Product"
    print *,"   Write_L2C_Text_Product"
    print *,"   Write_L2BC_Text_Product"
    print *,""
    print *,"If the name starts with TestRead then it will only validate"
    print *,"the Earth Explorer format of the input file, by reading it."
    print *,"In case of trouble it should give proper explanation on"
    print *,"what is wrong."
    print *,""
    print *,"If the name starts with Write and ends with Text_Product then"
    print *,"it will try to read the Earth Explorer input file and export"
    print *,"the data to an ASCII file with the same name but with"
    print *,"extension .TXT"
    print *,""
    print *,"The L2B/L2C/L2BC in the name is actually irrelevant, because"
    print *,"the tool detects the filetype from the given input file."
    print *,"This part of the naming of the tool is therefore for the"
    print *,"users convenience only"
    print *,""
    print *,"The tool requires the filename (without extension) of the"
    print *,"Earth Explorer input file to be used as first commandline option."
    print *,"(the only exception is the -help option described below)"
    print *,""
    print *,"The following optional commandline options are accepted after"
    print *,"the filename:"
    print *,""
    print *,"  [-ds <geoloc|pcd|mie|rayleigh>]         (L2B and L2C)"
    print *,"  [-ds <assimpcd|mievecwind|raylvecwind>] (L2C only)"
    print *,""
    print *,"the -ds option may be used to select using one or several"
    print *,"datasets present in the input file. When validating a file"
    print *,"this option selects which datasets are validated."
    print *,"When converting to ASCII this option selects which datasets"
    print *,"will be written to the output file."
    print *,"By default all datasets are used."
    print *,""
    print *,"  [-dsr <which_dataset_record>]"
    print *,""
    print *,"the -dsr option may be used to select a given BRC in the input"
    print *,"file."
    print *,"By default all BRC's in the input file are validated/exported"
    print *,""
    print *,"  [-noheaders] "
    print *,""
    print *,"the -noheaders may be used to suppress exporting the headers"
    print *,"to the output ASCII file. It has no influence on validating"
    print *,"a file."
    print *,"By default exporting of headers is switched on."
    print *,""
    print *,"  [-printall <true|false>] "
    print *,""
    print *,"the -printall option may be used to switch between exporting"
    print *,"the full data, or just a selection to the ASCII output file."
    print *,"When set to true all data in the file is exported."
    print *,"When set to false only the first and last element of all"
    print *,"arrays in the datastructure are printed."
    print *,"It has no influence on validating files."
    print *,"By default printall is set to true."
    print *,""
    print *,"  [-onlyhlos]"
    print *,""
    print *,"the -onlyhlos option may be used to quickly select printing"
    print *,"of only hlos results. It can be used in combination with"
    print *,"selecting the Mie or rayleigh channel using the -ds option."
    print *,"By default this option is not selected."
    print *,""
    print *,"  [-help] "
    print *,""
    print *,"the -help option is used to print the current help text."
    print *,""
    print *,"============================================"
    print *,""

  end subroutine Print_Extended_Help
    !  #]
  !--------------------
end program TestReadL2BCdata
