program TestWriteL2BCdata
  !  #[ Documentation
  !     a test program for the WriteL2BCdata module
  !     written by: J. de Kloe (KNMI)
  !     created:     02-09-2005
  !
  !     Modifications:
  !     11-Oct-2005  J. de Kloe   MPH/SPH improved.
  !     01-Dec-2005  P. Poli      Interfaced with WriteL2Bfile
  !     19-Dec-2005  J. de Kloe   small correction on the allocatable array
  !     11-Jan-2006  J. de Kloe   removed a not used USE statement
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     25-Oct-2006  D. Tan       introduce L2C_flag
  !     15-Mar-2007  J. de Kloe   removed use of SummaryVars
  !     15-Mar-2007  J. de Kloe   added option to test with N_Obs_* = 0
  !     16-Mar-2007  J. de Kloe   added option to generate a product file
  !                               without any datasets in it
  !     19-Mar-2007  J. de Kloe   reduced some numbers to limit the size of
  !                               the testfiles. (*.TXT file reduced from 62MB
  !                               to 1.1 MB !!!)
  !     27-Mar-2007  J. de Kloe   added -200BRC commandline option to allow
  !                               generation of a 200 BRC testfile
  !     28-Mar-2007  D. Tan       Pass L2C_Flag to Fill_L2BC_SPH.
  !                               Increase "actual" values for 200BRC file.
  !     04-Apr-2007 J. de Kloe    clean up of USE FixedHeader
  !     07-Jun-2007 J. de Kloe    added two int() calls convert int types
  !     13-Jun-2007 J. de Kloe    adapted to changed parameterlist of the
  !                               WriteL2BCfile routine
  !     30-Jul-2007 J. de Kloe    adapt to changed interface of Fill_FH_Items
  !     30-Jul-2007 J. de Kloe    adapted to changed Fill_L2BC_MPH routine
  !     03-Sep-2007 P. Poli       introduce L2BC_Headers, change L2BC_DataType 
  !                               to L2BC_BRC_DataType and L2BC_HDR_DataType
  !     10-Oct-2007 J. de Kloe    adapted to use FH_Default_Fields
  !     18-Oct-2007 J. de Kloe    adapted to use MPH_Default_Fields
  !     14-Nov-2007 J. de Kloe    made timestep 8 bytes and subtracted a 
  !                               millisecond to make the result more portable
  !     07-Dec-2007 J. de Kloe    adapted to changed FH/MPH interface
  !     16-Jan-2008 J. de Kloe    phase out integer kind i_
  !     25-Feb-2008 J. de Kloe    corrected filling of mph%ref_doc field
  !     26-Feb-2008 D. Tan        modified  filling of mph%ref_doc, more comments
  !
  !  #]
  !  #[ Modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BC_DS_Settings, only:   Set_Default_L2BCds_Settings, &
                                Get_Cmdline_Options_L2BCds,  &
                                Print_L2BCds_Settings, &
                                set_L2BC_string
  USE L2BC_DataStructure, only: L2BC_HDR_DataType, &
                                L2BC_BRC_DataType, &
                                Init_L2BC_HDR_DataStructure, &
                                Init_L2BC_BRC_DataStructure, &
                                Delete_L2BC_HDR_DataStructure, &
                                Delete_L2BC_BRC_DataStructure, &
                                GetStatisticsL2BMieDS, &
                                GetStatisticsL2BRayleighDS, &
                                Fill_L2BC_FH, Fill_Remaining_L2BC_MPH_Items, &
                                Fill_L2BC_SPH
  USE L2B_Rayleigh_MDS, only: Print_L2B_Rayleigh_hlos_profile
  USE L2B_Mie_MDS,      only: Print_L2B_Mie_hlos_profile
  USE L2B_AuxPar_DataStructure, only: FH_Default_Fields_Type, &
                          Init_FH_Default_Fields, Init_MPH_Default_Fields, &
                          MPH_Default_Fields_type
  USE L2B_Proc_Settings, only: L2BP_Settings_type,Init_L2B_Proc_Settings
  USE FixedHeader,  only: Init_FH, Fill_FH_items, Print_FH, &
                          systemstr_LOCAL_MET_INST
  USE MainProductHeader, only: Init_MPH, Fill_MPH_items, Print_MPH
  USE WriteL2BCdata, only: InitWriteL2BCfileModule, WriteL2BCfile

  USE Compiler_Features, only: newline
  USE ErrorHandler, only: no_error, program_abort, error_allocate
  USE Numerics, only: r8_
  USE EE_CFI_Datatypes, only: IntAs, IntAuc, IntAl, IntAus, &
                              DateTimeType, SetDateTime, IncrementDateTime
  USE Logging, only: logmsg,log_error
  USE AeolusConstants, only: software_ProcName, software_version, &
                             Ref_Doc_L2BC
  !  #]
  !  #[ local variables and parameters

  implicit none

  integer            :: error_flag, AllocStatus
  ! P. Poli: modified this structure so that it can store several BRCs
  TYPE(L2BC_BRC_DataType), dimension(:), pointer :: L2BC
  TYPE(L2BC_HDR_DataType),               pointer :: L2BC_Headers
  logical            :: ZeroObs_flag
  logical            :: Empty_flag
  logical            :: BRC200_flag

  ! used to fill FH
  type(DateTimeType) :: DateTimeStart
  type(DateTimeType) :: DateTimeStop
  real(r8_)          :: timestep
  TYPE(FH_Default_Fields_Type)  :: FH_Default_Fields
  TYPE(MPH_Default_Fields_Type) :: MPH_Default_Fields
  TYPE(L2BP_Settings_type)      :: L2BP_Settings

  ! used to fill Mie_MDS, Rayleigh_MDS
  integer            :: N_Meas,N_Range_Gates
  integer(IntAuc)    :: N_Meas_to_write
  integer(IntAuc)    :: N_Obs_Max_Rayleigh,N_Obs_Max_Mie
  integer(IntAuc)    :: N_Obs_Mie_to_write
  integer(IntAuc)    :: N_Obs_Rayleigh_to_write
  integer            :: iBRC, Num_BRC
  real(r8_)          :: Sat_Track
  logical            :: L2C_flag
  character(len=3)   :: L2BC_string
  integer(IntAl)     :: lat_start,lat_stop,lon_start,lon_stop
  integer(IntAuc)    :: M_Mie0
  integer(IntAuc)    :: M_Rayleigh0
  integer(IntAus) :: Num_Valid_Mie_Profiles,        Num_Valid_Rayleigh_Profiles
  integer(IntAus) :: Num_Clear_Mie_Profiles,        Num_Clear_Rayleigh_Profiles
  integer(IntAus) :: Num_Cloud_Mie_Profiles,        Num_Cloud_Rayleigh_Profiles
  integer(IntAus) :: Num_Mie_Prof_Warm_Pulses,      Num_Rayleigh_Prof_Warm_Pulses
  integer(IntAus) :: Num_Profiles_Surface_Mie,      Num_Profiles_Surface_Ray
  integer(IntAus) :: Num_Valid_Obs_Prof_L1B_Mie,    Num_Valid_Obs_Prof_L1B_Ray
  integer(IntAus) :: Num_Valid_Meas_Prof_L1B_Mie,   Num_Valid_Meas_Prof_L1B_Ray
  integer(IntAus) :: Num_Valid_Obs_Bins_L1B_Mie,    Num_Valid_Obs_Bins_L1B_Ray
  integer(IntAus) :: Num_Valid_Meas_Bins_L1B_Mie,   Num_Valid_Meas_Bins_L1B_Ray
  integer(IntAus) :: Num_Invalid_Meas_Prof_L1B_Mie, Num_Invalid_Meas_Prof_L1B_Ray
  integer(IntAus) :: Num_Invalid_Meas_Prof_L2B_Mie, Num_Invalid_Meas_Prof_L2B_Ray
  integer(IntAus) :: Num_Invalid_Meas_Bins_L1B_Mie, Num_Invalid_Meas_Bins_L1B_Ray
  integer(IntAus) :: Num_Invalid_Meas_Bins_L2B_Mie, Num_Invalid_Meas_Bins_L2B_Ray
  integer(IntAus) :: Num_Valid_Obs_L2B_Mie,         Num_Valid_Obs_L2B_Ray
  ! L2C parameters
  integer(IntAus) :: Num_Invalid_Obs_L2C_Mie,       Num_Invalid_Obs_L2C_Ray
  !  #]
  !  #[ program code

  print *,"starting TestWriteL2BCdata"

  !*** command line settings ***
  call Set_Default_L2BCds_Settings(L2C_flag, ZeroObs_flag, Empty_flag, &
                                   BRC200_flag)

  call Get_Cmdline_Options_L2BCds(L2C_flag, error_flag, &
                                  ZeroObs_flag, Empty_flag, BRC200_flag)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_error,"aborted Get_Cmdline_Options_L2BCds")
     call program_abort(error_flag,"Get_Cmdline_Options_L2BCds")
  END IF

  ! print the settings defined in Set_L2BCds_Settings
  ! and any changes added to them by the command line options
  call Print_L2BCds_Settings(L2C_flag, ZeroObs_flag, Empty_flag, BRC200_flag)

  !*** initialisations ***
  ! Initialize the L2BC_string
  L2BC_string=set_L2BC_string(L2C_flag)

  ! Initialize the WriteL2BCdata module 
  error_flag = no_error
  call InitWriteL2BCfileModule(error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitWriteL2BCfileModule")

  call Init_L2B_Proc_Settings(L2BP_Settings,error_flag,L2C_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2B_Proc_Settings")

  ! use the commandline switch -200BRC to produce a 200 BRC example
  ! test file with 5 observations per channel and 67 measurements per BRC
  IF (BRC200_flag) THEN
     Num_BRC            = 200 
     N_Obs_Max_Rayleigh = 5
     N_Obs_Max_Mie      = 5
     N_Meas             = 67
     N_Range_Gates      = 24
  ELSE
     ! a small test file for use in our automatic test suite
     Num_BRC            = 2 ! 20
     N_Obs_Max_Rayleigh = 4 ! 5
     N_Obs_Max_Mie      = 4 ! 5
     N_Meas             = 8 ! 67
     N_Range_Gates      =24
  END IF

  N_Meas_to_write         = int(N_Meas,IntAuc)

  IF (ZeroObs_flag) THEN
     ! this enables testing of empty datasets, which is needed, because for
     ! example due to instrument malfunctioning one channel may be 
     ! absent in the input data.
     N_Obs_Mie_to_write      = 0_IntAuc
     N_Obs_Rayleigh_to_write = 0_IntAuc
  ELSEIF (BRC200_flag) THEN
     N_Obs_Mie_to_write      = N_Obs_Max_Mie
     N_Obs_Rayleigh_to_write = N_Obs_Max_Rayleigh
  ELSE
     N_Obs_Mie_to_write      = 2_IntAuc ! 5_IntAuc
     N_Obs_Rayleigh_to_write = 2_IntAuc ! 5_IntAuc
  END IF
 
  allocate(L2BC(Num_BRC),stat=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_allocate,"TestWriteL2BCdata")

  do iBRC=1,Num_BRC

     print *,'TestWriteL2BCdata: Processing BRC # ',iBRC,' =============='

     IF (Empty_flag) THEN
        call Init_L2BC_BRC_DataStructure(L2BC(iBRC), &
                                     0, 0_IntAuc, 0_IntAuc, 0, error_flag)
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"Init_L2BC_BRC_DataStructure()")
     ELSE
        call Init_L2BC_BRC_DataStructure(L2BC(iBRC),N_meas, &
             N_Obs_Max_Mie, N_Obs_Max_Rayleigh, N_Range_Gates,&
             error_flag,L2C_flag=L2C_flag)
        IF (error_flag .ne. no_error) &
             call program_abort(error_flag,"Init_L2BC_BRC_DataStructure()")
     END IF

     print *,'TestWriteL2BCdata: initialized L2BC(',iBRC,')'

     IF (.not. Empty_flag) THEN
        call Print_L2B_Rayleigh_hlos_profile(L2BC(iBRC)%Rayleigh_MDS,&
                                             error_flag,Obs_Nr=1)

        call Print_L2B_Mie_hlos_profile(L2BC(iBRC)%Mie_MDS,error_flag,Obs_Nr=1)
     END IF
     ! Handle Headers
     IF (iBRC .eq. 1) THEN
        print *,'TestWriteL2BCdata: initializing L2BC_Headers'

        allocate(L2BC_Headers)
        call Init_L2BC_HDR_DataStructure(L2BC_Headers, error_flag)

        call SetDateTime(DateTimeStart,2001,03,12,12,34,56.0_r8_)
        timestep = 123.999_r8_ ! 2 minutes 3 seconds  999 milliseconds   
        ! NOTE: use a value just below 124 to make sure the ceil() returns
        ! with 124 If you give 124.0 here, the ceil() used to get
        ! FH%Validity_Period%Validity_Stop might return with 125
        ! due to machine inaccuracy ! (And this makes the result not portable!)

        DateTimeStop = DateTimeStart
        call IncrementDateTime(DateTimeStop,timestep)

        call Init_FH_Default_Fields(FH_Default_Fields)
        IF (L2C_flag) THEN 
           FH_Default_Fields%File_Description(:) = "Level 2C Product"
        ELSE
           FH_Default_Fields%File_Description(:) = "Level 2B Product"
        END IF
        FH_Default_Fields%Notes(:)            = "Room for some"//newline//&
                                                "additional remarks"
        FH_Default_Fields%Mission(:)          = "Aeolus"
        FH_Default_Fields%File_Class(:)       = "TEST"
        FH_Default_Fields%File_Version(:)     = "0001"
        FH_Default_Fields%System(:)           = "LOCAL"
        FH_Default_Fields%Creator(:)          = software_ProcName
        FH_Default_Fields%Creator_Version(:)  = software_version

        ! The L2BP_Settings struct is used to transfer some JobOrder
        ! items to the filling of the FH and MPH, 
        ! but since we do not use a JobOrder 
        ! file in the write program, we have to fill these manually.
        L2BP_Settings%System              = systemstr_LOCAL_MET_INST
        L2BP_Settings%Creator             = "L2BP"
        L2BP_Settings%Creator_Version     = software_version
        L2BP_Settings%File_Class          = "TEST"

        call Fill_L2BC_FH(L2BC_Headers, FH_Default_Fields, &
                          L2BP_Settings, DateTimeStart, DateTimeStop, &
                          error_flag,L2C_flag=L2C_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted Fill_L2BC_FH")
           call program_abort(error_flag,"Fill_L2BC_FH")
        END IF

        MPH_Default_Fields%Ref_Doc(:)      = Ref_Doc_L2BC
        MPH_Default_Fields%Software_Ver(:) = software_version
        MPH_Default_Fields%Proc_Center(:)  = "L2_MF"
                        ! the location where this file was produced
                        ! Obsolete.  Product files eventually filled with
                        ! L2BC_Headers%FH%System and/or L2BP_Settings%System.
                        ! Copy (possibly truncated) of FixedHeader system
                        ! (new requirement in spr 28)

        ! fill the L2B MPH
        call Fill_Remaining_L2BC_MPH_Items(L2BC_Headers%MPH, L2BC_Headers%FH,&
                                           L2BP_Settings, MPH_Default_Fields, &
                                           DateTimeStart, DateTimeStop, &
                                           error_flag, &
                                           L2C_flag=L2C_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted Fill_L2BC_MPH")
           call program_abort(error_flag,"Fill_L2BC_MPH")
        END IF
        ! this element is needed when filling the SPH, 
        ! after the looping is done, so save a copy here
        Sat_Track = 0.0

        print *,'TestWriteL2BCdata: initialized L2BC_Headers'
     ENDIF
 
  enddo

  print *,'TestWriteL2BCdata: finished processing BRCs'
  print *,'=========================================='

  lat_start =  5*1000000 ! in Microdegrees
  lat_stop  =  6*1000000 ! in Microdegrees
  lon_start = 55*1000000 ! in Microdegrees
  lon_stop  = 56*1000000 ! in Microdegrees

  
  IF (.not. Empty_flag) THEN
     ! this call yields no usefull numbers, since everything is
     ! missing in the current dummy dataset !
     call GetStatisticsL2BMieDS(L2BC,Num_BRC,         &
                       M_Mie0,                        &
                       Num_Valid_Mie_Profiles,        &
                       Num_Clear_Mie_Profiles,        &
                       Num_Cloud_Mie_Profiles,        &
                       Num_Mie_Prof_Warm_Pulses,      &
                       Num_Profiles_Surface_Mie,      &
                       Num_Invalid_Meas_Prof_L2B_Mie, &
                       Num_Invalid_Meas_Bins_L2B_Mie, &
                       Num_Valid_Obs_L2B_Mie,         &
                       error_flag)
     IF (error_flag .ne. no_error) THEN
        call logmsg(log_error,"aborted GetStatisticsMieDS")
        call program_abort(error_flag,"GetStatisticsMieDS")
     END IF

     IF (.not. ZeroObs_flag) THEN
        ! override, because otherwise they are all zero
        Num_Clear_Mie_Profiles = 1_IntAuc
        Num_Cloud_Mie_Profiles = 1_IntAuc
        Num_Valid_Mie_Profiles = Num_Clear_Mie_Profiles + &
                                 Num_Cloud_Mie_Profiles
        Num_Mie_Prof_Warm_Pulses = 0_IntAuc
        Num_Profiles_Surface_Mie = 0_IntAuc
        Num_Invalid_Meas_Prof_L2B_Mie = 0_IntAuc
        Num_Invalid_Meas_Bins_L2B_Mie = 0_IntAuc
        Num_Valid_Obs_L2B_Mie         = Num_Valid_Mie_Profiles
        M_Mie0                 = int(Num_Valid_Mie_Profiles,IntAuc)
        DO iBRC=1,size(L2BC)
           L2BC(iBRC)%Geoloc_ADS%N_Obs_Mie_Actual = N_Obs_Mie_to_write
           L2BC(iBRC)%PCD_ADS%N_Obs_Mie_Actual    = N_Obs_Mie_to_write
           L2BC(iBRC)%Mie_MDS%N_Obs_Mie_Actual    = N_Obs_Mie_to_write
           IF (L2C_flag) THEN
              L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Mie_Actual   = N_Obs_Mie_to_write
              L2BC(iBRC)%Mie_VecWind_MDS%N_Obs_Mie_Actual = N_Obs_Mie_to_write
           END IF
        END DO
     END IF

     ! this call yields no usefull numbers, since everything is
     ! missing in the current dummy dataset !
     call GetStatisticsL2BRayleighDS(L2BC,Num_BRC,&
                       M_Rayleigh0,                   &
                       Num_Valid_Rayleigh_Profiles,   &
                       Num_Clear_Rayleigh_Profiles,   &
                       Num_Cloud_Rayleigh_Profiles,   &
                       Num_Rayleigh_Prof_Warm_Pulses, &
                       Num_Profiles_Surface_Ray,      &
                       Num_Invalid_Meas_Prof_L2B_Ray, &
                       Num_Invalid_Meas_Bins_L2B_Ray, &
                       Num_Valid_Obs_L2B_Ray,         &
                       error_flag)
     IF (error_flag .ne. no_error) THEN
        call logmsg(log_error,"aborted GetStatisticsRayleighDS")
        call program_abort(error_flag,"GetStatisticsRayleighDS")
     END IF

     IF (.not. ZeroObs_flag) THEN
        ! override, because otherwise they are all zero
        Num_Clear_Rayleigh_Profiles   = 1_IntAus
        Num_Cloud_Rayleigh_Profiles   = 1_IntAus
        Num_Valid_Rayleigh_Profiles   = Num_Clear_Rayleigh_Profiles + &
                                        Num_Cloud_Rayleigh_Profiles
        Num_Rayleigh_Prof_Warm_Pulses = 0_IntAus
        Num_Profiles_Surface_Ray      = 0_IntAus
        Num_Invalid_Meas_Prof_L2B_Ray = 0_IntAus
        Num_Invalid_Meas_Bins_L2B_Ray = 0_IntAus
        Num_Valid_Obs_L2B_Ray         = Num_Valid_Rayleigh_Profiles
        M_Rayleigh0                   = int(Num_Valid_Rayleigh_Profiles,IntAuc)
        DO iBRC=1,size(L2BC)
           L2BC(iBRC)%Geoloc_ADS%N_Obs_Rayleigh_Actual   = N_Obs_Rayleigh_to_write
           L2BC(iBRC)%PCD_ADS%N_Obs_Rayleigh_Actual      = N_Obs_Rayleigh_to_write
           L2BC(iBRC)%Rayleigh_MDS%N_Obs_Rayleigh_Actual = N_Obs_Rayleigh_to_write
           IF (L2C_flag) THEN
              L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Rayleigh_Actual        = N_Obs_Rayleigh_to_write
              L2BC(iBRC)%Rayleigh_VecWind_MDS%N_Obs_Rayleigh_Actual = N_Obs_Rayleigh_to_write
           END IF
        END DO
     END IF
  ELSE
     M_Mie0                        = 0_IntAuc
     Num_Valid_Mie_Profiles        = 0_IntAus
     Num_Clear_Mie_Profiles        = 0_IntAus
     Num_Cloud_Mie_Profiles        = 0_IntAus
     Num_Mie_Prof_Warm_Pulses      = 0_IntAus
     Num_Profiles_Surface_Mie      = 0_IntAus
     Num_Valid_Obs_Prof_L1B_Mie    = 0_IntAus
     Num_Valid_Meas_Prof_L1B_Mie   = 0_IntAus
     Num_Valid_Obs_Bins_L1B_Mie    = 0_IntAus
     Num_Valid_Meas_Bins_L1B_Mie   = 0_IntAus
     Num_Invalid_Meas_Prof_L1B_Mie = 0_IntAus
     Num_Invalid_Meas_Prof_L2B_Mie = 0_IntAus
     Num_Invalid_Meas_Bins_L1B_Mie = 0_IntAus
     Num_Invalid_Meas_Bins_L2B_Mie = 0_IntAus
     Num_Valid_Obs_L2B_Mie         = 0_IntAus

     M_Rayleigh0                   = 0_IntAuc
     Num_Valid_Rayleigh_Profiles   = 0_IntAus
     Num_Clear_Rayleigh_Profiles   = 0_IntAus
     Num_Cloud_Rayleigh_Profiles   = 0_IntAus
     Num_Rayleigh_Prof_Warm_Pulses = 0_IntAus
     Num_Profiles_Surface_Ray      = 0_IntAus
     Num_Valid_Obs_Prof_L1B_Ray    = 0_IntAus
     Num_Valid_Meas_Prof_L1B_Ray   = 0_IntAus
     Num_Valid_Obs_Bins_L1B_Ray    = 0_IntAus
     Num_Valid_Meas_Bins_L1B_Ray   = 0_IntAus
     Num_Invalid_Meas_Prof_L1B_Ray = 0_IntAus
     Num_Invalid_Meas_Prof_L2B_Ray = 0_IntAus
     Num_Invalid_Meas_Bins_L1B_Ray = 0_IntAus
     Num_Invalid_Meas_Bins_L2B_Ray = 0_IntAus
     Num_Valid_Obs_L2B_Ray         = 0_IntAus

     N_Obs_Mie_to_write            = 0_IntAuc
     N_Obs_Rayleigh_to_write       = 0_IntAuc
     Num_BRC                       = 0
  END IF
  
  ! to fill (dummy value for now):
  Num_Valid_Obs_Prof_L1B_Mie    = 0_IntAus
  Num_Valid_Obs_Prof_L1B_Ray    = 0_IntAus
  Num_Valid_Meas_Prof_L1B_Mie   = 0_IntAus
  Num_Valid_Meas_Prof_L1B_Ray   = 0_IntAus
  Num_Valid_Obs_Bins_L1B_Mie    = 0_IntAus
  Num_Valid_Obs_Bins_L1B_Ray    = 0_IntAus
  Num_Valid_Meas_Bins_L1B_Mie   = 0_IntAus
  Num_Valid_Meas_Bins_L1B_Ray   = 0_IntAus
  Num_Invalid_Meas_Prof_L1B_Mie = 0_IntAus
  Num_Invalid_Meas_Prof_L1B_Ray = 0_IntAus
  Num_Invalid_Meas_Bins_L1B_Mie = 0_IntAus
  Num_Invalid_Meas_Bins_L1B_Ray = 0_IntAus

  ! adding these 2 in the call to fill L2B_SPH doesn't hurt
  ! they are just not used in that case
  Num_Invalid_Obs_L2C_Mie =  0_IntAus
  Num_Invalid_Obs_L2C_Ray =  0_IntAus

  print *,"filling L2BC%SPH"
  call Fill_L2BC_SPH(L2BC_Headers, Num_BRC, &
       lat_start, lat_stop, lon_start, lon_stop, &
       Sat_Track, &
       N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, N_Meas_to_write, &
       Num_Valid_Mie_Profiles,        Num_Valid_Rayleigh_Profiles,   &
       Num_Clear_Mie_Profiles,        Num_Clear_Rayleigh_Profiles,   &
       Num_Cloud_Mie_Profiles,        Num_Cloud_Rayleigh_Profiles,   &
       Num_Mie_Prof_Warm_Pulses,      Num_Rayleigh_Prof_Warm_Pulses, &
       Num_Profiles_Surface_Mie,      Num_Profiles_Surface_Ray,      &
       Num_Valid_Obs_Prof_L1B_Mie,    Num_Valid_Obs_Prof_L1B_Ray,    &
       Num_Valid_Meas_Prof_L1B_Mie,   Num_Valid_Meas_Prof_L1B_Ray,   &
       Num_Valid_Obs_Bins_L1B_Mie,    Num_Valid_Obs_Bins_L1B_Ray,    &
       Num_Valid_Meas_Bins_L1B_Mie,   Num_Valid_Meas_Bins_L1B_Ray,   &
       Num_Invalid_Meas_Prof_L1B_Mie, Num_Invalid_Meas_Prof_L1B_Ray, &
       Num_Invalid_Meas_Prof_L2B_Mie, Num_Invalid_Meas_Prof_L2B_Ray, &
       Num_Invalid_Meas_Bins_L1B_Mie, Num_Invalid_Meas_Bins_L1B_Ray, &
       Num_Invalid_Meas_Bins_L2B_Mie, Num_Invalid_Meas_Bins_L2B_Ray, &
       Num_Valid_Obs_L2B_Mie,         Num_Valid_Obs_L2B_Ray,         &
       error_flag, &
       L2C_flag=L2C_flag, &
       Num_Invalid_Obs_L2C_Mie=Num_Invalid_Obs_L2C_Mie,&
       Num_Invalid_Obs_L2C_Ray=Num_Invalid_Obs_L2C_Ray)

 IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted Fill_L2B_SPH")
     call program_abort(error_flag,"Fill_L2B_SPH")
  END IF

  print *,'TestWriteL2BCdata: calling WriteL2BCfile'
  call WriteL2BCfile(Num_BRC, L2BC_Headers, L2BC, N_Meas_to_write, &
                     N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, &
                     L2BCresultdir     = "./", &
                     L1Bfilename       = "[Undefined]", &
                     NWPfilename       = "[Undefined]", &
                     RBCfilename       = "[Undefined]", &
                     CLMfilename       = "[Undefined]", &
                     CALfilename       = "[Undefined]", &
                     Paramfilename     = "[Undefined]", &
                     error_flag        = error_flag, &
                     Param2Cfilename   = "[Undefined]", &
                     L2C_flag          = L2C_flag)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"WriteL2BCfile")

  ! clean the used memory
  DO iBRC=1,size(L2BC)
     call Delete_L2BC_BRC_DataStructure(L2BC(iBRC),L2C_flag=L2C_flag)
  END DO
  deallocate(L2BC)
  call Delete_L2BC_HDR_DataStructure(L2BC_Headers)
  deallocate(L2BC_Headers)

  print *,"done writing "//L2BC_string//" test datafile"

  !  #]
end program TestWriteL2BCdata
