module WriteL2BCdata
  !  #[ Description:
  !
  !     a module to write the 2 files describing the L2BC product
  !     of the Aeolus instrument
  !
  ! created:     29-Aug-2005 by J. de Kloe, KNMI
  !
  ! History:
  !    28-Nov-2005 P. Poli    coded WriteL2BtoBinaryFile
  !    03-Jan-2006 J. de Kloe changed 1_i8_ in 1 to prevent warnings
  !    03-Jan-2006 J. de Kloe small interface change in xml_module
  !    10-Jan-2006 P. Poli    calling writing of PCD_ADS
  !    11-Jan-2006 J. de Kloe removed no longer used DateTimeMod
  !    26-Jan-2006 P. Poli    calling writing of Geoloc_ADS
  !    02-Mar-2006 J. de Kloe adapted to changes in the XML_module
  !    30-Mar-2006 J. de Kloe Set M_Meas in
  !                           L2B_SPH, pending further merges from
  !                           nl8_aeolus0004_L2BtoTXT2
  !    12-May-2006 D. Tan     Null use of ParKind1 module
  !    13-Jul-2006 D. Tan     Re-order DSDs, remove 4 unnecessary messages
  !    17-Jul-2006 D. Tan     Add DSDs for L1B_Product and CLM_Product,
  !                           and RBC_ADS becomes RBC_Product
  !    08-Sep-2006 J. de Kloe activated the L2Bresultdir setting
  !    27-Oct-2006 D. Tan     Generalize to L2B/L2C alternatives.
  !    30-Oct-2006 J. de Kloe added subroutine WriteProductList()
  !    17-Nov-2006 J. de Kloe removed setting of empty DSD filenames
  !                           and small interface change in Fill_DSD_items
  !    20-Nov-2006 J. de Kloe adapted WriteProductList to take into account
  !                           the possible path name before the JobOrder
  !                           filename given on the commandline
  !    17-Feb-2007 J. de Kloe adapted to possibility of unprocessed BRC's
  !    23-Feb-2007 J. de Kloe small change in preparation of the removal
  !                           of the summaryvars datastructure
  !    02-Mar-2007 J. de Kloe added several folding marks 
  !    02-Mar-2007 J. de Kloe adapted to the latest IODD change 
  !                           (reordering of DSD's)
  !    02-Mar-2007 J. de Kloe corrected the bug in the DSD filling in case
  !                           of reference DSD's (all numbers should be set 
  !                           to zero then, but some attempt to calculate
  !                           the numbers was implemented anyway)
  !    15-Mar-2007 J. de Kloe removed use of Summaryvars
  !    16-Mar-2007 J. de Kloe small bugfix in filling of Num_Data_Sets field
  !                           of the MPH.
  !    19-Mar-2007 J. de Kloe remove an unused variable
  !    20-Mar-2007 J. de Kloe added consistency checks on M_Meas, M_Mie and
  !                           M_Rayleigh
  !    24-Apr-2007 J. de Kloe some cleanups
  !    06-Jun-2007 P. Poli    modified consisteny checks on M_Mie, M_Rayleigh:
  !                           they have to be larger than or equal to 
  !                           N_Obs_Mie_Actual and N_Obs_Rayleigh_Actual in 
  !                           each BRC (and not necessarily equal)
  !    07-Jun-2007 J. de Kloe adapted to the changed xml interface
  !    13-Jun-2007 J. de Kloe changed the interface of WriteL2BCfile
  !                           and removed truncation of filenames
  !    20-Jul-2007 J. de Kloe added call to xml_tree_set_schema()
  !    03-Sep-2007 P. Poli    modified WriteL2BCfile to allow using L2BC header
  !                           information from another datastructure,
  !                           change L2BC_DataType to L2BC_BRC_DataType 
  !                           and L2BC_HDR_DataType
  !    10-Oct-2007 J. de Kloe added a missing nullify()
  !    05-Nov-2007 J. de Kloe added setting of the xml namespace
  !    16-Jan-2008 J. de Kloe phase out integer kind i_
  !    21-Jan-2008 J. de kloe added workaround in call to Write_MPH_to_KVT
  !                           and to GetByteSize_MPH_KVT
  !                           to not change the fileformat for now.
  !    25-Jan-2008 J. de Kloe fix a last small 64-bit incompatibility
  !    21-Feb-2008 J. de Kloe removed the workaround again since it was
  !                           caused by a misunderstanding on L1B side
  !
  !  #]
  !  #[ Modules used:

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BC_DataStructure, only: L2BC_HDR_DataType, L2BC_BRC_DataType, &
                                set_L2BC_string
  USE FixedHeader, only: Copy_FH_to_xml_tree, file_type_L2B_N, file_type_L2C_N
  USE MainProductHeader, only: Copy_MPH_to_xml_tree, &
                               Write_MPH_to_KVT, Fill_MPH_Sizes, &
                               GetByteSize_MPH_KVT
  USE DataSetDescriptor, only: Init_DSD, Print_DSD, DSD_type, &
                               Fill_DSD_items, &
                               maxlen_Filename, GetByteSize_DSD_KVT
  USE L2BC_SPH, only: Copy_L2BC_SPH_to_xml_tree, GetByteSize_L2B_SPH_KVT, &
       Write_L2BC_SPH_to_KVT, &
       Num_DSD_L2B, Num_DS_L2B, Num_DSD_L2C, Num_DS_L2C, &
       Get_M_Meas, Get_M_Mie, Get_M_Rayleigh, &
       L2B_DS_Name_Geolocation_ADS, L2B_DS_Type_Geolocation_ADS, &
       L2B_DS_Name_PCD_ADS,         L2B_DS_Type_PCD_ADS, &
       L2B_DS_Name_Met_MDS,         L2B_DS_Type_Met_MDS, &
       L2B_DS_Name_Mie_MDS,         L2B_DS_Type_Mie_MDS, &
       L2B_DS_Name_Rayleigh_MDS,    L2B_DS_Type_Rayleigh_MDS, &
       L2B_DS_Name_L1B_Product,     L2B_DS_Type_L1B_Product, &
       L2B_DS_Name_CLM_Product,     L2B_DS_Type_CLM_Product, &
       L2B_DS_Name_CAL_Product,     L2B_DS_Type_CAL_Product, &
       L2B_DS_Name_L2B_Param_ADS,   L2B_DS_Type_L2B_Param_ADS, &
       L2B_DS_Name_RBC_Product,     L2B_DS_Type_RBC_Product, &
       L2C_DS_Name_L2B_Product,     L2C_DS_Type_L2B_Product, &
       L2C_DS_Name_L2C_Param_ADS,   L2C_DS_Type_L2C_Param_ADS, &
       L2C_DS_Name_AssimPCD_ADS,    L2C_DS_Type_AssimPCD_ADS, &
       L2C_DS_Name_MieVec_MDS,      L2C_DS_Type_MieVec_MDS, &
       L2C_DS_Name_RayleighVec_MDS, L2C_DS_Type_RayleighVec_MDS
  USE L2B_Geoloc_ADS, only: Write_L2B_Geoloc_ADS, GetByteSize_L2B_Geoloc_ADS
  USE L2B_PCD_ADS, only: Write_L2B_PCD_ADS, GetByteSize_L2B_PCD_ADS
  USE L2B_Mie_MDS, only: Write_L2B_Mie_MDS, Get_ByteSize_L2B_Mie_MDS
  USE L2B_Rayleigh_MDS, only: Write_L2B_Rayleigh_MDS, &
                              Get_ByteSize_L2B_Rayleigh_MDS
  USE L2C_AssimPCD_ADS,    only: Write_L2C_AssimPCD_ADS, &
                                 GetByteSize_L2C_AssimPCD_ADS
  USE L2C_MieVec_MDS,      only: Write_L2C_MieVec_MDS, &
                                 Get_ByteSize_L2C_MieVec_MDS
  USE L2C_RayleighVec_MDS, only: Write_L2C_RayleighVec_MDS, &
                                 Get_ByteSize_L2C_RaylVec_MDS

  USE XML_module, only: xml_node_type, &
       Init_XML, xml_tree_print, xml_tree_write, create_xml_tree, &
       xml_tree_create_header, delete_xml_tree, ee_h_name, &
       xml_tree_set_namespace, uri_ns_header

  USE KVT_module, only: max_str_len, Init_KVT_module
  USE DirectBinaryIO, only: Init_DirectBinaryIO_module , &
                            SetFilePointer, BigEndian

  ! support modules
  USE Logging, only: logmsg,log_debug,log_info,log_progress,log_warn,log_error
  USE ErrorHandler, only: no_error, error_xml, error_programming, &
                          error_opening_file, &
                          error_writing_file, error_allocate
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  USE LunManager, only: get_lun, free_lun
  USE EE_CFI_Datatypes, only: IntAd, IntAul, IntAuc, IntAl
  USE StringTools, only: Join_path_and_filename, Split_path_and_filename
  USE c_support, only: get_filesize
  USE AeolusConstants, only: iodd_version_L2B_N, iodd_version_L2C_N

  !  #]  
  !  #[ Parameter/Variable/Type declarations

  implicit none

  ! make all module contents private, unless explicitely stated otherwise
  private

  ! the default verbosity setting is Error (as silent as possible)
  integer, save :: LocalVerbosity = VerbosityError

  ! remember the initialisation state of the module
  logical, save :: module_is_initialised = .false.

  !  #]
  !  #[ Module interface
  public :: WriteL2BCfile, InitWriteL2BCfileModule, WriteProductList
  !  #]
contains
  !---------------------------------
  subroutine InitWriteL2BCfileModule(error_flag, ChoosenVerbosity)
    !  #[

    integer,           intent(out) :: error_flag
    integer, optional, intent(in)  :: ChoosenVerbosity

    error_flag = no_error

    IF (present(ChoosenVerbosity)) LocalVerbosity = ChoosenVerbosity

    IF (LocalVerbosity .ge. VerbosityReport) &
         call print_verbosity_setting(LocalVerbosity,ModuleName="WriteL2BCfile")

    call Init_KVT_module()

    call Init_DirectBinaryIO_module(error_flag,FileByteOrder=BigEndian)
    if (error_flag .ne. no_error) return

    call Init_XML(error_flag) ! (ChoosenVerbosity=VerbosityReport)
    if (error_flag .ne. no_error) return

    module_is_initialised = .true.

  end subroutine InitWriteL2BCfileModule

    !  #]
  subroutine WriteL2BCfile(Num_BRC, L2BC_Headers, L2BC, &
       N_Meas_to_write, N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, &
       L2BCresultdir, L1Bfilename, NWPfilename, RBCfilename, &
       CLMfilename, CALfilename, Paramfilename, error_flag, &
       L2Bfilename,Param2Cfilename, L2C_flag )
    !  #[

    integer,         intent(in) :: Num_BRC
    ! remark: Num_BRC may be less than the actual size of the L2BC array
    !         when a timewindow is applied, so use automatic dimensioning
    type(L2BC_HDR_DataType),               intent(inout) :: L2BC_Headers
    type(L2BC_BRC_DataType), dimension(:), intent(inout) :: L2BC
    integer(IntAuc), intent(in) :: N_Meas_to_write
    integer(IntAuc), intent(in) :: N_Obs_Mie_to_write
    integer(IntAuc), intent(in) :: N_Obs_Rayleigh_to_write
    character(len=*), intent(in)              :: L2BCresultdir
    character(len=*), intent(in)  :: L1Bfilename
    character(len=*), intent(in)  :: NWPfilename
    character(len=*), intent(in)  :: RBCfilename
    character(len=*), intent(in)  :: CLMfilename
    character(len=*), intent(in)  :: CALfilename
    character(len=*), intent(in)  :: Paramfilename
    integer,          intent(out) :: error_flag ! output
    ! WARNING: this L2Bfilename field is only to be used to import the L2B
    ! filename when writing an L2C file!
    ! When a L2B file is to be written, the filename is generated
    ! from the FixedHeader fields and is no input for this routine!!
    character(len=*), optional, intent(in) :: L2Bfilename     ! L2B only
    character(len=*), optional, intent(in) :: Param2Cfilename ! L2C only
    logical,          optional, intent(in) :: L2C_flag        ! switch

    ! local variables
    character(len=256) :: filename_hdr
    character(len=256) :: filename_dbl
    character(len=256) :: path
    character(len=256) :: tmp_filename
    logical            :: L2C_flag_local
    character(len=3)   :: L2BC_string
    character(len=12)  :: filesize_string
    character(len=256) :: L2Bfilename_local     ! only needed for L2C products
    character(len=256) :: Param2Cfilename_local ! only needed for L2C products
    character(len=256) :: xml_namespace_L2BC

    ! general xml items
    character(len=*), parameter :: xml_root = "Earth_Explorer_File"
    TYPE(xml_node_type), pointer :: xml_tree

    ! init
    error_flag = no_error
    tmp_filename(:) = ' '
    nullify(xml_tree)

    ! do this in 2 steps to circumvent a bug in ifort v8.1
    tmp_filename = Join_path_and_filename(L2BCresultdir, &
       L2BC_Headers%FH%File_Name)
    filename_hdr = trim(tmp_filename)//".HDR"
    filename_dbl = trim(tmp_filename)//".DBL"

    IF (.not. module_is_initialised) THEN
       print *,"ERROR in WriteL2BCfile:"
       print *,"the WriteL2BCfile module should be initialised before use !"
       error_flag = error_programming
       return
    END IF

    L2C_flag_local=.false.
    IF (present(L2C_flag)) L2C_flag_local=L2C_flag

    L2BC_string=set_L2BC_string(L2C_flag_local)

    call logmsg(log_info,"WriteL2BCfile: Writing "//L2BC_string//&
                         " output file...")

    ! Write the DBL file including MPH, SPH, and data and figure out size

    L2Bfilename_local = "[Undefined]"
    IF (L2C_flag_local) THEN
       IF (present(L2Bfilename)) THEN
          L2Bfilename_local = L2Bfilename
       ELSE
          ! use the old workaround to not break any tests
          L2Bfilename_local = L2BC_Headers%FH%File_Name
          L2Bfilename_local(18:18) = 'B'
          ! This should change "2C" into "2B".
       END IF
    END IF

    Param2Cfilename_local = "[Undefined]"
    IF (present(Param2Cfilename)) &
         Param2Cfilename_local = Param2Cfilename

    ! NOTE that this WriteL2BCtoBinaryFile fills the sizes in the MPH, SPH
    !      and DSD structures, and therefore it must be called first
    !       before trying to construct the XML header file !!!
    call WriteL2BCtoBinaryFile(Num_BRC, L2BC_Headers, L2BC, &
         N_Meas_to_write, N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, &
         L2BC_string, filename_dbl,&
         L1Bfilename, NWPfilename, CLMfilename, CALfilename, &
         Paramfilename, RBCfilename, &
         L2Bfilename_local, Param2Cfilename_local, &
         L2C_flag   = L2C_flag_local, &
         error_flag = error_flag)
    IF (error_flag .ne. no_error) return

    ! allocate a datastructure to hold the xml data
    ! and get a pointer to it, and fill it with the xml root tag pair:
    ! <Earth_Explorer_File><\Earth_Explorer_File>

    ! allocate and init the datastructure
    call create_xml_tree(xml_tree, error_flag)
    IF (error_flag .ne. no_error) return
    ! create a blank header (fixed+variable) and define the root
    call xml_tree_create_header(xml_tree, ee_h_name, error_flag)
    IF (error_flag .ne. no_error) return

    ! define which xml namespace should be used
    ! (this call must be done after the call to xml_tree_create_header()
    !  otherwise the simple_xml version cannot retrieve the needed 
    !  node_name of the root tag)
    !example: xmlns="http://www.esa.int/schemas/ae/ALD_U_N_2B_01.01"
    !uri_ns_header    ! defined in xml_module
    !file_type_L2B_N  ! defined in FixedHeader
    !iodd_version_L2B ! defined in AeolusConstants
    IF (L2C_flag_local) THEN
       xml_namespace_L2BC = trim(uri_ns_header)//trim(file_type_L2C_N)//&
                            "_"//trim(iodd_version_L2C_N)
    ELSE
       xml_namespace_L2BC = trim(uri_ns_header)//trim(file_type_L2B_N)//&
                            "_"//trim(iodd_version_L2B_N)
    END IF

    ! set the namespace tag
    call xml_tree_set_namespace(xml_tree,xml_namespace_L2BC,error_flag)
    IF (error_flag .ne. no_error) return

    ! fill all FH items in the xml tree
    path = "/"//ee_h_name
    call Copy_FH_to_xml_tree(L2BC_Headers%FH, path, xml_tree, error_flag)
    IF (error_flag .ne. no_error) return
    ! => write the MPH
    call Copy_MPH_to_xml_tree(L2BC_Headers%MPH, path, xml_tree, error_flag)
    IF (error_flag .ne. no_error) return
    ! => write the SPH
    call Copy_L2BC_SPH_to_xml_tree(L2BC_Headers%SPH, path, xml_tree, &
                                   error_flag, L2C_flag_local)
    IF (error_flag .ne. no_error) return

    ! Write the HDR file including FH, MPH, and SPH
    call xml_tree_write(xml_tree, filename_hdr, error_flag)
    IF (error_flag .ne. no_error) return

    call delete_xml_tree(xml_tree,error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_progress,"Successfully written productfile: "//&
                trim(L2BC_Headers%FH%File_Name))
    write(filesize_string,"(i12)",err=999) &
         get_filesize(trim(L2BC_Headers%FH%File_Name)//".DBL")
    call logmsg(log_progress,"Size of binary productfile was: "//&
                             trim(filesize_string)//" bytes")

    ! TODO: add a GetByteSize_L2BC_DataStructure routine
    !       and check if this number is as expected

    IF (LocalVerbosity .ge. VerbosityReport) &
         print *,"WriteL2BCfile: done writing L2BC output file"

    return

999 call logmsg(log_error,"ERROR in WriteL2BCfile: could not convert size "//&
                          "of binary product file to a string")
    error_flag = error_writing_file
    return

  end subroutine WriteL2BCfile
    !  #]
  !---------------------------------
  subroutine WriteL2BCtoBinaryFile(Num_BRC, L2BC_Headers, L2BC,&
               N_Meas_to_write, N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, &
               L2BC_string, filename_dbl,&
               L1Bfilename, NWPfilename, CLMfilename, CALfilename, &
               Paramfilename, RBCfilename, &
               L2Bfilename, Param2Cfilename, &
               L2C_flag, error_flag)
    !  #[
    integer,         intent(in) :: Num_BRC
    ! remark: Num_BRC may be less than the actual size of the L2BC array
    !         when a timewindow is applied, so use automatic dimensioning
    type(L2BC_HDR_DataType),              intent(inout) :: L2BC_Headers
    type(L2BC_BRC_DataType), dimension(:),intent(inout) :: L2BC
    integer(IntAuc), intent(in) :: N_Meas_to_write
    integer(IntAuc), intent(in) :: N_Obs_Mie_to_write
    integer(IntAuc), intent(in) :: N_Obs_Rayleigh_to_write
    character(len=3),   intent(in)  :: L2BC_string     ! "L2B" or "L2C"
    character(len=256), intent(in)  :: filename_dbl
    character(len=*),   intent(in)  :: L1Bfilename
    character(len=*),   intent(in)  :: NWPfilename
    character(len=*),   intent(in)  :: CLMfilename
    character(len=*),   intent(in)  :: CALfilename
    character(len=*),   intent(in)  :: Paramfilename
    character(len=*),   intent(in)  :: RBCfilename
    character(len=*),   intent(in)  :: L2Bfilename     ! for L2C products only
    character(len=*),   intent(in)  :: Param2Cfilename ! for L2C products only
    logical,            intent(in)  :: L2C_flag        ! switch
    integer,                    intent(out)   :: error_flag

    ! local variables
    integer            :: fileunit, AllocStatus
    integer(IntAl)     :: iBRC

    ! remark: this routine has no optional inputs anymore, which simplifies
    ! the code quite a bit.
    ! All optional input handling is done in WriteL2BCfile above
    ! which is the main entry point anyway for this module

    ! remark: variables used in the inquire() call must be default integers
    ! so don't force them to be 4 or 8 bytes 
    integer            :: endpos, begpos, loc_size, size_of_ds_calc
    integer            :: filepos_before, filepos_after
    integer(IntAl)     :: Dsr_Size, Num_Dsd, Num_Attached_DS
    integer(IntAd)     :: Ds_Offset
    integer(IntAd)     :: Tot_Product_Size
    integer(IntAuc)    :: M_Meas_test, M_Mie_test, M_Rayleigh_test
    character(len=4)   :: str_cntr
    logical            :: consistency_problem_found

    print *,"WriteL2BCtoBinaryFile: writing "//&
             L2BC_string//" data to binary file: "
    print *,trim(filename_dbl)

    !  #[ consistency check:

    ! for our own convenience we have N_Meas and 
    ! N_Obs_Mie_Actual/N_Obs_Rayleigh_Actual fields in all datasets that use
    ! them. These fields are redundant, because these values are also written
    ! to the SPH. Therefore we should at least make sure all copies of these
    ! values are consistent, since any programming mistake on our side on
    ! these fields could cause major confusion for external users.

    ! Poli: -- disagree: N_Obs_Mie_Actual is not redundant with
    ! M_Mie, since it may differ for each BRC, whereas M_Mie is constant for a
    ! given L2B product file. The consistency test is changed to verify that
    ! N_Obs_Mie_Actual does not exceed M_Mie (same for Rayleigh).

    consistency_problem_found = .false.
    call Get_M_Meas(L2BC_Headers%SPH,M_Meas_test,error_flag)
    call Get_M_Mie(L2BC_Headers%SPH,M_Mie_test,error_flag)
    call Get_M_Rayleigh(L2BC_Headers%SPH,M_Rayleigh_test,error_flag)
    !print *,"M_Meas_test     = ",M_Meas_test
    !print *,"M_Mie_test      = ",M_Mie_test
    !print *,"M_Rayleigh_test = ",M_Rayleigh_test

    DO iBRC=1,Num_BRC
       write(str_cntr,"(i4.4)",err=997) iBRC

       ! test M_Meas consistency

       ! REMARK: this nr_of_measurements field is part of the working data
       ! structure, and may be removed (or do we use it somewhere?)
       IF (L2BC(iBRC)%nr_of_measurements .ne. M_Meas_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%nr_of_measurements = ",&
               L2BC(iBRC)%nr_of_measurements
          print *,"does not match the SPH value of M_Meas = ",M_Meas_test
       END IF

       IF (L2BC(iBRC)%PCD_ADS%N_Meas .ne. M_Meas_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%PCD_ADS%N_Meas = ",&
               L2BC(iBRC)%PCD_ADS%N_Meas
          print *,"does not match the SPH value of M_Meas = ",M_Meas_test
       END IF

       IF (L2BC(iBRC)%Mie_MDS%N_Meas .ne. M_Meas_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%Mie_MDS%N_Meas = ",&
               L2BC(iBRC)%Mie_MDS%N_Meas
          print *,"does not match the SPH value of M_Meas = ",M_Meas_test
       END IF

       IF (L2BC(iBRC)%Rayleigh_MDS%N_Meas .ne. M_Meas_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%Rayleigh_MDS%N_Meas = ",&
               L2BC(iBRC)%Rayleigh_MDS%N_Meas
          print *,"does not match the SPH value of M_Meas = ",M_Meas_test
       END IF

       IF (L2C_flag) THEN
          IF (L2BC(iBRC)%Assim_PCD_ADS%N_Meas .ne. M_Meas_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//")%Assim_PCD_ADS%N_Meas = ",&
                  L2BC(iBRC)%Assim_PCD_ADS%N_Meas
             print *,"does not match the SPH value of M_Meas = ",M_Meas_test
          END IF

          IF (L2BC(iBRC)%Mie_VecWind_MDS%N_Meas .ne. M_Meas_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//")%Mie_VecWind_MDS = ",&
                  L2BC(iBRC)%Mie_VecWind_MDS%N_Meas
             print *,"does not match the SPH value of M_Meas = ",M_Meas_test
          END IF

          IF (L2BC(iBRC)%Rayleigh_VecWind_MDS%N_Meas .ne. M_Meas_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//")%Rayleigh_VecWind_MDS = ",&
                  L2BC(iBRC)%Rayleigh_VecWind_MDS%N_Meas
             print *,"does not match the SPH value of M_Meas = ",M_Meas_test
          END IF
       END IF ! L2C_flag

       ! test M_Mie consistency

       IF (L2BC(iBRC)%Geoloc_ADS%N_Obs_Mie_Actual .gt. M_Mie_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%Geoloc_ADS%N_Obs_Mie_Actual = ",&
               L2BC(iBRC)%Geoloc_ADS%N_Obs_Mie_Actual
          print *,"exceeds the SPH value of M_Mie = ",M_Mie_test
       END IF

       IF (L2BC(iBRC)%PCD_ADS%N_Obs_Mie_Actual .gt. M_Mie_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%PCD_ADS%N_Obs_Mie_Actual = ",&
               L2BC(iBRC)%PCD_ADS%N_Obs_Mie_Actual
          print *,"exceeds the SPH value of M_Mie = ",M_Mie_test
       END IF

       IF (L2BC(iBRC)%Mie_MDS%N_Obs_Mie_Actual .gt. M_Mie_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//")%Mie_MDS%N_Obs_Mie_Actual = ",&
               L2BC(iBRC)%Mie_MDS%N_Obs_Mie_Actual
          print *,"exceeds the SPH value of M_Mie = ",M_Mie_test
       END IF

       IF (L2C_flag) THEN
          IF (L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Mie_Actual .gt. M_Mie_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//&
                  ")%Assim_PCD_ADS%N_Obs_Mie_Actual = ",&
                  L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Mie_Actual
             print *,"exceeds the SPH value of M_Mie = ",M_Mie_test
          END IF

          IF (L2BC(iBRC)%Mie_VecWind_MDS%N_Obs_Mie_Actual .gt. M_Mie_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//&
                  ")%Mie_VecWind_MDS%N_Obs_Mie_Actual = ",&
                  L2BC(iBRC)%Mie_VecWind_MDS%N_Obs_Mie_Actual
             print *,"exceeds the SPH value of M_Mie = ",M_Mie_test
          END IF
       END IF ! L2C_flag

       ! test M_Rayleigh consistency

       IF (L2BC(iBRC)%Geoloc_ADS%N_Obs_Rayleigh_Actual .gt. &
                                                 M_Rayleigh_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//&
               ")%Geoloc_ADS%N_Obs_Rayleigh_Actual = ",&
               L2BC(iBRC)%Geoloc_ADS%N_Obs_Rayleigh_Actual
          print *,"exceeds the SPH value of M_Rayleigh = ",&
               M_Rayleigh_test
       END IF

       IF (L2BC(iBRC)%PCD_ADS%N_Obs_Rayleigh_Actual .gt. &
                                                 M_Rayleigh_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//&
               ")%PCD_ADS%N_Obs_Rayleigh_Actual = ",&
               L2BC(iBRC)%PCD_ADS%N_Obs_Rayleigh_Actual
          print *,"exceeds the SPH value of M_Rayleigh = ",&
               M_Rayleigh_test
       END IF

       IF (L2BC(iBRC)%Rayleigh_MDS%N_Obs_Rayleigh_Actual .gt. &
                                                 M_Rayleigh_test) THEN
          consistency_problem_found = .true.
          print *,"L2BC("//trim(str_cntr)//&
               ")%Rayleigh_MDS%N_Obs_Rayleigh_Actual = ",&
               L2BC(iBRC)%Rayleigh_MDS%N_Obs_Rayleigh_Actual
          print *,"exceeds the SPH value of M_Rayleigh = ",&
               M_Rayleigh_test
       END IF

       IF (L2C_flag) THEN
          IF (L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Rayleigh_Actual .gt. &
                                                 M_Rayleigh_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//&
                  ")%Assim_PCD_ADS%N_Obs_Rayleigh_Actual = ",&
                  L2BC(iBRC)%Assim_PCD_ADS%N_Obs_Rayleigh_Actual
             print *,"exceeds the SPH value of M_Rayleigh = ",&
                  M_Rayleigh_test
          END IF

          IF (L2BC(iBRC)%Rayleigh_VecWind_MDS%N_Obs_Rayleigh_Actual .gt. &
                                                 M_Rayleigh_test) THEN
             consistency_problem_found = .true.
             print *,"L2BC("//trim(str_cntr)//&
                  ")%Rayleigh_VecWind_MDS%N_Obs_Rayleigh_Actual = ",&
                  L2BC(iBRC)%Rayleigh_VecWind_MDS%N_Obs_Rayleigh_Actual
             print *,"exceeds the SPH value of M_Rayleigh = ",&
                  M_Rayleigh_test
          END IF
       END IF ! L2C_flag

    END DO ! iBRC

    IF (consistency_problem_found) THEN
       print *,"ERROR: consistency problem found in L2BC datastructure"
       print *,"This is a programming error"
       error_flag = error_programming
       goto 999
    END IF

    !  #]
    !  #[ fill Geolocation_DSD

    IF (.not. associated(L2BC_Headers%SPH%Geolocation_DSD)) THEN
       allocate(L2BC_Headers%SPH%Geolocation_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%Geolocation_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999    

    IF (associated(L2BC(1)%Geoloc_ADS)) THEN
       Dsr_size = GetByteSize_L2B_Geoloc_ADS(L2BC(1)%Geoloc_ADS,&
                                N_Obs_Mie_to_write,N_Obs_Rayleigh_to_write)
    ELSE
       ! an unassociated dataset may occur if no BRC's at all where
       ! processed dye to the <Time_Interval> setting of the JobOrder
       Dsr_size = 0
    END IF

    Ds_offset = GetByteSize_MPH_KVT() + &
                GetByteSize_L2B_SPH_KVT(L2C_flag)
    ! Geolocation is an attached annotation dataset, 
    ! so no need to set the filename.
    call Fill_DSD_items (L2BC_Headers%SPH%Geolocation_DSD, &
             L2B_DS_Name_Geolocation_ADS, L2B_DS_Type_Geolocation_ADS, &
             Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! remember this, for use in the next DSD
    Ds_offset = Ds_offset + Dsr_size*Num_BRC

    !  #]
    !  #[ fill PCD_DSD
    IF (.not. associated(L2BC_Headers%SPH%PCD_DSD)) THEN
       allocate(L2BC_Headers%SPH%PCD_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%PCD_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    IF (associated(L2BC(1)%PCD_ADS)) THEN
       Dsr_size = GetByteSize_L2B_PCD_ADS(L2BC(1)%PCD_ADS,N_Meas_to_write,&
                                  N_Obs_Mie_to_write,N_Obs_Rayleigh_to_write)
    ELSE
       Dsr_size = 0
    END IF
    ! Ds_offset has already been set above

    ! PCD is an attached annotation dataset, 
    ! so no need to set the filename.
    call Fill_DSD_items (L2BC_Headers%SPH%PCD_DSD, &
             L2B_DS_Name_PCD_ADS, L2B_DS_Type_PCD_ADS, &
             Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! remember this, for use in the next DSD
    Ds_offset = Ds_offset + Dsr_size*Num_BRC
    !  #]
    !  #[ fill Mie_DSD
    IF (.not. associated(L2BC_Headers%SPH%Mie_DSD)) THEN
       allocate(L2BC_Headers%SPH%Mie_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%Mie_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    IF (associated(L2BC(1)%Mie_MDS)) THEN
       Dsr_size = Get_ByteSize_L2B_Mie_MDS(L2BC(1)%Mie_MDS,N_Meas_to_write,&
                                            N_Obs_Mie_to_write)
    ELSE
       Dsr_size = 0
    END IF
    ! Ds_offset has already been set above

    ! Mie is an attached annotation dataset, 
    ! so no need to set the filename.
    call Fill_DSD_items (L2BC_Headers%SPH%Mie_DSD, &
             L2B_DS_Name_Mie_MDS, L2B_DS_Type_Mie_MDS, &
             Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! remember this, for use in the next DSD
    Ds_offset = Ds_offset + Dsr_size*Num_BRC
    !  #]
    !  #[ fill Rayleigh_DSD
    IF (.not. associated(L2BC_Headers%SPH%Rayleigh_DSD)) THEN
       allocate(L2BC_Headers%SPH%Rayleigh_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%Rayleigh_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999
    IF (associated(L2BC(1)%Rayleigh_MDS)) THEN
       Dsr_size = Get_ByteSize_L2B_Rayleigh_MDS(L2BC(1)%Rayleigh_MDS,&
                               N_Meas_to_write,N_Obs_Rayleigh_to_write)
    ELSE
       Dsr_size = 0
    END IF
    ! Ds_offset has already been set above
    ! Rayleigh is an attached annotation dataset, 
    ! so no need to set the filename.
    call Fill_DSD_items (L2BC_Headers%SPH%Rayleigh_DSD, &
             L2B_DS_Name_Rayleigh_MDS, L2B_DS_Type_Rayleigh_MDS, &
             Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
    IF (error_flag .ne. no_error) goto 999
    ! remember this, for use in the next DSD or in the MPH Tot_Prod_Size field
    Ds_offset = Ds_offset + Dsr_size*Num_BRC
    !  #]
    
    IF (L2C_flag) THEN
       !  #[ fill Assim_PCD_DSD
       IF (.not. associated(L2BC_Headers%SPH%Assim_PCD_DSD)) THEN
          allocate(L2BC_Headers%SPH%Assim_PCD_DSD,STAT=AllocStatus)
          IF (AllocStatus .ne. 0) THEN
             print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
             error_flag = error_allocate
             goto 999
          END IF
       END IF
       call Init_DSD(L2BC_Headers%SPH%Assim_PCD_DSD,error_flag)
       IF (error_flag .ne. no_error) goto 999

       IF (associated(L2BC(1)%Assim_PCD_ADS)) THEN
          Dsr_size = GetByteSize_L2C_AssimPCD_ADS(L2BC(1)%Assim_PCD_ADS,&
                                  N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write)
       ELSE
          Dsr_size = 0
       END IF
       ! Ds_offset has already been set above
       
       ! Mie is an attached annotation dataset, 
       ! so no need to set the filename.
       call Fill_DSD_items (L2BC_Headers%SPH%Assim_PCD_DSD, &
            L2C_DS_Name_AssimPCD_ADS, L2C_DS_Type_AssimPCD_ADS, &
            Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
       IF (error_flag .ne. no_error) goto 999
       
       ! remember this, for use in the next DSD
       Ds_offset = Ds_offset + Dsr_size*Num_BRC
       !  #]       
       !  #[ fill Mie_VecWind_DSD
       IF (.not. associated(L2BC_Headers%SPH%Mie_VecWind_DSD)) THEN
          allocate(L2BC_Headers%SPH%Mie_VecWind_DSD,STAT=AllocStatus)
          IF (AllocStatus .ne. 0) THEN
             print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
             error_flag = error_allocate
             goto 999
          END IF
       END IF
       call Init_DSD(L2BC_Headers%SPH%Mie_VecWind_DSD,error_flag)
       IF (error_flag .ne. no_error) goto 999

       IF (associated(L2BC(1)%Mie_VecWind_MDS)) THEN
          Dsr_size = Get_ByteSize_L2C_MieVec_MDS(L2BC(1)%Mie_VecWind_MDS,&
                                                 N_Obs_Mie_to_write)
       ELSE
          Dsr_size = 0
       END IF
       ! Ds_offset has already been set above
       
       ! Mie_VecWind is an attached annotation dataset, 
       ! so no need to set the filename.
       call Fill_DSD_items (L2BC_Headers%SPH%Mie_VecWind_DSD, &
            L2C_DS_Name_MieVec_MDS, L2C_DS_Type_MieVec_MDS, &
            Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
       IF (error_flag .ne. no_error) goto 999
       
       ! remember this, for use in the next DSD
       Ds_offset = Ds_offset + Dsr_size*Num_BRC
       !  #]
       !  #[ fill Rayleigh_VecWind_DSD
       IF (.not. associated(L2BC_Headers%SPH%Rayleigh_VecWind_DSD)) THEN
          allocate(L2BC_Headers%SPH%Rayleigh_VecWind_DSD,STAT=AllocStatus)
          IF (AllocStatus .ne. 0) THEN
             print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
             error_flag = error_allocate
             goto 999
          END IF
       END IF
       call Init_DSD(L2BC_Headers%SPH%Rayleigh_VecWind_DSD,error_flag)
       IF (error_flag .ne. no_error) goto 999

       IF (associated(L2BC(1)%Rayleigh_VecWind_MDS)) THEN
          Dsr_size = Get_ByteSize_L2C_RaylVec_MDS(&
                                L2BC(1)%Rayleigh_VecWind_MDS,&
                                N_Obs_Rayleigh_to_write)
       ELSE
          Dsr_size = 0
       END IF
       ! Ds_offset has already been set above
       
       ! RayleighVec is an attached measurement dataset, 
       ! so no need to set the filename.
       call Fill_DSD_items (L2BC_Headers%SPH%Rayleigh_VecWind_DSD, &
            L2C_DS_Name_RayleighVec_MDS, L2C_DS_Type_RayleighVec_MDS, &
            Ds_Offset, int(Num_BRC,IntAl), Dsr_Size, error_flag)
       IF (error_flag .ne. no_error) goto 999
       
       ! remember this, for use in the MPH product_size field
        Ds_offset = Ds_offset + Dsr_size*Num_BRC
       !  #]
    END IF
    
    !  #[ fill L1B Product DSD
    ! L1B_DSD.1 initialize L1B_DSD
    IF (.not. associated(L2BC_Headers%SPH%l1B_DSD)) THEN
       allocate(L2BC_Headers%SPH%L1B_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%L1B_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! L1B_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%L1B_DSD, &
         L2B_DS_Name_L1B_Product, L2B_DS_Type_L1B_Product, &
         Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
         error_flag=error_flag, &
         Filename = L1Bfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]
    !  #[ fill L2B Met MDS DSD
    ! Met_DSD.1 initialize Met_DSD
    IF (.not. associated(L2BC_Headers%SPH%Met_DSD)) THEN
       allocate(L2BC_Headers%SPH%Met_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%Met_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! Met_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%MET_DSD, &
         L2B_DS_Name_Met_MDS, L2B_DS_Type_Met_MDS, &
         Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
         error_flag=error_flag, &
         Filename = NWPfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]
    !  #[ fill L2B RBC ADS DSD
    ! RBC_DSD.1 initialize RBC
    IF (.not. associated(L2BC_Headers%SPH%RBC_DSD)) THEN
       allocate(L2BC_Headers%SPH%RBC_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%RBC_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! RBC_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%RBC_DSD, &
       L2B_DS_Name_RBC_Product, L2B_DS_Type_RBC_Product, &
       Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
       error_flag=error_flag, &
       Filename = RBCfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]
    !  #[ fill L2B CLM DSD
    ! CLM_DSD.1 initialize CLM
    IF (.not. associated(L2BC_Headers%SPH%CLM_DSD)) THEN
       allocate(L2BC_Headers%SPH%CLM_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%CLM_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! CLM_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%CLM_DSD, &
       L2B_DS_Name_CLM_Product, L2B_DS_Type_CLM_Product, &
       Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
       error_flag=error_flag, &
       Filename = CLMfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]
    !  #[ fill L2B CAL DSD
    ! CAL_DSD.1 initialize CAL
    IF (.not. associated(L2BC_Headers%SPH%CAL_DSD)) THEN
       allocate(L2BC_Headers%SPH%CAL_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%CAL_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! CAL_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%CAL_DSD, &
       L2B_DS_Name_CAL_Product, L2B_DS_Type_CAL_Product, &
       Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
       error_flag=error_flag, &
       Filename = CALfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]
    !  #[ fill L2B_Param ADS
    ! Par2B_DSD.1 initialize L2B_Param_DSD
    IF (.not. associated(L2BC_Headers%SPH%L2B_Param_DSD)) THEN
       allocate(L2BC_Headers%SPH%L2B_Param_DSD,STAT=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
          error_flag = error_allocate
          goto 999
       END IF
    END IF
    call Init_DSD(L2BC_Headers%SPH%L2B_Param_DSD,error_flag)
    IF (error_flag .ne. no_error) goto 999

    ! Par2B_DSD.5 Update DSD information
    ! this is a referenced DS, so set positions and sizes to zero
    call Fill_DSD_items (L2BC_Headers%SPH%L2B_Param_DSD, &
       L2B_DS_Name_L2B_Param_ADS, L2B_DS_Type_L2B_Param_ADS, &
       Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
       error_flag=error_flag, &
       Filename = Paramfilename)
    IF (error_flag .ne. no_error) goto 999
    !  #]

    ! TODO: (possibly) consistency_dsds(L2BC,L2C_flag)

    IF (L2C_flag) THEN
       !  #[ fill L2B Product DSD
       ! L2B_DSD.1 initialize L2B_DSD
       IF (.not. associated(L2BC_Headers%SPH%L2B_DSD)) THEN
          allocate(L2BC_Headers%SPH%L2B_DSD,STAT=AllocStatus)
          IF (AllocStatus .ne. 0) THEN
             print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
             error_flag = error_allocate
             goto 999
          END IF
       END IF
       call Init_DSD(L2BC_Headers%SPH%L2B_DSD,error_flag)
       IF (error_flag .ne. no_error) goto 999

       ! L2B_DSD.5 Update DSD information
       ! this is a referenced DS, so set positions and sizes to zero
       call Fill_DSD_items (L2BC_Headers%SPH%L2B_DSD, &
            L2C_DS_Name_L2B_Product, L2C_DS_Type_L2B_Product, &
            Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
            error_flag=error_flag, &
            Filename = L2Bfilename)
       IF (error_flag .ne. no_error) goto 999
       !  #]
       !  #[ fill L2C_Param DSD
       ! Par2C_DSD.1 initialize L2C_Param_DSD
       IF (.not. associated(L2BC_Headers%SPH%L2C_Param_DSD)) THEN
          allocate(L2BC_Headers%SPH%L2C_Param_DSD,STAT=AllocStatus)
          IF (AllocStatus .ne. 0) THEN
             print *,"WriteL2BCtoBinaryFile: ERROR: Allocation failure:"
             error_flag = error_allocate
             goto 999
          END IF
       END IF
       call Init_DSD(L2BC_Headers%SPH%L2C_Param_DSD,error_flag)
       IF (error_flag .ne. no_error) goto 999

       ! Par2C_DSD.5 Update DSD information
       ! this is a referenced DS, so set positions and sizes to zero
       call Fill_DSD_items (L2BC_Headers%SPH%L2C_Param_DSD, &
            L2C_DS_Name_L2C_Param_ADS, L2C_DS_Type_L2C_Param_ADS, &
            Ds_Offset=0_intAd, Num_Dsr=0_IntAl, Dsr_Size=0_IntAl, &
            error_flag=error_flag, &
            Filename = Param2Cfilename)
       IF (error_flag .ne. no_error) goto 999
       !  #]
    END IF

    !  #[ fill L2B MPH sizes
    Tot_Product_Size = Ds_offset

    IF (L2C_flag) THEN
       Num_Dsd         = Num_DSD_L2C
       Num_Attached_DS = Num_DS_L2C
    ELSE
       Num_Dsd         = Num_DSD_L2B
       Num_Attached_DS = Num_DS_L2B
    END IF
    call Fill_MPH_Sizes(L2BC_Headers%MPH,&
                        Tot_Size      = Tot_Product_Size,&
                        Sph_Size      = GetByteSize_L2B_SPH_KVT(L2C_flag),&
                        Num_Dsd       = Num_Dsd,&
                        Dsd_Size      = GetByteSize_DSD_KVT(),&
                        Num_Data_Sets = Num_Attached_DS,&
                        error_flag    = error_flag)
    IF (error_flag .ne. no_error) goto 999
    !  #]

    !  #[ 1. open file
    fileunit = get_lun()
    IF (fileunit .lt. 0) THEN
       ! no fileunits available, so report an error
       error_flag = error_opening_file
       return
    END IF

    open(unit=fileunit,file=filename_dbl,status="replace",&
         action="write",access="direct",recl=1,form="UNFORMATTED",ERR=996)
    !  #]
    !  #[ 2. write L2BC MPH
    call logmsg(log_info,'WriteL2BCtoBinaryFile: writing MPH')
    call Write_MPH_to_KVT(L2BC_Headers%MPH,fileunit,error_flag)
    IF (error_flag .ne. no_error) goto 998 ! abort writing the file
    !  #]
    !  #[ 3. write L2BC SPH
    call logmsg(log_info,'WriteL2BCtoBinaryFile: writing SPH')
    call Write_L2BC_SPH_to_KVT(L2BC_Headers%SPH,fileunit,&
                               error_flag,L2C_flag=L2C_flag)
    IF (error_flag .ne. no_error) goto 998 ! abort writing the file
    !  #]

    !  #[ 4. write L2B Geol ADS

    ! 4.2 get position before writing Geolocation ADS
    inquire(unit=fileunit,nextrec=filepos_before)
    ! 4.3 writing Geolocation_ADS, get size of one DS and check sizes are
    ! constant
    do iBRC=1,Num_BRC ! remind that Num_BRC might be zero !
       write(str_cntr,"(i4.4)",err=997) iBRC
       call logmsg(log_info,'WriteL2BCtoBinaryFile: '//&
                   'writing Geolocation_ADS for BRC #'//trim(str_cntr))

       inquire(unit=fileunit,nextrec=begpos)
       call Write_L2B_Geoloc_ADS(fileunit,L2BC(iBRC)%Geoloc_ADS, &
                                 N_Obs_Mie_to_write, &
                                 N_Obs_Rayleigh_to_write, error_flag)
       IF (error_flag .ne. no_error) goto 998
       inquire(unit=fileunit,nextrec=endpos)
       endpos   = endpos-1
       loc_size = endpos-begpos+1
       ! verify that all the DSRs have the same size
       IF (loc_size.ne.L2BC_Headers%SPH%Geolocation_DSD%size_of_one_dsr) THEN
          call logmsg(log_warn,"WriteL2BCtoBinaryFile: WARNING: "//&
                      "Geolocation_ADS DSRs have variable sizes")
       ENDIF
    enddo
    ! 4.4 get position and size after writing this DS
    inquire(unit=fileunit,nextrec=filepos_after)

    size_of_ds_calc = L2BC_Headers%SPH%Geolocation_DSD%size_of_one_dsr * &
                      L2BC_Headers%SPH%Geolocation_DSD%nr_of_dataset_records
    IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
       print *,"ERROR in WriteL2BCtoBinaryFile: Total size of Geolocation_ADS"
       print *,"is not consistent with DSD content !!!"
       print *,'actual DS size:            ',filepos_after-filepos_before
       print *,'GetByteSize result in DSD: ',size_of_ds_calc
       goto 998
    ENDIF
    !  #]
    !  #[ 5. write L2B PCD ADS

    ! 5.2 get position before writing PCD ADS
    inquire(unit=fileunit,nextrec=filepos_before)

    ! 5.3 writing PCD_ADS, get size of one DS and check sizes are
    ! constant
    do iBRC=1,Num_BRC
       inquire(unit=fileunit,nextrec=begpos)
       print *,'WriteL2BCtoBinaryFile: writing PCD_ADS for BRC #',iBRC
       call Write_L2B_PCD_ADS(fileunit,L2BC(iBRC)%PCD_ADS, &
                              N_Meas_to_write, N_Obs_Mie_to_write, &
                              N_Obs_Rayleigh_to_write ,error_flag)
       IF (error_flag .ne. no_error) goto 998
       inquire(unit=fileunit,nextrec=endpos)
       endpos   = endpos-1
       loc_size = endpos-begpos+1
       IF (iBRC.eq.1) THEN
          L2BC_Headers%SPH%PCD_DSD%size_of_one_dsr=loc_size
       ELSE
          IF (loc_size.ne.L2BC_Headers%SPH%PCD_DSD%size_of_one_dsr) THEN
             print *,'WriteL2BCtoBinaryFile: WARNING: PCD_ADS DSRs have variable sizes'
          ENDIF
       ENDIF
    enddo
    ! 5.4 get position and size after writing one DS
    inquire(unit=fileunit,nextrec=filepos_after)

    size_of_ds_calc = L2BC_Headers%SPH%PCD_DSD%size_of_one_dsr * &
                      L2BC_Headers%SPH%PCD_DSD%nr_of_dataset_records
    IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
       print *,"ERROR in WriteL2BCtoBinaryFile: Total size of PCD_ADS"
       print *,"is not consistent with DSD content !!!"
       print *,'actual DS size:            ',filepos_after-filepos_before
       print *,'GetByteSize result in DSD: ',size_of_ds_calc
       goto 998
    ENDIF
    !  #]
    !  #[ 6. write L2B Mie MDS

    ! 6.2 get position before writing Mie ADS
    inquire(unit=fileunit,nextrec=filepos_before)

    ! 6.3 writing Mie_MDS, get size of one DS and check sizes are
    ! constant
    do iBRC=1,Num_BRC
       inquire(unit=fileunit,nextrec=begpos)
       print *,'WriteL2BCtoBinaryFile: writing Mie_MDS for BRC #',iBRC
       call Write_L2B_Mie_MDS(fileunit,L2BC(iBRC)%Mie_MDS,&
                               N_Meas_to_write, N_Obs_Mie_to_write, &
                               error_flag)
       IF (error_flag .ne. no_error) goto 998
       inquire(unit=fileunit,nextrec=endpos)
       endpos   = endpos-1
       loc_size = endpos-begpos+1
       IF (iBRC.eq.1) THEN
          L2BC_Headers%SPH%Mie_DSD%size_of_one_dsr=loc_size
       ELSE
          IF (loc_size.ne.L2BC_Headers%SPH%Mie_DSD%size_of_one_dsr) THEN
             print *,'WriteL2BCtoBinaryFile: WARNING: Mie_MDS DSRs have variable sizes'
          ENDIF
       ENDIF
    enddo

    ! 6.4 get position and size after writing one DS
    inquire(unit=fileunit,nextrec=filepos_after)

    size_of_ds_calc = L2BC_Headers%SPH%Mie_DSD%size_of_one_dsr * &
                      L2BC_Headers%SPH%Mie_DSD%nr_of_dataset_records
    IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
       print *,"ERROR in WriteL2BCtoBinaryFile: Total size of Mie_MDS"
       print *,"is not consistent with DSD content !!!"
       print *,'actual DS size:            ',filepos_after-filepos_before
       print *,'GetByteSize result in DSD: ',size_of_ds_calc
       goto 998
    ENDIF
    !  #]
    !  #[ 7. write L2B Rayleigh MDS

    ! 7.2 get position before writing Rayleigh ADS
    inquire(unit=fileunit,nextrec=filepos_before)

    ! 7.3 writing Rayleigh_MDS, get size of one DS and check sizes are
    ! constant
    do iBRC=1,Num_BRC
       inquire(unit=fileunit,nextrec=begpos)
       print *,'WriteL2BCtoBinaryFile: writing Rayleigh_MDS for BRC #',iBRC
       call Write_L2B_Rayleigh_MDS(fileunit,L2BC(iBRC)%Rayleigh_MDS, &
                                   N_Meas_to_write, N_Obs_Rayleigh_to_write, &
                                   error_flag)
       IF (error_flag .ne. no_error) goto 998
       inquire(unit=fileunit,nextrec=endpos)
       endpos   = endpos-1
       loc_size = endpos-begpos+1
       IF (iBRC.eq.1) THEN
          L2BC_Headers%SPH%Rayleigh_DSD%size_of_one_dsr=loc_size
       ELSE
          IF (loc_size.ne.L2BC_Headers%SPH%Rayleigh_DSD%size_of_one_dsr) THEN
             print *,'WriteL2BCtoBinaryFile: WARNING: '//&
                  'Rayleigh_MDS DSRs have variable sizes'
          ENDIF
       ENDIF
    enddo

    ! 7.4 get position and size after writing one DS
    inquire(unit=fileunit,nextrec=filepos_after)
    size_of_ds_calc = L2BC_Headers%SPH%Rayleigh_DSD%size_of_one_dsr * &
                      L2BC_Headers%SPH%Rayleigh_DSD%nr_of_dataset_records
    IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
       print *,"ERROR in WriteL2BCtoBinaryFile: Total size of Rayleigh_MDS"
       print *,"is not consistent with DSD content !!!"
       print *,'actual DS size:            ',filepos_after-filepos_before
       print *,'GetByteSize result in DSD: ',size_of_ds_calc
       goto 998
    ENDIF
    !  #]

    IF (L2C_flag) THEN
       !  #[ write L2C AssimPCD ADS

       ! Assim_PCD.2 get position before writing Assim_PCD ADS
       inquire(unit=fileunit,nextrec=filepos_before)

       ! Assim_PCD.3 writing Assim_PCD_ADS, get size of one DS and check sizes
       ! are constant
       do iBRC=1,Num_BRC
          inquire(unit=fileunit,nextrec=begpos)
          print *,'WriteL2BCtoBinaryFile: writing Assim_PCD_ADS for BRC #',iBRC
          call Write_L2C_AssimPCD_ADS(fileunit,L2BC(iBRC)%Assim_PCD_ADS,&
                                      N_Obs_Mie_to_write,&
                                      N_Obs_Rayleigh_to_write,error_flag)
          IF (error_flag .ne. no_error) goto 998
          inquire(unit=fileunit,nextrec=endpos)
          endpos   = endpos-1
          loc_size = endpos-begpos+1
          IF (iBRC.eq.1) THEN
             L2BC_Headers%SPH%Assim_PCD_DSD%size_of_one_dsr=loc_size
          ELSE
             IF (loc_size.ne.L2BC_Headers%SPH%Assim_PCD_DSD%size_of_one_dsr) THEN
                print *,'WriteL2BCtoBinaryFile: WARNING: '//&
                     'Assim_PCD_ADS DSRs have variable sizes'
             ENDIF
          ENDIF
       enddo

       ! Assim_PCD.4 get position and size after writing one DS
       inquire(unit=fileunit,nextrec=filepos_after)

       size_of_ds_calc = L2BC_Headers%SPH%Assim_PCD_DSD%size_of_one_dsr * &
            L2BC_Headers%SPH%Assim_PCD_DSD%nr_of_dataset_records

       IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
          print *,"ERROR in WriteL2BCtoBinaryFile: Total size of Assim_PCD_ADS"
          print *,"is not consistent with DSD content !!!"
          print *,'actual DS size:            ',filepos_after-filepos_before
          print *,'GetByteSize result in DSD: ',size_of_ds_calc
          goto 998
       ENDIF
       !  #]
       !  #[ write L2C Mie VecWind MDS

       ! L2C_Mie_VecWind_MDS.2 get position before writing Mie VecWind MDS
       inquire(unit=fileunit,nextrec=filepos_before)

       ! L2C_Mie_VecWind_MDS.3 writing Mie_VecWind_MDS, 
       ! get size of one DS and check sizes are constant
       do iBRC=1,Num_BRC
          inquire(unit=fileunit,nextrec=begpos)
          print *,'WriteL2BCtoBinaryFile: writing Mie_VecWind_MDS for BRC #',iBRC
          call Write_L2C_MieVec_MDS(fileunit,L2BC(iBRC)%Mie_VecWind_MDS,&
                                    N_Obs_Mie_to_write,error_flag)
          IF (error_flag .ne. no_error) goto 998
          inquire(unit=fileunit,nextrec=endpos)
          endpos   = endpos-1
          loc_size = endpos-begpos+1
          IF (iBRC.eq.1) THEN
             L2BC_Headers%SPH%Mie_VecWind_DSD%size_of_one_dsr=loc_size
          ELSE
             IF (loc_size.ne.L2BC_Headers%SPH%Mie_VecWind_DSD%size_of_one_dsr) THEN
                print *,'WriteL2BCtoBinaryFile: WARNING: MieVec_MDS DSRs have variable sizes'
             ENDIF
          ENDIF
       enddo

       ! L2C_Mie_VecWind_MDS.4 get position and size after writing one DS
       inquire(unit=fileunit,nextrec=filepos_after)

       size_of_ds_calc = L2BC_Headers%SPH%Mie_VecWind_DSD%size_of_one_dsr * &
            L2BC_Headers%SPH%Mie_VecWind_DSD%nr_of_dataset_records
       IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
          print *,"ERROR in WriteL2BCtoBinaryFile: Total size of"
          print *,"Mie_WindVec_MDS is not consistent with DSD content !!!"
          print *,'actual DS size:            ',filepos_after-filepos_before
          print *,'GetByteSize result in DSD: ',size_of_ds_calc
          goto 998
       ENDIF
       !  #]
       !  #[ write L2C Rayleigh VecWind MDS

       ! L2C_Ray_VecWind.2 get position before writing Rayleigh VecWind MDS
       inquire(unit=fileunit,nextrec=filepos_before)

       ! L2C_Ray_VecWind.3 writing Rayleigh_VecWind_MDS,
       ! get size of one DS and check sizes are constant
       do iBRC=1,Num_BRC
          inquire(unit=fileunit,nextrec=begpos)
          print *,'WriteL2BCtoBinaryFile: writing Rayleigh_VecWind_MDS for BRC #',iBRC
          call Write_L2C_RayleighVec_MDS(fileunit,&
                                         L2BC(iBRC)%Rayleigh_VecWind_MDS,&
                                         N_Obs_Rayleigh_to_write,error_flag)
          IF (error_flag .ne. no_error) goto 998
          inquire(unit=fileunit,nextrec=endpos)
          endpos   = endpos-1
          loc_size = endpos-begpos+1
          IF (iBRC.eq.1) THEN
             L2BC_Headers%SPH%Rayleigh_VecWind_DSD%size_of_one_dsr=loc_size
          ELSE
             IF (loc_size.ne.L2BC_Headers%SPH%Rayleigh_VecWind_DSD%size_of_one_dsr) THEN
                print *,'WriteL2BCtoBinaryFile: WARNING: Rayleigh_VecWind_MDS DSRs have variable sizes'
             ENDIF
          ENDIF
       enddo
       ! L2C_Ray_VecWind.4 get position and size after writing one DS
       inquire(unit=fileunit,nextrec=filepos_after)

       size_of_ds_calc = L2BC_Headers%SPH%Rayleigh_VecWind_DSD%size_of_one_dsr * &
            L2BC_Headers%SPH%Rayleigh_VecWind_DSD%nr_of_dataset_records
       IF (size_of_ds_calc .ne. (filepos_after-filepos_before)) THEN
          print *,"ERROR in WriteL2BCtoBinaryFile: Total size of"
          print *,"Rayleigh_WindVec_MDS is not consistent with DSD content !!!"
          print *,'actual DS size:            ',filepos_after-filepos_before
          print *,'GetByteSize result in DSD: ',size_of_ds_calc
          goto 998
       ENDIF
       !  #]
    END IF

    !------------------------------
    ! close the file
    !------------------------------
    close(unit=fileunit)

    !------------------------------
    ! done
    !------------------------------
    return

    !  #[ exception/error handlers 
996 print *,"ERROR could not open outputfile: ",trim(filename_dbl)
    error_flag = error_opening_file
    return

997 print *,"conversion error in WriteL2BCtoBinaryFile: "
    print *,"could not convert the counter iBRC = ",iBRC," to a string .."
    error_flag = error_writing_file
    return

    ! jump point in case the file was open
998 close(unit=fileunit)
    call free_lun(fileunit)    
    ! no return here, since we use the ABORT print below

    ! jump point in case the file was not yet open
999 print *,"ABORTING subroutine WriteL2BCtoBinaryFile"
    return
    !  #]

  end subroutine WriteL2BCtoBinaryFile
    !  #]  
  !---------------------------------
  subroutine WriteProductList(L2BC_Headers,path_and_filename_JobOrder,error_flag)
    !  #[

    ! create the little Product List file, required by the 
    ! Thin Layer system:
    type(L2BC_HDR_DataType), intent(in)  :: L2BC_Headers
    character(len=256),  intent(in)  :: path_and_filename_JobOrder
    integer,             intent(out) :: error_flag

    ! local variable
    integer            :: fileunit, l
    character(len=256) :: order_id, ProductListFileName
    character(len=256) :: filename_JobOrder
    character(len=256) :: pathname_JobOrder

    ! init
    error_flag = no_error

    call Split_path_and_filename(path_and_filename_JobOrder,&
                                 pathname_JobOrder,filename_JobOrder)

    ! expected format for filename_JobOrder is:
    ! JobOrder.<order_id>.xml
    ! so remove leading "JobOrder." and tailing ".xml"
    ! to get the <order_id> string.

    l = len_trim(filename_JobOrder)

    if (l .le. 13) then
       call logmsg(log_error,"could not extract <order_id> from "//&
            "filename_JobOrder in subroutine WriteProductList; reason: "//&
            "length filename_JobOrder is smaller than the expected minimum "//&
            "of 13 characters")
       error_flag = error_writing_file
       return
    end if
    if (filename_JobOrder(1:9) .ne. "JobOrder.") then
       call logmsg(log_error,"could not extract <order_id> from "//&
            "filename_JobOrder in subroutine WriteProductList; reason: "//&
            "filename_JobOrder does not start with string 'JobOrder'")
       error_flag = error_writing_file
       return
    end if
    if (filename_JobOrder(l-3:l) .ne. ".xml") then 
       call logmsg(log_error,"could not extract <order_id> from "//&
            "filename_JobOrder in subroutine WriteProductList; reason: "//&
            "filename_JobOrder does not have '.xml' extention")
       error_flag = error_writing_file
       return
    end if
    order_id = filename_JobOrder(10:l-4)
    !print *,"test: order_id = ",trim(order_id)
    
    ProductListFileName = &
         Join_path_and_filename(pathname_JobOrder,trim(order_id)//".LIST")
    !print *,"test: ProductListFileName = ",trim(ProductListFileName)

    fileunit = get_lun()
    open(unit=fileunit,file=ProductListFileName,status="replace",&
         action="write",err=999)
    write(fileunit,"(a)",err=998) trim(L2BC_Headers%FH%File_Name)//".HDR"
    write(fileunit,"(a)",err=998) trim(L2BC_Headers%FH%File_Name)//".DBL"
    close(fileunit)
    call free_lun(fileunit)

    return

998 call logmsg(log_error,"could not write ProductList: "//&
                          trim(ProductListFileName))
       error_flag = error_writing_file
       return

999 call logmsg(log_error,"could not open ProductList: "//&
                          trim(ProductListFileName))
       error_flag = error_writing_file
       return

  end subroutine WriteProductList

    !  #]
  !---------------------------------
end module WriteL2BCdata
