program Test_Match_AMD
  !  #[ Description
  !        a test program for the match_nwp_module
  !     
  !     created 10-Nov-2006 by: J. de Kloe
  !        
  !     Modifications:
  !        23-Nov-2006 J. de Kloe  bugfix
  !        15-Mar-2007 J. de Kloe  remove unused routine from USE statement
  !        28-Aug-2007 D. Tan      remove references to obsolete routines
  !        03-Sep-2007 P. Poli     Introduced L1B_Headers
  !        04-Sep-2007 P. Poli     change L1B_DataType to L1B_BRC_DataType, 
  !                                change L2BC_DataType to L2BC_BRC_DataType
  !        03-Oct-2007 J. de Kloe  corrected an error string
  !        16-Jan-2008 J. de Kloe  phase out integer kind i_
  !  #]
  !  #[ modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Match_AMD_Module, only: Find_Matching_AMD_Profile
  USE L2B_Proc_Settings, only: L2BP_Settings_type, &
                               Init_L2B_Proc_Settings, &
                               Delete_L2B_Proc_Settings, &
                               Set_Derived_L2B_Proc_Settings, &
                               Print_L2B_Proc_Settings, &
                               Get_JobOrderName_from_CmdLine, &
                               Get_Additional_Cmdline_Options, &
                               Print_L2B_Proc_Settings, &
                               Copy_JobOrderData_Items
  USE Read_L2B_AuxPar_data, only: InitRead_L2B_AuxPar_fileModule, &
                                  Read_L2B_AuxPar_file
  USE L2B_AuxPar_DataStructure, only: Init_L2B_AuxPar, L2B_AuxPar_Type, &
                                      Print_L2B_AuxPar,  Delete_L2B_AuxPar
  USE ReadL1Bdata, only: InitReadL1BfileModule, ReadL1Bfile
  USE ReadJobOrderData, only: ReadJobOrderFile
  USE JobOrder_DataStructure, only: JobOrderData_type, Print_JobOrderData, &
                                    Init_JobOrderData, Delete_JobOrderData
  USE ReadL1Bdata, only: ReadL1Bfile
  USE L1B_DataStructure, only: L1B_HDR_DataType, &
                               L1B_BRC_DataType, &
                               Delete_L1B_BRC_DataStructure, &
                               Delete_L1B_HDR_DataStructure, &
                               Init_L1B_BRC_DataStructure, &
                               Init_L1B_HDR_DataStructure, &
                               Empty_L1B_BinaryDatasets
  USE L2BC_DataStructure, only: L2BC_BRC_DataType, Delete_L2BC_BRC_DataStructure, &
                                Init_L2BC_BRC_DataStructure
  USE AMD_DataStructure, only: AMD_HDR_DataType, &
                               AMD_BRC_DataType, &
                               Delete_AMD_HDR_DataStructure, &
                               Delete_AMD_BRC_DataStructure
  USE ReadAMDdata, only: InitReadAMDfileModule, Load_All_AMD_Profiles
  USE EE_CFI_Datatypes, only: IntAuc, IntAs
  USE Working_DataStructure, only: WorkingDataType, InitWorkingData, &
                                   Copy_L1B_fields2WorkingArrays, &
                                   DeleteWorkingData
  ! support modules
  USE Logging, only: init_logging, &
                     logmsg,log_debug,log_info,log_progress,log_warn,log_error
  ! define some parameters for verbosity control
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, VerbosityReport
  USE ErrorHandler, only: no_error, program_abort,request_program_to_stop,&
                          error_allocate

  !  #]
  !  #[ Variables

  implicit none 

  ! local variables
  integer :: error_flag, Task_Number, iBRC, Num_BRC, iAMD
  integer :: AllocStatus
  integer :: N_meas, which_AMD_DSR

  integer,         parameter :: N_range_gates  = 24 ! current used nr of gates
  integer(IntAuc), parameter :: N_Obs_Max_Mie      = 2 ! max. nr. of obs. Mie
  integer(IntAuc), parameter :: N_Obs_Max_Rayleigh = 2 ! max. nr. of obs. Rayl.

  ! a structure to hold the JobOrder settings
  TYPE(JobOrderData_type) :: JobOrderData
  character(len=256)      :: filename_JobOrder

  ! a structure to hold all processor settings
  TYPE(L2BP_Settings_type) :: L2BP_Settings
  ! a struct to hold the necessary input data for the processor
  TYPE(L1B_BRC_DataType)   :: L1B
  TYPE(L1B_HDR_DataType)   :: L1B_Headers
  ! a struct to hold all results from the processor
  TYPE(L2BC_BRC_DataType), dimension(:), pointer :: L2B
  ! a struct to hold all NWP data
  TYPE(AMD_BRC_DataType), dimension(:), pointer :: AMD
  TYPE(AMD_HDR_DataType)                        :: AMD_Headers
  ! the local working datastructure to store temporary processing results
  TYPE(WorkingDataType) :: WorkingData
  !  #]
  !  #[ Program code

  ! init
  error_flag = no_error
  nullify(L2B)
  nullify(AMD)

  call logmsg(log_progress,"Starting Test_Match_NWP")

  call Init_L2B_Proc_Settings(L2BP_Settings,error_flag)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_error,"aborted Init_L2B_Proc_Settings")
     call program_abort(error_flag,"Init_L2B_Proc_Settings")
  END IF

  !  #[ read the JobOrder file

  call Get_JobOrderName_from_CmdLine(filename_JobOrder,error_flag)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_progress,"Reading JobOrder from the commandline failed")
     call program_abort(error_flag,"Get_JobOrderName_from_CmdLine")
  END IF

  print *,"Reading JobOrder file: ",trim(filename_JobOrder)
  ! initialise the JobOrder datastructure
  call Init_JobOrderData(JobOrderData)
  ! load the JobOrder file
  call ReadJobOrderFile(JobOrderData,filename_JobOrder,&
                        Task_Number,error_flag,workaround=.true.)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_progress,"Loading Joborder file: "//&
                 trim(filename_JobOrder)//" failed")
     call program_abort(error_flag,"ReadJobOrderFile")
  END IF
  call logmsg(log_progress,"Joborder file: "//&
              trim(filename_JobOrder)//" loaded")

  ! do a test print of these settings
  !call Print_JobOrderData(JobOrderData,Task_Number,&
  !     "JobOrderData",error_flag)
  !IF (error_flag .ne. no_error) &
  !     call program_abort(error_flag,"Print_JobOrderData")
  
  ! copy items from the JobOrderData to the variables in the
  ! L2B_Proc_Settings module, so the test program will use them
  call Copy_JobOrderData_Items(JobOrderData,L2BP_Settings,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Copy_JobOrderData_Items")

  !  #]

  !  #[ read the AuxPar file
  call Init_L2B_AuxPar(L2BP_Settings%L2B_AuxPar,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_L2B_AuxPar")
     
  ! load the L2BAuxPar input file
  call InitRead_L2B_AuxPar_fileModule(error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitRead_L2B_AuxPar_fileModule")
  call Read_L2B_AuxPar_file(L2BP_Settings%L2B_AuxPar, &
                            L2BP_Settings%L2BAuxPar_filename,error_flag)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_progress,"loading L2B_AuxPar file: "//&
              trim(L2BP_Settings%L2BAuxPar_filename)//" failed")
     call program_abort(error_flag,"Read_L2B_AuxPar_file")
  END IF

  call logmsg(log_progress,"L2B_AuxPar file: "//&
              trim(L2BP_Settings%L2BAuxPar_filename)//" loaded")
  !  #]

  ! print the settings defined in Set_L2B_Proc_Settings
  ! and any changed added to them by the command line options
  call Print_L2B_Proc_Settings(L2BP_Settings,"L2BP_Settings",error_flag)

  !call Init_Logging("DEBUG","NOOP",error_flag)
  call Init_Logging(L2BP_Settings%Stdout_Log_Level,&
                    L2BP_Settings%Stderr_Log_Level,error_flag)
  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_Logging")

  call InitReadAMDfileModule(error_flag,ChoosenVerbosity=VerbosityError)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitReadAMDfileModule")

  ! Initialize reading of L1B file (EE-format)
  call InitReadL1BfileModule(error_flag)!,ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitReadL1BfileModule")

  ! initialise the L1B datastructure, but do not yet allocate its 
  ! substructures, that is only possible after reading the L1B_SPH
  call Init_L1B_HDR_DataStructure(L1B_Headers, error_flag)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted Init_L1B_HDR_DataStructure")
     call program_abort(error_flag,"Init_L1B_HDR_DataStructure")
  END IF

  call Init_L1B_BRC_DataStructure(L1B, error_flag)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted Init_L1B_BRC_DataStructure")
     call program_abort(error_flag,"Init_L1B_BRC_DataStructure")
  END IF

  ! read the headers of the L1B file. This is needed to get
  ! the Num_BRC value before entering the BRC loop
  ! (doing this outside the loop is required if you wish
  !  to parallelise this loop on a supercomputer!)
  call ReadL1Bfile(L1B_Headers, L2BP_Settings%L1Bfile%name, error_flag, &
                   L1B = L1B, Which_DSR =  1, ReadHeaders=.true.)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted ReadL1Bfile")
     call program_abort(error_flag,"ReadL1Bfile")
  END IF

  ! Set the number of BRCs
  Num_BRC = L1B_Headers%SPH%Geolocation_DSD%nr_of_dataset_records

  ! allocate L2B structure array
  allocate(L2B(Num_BRC),stat=AllocStatus)
  IF (AllocStatus .ne. 0) THEN
     print *,"Allocation problem in L2B_processor()"
     error_flag = error_allocate
     call logmsg(log_error,"L2B_processor: Allocation problem"//&
          " could not get memory to allocate: L2B(Num_BRC)")
     call program_abort(error_flag,"L2B_processor")
  END IF
  
  ! load all AMD profiles into an array of raw profiles
  call Load_All_AMD_Profiles(L2BP_Settings%NWPfile(1)%name,AMD_Headers,AMD,error_flag)
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted Load_All_AMD_Profiles")
     call program_abort(error_flag,"Load_All_AMD_Profiles")
  END IF

  ! loop over all BRCs in the file
  brc_loop: DO iBRC=1,Num_BRC
     !  #[
     print *,"processing BRC nr.:",iBRC
     call ReadL1Bfile(L1B_Headers, L2BP_Settings%L1Bfile%name, error_flag, &
          L1B = L1B, Which_DSR =  iBRC, &
          ReadHeaders=.false., ReadGeoloc=.true., ReadPCD=.true., & 
          ReadGWD=.true., ReadMeas=.true., ReadUS=.true., ReadWV=.true., &
          ReadCal=.true.)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"ReadL1Bfile")
     
     N_meas = L1B_Headers%SPH%N_Meas_Max
     
     print *,"initialising L2B data structure"
     call Init_L2BC_BRC_DataStructure(L2B(iBRC),N_meas, &
          N_Obs_Max_Mie, N_Obs_Max_Rayleigh, N_range_gates, error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Init_L2BC_BRC_DataStructure")
     
     print *,"initialising L1B working arrays..."
     print *,"done initialising L1B working arrays"
     print *,"N_meas = ",N_meas
     ! TODO: revise these prints when testoutput changes acceptable

     print *,"initialising Working arrays..."
     call InitWorkingData(WorkingData, int(N_range_gates,IntAs), &
          int(N_Meas,IntAs), int(N_Obs_Max_Mie,IntAs), &
          int(N_Obs_Max_Rayleigh,IntAs), error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"InitWorkingData")
     print *,"done initialising Working arrays"
   
     call Copy_L1B_fields2WorkingArrays(L1B,WorkingData)
     
     !call Screen_L1B_Data(L1B,WorkingData,L2B%PCD_ADS%L1B_Input_Screening,&
     !     L2B_AuxPar%Data%Screening_Params%L1B_Screening_Params,error_flag)
     !IF (error_flag .ne. no_error) &
     !     call program_abort(error_flag,"Screen_L1B_Data")
     
     ! do_matching to select the proper AMD struct
     ! remark: the result may be: which_AMD_DSR=-1 
     !         when no suitable profile was found !
     call Find_Matching_AMD_Profile(AMD,WorkingData,&
          L2BP_Settings%L2B_AuxPar%Data%WVM_Params%AMD_Matchup_Params,&
          L2B(iBRC)%PCD_ADS%L2B_AMD_Screening%L2B_AMD_Collocations, &
          which_AMD_DSR, iBRC,error_flag)
     IF (error_flag .ne. no_error) &
          call program_abort(error_flag,"Find_Matching_AMD_Profile")
     
     print *,"Selected AMD profile nr.: ",which_AMD_DSR
     
     ! clean up to enable handling the next BRC

     call Empty_L1B_BinaryDatasets(L1B)
     call DeleteWorkingData(WorkingData)
     !  #]
  END DO brc_loop

  print *,"=================================="
  print *,"nr of processed BRC's was: ",Num_BRC
  print *,"=================================="
  
  ! clean up, free allocated memory
  ! free allocated memory when done with this BRC                             
  call Delete_L1B_BRC_DataStructure(L1B)
  call Delete_L1B_HDR_DataStructure(L1B_Headers)
  IF (associated(L2B)) THEN
     DO iBRC = 1, size(L2B) ! Num_BRC
        call Delete_L2BC_BRC_DataStructure(L2B(iBRC))
     ENDDO
  END IF
  deallocate(L2B)
  IF (associated(AMD)) THEN
     DO iAMD = 1, size(AMD) ! Num_BRC
        call Delete_AMD_BRC_DataStructure(AMD(iAMD))
     ENDDO
  END IF
  deallocate(AMD)
  call Delete_AMD_HDR_DataStructure(AMD_Headers)

  call Delete_L2B_Proc_Settings(L2BP_Settings)
  call Delete_JobOrderData(JobOrderData)
  
  call logmsg(log_progress,"Finished L2B-Processor")
  !  #]
end program Test_Match_AMD
