function CreateData(scenarioName)


% - SCENARIO NAME is the name of the directory containing all scenario xml files, instrument data and aisp products directory 
%
% - DATADIR is the name of the directory containing the ascii data ('single_RMA_midlat_winter_and_altostratus', 'TestScenario_1.1_u00' )
% 
% - START TIME is the start sensing time of the scenario. If the default value
%   '2007-10-02T00:00:00.000000' is correct, set to 'default' the variable
%   startTime. If an other date is necessary, set the startTime variable following the format YYYY-MM-DDThh:mm:ss.uuuuuu
% 
% - DURSEG is a vector with the duration of each segment.
%   You have to precise NbSeg-1 values where NbSeg is the number of segment.
% 
% - ALLNOISE is a flag corresponding to the E2S AllNoise flag. Set this
%   flag to 0 to not introduce noise 
%  
% - SIMPOISSONNOISE is a flag corresponding to the E2S SimPoissonFlag. If AllNoise is set to 1 
% 
% - FLAGINTERPOL is a flag corresponding to the 6 interpolation flag of
%   atmospheric data. Set this flag to 'false' to have constant values on the
%   segment and 'true' to have interpolated values. The current version
%   doesn't allowed to have different values for the 6 flag. 
% 
% - JOBORDERNAME will be used to create the name of the job
%   order : JobOrder.JOBORDERNAME.xml. For example, JOBORDERNAME can take the
%   values : '00001' or 'test01'
% 
% - N is the number of measurement. All segments will have the same number
%   of measurement per observation.
% 
% - P is the number of laser shots. All segments will have the same number
%   of laser shots per measurement.
% 
% - MIEFLAG is a variable selecting the mie contribution from the knmi
%   ascii file. MIEFLAG can take the values : 'all' (aerosol + cloud
%   contribution), 'cloud' (only cloud contribution), 'aerosol' (only
%   aerosol contribution), 'none' (no mie contribution)
%
% - AOCSERROR is a flag. Set this flag to 'no' will set all AOCS error
%   parameters equal to 0. Set this flag to 'yes' won't change default values of Aocs Error Parameters.  
 

%scenarioName='testCFI';    
DataDir = 'single_RMA_midlat_winter_and_altostratus/';
startTime = 'default';  
durSeg = [30];
AllNoise = 0;
SimPoissonNoise = 0;
FlagInterpol='false';
JobOrderName=scenarioName;
N=1;
P=1005;
mieflag = 'all';
AocsError='no';

%ResetProductCounterFile;

CreateScenario(scenarioName,DataDir,startTime,durSeg,N,P,AllNoise,SimPoissonNoise,FlagInterpol,mieflag,AocsError) 

statusE2S = CreateE2SData(scenarioName);

if statusE2S == 0,
    CreateL1BData(JobOrderName, scenarioName);
end
