function CreateScenario(scenarioName,DataDir,startTime,durSeg,AllNoise,N,P,SimPoissonNoise,FlagInterpol,mieflag,AocsError)


% Function CreateScenario

% This function checks if a directory with the proposed scenario name
% already exists. If it finds one, it propose to overwrite the scenario. If
% the response is yes, the program will call the CreateScenario function . If the 
% response is No or if the warning dialog box is closed, the program will stop. 
%
% Input arguments :
% 
% - SCENARIO NAME is the name of the scenario where you will have all scenario xml files, instrument data and aisp products directory 
%
% - DATADIR is the name of the directory where are the ascii data you want to
% 
% - START TIME is the start sensing time of the scenario. If the default value
%   '2007-10-02T00:00:00.000000' is correct, set to 'default' the variable
%   startTime. If a precised date is necessary, set the startTime variable following the format YYYY-MM-DDThh:mm:ss.uuuuuu
% 
% - DURSEG is a vector with the duration of each segment.
%   You have to precise NbSeg-1 values where NbSeg is the number of segment.
% 
% - ALLNOISE is a flag corresponding to the E2S AllNoise flag. Set this
%   flag to 0 to not introduce noise 
% 
% - SIMPOISSONNOISE is a flag corresponding to the E2S SimPoissonFlag. 
% 
% - FLAGINTERPOL is a flag corresponding to the 6 interpolation flag of
%   atmospheric data. Set this flag to 'false' to have constant values on the
%   segment and 'true' to have interpolated values. The current version
%   doesn't allowed to have different values for the 6 flag. If it is
%   necessary, it can be done easily.
% 
% - JOBORDERNAME is a character array that will be used to create the name job
%   order : JobOrder.JOBORDERNAME.xml. For example, JOBORDERNAME can take the
%   values : '00001' or 'test01'
% 
% - N is the number of measurement. All segments will have the same number
%   of measurement per observation.
% 
% - P is the number of laser shots. All segments will have the same number
%   of laser shots per mezasurement.
% 
% - MIEFLAG is a variable selecting the mie contribution from the knmi
%   ascii file. MIEFLAG can take the values : 'all' (aerosol + cloud
%   contribution), 'cloud' (only cloud contribution), 'aerosol' (only
%   aerosol contribution), 'none' (no mie contribution)
% - AOCSERROR is a flag. Set this flag to 'no' will set all AOCS error
%   parameters equal to 0. Set this flag to 'yes' won't change default values of Aocs Error Parameters.  
%
% Revision 1.1   19 Dec 2005 Marie-Laure Denneulin
% Initial revision

DefinePaths;
scenarioPath=fullfile(SimPathE2S,'scenarios',scenarioName);

fprintf('Creation Scenario\n')

% Creation du scenario
TestExistScenario = exist(scenarioPath,'dir');
if (TestExistScenario==7),
    button = questdlg(sprintf('the %s directory already exists! Do you want to overwrite?',scenarioName),'Warning','Yes','No','Yes');
    switch button
        case 'Yes'
            CreateScenarioFromAsciData(scenarioName,DataDir,startTime,durSeg,AllNoise,N,P,SimPoissonNoise,FlagInterpol,mieflag,AocsError);
        case 'No'
            return;
        case isempty(button)
            return;      
    end
else
    mkdir(scenarioPath);
    CreateScenarioFromAsciData(scenarioName,DataDir,startTime,durSeg,AllNoise,N,P,SimPoissonNoise,FlagInterpol,mieflag,AocsError);
end
