function CreateScenarioFromAsciData(scenarioName,DataDir,startTime,durSeg,N,P,AllNoise,SimPoissonNoise,FlagInterpol,mieflag,AocsError)

% function CreateScenario(scenarioName,DataDir,startTime,durSeg,AllNoise,SimPoissonNoise,FlagInterpol)
% 
% Create a directory with all scenario files. 
% 
% Input arguments :
%
% - SCENARIO NAME is the name of the scenario where you will have all scenario xml files, instrument data and aisp products directory 
%
% - DATADIR is the name of the directory where are the ascii data you want to
% 
% - START TIME is the start sensing time of the scenario. If the default value
%   '2007-10-02T00:00:00.000000' is correct, set to 'default' the variable
%   startTime. If a precised date is necessary, set the startTime variable following the format YYYY-MM-DDThh:mm:ss.uuuuuu
% 
% - DURSEG is a vector with the duration of each segment.
%   You have to precise NbSeg-1 values where NbSeg is the number of segment.
% 
% - ALLNOISE is a flag corresponding to the E2S AllNoise flag. Set this
%   flag to 0 to not introduce noise 
% 
% - SIMPOISSONNOISE is a flag corresponding to the E2S SimPoissonFlag. 
% 
% - FLAGINTERPOL is a flag corresponding to the 6 interpolation flag of
%   atmospheric data. Set this flag to 'false' to have constant values on the
%   segment and 'true' to have interpolated values. The current version
%   doesn't allowed to have different values for the 6 flag. If it is
%   necessary, it can be done easily.
% 
% - JOBORDERNAME is a character array that will be used to create the name job
%   order : JobOrder.JOBORDERNAME.xml. For example, JOBORDERNAME can take the
%   values : '00001' or 'test01'
% 
% - N is the number of measurement. All segments will have the same number
%   of measurement per observation.
% 
% - P is the number of laser shots. All segments will have the same number
%   of laser shots per mezasurement.
% 
% - MIEFLAG is a variable selecting the mie contribution from the knmi
%   ascii file. MIEFLAG can take the values : 'all' (aerosol + cloud
%   contribution), 'cloud' (only cloud contribution), 'aerosol' (only
%   aerosol contribution), 'none' (no mie contribution)
%
%
% Revision 1.1   19 Dec 2005 Marie-Laure Denneulin
% Initial revision
%
% Revision 1.2   6 Jan 2006 Marie-Laure Denneulin
% Change on the creation atmosphereProfile xml files


DefinePaths;


scenarioPath=fullfile(SimPathE2S,'scenarios',scenarioName)
[startTimeKnmi,nbSegment] = ConvertTxtScenarioToXML(DataDir,scenarioPath,mieflag);

if strcmp(startTime,'default'),
    startTime = '2007-10-02T00:00:00.000000';
end
TimeStepKNMI = 0.074627;



% Check that the length of vector durSeg is equal to nbSeg-1 where nbSeg is
% the number of segment
if (length(durSeg)~= nbSegment-1) & (nbSegment~=1) & (length(durSeg)~=0),
    fprintf('verifier la taille du vecteur durSeg\n');
    return;
end

% Copy the files defined for the scenario in the directory scenarioName
copyfile(fullfile(SimPathE2S,'defaults','aispDefaultParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','aispErrorDefaultParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','aispHousekeepingParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','lidarInstrumentDetectorParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','lidarInstrumentLinkParameters.xml'),  scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','lidarInstrumentRayleighParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','lidarInstrumentMieParameters.xml'), scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','lidarInstrumentCalibrationModeParameters.xml'), scenarioPath);

copyfile(fullfile(SimPathE2S,'defaults','orbitScenarioFile.xml'),scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','satelliteCharacterisation.xml'),scenarioPath);
copyfile(fullfile(SimPathE2S,'defaults','satelliteParameters.xml'),scenarioPath);


% Copy the dem parameters file in the directory scenarioName
copyfile(fullfile(SimPathE2S,'defaults','demParameters.xml'),scenarioPath);

% Copy the image DefocusImage in the directory scenarioName 
copyfile(fullfile(SimPathE2S,'defaults','DefocusImage.tiff'),scenarioPath);


SegFile1 = 'aispErrorParameters.xml';
SegFile2 = 'aocsErrorParameters.xml';
SegFile3 = 'atmosphereCloudParameters.xml';
SegFile4 = 'atmosphereGroundParameters.xml';
SegFile5 = 'lidarInstrumentModeParameters.xml';

% Creation of xml files defined for a segment
% For the case with only one profile, the other atmopshereProfileParameter 
% file are created by a copy of the initial file. The startTime of the
% second segment is computed from the duration segment.

if nbSegment==1,
    nbSegment=length(durSeg)+1;
  
    startTimeVector = CreateStartTimeVector(startTime, durSeg);
    for kSeg=1: nbSegment,

        copyfile(fullfile(SimPathE2S,'defaults',SegFile1), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile1]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile2), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile2]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile3), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile3]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile4), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile4]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile5), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile5]));
        if kSeg~=1,
            copyfile(fullfile(scenarioPath,'Segment01_atmosphereProfileParameters.xml'),...
                fullfile(scenarioPath,sprintf('Segment%02d_atmosphereProfileParameters.xml',kSeg)));
        end
        CreateSegDefFile(kSeg,startTimeVector(kSeg),scenarioPath);
    end
    % Creation of scenario xml file
    CreateScenarioDefFile(nbSegment,scenarioName);
    
else
    
    switch length(durSeg)
        case 0
            %durSegInit = diff(startTimeKnmi,1);
            %durSeg = durSegInit(:,3)*24*60*60 + durSegInit(:,4)*60*60 + durSegInit(:,5)*60 + durSegInit(:,6);
            %startTimeVector = CreateStartTimeVector(startTime, durSeg);
	    durSeg = ones(1,nbSegment)*TimeStepKNMI;
	    startTimeVector = CreateStartTimeVector(startTime, durSeg);

        otherwise 
            startTimeVector = CreateStartTimeVector(startTime, durSeg);
    end
    
    for kSeg=1: nbSegment,
        copyfile(fullfile(SimPathE2S,'defaults',SegFile1), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile1]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile2), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile2]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile3), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile3]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile4), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile4]));
        copyfile(fullfile(SimPathE2S,'defaults',SegFile5), fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile5]));
         
        CreateSegDefFile(kSeg,startTimeVector(kSeg),scenarioPath);
    end

    % Creation of scenario xml file
    CreateScenarioDefFile(nbSegment,scenarioName);
end

% 
% Set the field allNoiseFlag with the value AllNoise in all
% Segmentxx_lidarInstrumentModeParameters.xml files
% Set the N and P fields with the the values N and P in all
% Segmentxx_lidarInstrumentModeParameters.xml files
for kSeg = 1:nbSegment,
    xml = xmlread(fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile5]));
    position = xml.getChildNodes.getLength -1;
    allnoise =  xml.getChildNodes.item(position).getChildNodes.item(13).getChildNodes.item(0);
    allnoise.setData(num2str(AllNoise));
    nbMeas = xml.getChildNodes.item(position).getChildNodes.item(5).getChildNodes.item(0);
    nbMeas.setData(num2str(N));
    nbShot = xml.getChildNodes.item(position).getChildNodes.item(3).getChildNodes.item(0);
    nbShot.setData(num2str(P));
    xmlwrite(fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile5]),xml);
end

% Set the variable poissonNoiseFlag with the value SimPoissonNoise in the  
% lidarInstrumentDetectorParameters.xml file
xml = xmlread(fullfile(scenarioPath,'lidarInstrumentDetectorParameters.xml'));
poissonNoise = xml.getChildNodes.item(2).getChildNodes.item(41)...
    .getChildNodes.item(0);
poissonNoise.setData(num2str(SimPoissonNoise));   
xmlwrite(fullfile(scenarioPath,'lidarInstrumentDetectorParameters.xml'),xml);

% Set the interpolation flags in all the
% Segmentxx_atmosphereProfileParameters.xml file
for kSeg = 1:nbSegment,
    xml2 = xmlread(fullfile(scenarioPath,sprintf('Segment%02d_atmosphereProfileParameters.xml',kSeg)));
    flagInterpol = xml2.getChildNodes.item(0).getChildNodes;   
    for kFlag = 1:6,
        flag = flagInterpol.item(417+(kFlag-1)*2).getChildNodes.item(0);
        flag.setData(FlagInterpol);
    end
    xmlwrite(fullfile(scenarioPath,sprintf('Segment%02d_atmosphereProfileParameters.xml',kSeg)),xml2);
end

% If AocsError is set to 'no', all the aocs error parameters in the Segmentxx_AocsErrorParameters.xml 
% files will be set to 0 or false 
if strcmp(AocsError,'no'),
    for kSeg = 1:nbSegment,
        xml3 = xmlread(fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile2]));
        aocsParamNode = xml3.getChildNodes.item(2).getChildNodes;
        for k=1:(aocsParamNode.getLength-1)/2,
            if ((k~=2)&(k~=4)),
                aocsParamNode.item(2*k-1).getChildNodes.item(0).setData('0');
            else
                aocsParamNode.item(2*k-1).getChildNodes.item(0).setData('false');
            end
        end
        xmlwrite(fullfile(scenarioPath,[sprintf('Segment%02d_',kSeg),SegFile2]),xml3);
    end
else
	return;
end
        
    
