function brc=CreateAuxMet(Scenario,varargin)
%  function brc=CreatAuxMet(Scenario,[storeoption,[JobOrder]])
%
%  This function open the BRC files SLS_Input_BRC_nnn.m in the InstrumentData directory of scenario Scenario and stores data
%  relative to the atmosphere and the LOS in the output structure brc. The option argument storeoption is optional. If present
%  and equal to 'store', then the output structure is saved in a mat file. By default, the mat file name is copied from the L1B
%  file name obtained from the scenario with the name Scenario. If JobOrder (optional argument) is present, the L1B file is
%  searched in this directory, ortherwise it is assumed thet JobOrder=Scenario.
%
%  A. Dabas, Meteo-France
%  07/01/2006
%

% Check input arguments
if nargin==1,
    option='nostore';
elseif nargin==2,
    option=varargin{1};
    JobOrder=Scenario;
elseif nargin==3,
    option=varargin{1};
    JobOrder=varargin{2};
else
    error('CreateAuxMet: Wrong number of input arguments.');
end

% Built path name to the InstrumentData directory of the E2S scenario
DefinePaths;
DirName=fullfile(SimPathE2S,'scenarios',Scenario,'InstrumentData','');

% Find SLS_Input_BRC_nnn.m files where AuxMetData at observation and measurement scales can be found.
d=dir(fullfile(DirName,'SLS_Input_BRC_*.m'));
if isempty(d),
    error('CreateAuxMet : Cannot find SLS_Input_BRC_nnn.m files for this scenario.');
end

% Read data in files and store them in output structure.
numFiles=length(d);

brc=struct([]);

for kFiles=1:numFiles,
    
    % Open SLS_Input_BRC_nnn.m file
    fid=fopen(fullfile(DirName,d(kFiles).name),'rt');
    if fid < 0,
        error(sprintf('CreateAuxMet : Cannot open %s.'),d(kFiles).name); 
    end

    % Read and execute line by line (faster than executing the whole file as a script)
    while(~feof(fid)),
        line=fgetl(fid);
        eval(line);
    end
    fclose(fid);
    
    % Store BRC data in instance kFiles of the output structure.
    brc(kFiles).Atmosphere=Atmosphere;
    brc(kFiles).losInfo=losInfo;

    % Monitoring progress if more than 5 segments.
    if numFiles>5,
        fprintf(1,'BRC n %3d of %3d completed\r',kFiles,numFiles);
    end
end

% If 'store' option set
if strcmp(option,'store'),
    
    % Build file name for storing the output structure.
    DirName=fullfile(SimPathL1b,'working',JobOrder,'');
    d=dir(fullfile(DirName,'AE_*_*_U_N_1B_*.DBL'));

    % Message monitoring progress
    fprintf(1,'Now storing output result.');
    
    if isempty(d),
        error('CreateAuxMet : Cannot locate L1B output file.');
    end

    if length(d) > 1,
        warning('CreateAuxMet : More than one L1B file, I take the youngest one.');
    end

    [lastdate,index]=max(datenum(vertcat(d.date)));
    [pathname,filename,ext]=fileparts(d(index(1)).name);
    elements=strread(filename,'%s','delimiter','_');
    fname=[elements{1},'_TEST_MET_U_N_2B_',elements{7},'_',elements{8},'_',elements{9},'_',elements{10},'.mat'];

    % Save BRC data in mat file bearing the name AE_TEST_MET_U_N_2B_*.mat
    save(fullfile(DirName,fname),'brc','-mat');
    
    fprintf(1,'\n');

end        


    

