function dep_out=Decompose_Struct(data,dl,dep_in);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% History
% P. Poli   12 Feb 2007   Original code
%
% DOCUMENTATION
%
% The present code scans a structure and 
% decomposes it into its sub-elements using a recursive approach.
% This can be useful to generate code automatically --
% for example code to read/write virtually *any* structure.
%
% Calling sequence:
% > depth = Decompose_Struct(xxxData,'',0);
% returns a value which is the maximum depth found in the structure.
% Depending on the options set in the configuration area,
% the program can also display the names of the elements.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Configuration area
flag_print=1;                  % if set, prints names of variables;
                               % otherwise, does not print anything
flag_print_whole_arrays=0;     % if set, loops inside arrays;
              % otherwise scans only the first element of each array
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin == 1),

  % entry point
  dep_out=Decompose_Struct(data,'',0);

else

  % recursive call
  ns=length(data);
  if (ns ~= numel(data)),
	% 2D array
	ns1=size(data,1);
	ns2=size(data,2);
	if (flag_print_whole_arrays),
		ns1_loop=ns1;
		ns2_loop=ns2;
	else
		ns1_loop=1;
		ns2_loop=1;
	end
	for i=1:ns1_loop
		for j=1:ns2_loop
			if (flag_print),
				if (flag_print_whole_arrays),
					fprintf(...
	'%s--2D element number %i out of %i, %i out of %i\n',dl,i,ns1,j,ns2)
				else
					fprintf(...
				'\b    2D array of size %i %i\n',ns1,ns2)
				end
			end
			dl2=[dl ' '];
			dep2=dep_in+1;
			dep_out=Decompose_Struct(data(i,j),dl2,dep2);
		end
	end
  else
	if (ns > 1 & ~isstr(data)),
		% 1D array
		if (flag_print_whole_arrays),
			ns_loop=ns;
		else
			ns_loop=1;
		end
		for i=1:ns_loop
			if (flag_print),
				if (flag_print_whole_arrays),
					fprintf(...
				'%s--1D element number %i out of %i\n',dl,i,ns)
				else
					fprintf(...
				'\b    1D array of size %i\n',ns)
				end
			end
			dl2=[dl ' '];
			dep2=dep_in+1;
			dep_out=Decompose_Struct(data(i),dl2,dep2);
		end
	else
		if (isstruct(data)),
			ff=fieldnames(data);
			nff=numel(ff);
			for iff=1:nff
				ff0=strvcat(ff(iff));
				if (flag_print),
					fprintf('%s...%-50s\n',dl,ff0)
				end
				dl2=[dl ' '];
				dep2=dep_in+1;
				dep_out=Decompose_Struct(data.(ff0),dl2,dep2);
			end
		else
			fprintf('\b    %s\n',class(data))
			if (isstr(data)),
				fprintf('\b (length=%i)\n',length(data))
			end
			dep_out=dep_in;
		end
	end
  end
end

end

