function [surf,prof,header]=ReadKNMIData(ficname)
%
% FUNCTION [SURF,PROF,HEADER]=ReadKNMIData(FICNAME)
% 
% Read KNMI ASCII files. FICNAME is the name of KNMI file. Surface data are stored in SURF structure while PROF is a structure
% that contains all the vertical profiles of meteorological and optical parameters of the atmosphere. HEADER is a cell of
% character arrays, each cell containing one line of the KNMI file header.
%
% A. Dabas, Meteo-France, 06 December 2005
%

% Open input file
fid=fopen(ficname);
if fid<0,
    error('ReadKNMIData : cannot open input file.');
end

% Read header and stores the lines in the cell array header
for k=1:36,
    header{k}=fgetl(fid);
end

% Read number of profiles and number of levels per profil
ligne=fgetl(fid);
tmp=sscanf(ligne,'%3d,%3d');
nprof=tmp(1);
nlev=tmp(2);

% Loop on profiles
for kprof=1:nprof,

	% Read surface parameters
	ligne=fgetl(fid);
	tmp=sscanf(ligne,'%f,%f,%f,%f,%f,%f,%d,%d,%d,%d,%d,%d,%f');
	surf(kprof).lat=tmp(1);
	surf(kprof).lon=tmp(2);
	surf(kprof).oro=tmp(3);
	surf(kprof).beta_g=tmp(4);
	surf(kprof).B=tmp(5);
	surf(kprof).albedo=tmp(6);
	surf(kprof).mask=tmp(7);
	surf(kprof).year=tmp(8);
	surf(kprof).month=tmp(9);
	surf(kprof).day=tmp(10);
	surf(kprof).hr=tmp(11);
	surf(kprof).min=tmp(12);
	surf(kprof).sec=tmp(13);

	% Read profiles
	for klev=1:nlev,
		ligne=fgetl(fid);
		tmp=sscanf(ligne,'%6d,%6d,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f');
		prof(kprof).zmin(klev)=tmp(1);
		prof(kprof).zmax(klev)=tmp(2);
		prof(kprof).P(klev)=tmp(3);
		prof(kprof).T(klev)=tmp(4);
		prof(kprof).u(klev)=tmp(5);
		prof(kprof).v(klev)=tmp(6);
		prof(kprof).w(klev)=tmp(7);
		prof(kprof).wp(klev)=tmp(8);
		prof(kprof).beta_a(klev)=tmp(9);
		prof(kprof).sigma_a(klev)=tmp(10);
		prof(kprof).beta_c(klev)=tmp(11);
		prof(kprof).sigma_c(klev)=tmp(12);
		prof(kprof).beta_m(klev)=tmp(13);
		prof(kprof).beta_pr(klev)=tmp(14);
	end  %end loop on levels
end  % end loop on profiles


% Close file
fclose(fid);
