function l1aData = ReadL1AData(JobOrderName,rank)

DefinePaths;
l1aData = dir(fullfile(SimPathL1b,'working',JobOrderName,'*_1A_*.DBL'))

if nargin==2,
    l1aDataFile = l1aData(rank).name
else
    l1aDataFile = l1aData(1).name
end

fid = fopen(fullfile(SimPathL1b,'working',JobOrderName,l1aDataFile),'r','b');

nbByteMPH = 1247;
nbByteSPH = 4386;
MPH = fread(fid, nbByteMPH, '*char');
SPH = fread(fid, nbByteSPH, '*char');

posnbDSR = findstr(SPH','NUM_DSR')+8;
nbDSR = str2num(SPH(posnbDSR(1):posnbDSR(1)+11)');
Nmax = 67;

l1aData=[];
l1aData.MPH=MPH;
l1aData.SPH=SPH;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Geolocation ADSR %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr = 1:nbDSR,
    l1aData.GeoDSR(idsr).startObsTime = fread(fid, 3,'*int32');
    l1aData.GeoDSR(idsr).ObsAOCS.obsCentroidTime = fread(fid, 3,'*int32');
    l1aData.GeoDSR(idsr).ObsAOCS.xPosition = fread(fid, 1,'*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.yPosition = fread(fid, 1,'*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.zPosition = fread(fid, 1,'*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.xVelocity = fread(fid, 1, '*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.yvelocity = fread(fid, 1, '*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.zvelocity = fread(fid, 1, '*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.Rollangle = fread(fid, 1, '*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.PitchAngle = fread(fid, 1, '*float64');
    l1aData.GeoDSR(idsr).ObsAOCS.YawAngle = fread(fid, 1, '*float64');
    spare1 = fread(fid, 64, '*ubit1');

    %List of Measurement AOCS
    for imeas = 1:Nmax,
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).MeasCentroidTime = fread(fid, 3,'*int32');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).xPositionMeas = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).yPositionMeas = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).zPositionMeas = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).xVelocityMeas = fread(fid, 1, '*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).yvelocityMeas = fread(fid, 1, '*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).zvelocityMeas = fread(fid, 1, '*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).RollangleMeas = fread(fid, 1, '*float64');
	l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).pitchAngleMeas = fread(fid, 1, '*float64');
        l1aData.GeoDSR(idsr).ListMeasAOCS(imeas).YawAngleMeas = fread(fid, 1, '*float64');
        spare2 = fread(fid, 64, '*ubit1');
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % OBSERVATION GEOLOCATION
    %	Observation Mie Geolocation
    for ibin = 1:25,  
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).longHeightBin = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).latHeightBin = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).altHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).topoAzHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).topoElevHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).targetToSunVisibilityFlag = fread(fid, 1,'*int16');
        l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).satelliteRangeHeightBin = fread(fid, 1, '*float64');
    end
    % Observation Rayleigh Geolocation
    for ibin = 1:25,
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LatHeightBin = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).AltHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TopoAzHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TopoElevHeightBin = fread(fid, 1,'*float64');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TargetToSunVisibilityFlag = fread(fid, 1,'*int16');
        l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).SatelliteRangeHeightBin = fread(fid, 1, '*float64');
    end
    % Geolocation of DEM intersection
    l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LatDemIntersection = fread(fid, 1,'*int32');
    l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LongDemIntersection = fread(fid, 1,'*int32');
    l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.AltDemIntersection = fread(fid, 1,'*float64');

    % Los Velocity
    l1aData.GeoDSR(idsr).ObsGeolocation.AOCSLosVelocity = fread(fid, 1,'*float64');
    %spare3 = fread(fid, 64, '*ubit1');

    % Geoid separation
    l1aData.GeoDSR(idsr).ObsGeolocation.GeoidSeparation = fread(fid, 1, '*float64');
    spare3 = fread(fid, 64, '*ubit1');

    % LIST OF MEASUREMENT GEOLOCATION
    for imeas = 1:Nmax,
        % Measurement Mie Geolocation
        for ibin = 1:25,
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LatHeightBinMie = fread(fid, 1,'*int32');
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).AltHeightBinMie = fread(fid, 1,'*float64');
        end
        % Measurement Rayleigh Geolocation
        for ibin = 1:25,
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LatHeightBinMie = fread(fid, 1,'*int32');
            l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).AltHeightBinMie = fread(fid, 1,'*float64');
        end
        % Geolocation of DEM intersection
        l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LatDemIntersection = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LongDemIntersection = fread(fid, 1,'*int32');
        l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.AltDemIntersection = fread(fid, 1,'*float64');

        % AOCS Los Velocity
        l1aData.GeoDSR(idsr).MeasGeolocation(imeas).AOCSLosVelocity = fread(fid, 1,'*float64');
        spare4 = fread(fid, 64, '*ubit1');
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%     % Observation Geolocation
%     for ibin = 1:25,   % Mie Geolocation
%         l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).LatHeightBin = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).AltHeightBin = fread(fid, 1,'*float64');
%     end
%     for ibin = 1:25,   % Ray Geolocation
%         l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LatHeightBin = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).AltHeightBin = fread(fid, 1,'*float64');
%     end
%     % Geolocation of DEM intersection
%     l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LatDemIntersection = fread(fid, 1,'*int32');
%     l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LongDemIntersection = fread(fid, 1,'*int32');
%     l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.AltDemIntersection = fread(fid, 1,'*float64');
%     
%     l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.TopoAzDemIntersec = fread(fid, 1,'*float64');
%     l1aData.GeoDSR(idsr).ObsGeolocation.DemGeolocation.TopoElevDemIntersec = fread(fid, 1,'*float64');
%     % AOCS Los Velocity
%     l1aData.GeoDSR(idsr).ObsGeolocation.AOCSLosVelocity = fread(fid, 1,'*float64');
%     spare3 = fread(fid, 64, '*ubit1');
%     
%     %   List of Measurement Geolocations
%     for imeas = 1:Nmax,
%         for ibin = 1:25,   % Mie Geolocation
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LatHeightBinMie = fread(fid, 1,'*int32');
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).AltHeightBinMie = fread(fid, 1,'*float64');
%         end
%         for ibin = 1:25,   % Ray Geolocation
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LongHeightBin = fread(fid, 1,'*int32');
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LatHeightBinMie = fread(fid, 1,'*int32');
%             l1aData.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).AltHeightBinMie = fread(fid, 1,'*float64');
%         end
%         % Geolocation of DEM intersection
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LatDemIntersection = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LongDemIntersection = fread(fid, 1,'*int32');
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.AltDemIntersection = fread(fid, 1,'*float64');
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.TopoAzDemIntersec = fread(fid, 1,'*float64');
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.TopoElevDemIntersec = fread(fid, 1,'*float64');
%         % AOCS Los Velocity
%         l1aData.GeoDSR(idsr).MeasGeolocation(imeas).AOCSLosVelocity = fread(fid, 1,'*float64');
%         spare4 = fread(fid, 64, '*ubit1');
%     end
end % for idsr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Data Quality ADS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
   l1aData.DqDSR(idsr).startObsTime = fread(fid, 3,'*int32');
   l1aData.DqDSR(idsr).ObsValidityIndicator = fread(fid,1,'uint8');
   for imeas = 1:67,
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).measDataPresent = fread(fid,1,'uint8');
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).mieMeasSPValid = fread(fid,1,'uint8');
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).rayMeasSPValid = fread(fid,1,'uint8');
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).measLaserFreqLocked = fread(fid,1,'uint8');
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).AttitudeOnTarget = fread(fid,1,'uint8');
       l1aData.DqDSR(idsr).ListMeasValIndic(imeas).VelAttError = fread(fid,1,'float64');
       spare5 = fread(fid,24,'*ubit1');
   end
   l1aData.DqDSR(idsr).RefPulseValIndic.mieRefPulsesPresent = fread(fid,1,'uint8');
   l1aData.DqDSR(idsr).RefPulseValIndic.rayRefPulsesPresent = fread(fid,1,'uint8');
   l1aData.DqDSR(idsr).RefPulseValIndic.mieRefPulsesSPValid = fread(fid,1,'uint8');
   l1aData.DqDSR(idsr).RefPulseValIndic.rayRefPulsesSPValid = fread(fid,1,'uint8');
   l1aData.DqDSR(idsr).RefPulseValIndic.ListLaserFreqLocked = fread(fid,1003,'uint8');
   spare6 = fread(fid,17*8,'ubit1');
   
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Reference Pulse ADS %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
    l1aData.RpDSR(idsr).startObsTime = fread(fid, 3,'*int32');
    l1aData.RpDSR(idsr).numRefPulses = fread(fid,1,'*uint32');
    %spare7 = fread(fid,32,'*ubit1')
    for ishot=1:1003,
        l1aData.RpDSR(idsr).mieRefPulse(ishot).col = fread(fid,20,'uint16');
    end
    %spare7 = fread(fid,32,'*ubit1')
    for ishot=1:1003,
        l1aData.RpDSR(idsr).rayRefPulse(ishot).col = fread(fid,20,'uint16');
    end
    spare7 = fread(fid,32,'*ubit1');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Housekeeping Data ADS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
    l1aData.HkDSR(idsr).startObsTime = fread(fid,3,'*int32');
    l1aData.HkDSR(idsr).instMode = fread(fid,1,'*uchar');
    l1aData.HkDSR(idsr).P = fread(fid,1,'*uchar');
    l1aData.HkDSR(idsr).N = fread(fid,1,'*uchar');
    spare8 = fread(fid,13*8,'*ubit1');
    % Laser Pulse Attributes
    l1aData.HkDSR(idsr).LaserPulseAttributes.avgLaserFreqOffset = fread(fid, 1, '*float64');
    l1aData.HkDSR(idsr).LaserPulseAttributes.avgUVEnergy = fread(fid, 1, '*float64');
    l1aData.HkDSR(idsr).LaserPulseAttributes.laserFreqStdDev = fread(fid, 1, '*float64');
    l1aData.HkDSR(idsr).LaserPulseAttributes.uvEnergyStdDev = fread(fid, 1, '*float64');
    for ishot= 1:1003,
        l1aData.HkDSR(idsr).LaserPulseAttributes.ListPulseAttributes(ishot).laserFreqOffset = ...
            fread(fid,1,'*float64');
        l1aData.HkDSR(idsr).LaserPulseAttributes.ListPulseAttributes(ishot).uvEnergyPerPulse = ...
            fread(fid,1,'*float64');
    end
    % Pulse Time Delays
    l1aData.HkDSR(idsr).pulseTimeDelays.dt1 = fread(fid,1,'*uint32');  
    l1aData.HkDSR(idsr).pulseTimeDelays.dt2 = fread(fid,1,'*uint32'); 
    l1aData.HkDSR(idsr).pulseTimeDelays.dt3Fixed = fread(fid,1,'*uint32'); 
    l1aData.HkDSR(idsr).pulseTimeDelays.dt4 = fread(fid,1,'*uint32') ;
    l1aData.HkDSR(idsr).pulseTimeDelays.dt5 = fread(fid,1,'*uint32') ;
    l1aData.HkDSR(idsr).pulseTimeDelays.tdRayMie = fread(fid,1,'*uint32') ;
    l1aData.HkDSR(idsr).pulseTimeDelays.Listdt3Variable = fread(fid,67,'*uint32');
    
    % Mie Time Delays
    l1aData.HkDSR(idsr).mieTimeDelays.ListbinLayerIntegrationTimes = fread(fid,24,'*int32');
    l1aData.HkDSR(idsr).mieTimeDelays.bkgIntegrationTime = fread(fid,1,'*int32');
    
    % Ray Time Delays
    l1aData.HkDSR(idsr).rayTimeDelays.ListbinLayerIntegrationTimes = fread(fid,24,'*int32');
    l1aData.HkDSR(idsr).rayTimeDelays.bkgIntegrationTime = fread(fid,1,'*int32');
    
    l1aData.HkDSR(idsr).heightRayBinOne = fread(fid,1,'*int32');
    l1aData.HkDSR(idsr).avgMieAccdDieTemp = fread(fid,1,'*float64');
    l1aData.HkDSR(idsr).avgRayAccdDieTemp = fread(fid,1,'*float64');
    spare9 = fread(fid,16*8,'*ubit1'); 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Mie Channel Measurement Data ADS %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for idsr=1:nbDSR,
    l1aData.MieMDDSR(idsr).startObsTime = fread(fid,3,'*int32');
    l1aData.MieMDDSR(idsr).numMeas = fread(fid,1,'uint16');
    l1aData.MieMDDSR(idsr).numHeightBin = fread(fid,1,'uint16');
    l1aData.MieMDDSR(idsr).numAccdColumns = fread(fid,1,'uint16');
    for imeas=1:67,
        for ibin=1:25,
            l1aData.MieMDDSR(idsr).mieMeasData(imeas).mieBinMeasData(ibin).col = fread(fid,20,'uint16');
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Ray Channel Measurement Data ADS %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for idsr=1:nbDSR,
    l1aData.RayMDDSR(idsr).startObsTime = fread(fid,3,'*int32');
    l1aData.RayMDDSR(idsr).numMeas = fread(fid,1,'uint16');
    l1aData.RayMDDSR(idsr).numHeightBin = fread(fid,1,'uint16');
    l1aData.RayMDDSR(idsr).numAccdColumns = fread(fid,1,'uint16');
    for imeas=1:67,
        for ibin=1:25,
            l1aData.RayMDDSR(idsr).rayMeasData(imeas).rayBinMeasData(ibin).col = fread(fid,20,'uint16');
        end
    end
end

fclose(fid);


