function WriteL1AData(struct,filename)

% Function WriteL1AData (STRUCT, filename)
%
% Open a L1A file from the Matlab structure and save it with the name
% filename. The structure is the result of the ReadL1AData function.
% 
% Input arguments : 
%
% - Struct : is a matlab structure created by the ReadL1AData program. 
%
% - filename : is the name of the created file. The filename must contain the absolute path of the file.
%
% Revision 1.1  20 Dec 2005     M-L Denneulin
% Initial Revision
% 
% Revision 1.2  30 Jan 2006     M-L Denneulin
% Modification of the GeoDSR format to be compliant with the DLR version 1.2 of the L1B processor
%
%



DefinePaths;
filename
fid = fopen(filename,'w+','b');

fwrite(fid,struct.MPH,'char');
fwrite(fid,struct.SPH,'char');

nbDSR = length(struct.GeoDSR);
Nmax=67;
for idsr = 1:nbDSR,
    fwrite(fid,struct.GeoDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.obsCentroidTime,'int32');


    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.xPosition,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.yPosition,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.zPosition,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.xVelocity,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.yvelocity,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.zvelocity,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.Rollangle,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.PitchAngle,'float64');
    fwrite(fid,struct.GeoDSR(idsr).ObsAOCS.YawAngle,'float64');
    fwrite(fid,zeros(1,64),'ubit1');

    %List of Measurement AOCS
    for imeas = 1:Nmax,
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).MeasCentroidTime,'int32');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).xPositionMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).yPositionMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).zPositionMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).xVelocityMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).yvelocityMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).zvelocityMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).RollangleMeas,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ListMeasAOCS(imeas).YawAngleMeas,'float64');
        fwrite(fid,zeros(1,128),'ubit1');
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % OBSERVATION GEOLOCATION
    %	Observation Mie Geolocation
    for ibin = 1:25,  
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).longHeightBin,'int32');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).latHeightBin,'int32');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).altHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).topoAzHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).topoElevHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).targetToSunVisibilityFlag,'int16');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).satelliteRangeHeightBin, 'float64');
    end
    % Observation Rayleigh Geolocation
    for ibin = 1:25,
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LongHeightBin,'int32');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LatHeightBin,'int32');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).AltHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TopoAzHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TopoElevHeightBin,'float64');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).TargetToSunVisibilityFlag,'int16');
        fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).SatelliteRangeHeightBin , 'float64');
    end
    % Geolocation of DEM intersection
    fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LatDemIntersection,'int32');
    fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LongDemIntersection,'int32');
    fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.AltDemIntersection,'float64');

    % Los Velocity
    fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.AOCSLosVelocity,'float64');
    %spare3 = fread(fid, 64, '*ubit1');

    % Geoid separation
    fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.GeoidSeparation, 'float64');
    fwrite(fid, zeros(1,64), 'ubit1');

    % LIST OF MEASUREMENT GEOLOCATION
    for imeas = 1:Nmax,
        % Measurement Mie Geolocation
        for ibin = 1:25,
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LongHeightBin,'int32');
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LatHeightBinMie,'int32');
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).AltHeightBinMie,'float64');
        end
        % Measurement Rayleigh Geolocation
        for ibin = 1:25,
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LongHeightBin,'int32');
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LatHeightBinMie,'int32');
            fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).AltHeightBinMie,'float64');
        end
        % Geolocation of DEM intersection
        fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LatDemIntersection,'int32');
        fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LongDemIntersection,'int32');
        fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.AltDemIntersection,'float64');

        % AOCS Los Velocity
        fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).AOCSLosVelocity,'float64');
        fwrite(fid,zeros(1,64),'ubit1');
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     % Observation Geolocation
%     for ibin = 1:25,   % Mie Geolocation
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).LongHeightBin,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).LatHeightBin,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.MieGeolocation(ibin).AltHeightBin,'float64');
%     end
%     for ibin = 1:25,   % Ray Geolocation
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LongHeightBin,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).LatHeightBin,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.RayGeolocation(ibin).AltHeightBin,'float64');
%     end
%     % Geolocation of DEM intersection
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LatDemIntersection,'int32');
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.LongDemIntersection,'int32');
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.AltDemIntersection,'float64');
% 
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.TopoAzDemIntersec,'float64');
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.DemGeolocation.TopoElevDemIntersec,'float64');
%     % AOCS Los Velocity
%     fwrite(fid,struct.GeoDSR(idsr).ObsGeolocation.AOCSLosVelocity,'float64');
%     fwrite(fid,zeros(1,64),'ubit1');
% 
%     %   List of Measurement Geolocations
%     for imeas = 1:Nmax,
%         for ibin = 1:25,   % Mie Geolocation
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LongHeightBin,'int32');
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).LatHeightBinMie,'int32');
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).MieGeolocation(ibin).AltHeightBinMie,'float64');
%         end
%         for ibin = 1:25,   % Ray Geolocation
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LongHeightBin,'int32');
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).LatHeightBinMie,'int32');
%             fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).RayGeolocation(ibin).AltHeightBinMie,'float64');
%         end
%         % Geolocation of DEM intersection
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LatDemIntersection,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.LongDemIntersection,'int32');
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.AltDemIntersection,'float64');
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.TopoAzDemIntersec,'float64');
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).DemGeolocation.TopoElevDemIntersec,'float64');
%         % AOCS Los Velocity
%         fwrite(fid,struct.GeoDSR(idsr).MeasGeolocation(imeas).AOCSLosVelocity,'float64');
%         fwrite(fid,zeros(1,64),'ubit1');
%     end

end % for idsr

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Data Quality ADS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
    fwrite(fid,struct.DqDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.DqDSR(idsr).ObsValidityIndicator,'uint8');
    for imeas = 1:67,
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).measDataPresent,'uint8');
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).mieMeasSPValid,'uint8');
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).rayMeasSPValid,'uint8');
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).measLaserFreqLocked,'uint8');
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).AttitudeOnTarget,'uint8');
        fwrite(fid,struct.DqDSR(idsr).ListMeasValIndic(imeas).VelAttError,'float64');
        fwrite(fid,zeros(1,24),'ubit1');
    end
    fwrite(fid,struct.DqDSR(idsr).RefPulseValIndic.mieRefPulsesPresent,'uint8');
    fwrite(fid,struct.DqDSR(idsr).RefPulseValIndic.rayRefPulsesPresent,'uint8');
    fwrite(fid,struct.DqDSR(idsr).RefPulseValIndic.mieRefPulsesSPValid,'uint8');
    fwrite(fid,struct.DqDSR(idsr).RefPulseValIndic.rayRefPulsesSPValid,'uint8');
    fwrite(fid,struct.DqDSR(idsr).RefPulseValIndic.ListLaserFreqLocked,'uint8');
    fwrite(fid,zeros(1,17*8),'ubit1');

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Reference Pulse ADS %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
    fwrite(fid,struct.RpDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.RpDSR(idsr).numRefPulses,'uint32');
    fwrite(fid,zeros(1,32),'ubit1');
    for ishot=1:1003,
        fwrite(fid,struct.RpDSR(idsr).mieRefPulse(ishot).col,'uint16');
    end
    for ishot=1:1003,
        fwrite(fid,struct.RpDSR(idsr).rayRefPulse(ishot).col,'uint16');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Housekeeping Data ADS %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for idsr=1:nbDSR,
    fwrite(fid,struct.HkDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.HkDSR(idsr).instMode,'uchar');
    fwrite(fid,struct.HkDSR(idsr).P,'uchar');
    fwrite(fid,struct.HkDSR(idsr).N,'uchar');
    fwrite(fid,zeros(1,13*8),'ubit1');
    % Laser Pulse Attributes
    fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.avgLaserFreqOffset,'float64');
    fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.avgUVEnergy,'float64');
    fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.laserFreqStdDev,'float64');
    fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.uvEnergyStdDev,'float64');
    for ishot= 1:1003,
        fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.ListPulseAttributes(ishot).laserFreqOffset,'float64');
        fwrite(fid,struct.HkDSR(idsr).LaserPulseAttributes.ListPulseAttributes(ishot).uvEnergyPerPulse,'float64');
    end
    % Pulse Time Delays
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.dt1,'uint32');
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.dt2,'uint32');
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.dt3Fixed,'uint32');
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.dt4,'uint32') ;
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.dt5,'uint32') ;
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.tdRayMie,'uint32') ;
    fwrite(fid,struct.HkDSR(idsr).pulseTimeDelays.Listdt3Variable,'uint32');

    % Mie Time Delays
    fwrite(fid,struct.HkDSR(idsr).mieTimeDelays.ListbinLayerIntegrationTimes,'int32');
    fwrite(fid,struct.HkDSR(idsr).mieTimeDelays.bkgIntegrationTime,'int32');

    % Ray Time Delays
    fwrite(fid,struct.HkDSR(idsr).rayTimeDelays.ListbinLayerIntegrationTimes,'int32');
    fwrite(fid,struct.HkDSR(idsr).rayTimeDelays.bkgIntegrationTime,'int32');

    fwrite(fid,struct.HkDSR(idsr).heightRayBinOne,'int32');
    fwrite(fid,struct.HkDSR(idsr).avgMieAccdDieTemp,'float64');
    fwrite(fid,struct.HkDSR(idsr).avgRayAccdDieTemp,'float64');
    fwrite(fid,zeros(1,16*8),'ubit1');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Mie Channel Measurement Data ADS %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for idsr=1:nbDSR,
    fwrite(fid,struct.MieMDDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.MieMDDSR(idsr).numMeas,'uint16');
    fwrite(fid,struct.MieMDDSR(idsr).numHeightBin,'uint16');
    fwrite(fid,struct.MieMDDSR(idsr).numAccdColumns,'uint16');
    for imeas=1:67,
        for ibin=1:25,
            fwrite(fid,struct.MieMDDSR(idsr).mieMeasData(imeas).mieBinMeasData(ibin).col,'uint16');
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Ray Channel Measurement Data ADS %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for idsr=1:nbDSR,
    fwrite(fid,struct.RayMDDSR(idsr).startObsTime,'int32');
    fwrite(fid,struct.RayMDDSR(idsr).numMeas,'uint16');
    fwrite(fid,struct.RayMDDSR(idsr).numHeightBin,'uint16');
    fwrite(fid,struct.RayMDDSR(idsr).numAccdColumns,'uint16');
    for imeas=1:67,
        for ibin=1:25,
            fwrite(fid,struct.RayMDDSR(idsr).rayMeasData(imeas).rayBinMeasData(ibin).col,'uint16');
        end
    end
end

fclose(fid);



