function varargout = gui_102(varargin)

% GENERIC STRUCTURE VISUALIZER AND EDITOR
%
% Paul Poli, 14 Feb 2007
%
% Usage (from the Matlab workspace):
%
% > mydata=ReadBinaryData(myL1Bfile,'all');
% > myrbcdata=ReadBinaryData(myRBCfile,'all');
% (etc...)
% > gui_102
%
% This program allows to:
% - browse through the contents of a structure
% - make a selection of the elements (up to 3 dimensions)
% - plot that selection
% - export that selection as a variable to the workspace
% - edit that variable
% - replace that selection by data imported from the workspace
%
% The pre-requisites are that the data must have been loaded beforehand.
% For ADM products, use ReadBinaryData.m
%
% Technical details:
%
% The following variables are used in the workspace:
% var_name [default: 'xval','xch'] : names of the variables where the program saves
%              - the information extracted from the structures 
%              - the substructure name
% xval [unless otherwise specified in the GUI, and if the save option is used]
%               contains the saved variable (may be numeric or character)
%               examples: [0 1 2 3]
%                         'Kiruna'
%                         [[0 1 2 3];[4 5 6 7]]
% xch  [unless otherwise specified in the GUI, and if the save option is used]
%               contains the name of the structure subset (character)
%               example: 'struct1.substruct4(2:10).subsubstruct2'
% flag_display_value : 1 or 0. If 1, displays the variables
%               in the workspace before plotting or saving
%
% LB Application M-file for gui_102.fig
%   LB, by itself, creates a new LB or raises the existing
%   singleton*.
%
%   H = LB returns the handle to a new LB or the handle to
%   the existing singleton*.
%
%   LB('CALLBACK',hObject,eventData,handles,...) calls the local
%   function named CALLBACK in LB.M with the given input arguments.
%
%   LB('Property','Value',...) creates a new LB or raises the
%   existing singleton*.  Starting from the left, property value pairs are
%   applied to the GUI before gui_102_OpeningFunction gets called.  An
%   unrecognized property name or invalid value makes property application
%   stop.  All inputs are passed to gui_102_OpeningFcn via varargin.
%
%   *See GUI Options - GUI allows only one instance to run (singleton).
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Copyright 2000-2002 The MathWorks, Inc.

% Edit the above text to modify the response to help gui_102

% Last Modified by GUIDE v2.5 14-Feb-2008 15:54:22

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',          mfilename, ...
                   'gui_Singleton',     gui_Singleton, ...
                   'gui_OpeningFcn',    @gui_102_OpeningFcn, ...
                   'gui_OutputFcn',     @gui_102_OutputFcn, ...
                   'gui_LayoutFcn',     [], ...
                   'gui_Callback',      []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    varargout{1:nargout} = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before gui_102 is made visible.
function gui_102_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to gui_102 (see VARARGIN)
global guigsv_struct_label
global guigsv_values
guigsv_struct_label='';
guigsv_values=[];
clear('guigsv_struct_label')
clear('guigsv_values')
% Choose default command line output for gui_102
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% Populate the listbox
update_listbox1(handles.listbox1)
set(handles.listbox1,'Value',[])
evalin('base','var_name{1}=''xval'';');
evalin('base','var_name{2}=''xch'';');
evalin('base','flag_dislay_values=1;')

% UIWAIT makes gui_102 wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = gui_102_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


function varargout = update_button_Callback(h, eventdata, handles, varargin)
% hObject    handle to update_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

update_listbox1(handles.listbox1)
set(handles.listbox1,'Value',[])

function [var1,var2] = get_var_names(handles)
% Returns the names of the two variables to plot
list_entries = get(handles.listbox1,'String');
index_selected = get(handles.listbox1,'Value');
if length(index_selected) ~= 2
    errordlg('You must select two variables','Incorrect Selection','modal')
else
    var1 = list_entries{index_selected(1)};
    var2 = list_entries{index_selected(2)};
end 

function varargout = plot_button_Callback(h, eventdata, handles, varargin)
% hObject    handle to plot_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[x,y] = get_var_names(handles);
figure(gcf)
evalin('base',['plot(',x,',',y,')'],'errordlg(lasterr,''Error generating plots'',''modal'')')

function varargout = semilogx_button_Callback(h, eventdata, handles, varargin)
% hObject    handle to semilogx_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[x,y] = get_var_names(handles);
figure(gcf)
evalin('base',['semilogx(',x,',',y,')'],'errordlg(lasterr,''Error generating plots'',''modal'')')

function varargout = semilogy_button_Callback(h, eventdata, handles, varargin)
% hObject    handle to semilogy_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[x,y] = get_var_names(handles);
figure(gcf)
evalin('base',['semilogy(',x,',',y,')'],'errordlg(lasterr,''Error generating plots'',''modal'')')



% --- Executes during object creation, after setting all properties.
function listbox1_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox2_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox3_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox4_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox5_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox6_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox7_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox8_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox9_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox10_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox11_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox12_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox13_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function listbox14_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function edit1_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function edit2_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function uipanel1_CreateFcn(hObject, eventdata, handles)
generic_CreateFcn(hObject, eventdata, handles)

function generic_CreateFcn(hObject, eventdata, handles)
usewhitebg = ispc;
if usewhitebg
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- Executes on selection change in listbox1.
function listbox1_Callback(hObject, eventdata, handles)
% hObject    handle to listbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox1 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox1
generic_listbox_Callback(hObject, eventdata, handles,1,14)
%Complete call:
%list_entries=get(hObject,'String');
%if (~isempty(list_entries))
%index_selected = get(hObject,'Value');
%var = list_entries{index_selected(1)};
%update_next_listbox(handles.listbox1,handles.listbox2,var,index_selected)
%set(handles.listbox2,'Value',[])
%clear_next_listboxes([3:14],handles)
%set_char_var('guigsv_struct_label',get_second_half_TooltipString(handles.listbox2))
%end

function listbox2_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,2,14)

function listbox3_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,3,14)

function listbox4_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,4,14)

function listbox5_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,5,14)

function listbox6_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,6,14)

function listbox7_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,7,14)

function listbox8_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,8,14)

function listbox9_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,9,14)

function listbox10_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,10,14)

function listbox11_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,11,14)

function listbox12_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,12,14)

function listbox13_Callback(hObject, eventdata, handles)
generic_listbox_Callback(hObject, eventdata, handles,13,14)

function listbox14_Callback(hObject, eventdata, handles)

function generic_listbox_Callback(hObject, eventdata, handles,io,iomax)
global guigsv_struct_label
if (io~=iomax),
	list_entries=get(hObject,'String');
	if (~isempty(list_entries))
		index_selected = get(hObject,'Value');
		var = list_entries{index_selected(1)};
		eval(sprintf('update_next_listbox(handles.listbox%i,handles.listbox%i,var,index_selected)',io,io+1))
		eval(sprintf('set(handles.listbox%i,''Value'',[])',io+1))
		eval(sprintf('clear_next_listboxes([%i:%i],handles)',io+2,iomax))
		eval(sprintf('set_char_var(''guigsv_struct_label'',get_second_half_TooltipString(handles.listbox%i))',io+1))
	end
end

function clear_next_listboxes(nos,handles)
if (~isempty(nos))
	nnos=length(nos);
	for i=1:nnos,
		eval(sprintf('set(handles.listbox%i,''String'',[])',nos(i)))
	end
end

function update_listbox1(handles_listbox)
% hObject    handle to update (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox1 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox1

% Updates the listbox to match the current workspace
vars = evalin('base','who');
var2=[];
j=0;
for is=1:size(vars)
	if (evalin('base',sprintf('isstruct(%s)',vars{is}))),
		j=j+1;
		var2{j}=vars{is};
	end
end
set(handles_listbox,'String',var2)
set_first_half_TooltipString(handles_listbox,'')

function update_next_listbox(handles_prev_listbox,handles_listbox,var,opt_mask)
% hObject    handle to update (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox1 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox1

% Updates the listbox to match the current workspace
%vars = evalin('base','who');
% if it is an array or a structure, display next level
var_2nd=var;
if (nargin > 3),
	if (length(opt_mask) > 1),
%%		mask=sprintf('(%i',opt_mask(1));
%%		for im=2:length(opt_mask)
%%			mask=[mask sprintf(',%i',opt_mask(im))];
%%		end
%%		mask=[mask ')'];
		mask=sprintf('(%i:%i)',min(opt_mask),max(opt_mask));
		var_2nd=strvcat(mask);
	end
end
parentvar_2nd=get_second_half_TooltipString(handles_prev_listbox);
var_local_2nd=sprintf('%s%s',parentvar_2nd,var_2nd);
parentvar=get_first_half_TooltipString(handles_prev_listbox);
var_local=sprintf('%s%s',parentvar,var);
flag_struct=evalin('base',sprintf('isstruct(%s)',var_local));
length_struct=evalin('base',sprintf('length(%s)',var_local));
numel_struct=evalin('base',sprintf('numel(%s)',var_local));
if (length_struct > 1),
	vars={};
	if (length_struct ~= numel_struct)
		% 2D array
		ns1=evalin('base',sprintf('size(%s,1)',var_local));
		ns2=evalin('base',sprintf('size(%s,2)',var_local));
		is=0;
		for is1=1:ns1
			for is2=1:ns2
				is=is+1;
				vars{is}=sprintf('(%i,%i)',is1,is2);
			end
		end
	else
		% 1D array
		for is=1:length_struct
			vars{is}=sprintf('(%i)',is);
		end
	end
	set(handles_listbox,'String',vars)
	set_first_half_TooltipString(handles_listbox,sprintf('%s',var_local))
	set_second_half_TooltipString(handles_listbox,sprintf('%s',var_local_2nd))
else
	if (flag_struct),
		vars=[];
		vars=evalin('base',sprintf('fieldnames(%s)',var_local));
		set(handles_listbox,'String',vars)
		set_first_half_TooltipString(handles_listbox,sprintf('%s.',var_local))
		set_second_half_TooltipString(handles_listbox,sprintf('%s.',var_local_2nd))
	else
		% neither an array, nor a structure... nothing to do
		set(handles_listbox,'String',[])
		set_first_half_TooltipString(handles_listbox,sprintf('%s',var_local))
		set_second_half_TooltipString(handles_listbox,sprintf('%s',var_local_2nd))
	end
end

function chain = get_first_half_TooltipString(handles)
ch=get(handles,'TooltipString');
ich=strfind(ch,'|');
nch=length(ch);
if (isempty(ich)),
	chain='';
else
	if (ich==1),
		chain='';
	else
		chain=ch(1:ich-1);
	end
end

function chain = get_second_half_TooltipString(handles)
ch=get(handles,'TooltipString');
ich=strfind(ch,'|');
nch=length(ch);
if (isempty(ich)),
	chain='';
else
	if (ich==nch),
		chain='';
	else
		chain=ch(ich+1:nch);
	end
end

function set_first_half_TooltipString(handles,chain1)
chain2 = get_second_half_TooltipString(handles);
chain = [chain1 '|' chain2];
set(handles,'TooltipString',chain);

function set_second_half_TooltipString(handles,chain2)
chain1 = get_first_half_TooltipString(handles);
chain = [chain1 '|' chain2];
set(handles,'TooltipString',chain);

function edit1_Callback(hObject, eventdata, handles)
% hObject    handle to edit1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit1 as text
%        str2double(get(hObject,'String')) returns contents of edit1 as a double
edit1=['''' get(hObject,'String') ''''];
evalin('base',sprintf('var_name{1}=%s;',edit1));

function edit2_Callback(hObject, eventdata, handles)
edit2=['''' get(hObject,'String') ''''];
evalin('base',sprintf('var_name{2}=%s;',edit2));

function set_char_var (variable_name,contents)
eval(sprintf('global %s',variable_name));
eval(sprintf('%s=''%s'';',variable_name,contents));

function set_var (variable_name,contents)
evalin('base',sprintf('global %s',contents));
%str_contents=num2str(contents)
evalin('base',sprintf('%s=%s;',variable_name,contents));
evalin('base',sprintf('clear %s',contents));

function flag_return = get_var (variable_name,contents)
flag_return=0;
evalin('base',sprintf('global %s',contents));
if (evalin('base',sprintf('isempty(%s)',variable_name)))
	errordlg(sprintf('Input data are not available in the workspace under the name:\n %s',variable_name),'Incorrect Input','modal')
	flag_return=1;
else
	evalin('base',sprintf('%s=%s;',contents,variable_name));
end

% --- Executes on button press in pushbutton.
function pushbutton1_Callback(hObject, eventdata, handles)
[val,tit]=Get_or_Set_Data('Get',1);
if (~isempty(val))
if (numel(val) == 1)
	errordlg('You have selected only one value.','Incorrect Selection','modal')
else
	plot(val)
	xlabel('Index')
	title(sprintf('%s',tit),'Interpreter','none')
end
end

function pushbutton2_Callback(hObject, eventdata, handles)
[val,tit]=Get_or_Set_Data('Get',1);
if (~isempty(val))
if (numel(val) == 1)
	errordlg('You have selected only one value.','Incorrect Selection','modal')
else
	semilogy(val)
	xlabel('Index')
	title(sprintf('%s',tit),'Interpreter','none')
end
end

function pushbutton3_Callback(hObject, eventdata, handles)
[val,tit]=Get_or_Set_Data('Get',1);
if (~isempty(val))
if (numel(val) == length(val))
	errordlg('You have selected only one dimension.','Incorrect Selection','modal')
else
	tabplot=[[val,val(:,end)];[val(end,:),val(end,end)]];
	surface(tabplot)
	title(sprintf('%s',tit),'Interpreter','none')
end
end

function pushbutton4_Callback(hObject, eventdata, handles)
[val,tit]=Get_or_Set_Data('Get',1);
if (~isempty(val))
if (numel(val) == length(val))
	errordlg('You have selected only one dimension.','Incorrect Selection','modal')
else
	tabplot=[[val,val(:,end)];[val(end,:),val(end,end)]];
	h=pcolor(tabplot);
	if (length(tabplot) > 100) % do not display edges if one dimension exceeds 100
		set(h,'EdgeAlpha',0)
	end
	title(sprintf('%s',tit),'Interpreter','none')
end
end

function exportbutton_Callback(hObject, eventdata, handles)
global guigsv_struct_label
if (isempty(guigsv_struct_label))
	errordlg('Your selection is empty.','Incorrect Selection','modal')
else
	[output_val,output_tit]=Get_or_Set_Data('Get');
	if (~isempty(output_val))
		set_var(evalin('base','var_name{1}'),'guigsv_values');
		set_var(evalin('base','var_name{2}'),'guigsv_struct_label');
	end
end

function importbutton_Callback(hObject, eventdata, handles)
global guigsv_struct_label
if (isempty(guigsv_struct_label))
	errordlg('Your selection is empty.','Incorrect Selection','modal')
else
	flag_return=get_var(evalin('base','var_name{1}'),'guigsv_values');
	if (flag_return == 0)
		if (flag_return == 0)
			[output_val,output_tit]=Get_or_Set_Data('Set');
		end
		evalin('base','clear guigsv_values');
	end
end

function modifybutton_Callback(hObject, eventdata, handles)
var_to_edit=evalin('base','var_name{1}');
evalin('base',sprintf('openvar(''%s'')',var_to_edit));

function [output_val,output_tit] = Get_or_Set_Data(direction,flag_numeric_only)
% This function decomposes a variable definition into arrays:
% for example: struct1.substruct4(5:10).subsubstruct2.subsubsubstruct3(2:4) => array of size (1:10,1:4)
%global val
global guigsv_values
global guigsv_struct_label
chain=guigsv_struct_label;
flag_return=0;
if (direction == 'Get')
	fprintf('Trying to get the following data: \n%s\n',chain)
	flag_from_struct_to_ws=1; % copy from structure to workspace
else
	fprintf('Trying to set the following data: \n%s\n',chain)
	flag_from_struct_to_ws=0; % copy from workspace to structure
end
nch=length(chain);
output_tit='';
finderr=strfind(chain,'.(');
if (chain(nch:nch) == '.' | ~isempty(finderr))
	if (chain(nch:nch) == '.')
		errordlg('Your selection is incomplete: you have not yet completely defined a variable.','Incorrect Selection','modal')
		flag_return=1;
	elseif (~isempty(finderr))
		errordlg('You have selected more than one variable.','Incorrect Selection','modal')
		flag_return=1;
	end
else
icolon=strfind(chain,':'); % the number of colons indicates how many arrays are in this variable definition
if (~isempty(icolon))
	icolon=sort(icolon);
	ncolon=length(icolon);
	if (ncolon > 3)
		errordlg('You have selected more than three dimensions.','Incorrect Selection','modal')
		flag_return=1;
	else
		if (ncolon == 3)
		uiwait(msgbox('You have selected three dimensions. The first and second dimensions will be merged.','Warning','modal'));
		end
		for lcolon=1:ncolon % for each colon, look for the range of indices
			% look for lower index
			jcolon(lcolon)=icolon(lcolon);
			jok=1;
			while (jcolon(lcolon) > 1 & jok)
				jcolon(lcolon)=jcolon(lcolon)-1;
				sub=chain(jcolon(lcolon):jcolon(lcolon));
				if (~isempty(str2num(sub)))
					jok=1;
				else
					jok=0;
				end
			end
			jcolon(lcolon)=jcolon(lcolon)+1;
			imin(lcolon)=str2num(chain(jcolon(lcolon):icolon(lcolon)-1));
			% look for upper index
			kcolon(lcolon)=icolon(lcolon);
			kok=1;
			while (kcolon(lcolon) < length(chain) & kok)
				kcolon(lcolon)=kcolon(lcolon)+1;
				sub=chain(kcolon(lcolon):kcolon(lcolon));
				if (~isempty(str2num(sub)))
					kok=1;
				else
					kok=0;
				end
			end
			kcolon(lcolon)=kcolon(lcolon)-1;
			imax(lcolon)=str2num(chain(icolon(lcolon)+1:kcolon(lcolon)));
		end
		chain_left=chain(1:jcolon(1)-1);
		chain_right=chain(kcolon(ncolon)+1:length(chain));
		if (ncolon == 1) % decompose the 1D array structure into a 1D array
			for i=imin(1):imax(1),
				chain0=[chain_left num2str(i) chain_right];
				output_val(i)=evalin('base',chain0);
				if (~flag_from_struct_to_ws)
					% check the position exists in the input array
					if (i > length(guigsv_values))
						guigsv_values
						size(guigsv_values)
						class(guigsv_values)
						flag_return=2;
					else % check the numbers are of the same type
						if (~strcmp(class(output_val(i)),class(guigsv_values(i))))
							flag_return=3;
						else
							evalin('base',sprintf('%s=guigsv_values(%i);',chain0,i));
						end
					end
				end
			end
		elseif (ncolon == 2) % decompose the 2D array structure into a 2D array
			chain_middle=chain(kcolon(1)+1:jcolon(2)-1);
			for i=imin(1):imax(1),
				for j=imin(2):imax(2),
					chain0=[chain_left num2str(i) chain_middle num2str(j) chain_right];
					output_val(i,j)=evalin('base',chain0);
					if (~flag_from_struct_to_ws)
						% check the position exists in the input array
						if (i > size(guigsv_values,1) | j > size(guigsv_values,2))
							flag_return=2;
						else % check the numbers are of the same type
							if (~strcmp(class(output_val(i,j)),class(guigsv_values(i,j))))
								flag_return=3;
							else
								evalin('base',sprintf('%s=guigsv_values(%i,%i);',chain0,i,j));
							end
						end
					end
				end
			end
		else % ncolon =3 decompose the 3D array structure into a 2D array, by merging the first and 2nd dimensions
			chain_middle1=chain(kcolon(1)+1:jcolon(2)-1);
			chain_middle2=chain(kcolon(2)+1:jcolon(3)-1);
			l=0;
			for i=imin(1):imax(1),
				for j=imin(2):imax(2),
					l=l+1;
					for k=imin(3):imax(3),
						chain0=[chain_left num2str(i) chain_middle1 num2str(j) chain_middle2 num2str(k) chain_right];
						output_val(l,k)=evalin('base',chain0);
						if (~flag_from_struct_to_ws)
							% check the position exists in the input array
							if (l > size(guigsv_values,1) | k > size(guigsv_values,2))
								flag_return=2;
							else % check the numbers are of the same type
								if (~strcmp(class(output_val(l,k)),class(guigsv_values(l,k))))
									flag_return=3;
								else
									evalin('base',sprintf('%s=guigsv_values(%i,%i);',chain0,l,k));
								end
							end
						end
					end
				end
			end
		end
		if (~flag_from_struct_to_ws)
			if (flag_return == 2)
				errordlg('The size of the structure is larger than the size of your input. Import not complete.','Incorrect Input','modal')
			elseif (flag_return == 3)
				errordlg('The class of the structure differs from that of your input for at least one element. Import not complete.','Incorrect Input','modal')
			end
		end
	end
else
	output_val=evalin('base',chain);
	if (~flag_from_struct_to_ws)
		if (~isnumeric(output_val)) % check character strings are the same length
			if (length(output_val) ~= length(guigsv_values))
				errordlg(sprintf('The length of the character string in the structure is %i\nwhile the length of your input is %i',length(output_val),length(guigsv_values)),'Incorrect Input','modal')
				flag_return=1;
			else
				evalin('base',sprintf('%s=guigsv_values;',chain));
			end
		else % check single numeric input
			if (length(guigsv_values) > 1) % check there is only one element
				errordlg(sprintf('The selection contains one element while your input contains %i elements.',length(guigsv_values)),'Incorrect Input','modal')
				flag_return=1;
			else
				if (~strcmp(class(output_val),class(guigsv_values))) % check the numbers belong to the same types
					errordlg(sprintf('The class of the selection in the structure is %s\nwhile the class of your input is %s',class(output_val),class(guigsv_values)),'Incorrect Input','modal')
					flag_return=1;
				else
					evalin('base',sprintf('%s=guigsv_values;',chain));
				end
			end
		end
	end
end
if (flag_return == 0 & evalin('base','flag_dislay_values == 1'))
	if (flag_from_struct_to_ws)
		fprintf('Found the following values in the structure: \n')
		output_val
	else
		if (flag_return==0)
			fprintf('Imported the following values in the structure: \n')
			evalin('base','guigsv_values')
		end
%		gui_102('exportbutton_Callback',gcbo,[],guidata(gcbo))
	end
end
output_tit=chain;
if (nargin > 1) % if flag_numeric_only is true, verify the output array only contains numbers
	if (flag_numeric_only & ~isnumeric(output_val))
		errordlg('There are some non-numeric values in your selection.','Incorrect Selection','modal')
		output_val=[];
		output_tit='';
	end
end
end
if (flag_return ~= 0)
	output_val=[];
end
guigsv_values=output_val;

% --- Executes on button press in checkbox1.
function checkbox1_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox1
if (get(hObject,'Value'))
	evalin('base','flag_dislay_values=1;')
else
	evalin('base','flag_dislay_values=0;')
end

