program GenerateTestAccdCounts
  !  #[ Description
  !---------------------------------------------------
  ! purpose: a program to generate test data (accd counts)
  !          to enable testing the MieResponse module
  !          
  !    Written by:  Jos de Kloe.
  !    created:     28-Jan-2008
  !
  !    Modifications
  !    -------------
  !    30-JAN-2008  D. Tan     Remove sqrt from parameter
  !    31-JAN-2008  D. Tan     Add one IntAus type conversion
  !---------------------------------------------------
  !  #]
  !  #[ Modules used
  !
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE EE_CFI_Datatypes, only: IntAus, IntAuc, FAdoxy, maxval_IntAus
  USE numerics, only: r8_
  !  #]
  !  #[ variables
  implicit none 

  integer(IntAuc), parameter :: numACCDCols        = 20
  integer(IntAuc), parameter :: numUsefulACCDCols  = 16
  integer(IntAuc), parameter :: firstUsefulACCDCol = 3
  integer(IntAus), dimension(numACCDCols) :: AccdInt
  real(FAdoxy),   dimension(numACCDCols) :: AccdReal
  real(FAdoxy) :: pixel_location
  integer :: i
  !  #]
  !  #[ definition of testcase 2
  !character(len=*), parameter :: case_text = "no noise, zero wind"
  !real(FAdoxy), parameter :: BackgroundLevel = 1580._r8_
  !real(FAdoxy), parameter :: NoiseLevel      =    0._r8_
  !real(FAdoxy), parameter :: PeakHeight      = 1400._r8_ ! above background
  !real(FAdoxy), parameter :: PeakWidth       =    0.5_r8_
  !real(FAdoxy), parameter :: PeakOffset      =    0.0_r8_
  !real(FAdoxy) :: pixel_location
  !  #]
  !  #[ definition of testcase 3
  !character(len=*), parameter :: case_text = "noise, zero wind"
  !real(FAdoxy), parameter :: BackgroundLevel = 1580._r8_
  !real(FAdoxy), parameter :: NoiseLevel      =   44._r8_
  !real(FAdoxy), parameter :: PeakHeight      = 1400._r8_ ! above background
  !real(FAdoxy), parameter :: PeakWidth       =    1.4_r8_
  !real(FAdoxy), parameter :: PeakOffset      =    0.0_r8_
  !  #]
  !  #[ definition of testcase 4
  !character(len=*), parameter :: case_text = "noise, nonzero wind"
  !real(FAdoxy), parameter :: BackgroundLevel = 1580._r8_
  !real(FAdoxy), parameter :: NoiseLevel      =   44._r8_
  !real(FAdoxy), parameter :: PeakHeight      = 1400._r8_ ! above background
  !real(FAdoxy), parameter :: PeakWidth       =    1.4_r8_
  !real(FAdoxy), parameter :: PeakOffset      =    2.5_r8_
  !  #]
  !  #[ definition of testcase 5
  !character(len=*), parameter :: case_text = "no noise, zero wind, overflow"
  !real(FAdoxy), parameter :: BackgroundLevel = 1580._r8_
  !real(FAdoxy), parameter :: NoiseLevel      =    0._r8_
  !real(FAdoxy), parameter :: PeakHeight      = 280000._r8_ ! above background
  !real(FAdoxy), parameter :: PeakWidth       =    1.4_r8_
  !real(FAdoxy), parameter :: PeakOffset      =    0._r8_
  !  #]
  !  #[ definition of testcase 6
  character(len=*), parameter :: case_text = "noise, nonzero wind, overflow"
  real(FAdoxy), parameter :: BackgroundLevel = 1580._r8_
  real(FAdoxy), parameter :: NoiseLevel      =   22._r8_
  real(FAdoxy), parameter :: PeakHeight      = 280000._r8_ ! above background
  real(FAdoxy), parameter :: PeakWidth       =    1.4_r8_
  real(FAdoxy), parameter :: PeakOffset      =    2.3_r8_
  !  #]
  !  #[ calc accd counts
  ! set the background level
  AccdReal(:) = BackgroundLevel
  
  call init_random_nr_generator(975123,219)

  do i=firstUsefulACCDCol,firstUsefulACCDCol+numUsefulACCDCols-1
     ! add some noise
     AccdReal(i) = AccdReal(i) + NoiseLevel*gaussian_rnd_nr()
     ! add the peak
     pixel_location = real(i,r8_)-10.5_r8_
     AccdReal(i) = AccdReal(i) + &
                   PeakHeight*gaussian(pixel_location,PeakOffset,PeakWidth)
     ! convert to 2 byte unsigned integer 
     ! to simulate storage in a L1B datafile
     AccdInt(i)  = int(AccdReal(i),IntAus)
     if (AccdInt(i) .gt. maxval_IntAus) &
          AccdInt(i) = modulo(AccdInt(i),maxval_IntAus+1_IntAus)
  end do
  !  #]
  !  #[ print results
  print *,"case: "//trim(case_text)
  print "(20(i5,1x))",(AccdInt(i),i=1,numACCDCols)
  !  #]
contains
  !-------------------
  subroutine init_random_nr_generator(s1,s2)
    !  #[
    integer, intent(in) :: s1,s2

    ! local variables
    integer :: nrandomseeds, i
    integer, dimension(:), allocatable :: randomseeds

    ! prepare the default Fortran90 random generator
    call random_seed(size=nrandomseeds)
    !print *,"size of seeds array for this compiler: n=",nrandomseeds
    allocate(randomseeds(nrandomseeds))
    do i=1,nrandomseeds
       randomseeds(i) = s1 + s2*i
    end do
    CALL random_seed(put=randomseeds(1:nrandomseeds))
    deallocate(randomseeds)

  end subroutine init_random_nr_generator
    !  #]
  function gaussian_rnd_nr() result(r)
    !  #[
    !-------------------------------------------
    ! a function, inspired on gasdev from numerical recipes
    ! to calculate a gaussian distributed random number
    !-------------------------------------------
    ! see http://www.bearcave.com/misl/misl_tech/wavelets/hurst/random.html
    ! or  http://www.taygeta.com/random/gaussian.html
    ! which make clear that this algorithm is only used by,
    ! and certainly not invented by numerical recipes.
    !-------------------------------------------
    real(r8_) :: r ! result

    integer,   save :: iset = 0
    real(r8_), save :: gset
    real(r8_) :: v1,v2,rsq,fac,u1,u2

    if (iset.eq.0) then
1      call random_number(u1)
       call random_number(u2)
       v1  = u1*2 - 1.0_r8_
       v2  = u2*2 - 1.0_r8_
       rsq = v1*v1 + v2*v2
       if ((rsq .ge. 1.0_r8_) .or. (rsq .eq. 0.0_r8_) ) goto 1
       fac  = sqrt(-2.0_r8_*log(rsq)/rsq)
       gset = v1*fac
       r    = v2*fac
       iset = 1
    else
       r    = gset
       iset = 0
    end if

    return

  end function gaussian_rnd_nr
    !  #]
  function gaussian(x,dx,sigma) result(y)
    !  #[
    real(r8_), intent(in) :: x,dx,sigma ! input
    real(r8_)             :: y          ! result
    
    ! local variable
    real(r8_) :: fac

    ! local parameters
    real(r8_), parameter :: pi  = 3.14159265358979323846264338328_r8_
    real(r8_), parameter :: two_pi      = pi*2._r8_
    real(r8_)            :: sqrt_two_pi

    sqrt_two_pi = sqrt(two_pi)

    fac = -0.5_r8_*(x-dx)*(x-dx)/(sigma*sigma)
    y = exp(fac)/(sigma*sqrt_two_pi)

  end function gaussian
    !  #]
  !-------------------
end program GenerateTestAccdCounts
