program TestMolScat
  !  #[ Description
  !---------------------------------------------------
  ! purpose: a test program for the MolScat module
  !          
  !    Written by:  Jos de Kloe.
  !    created:     03-03-2005
  !
  !    Modifications:
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !
  !---------------------------------------------------
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  use MolScat, only: calc_molecular_extinction
  use MolScat, only: calc_molecular_backscatter
  use MolScat, only: InitMolScatModule
  use MolScat, only: get_ref_mol_scat 
  USE AeolusConstants, only: pi

  use numerics,   only: r8_
  use LunManager, only: get_lun, free_lun
  !  #]
  !  #[ variables and parameters
  implicit none

  ! local parameter 
  real(r8_), parameter :: eps = 1.e-3 ! allowed rel. diff. while comparing

  ! local variables
  real(r8_) :: sigma_m1, beta_m1, RMA_sigma_m1, RMA_beta_m1, p, t, z
  real(r8_) :: sigma_m2, beta_m2, RMA_sigma_m2, RMA_beta_m2
  real(r8_) :: rel_diff
  integer  :: nr_of_levels, i, scene, fileunit, ios
  character(len=100) :: filename
  !  #]
  !  #[ the program code
  ! init the MolScat module
  call InitMolScatModule

  ! read the datafile  
  filename = "TestMolScatProfile.dat"
  fileunit = get_lun()
  open(unit=fileunit,file=filename,status="old",&
       action="read",form="FORMATTED",ERR=99)
  read(fileunit,*,iostat=ios) nr_of_levels
  IF (ios .ne. 0) goto 98

  DO scene=1,3
     select case(scene)
     case(1)
        print *,"Testing scene = scene_tropical"
     case(2)
        print *,"Testing scene = scene_sub_arctic_winter"
     case(3)
        print *,"Testing scene = scene_mid_latitude_winter"
     end select

     DO i=1,nr_of_levels
        read(fileunit,*,iostat=ios) z, p, t, sigma_m1, beta_m1, &
                                    RMA_sigma_m1, RMA_beta_m1
        IF (ios .ne. 0) goto 98
        ! z in [m]
        ! p in [hPa]
        ! t in [K]
        sigma_m2     = calc_molecular_extinction(p,t)
        beta_m2      = calc_molecular_backscatter(p,t)
        RMA_beta_m2  = get_ref_mol_scat(z)
        RMA_sigma_m2 = 8.*pi*RMA_beta_m2/3.

        ! check with reference results in the file
        rel_diff = abs(sigma_m1-sigma_m2)/max(sigma_m1,sigma_m2)
        IF (rel_diff .gt. eps) THEN
           print *,"sigma_m differs!"
           print *,"sigma_m1 = ",sigma_m1
           print *,"sigma_m2 = ",sigma_m2
           print *,"rel_diff = ",rel_diff
        END IF
        rel_diff = abs(beta_m1-beta_m2)/max(beta_m1,beta_m2)
        IF (rel_diff .gt. eps) THEN
           print *,"beta_m differs!"
           print *,"beta_m1  = ",beta_m1
           print *,"beta_m2  = ",beta_m2
           print *,"rel_diff = ",rel_diff
        END IF
        rel_diff = abs(RMA_sigma_m1-RMA_sigma_m2)/&
                   max(RMA_sigma_m1,RMA_sigma_m2) 
        IF (rel_diff .gt. eps) THEN
           print *,"RMA_sigma_m differs!"
           print *,"RMA_sigma_m1 = ",RMA_sigma_m1
           print *,"RMA_sigma_m2 = ",RMA_sigma_m2
           print *,"rel_diff     = ",rel_diff
        END IF
        rel_diff = abs(RMA_beta_m1-RMA_beta_m2)/&
                   max(RMA_beta_m1,RMA_beta_m2)
        IF (rel_diff .gt. eps) THEN
           print *,"RMA_beta_m differs!"
           print *,"RMA_beta_m1  = ",RMA_beta_m1
           print *,"RMA_beta_m2  = ",RMA_beta_m2
           print *,"rel_diff     = ",rel_diff
        END IF

        !print *,"z,p,t,s,b = ",z,p,t,sigma_m1,beta_m1
     END DO ! level loop
     print *,"done with this scene"
  END DO ! scene loop
  close(fileunit)
  call free_lun(fileunit)

  print *,"done with all tests"

  goto 11 ! goto the end of this program

98 print *,"ERROR: unable to read data from file: ",trim(filename)
  stop

99 print *,"ERROR: unable to open input file: ",trim(filename)
  stop

11 continue
  ! end of this program

  !  #]
end program TestMolScat
