program Test_LUT_RayCalib_handling
  !  #[ Description:
  !     Test routine to read ILIAD tables of Rayleigh channel A and B counts.
  !     Input files are binary files generated in
  !
  !	    /nobackup_1/users/marseill/dwl/L2B/files_Alain/iliadtables
  !
  !	Next, tables are interpolated to the actual temperature and pressure
  !	(in practice obtained from the Auxiliary Meteorological data)

  ! Modifications:
  !   30-Jun-2006  D. Tan       Null use of ParKind1 module
  !   19-Sep-2006  J. de Kloe   some small interface changes
  !   16-Jan-2008  J. de Kloe   phase out integer kind i_

  !  #]
  !  #[ Modules used
USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
USE numerics, only: r8_
USE MolScat,  only: InitMolScatModule,calc_molecular_extinction, &
                    calc_molecular_backscatter
USE ErrorHandler, only: no_error,program_abort
USE LUT_RayCalib_handling, only: Load_LUT_RayCalib, LUT_RayCalib_Intpol, &
                                 Unload_LUT_RayCalib
  !  #]
  !  #[ Variables
  implicit none

  integer   :: error_flag !,i ! not used for now
  real(r8_) :: Temp,PresshPa,Freq_wind ! ,Wind_hlos ! not used for now
  real(r8_) :: RayCal
  real(r8_) :: sigma,beta
  !  #]
  !  #[ Program code
  Temp         = 288.     ! (K)
  PresshPa     = 1000.0   ! (hPa)
  !  Wind_hlos = 10.      ! (m/s) ! not used for now
  Freq_wind = 33.      ! (MHz)
  
  call InitMolScatModule()
  sigma = calc_molecular_extinction(PresshPa,Temp)
  beta  = calc_molecular_backscatter(PresshPa,Temp)
  print*,'sigma mol ',sigma
  print*,'beta mol  ',beta

  call Load_LUT_RayCalib(error_flag)
  if (error_flag .ne. no_error) then
     call program_abort(error_flag,"Load_LUT_RayCalib")
  endif

  !do i=1,100
  !   Wind_hlos = i
  !   call Read_LUT_RayCalib(Temp,PresshPa,Wind_hlos,RayCal,error_flag)
  !   if (error_flag .ne. no_error) &
  !        call program_abort(error_flag,"Read_LUT_RayCalib")
     call LUT_RayCalib_Intpol(Temp,PresshPa,Freq_wind,RayCal,error_flag)
     if (error_flag .ne. no_error) &
          call program_abort(error_flag,"LUT_RayCalib_Intpol")
     print*,'RayCal ',RayCal
  !   print*,'error_flag ',error_flag
  !enddo
  call Unload_LUT_RayCalib

  !  #]
END program Test_LUT_RayCalib_handling


