program TestReadRBCdata
  !  #[ Documentation
  !
  ! a test program for the ReadRBCdata module
  ! written by: J. de Kloe (KNMI), 16-02-2006
  !
  ! Modifications:
  !     12-May-2006 D. Tan       Null use of ParKind1 module
  !     16-Jan-2008 J. de Kloe   phase out integer kind i_
  !
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE RBC_DataStructure, only: RBC_DataType, Init_RBC_DataStructure, &
                               Delete_RBC_DataStructure, &
                               Check_RBC_DS_with_Dummy_data
  USE ReadRBCdata, only: ReadRBCfile, InitReadRBCfileModule
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport
  USE ErrorHandler, only: no_error, program_abort, error_cmdline_options
  USE Compiler_Features, only: iargc_aeolus,getarg_aeolus
  !  #]
  !  #[ variables
  implicit none

  integer            :: error_flag, nr_of_arguments
  character(len=256) :: filename ! without extension
  TYPE(RBC_DataType) :: RBC
  !  #]
  !  #[ program code
  error_flag = no_error

  call Init_RBC_DataStructure(RBC, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DataStructure")  

  !call InitReadRBCfileModule(error_flag)
  call InitReadRBCfileModule(error_flag,ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitReadRBCfileModule")

  ! read the filename without extension as 1st commandline parameter:
  nr_of_arguments = iargc_aeolus()
  IF (nr_of_arguments .ne. 1) THEN
     print *,"Usage: TestReadRBCdata <filename[without extension]>"
     call program_abort(error_cmdline_options,"TestReadRBCdata")
  END IF

  call getarg_aeolus(1,filename)

  ! read all datablocks
  call ReadRBCfile(RBC, filename, error_flag)
  !call ReadRBCfile(RBC, filename, error_flag, Only_DBL=.true.)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"ReadRBCfile")

  print *,"Checking datacontents of dummy RBC_DS"
  call Check_RBC_DS_with_Dummy_data(RBC%RBC_DS,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Check_RBC_DS_with_Dummy_data")

  print *,"Test succesfully completed"

  call Delete_RBC_DataStructure(RBC)
  !  #]
end program TestReadRBCdata
