module WriteRBCdata
  !  #[ Description:

  !
  !     a module to write the 2 files describing the RBC look-up table
  !     of the Aeolus instrument
  !
  ! created:     15-Feb-2006 by J. de Kloe
  !
  ! History:
  !    16-Feb-2006 J. de Kloe  added MPH handling
  !    22-Feb-2006 J. de Kloe  added SPH handling
  !    02-Mar-2006 J. de Kloe  adapted to changes in the XML_module
  !    07-Mar-2006 J. de Kloe  added writing of binary *.DBL file
  !    27-Mar-2006 D. Tan      added FP_DSD and quality parameters
  !    12-May-2006 D. Tan      Null use of ParKind1 module
  !    18-Jul-2006 D. Tan      Use RBC_DS_[Name/Type]_RBC_ADS.
  !    17-Nov-2006 J. de Kloe  small interface change in Fill_DSD_items
  !    21-Nov-2006 J. de Kloe  removed filename from 1st Fill_DSD_items call
  !    24-Apr-2007 J. de Kloe  phase out the SummaryVars code
  !    07-Jun-2007 J. de Kloe  adapted to changed xml interface
  !    20-Jul-2007 J. de Kloe  added call to xml_tree_set_schema()
  !    30-Jul-2007 J. de Kloe  adapt to changed interface of Fill_FH_Items
  !    18-Oct-2007 J. de Kloe  adapt to changed interface of Fill_MPH_Items
  !    05-Nov-2007 J. de Kloe  added setting of the xml namespace
  !    07-Dec-2007 J. de Kloe  adapted to changed FH/MPH interface
  !    16-Jan-2008 J. de Kloe  phase out integer kind i_
  !    21-Jan-2008 J. de kloe  added workaround in call to Write_MPH_to_KVT
  !                            to not change the fileformat for now.
  !    21-Feb-2008 J. de Kloe  removed the workaround again since it was
  !                            caused by a misunderstanding on L1B side
  !    22-Feb-2008 D. Tan      Change Ref_Doc value
  !
  !  #]
  !  #[ Modules used:

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE RBC_DataStructure, only: RBC_DataType, &
                               GetByteSize_RBC_DataStruct_KVT, &
                               GetByteSize_RBC_DS, Write_RBC_DS
  USE FixedHeader, only: Init_FH, Copy_FH_to_xml_tree, &
                         Fill_FH_Items, file_type_AuxRBC, &
                         maxlen_file_type, maxlen_file_description, &
                         maxlen_notes, maxlen_creator, &
                         maxlen_creator_version, maxlen_system
  USE MainProductHeader, only: Init_MPH, Copy_MPH_to_xml_tree, &
                               Write_MPH_to_KVT, Fill_MPH_items, &
                               GetByteSize_MPH_KVT, &
                               maxlen_Product, maxlen_Ref_Doc, &
                               maxlen_Software_Ver
  USE RBC_SPH, only: Copy_RBC_SPH_to_xml_tree, Write_RBC_SPH_to_KVT, &
                     maxlen_SPH_Descriptor, maxlen_Ref_RBC_Suite, &
                     Fill_RBC_SPH_items, Init_RBC_SPH, &
                     GetByteSize_RBC_SPH_KVT,&
                     RBC_DS_Name_RBC_ADS, RBC_DS_Type_RBC_ADS, &
                     RBC_DS_Name_FP_MDS, RBC_DS_Type_FP_MDS
  USE DataSetDescriptor, only: Init_DSD, Print_DSD, DSD_type, &
       Fill_DSD_items,  maxlen_DS_Name, maxlen_DS_Type, &
       maxlen_DSD_Filename => maxlen_Filename

  USE XML_module, only: xml_node_type, &
       Init_XML, xml_tree_print, xml_tree_write, create_xml_tree, &
       xml_tree_create_header, delete_xml_tree, ee_h_name, &
       xml_tree_set_namespace, uri_ns_header

  USE KVT_module, only: max_str_len, Init_KVT_module
  USE DirectBinaryIO, only: Init_DirectBinaryIO_module , &
                            OpenDirectBinaryFile, CloseDirectBinaryFile, &
                            BigEndian, write_zero_bytes_bin_file

  USE Numerics, only: r8_
  USE ErrorHandler, only: no_error, error_xml, error_programming, &
                          error_opening_file, & ! error_reading_file,&
                          error_writing_file, error_allocate
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  USE LunManager, only: get_lun, free_lun
  USE Compiler_Features, only: newline
  USE EE_CFI_Datatypes, only: IntAd, IntAul, IntAl, IntAs, IntAus, FAdoxy, &
                              FAdoxy_to_IntAul, FAdoxy_to_IntAus

  USE EE_CFI_Datatypes, only: DateTimeType,Construct_KVT_DateTime, &
                              SetDateTime, PrintDateTime
!                              InitDateTime, IncrementDateTime, &
!                              TestDateTime, &
!                              DateTime2String, GetCurrentDateTime
  USE AeolusConstants, only: software_version_L2BP => software_version, &
                             iodd_version_AuxRBC, Ref_Doc_AuxRBC

  !  #]  
  !  #[ Parameter/Variable/Type declarations

  implicit none

  ! make all module contents public, unless explicitely stated otherwise
  public

  ! the default verbosity setting is Error (as silent as possible)
  integer, private, save :: LocalVerbosity = VerbosityError

  ! remember the initialisation state of the module
  logical, private, save :: module_is_initialised = .false.

  !  #]
  !  #[ Module interface
  !private :: 
  !  #]
contains
  !---------------------------------
  subroutine InitWriteRBCfileModule(error_flag, ChoosenVerbosity)
    !  #[

    integer,           intent(out) :: error_flag
    integer, optional, intent(in)  :: ChoosenVerbosity

    error_flag = no_error

    IF (present(ChoosenVerbosity)) LocalVerbosity = ChoosenVerbosity

    IF (LocalVerbosity .ge. VerbosityReport) &
         call print_verbosity_setting(LocalVerbosity,ModuleName="WriteRBCfile")

    call Init_KVT_module()

    call Init_DirectBinaryIO_module(error_flag,FileByteOrder=BigEndian)
    if (error_flag .ne. no_error) return

    call Init_XML(error_flag) ! (ChoosenVerbosity=VerbosityReport)
    if (error_flag .ne. no_error) return

    module_is_initialised = .true.

  end subroutine InitWriteRBCfileModule
    !  #]
  subroutine WriteRBCfile(RBC, error_flag)
    !  #[
    type(RBC_DataType), intent(inout) :: RBC
    integer,            intent(out)   :: error_flag

    ! local variables
    character(len=256) :: filename_hdr
    character(len=256) :: filename_dbl
    character(len=256) :: path
    character(len=256) :: xml_namespace_RBC

    ! general xml items
    TYPE(xml_node_type), pointer :: xml_tree

    ! init
    error_flag = no_error

    ! check if we have a FH
    IF (.not. associated(RBC%FH)) THEN
       print *,"ERROR in   WriteRBCfile():"
       print *,"the RBC%FH pointer seems not associated !!!!"
       print *,"Unable to figure out what file to read from ...."
       error_flag = error_programming
       return
    END IF

    ! check if we have a MPH
    IF (.not. associated(RBC%MPH)) THEN
       print *,"ERROR in   WriteRBCfile():"
       print *,"the RBC%MPH pointer seems not associated !!!!"
       error_flag = error_programming
       return
    END IF

    ! check if we have a SPH
    IF (.not. associated(RBC%SPH)) THEN
       print *,"ERROR in   WriteRBCfile():"
       print *,"the RBC%SPH pointer seems not associated !!!!"
       error_flag = error_programming
       return
    END IF

    ! construct the filenames to be used by adding the extensions
    filename_hdr = trim(RBC%FH%File_Name)//".HDR"
    filename_dbl = trim(RBC%FH%File_Name)//".DBL"

    IF (.not. module_is_initialised) THEN
       print *,"ERROR in WriteRBCfile:"
       print *,"the WriteRBCfile module should be initialised before use !"
       error_flag = error_programming
       return
    END IF

    !IF (LocalVerbosity .ge. VerbosityReport) &
    print *,"WriteRBCfile: Writing RBC output file..."

    ! allocate a datastructure to hold the xml data
    ! get a pointer to it, and fill it with the xml root tag pair:
    ! <Earth_Explorer_File><\Earth_Explorer_File>

    ! allocate and init the datastructure
    nullify(xml_tree)
    call create_xml_tree(xml_tree, error_flag)
    IF (error_flag .ne. no_error) return

    ! create a blank header (fixed+variable) and define the root
    call xml_tree_create_header(xml_tree, ee_h_name, error_flag)
    IF (error_flag .ne. no_error) return

    !example: xmlns="http://www.esa.int/schemas/ae/AUX_RBC_L2_01.01"
    !uri_ns_header       ! defined in xml_module
    !file_type_AuxRBC    ! defined in FixedHeader
    !iodd_version_AuxRBC ! defined in AeolusConstants
    xml_namespace_RBC = trim(uri_ns_header)//trim(file_type_AuxRBC)//&
                        "_"//trim(iodd_version_AuxRBC)

    ! define which xml namespace should be used
    call xml_tree_set_namespace(xml_tree,xml_namespace_RBC,error_flag)
    IF (error_flag .ne. no_error) return

    ! fill all FH items in the xml tree
    path = "/"//ee_h_name
    call Copy_FH_to_xml_tree(RBC%FH, path, xml_tree, error_flag)
    IF (error_flag .ne. no_error) return

    ! => write the MPH
    call Copy_MPH_to_xml_tree(RBC%MPH, path, xml_tree, error_flag)
    IF (error_flag .ne. no_error) return

    ! => write the SPH
    call Copy_RBC_SPH_to_xml_tree(RBC%SPH, path, xml_tree, error_flag)
    IF (error_flag .ne. no_error) return

    IF (LocalVerbosity .ge. VerbosityReport) THEN
       print *,"WriteRBCfile: Writing XML header file ..."
       print *,trim(filename_hdr)
    END IF

    ! Write the HDR file including FH, MPH, and SPH (with DSD)
    call xml_tree_write(xml_tree, filename_hdr, error_flag)
    IF (error_flag .ne. no_error) return

    IF (LocalVerbosity .ge. VerbosityReport) THEN
       !print *,"==================="
       !print *,"the generated xml tree was successfully written to file"
       !print *,"==================="
       !call xml_tree_print(xml_tree, error_flag)
       !IF (error_flag .ne. no_error) return
       !print *,"==================="
    END IF

    call delete_xml_tree(xml_tree,error_flag)
    IF (error_flag .ne. no_error) return

    ! Write the DBL file including MPH, SPH, and data and figure out size
    call WriteRBCtoBinaryFile(RBC,filename_dbl,error_flag)

    IF (LocalVerbosity .ge. VerbosityReport) &
         print *,"WriteRBCfile: done writing RBC output file"

  end subroutine WriteRBCfile
    !  #]
  !---------------------------------
  subroutine WriteRBCtoBinaryFile(RBC,filename_dbl,error_flag)
    !  #[
    type(RBC_DataType), intent(inout) :: RBC
    integer,            intent(out)   :: error_flag
    character(len=256), intent(in)    :: filename_dbl

    ! local variables
    integer           :: fileunit!, AllocStatus
    !integer           :: nr_of_zero_bytes
    integer           :: pos_start, pos_end
    integer           :: nbytes_written, nbytes_expected
    
    error_flag = no_error

    IF (LocalVerbosity .ge. VerbosityReport) THEN
       print *,"WriteRBCtoBinaryFile: writing RBC data to binary file: "
       print *,trim(filename_dbl)
    END IF

    call OpenDirectBinaryFile(filename_dbl,"write",fileunit,error_flag)
    IF (error_flag .ne. no_error) return

    ! 2. write RBC MPH
    IF (LocalVerbosity .ge. VerbosityReport) &
         print *,'WriteRBCtoBinaryFile: writing MPH'
    call Write_MPH_to_KVT(RBC%MPH,fileunit,error_flag)
    IF (error_flag .ne. no_error) goto 999 ! abort writing the file

    ! 3. write RBC SPH
    IF (LocalVerbosity .ge. VerbosityReport) &
         print *,'WriteRBCtoBinaryFile: writing SPH'
    call Write_RBC_SPH_to_KVT(RBC%SPH,fileunit,error_flag)
    IF (error_flag .ne. no_error) goto 999 ! abort writing the file

    ! 4. write RBC DS
    IF (associated(RBC%RBC_DS)) THEN
       inquire(unit=fileunit,nextrec=pos_start)
       print *,'WriteRBCtoBinaryFile: writing RBC_DS'

       !nr_of_zero_bytes = 100 ! GetByteSize_RBC_DS(RBC%RBC_DS)
       !call write_zero_bytes_bin_file(fileunit,nr_of_zero_bytes,error_flag)

       call Write_RBC_DS(fileunit,RBC%RBC_DS,error_flag)
       IF (error_flag .ne. no_error) goto 999

       inquire(unit=fileunit,nextrec=pos_end)
       nbytes_written  = pos_end-pos_start
       nbytes_expected = GetByteSize_RBC_DS(RBC%RBC_DS)

       IF (nbytes_written .ne. nbytes_expected) THEN
          print *,"ERROR in WriteRBCtoBinaryFile: writing RBC_DS"
          print *,"pos_start       = ",pos_start
          print *,"pos_end         = ",pos_end
          print *,"nbytes_written  = ",nbytes_written
          print *,"nbytes_expected = ",nbytes_expected
          print *,"something went wrong when writing the RBC_DS to the"
          print *,"binary file, probably a programming error ..."
          error_flag = error_programming
          goto 999
       END IF

    ELSE
       print *,"ERROR in WriteRBCtoBinaryFile:"
       print *,"RBC%RBC_DS: [seems not associated]"
       error_flag = error_writing_file
       goto 999
    END IF

    !------------------------------
    ! done
    !------------------------------
    call CloseDirectBinaryFile(fileunit)
    return

999 print *,"ABORTING subroutine WriteRBCtoBinaryFile"
    call CloseDirectBinaryFile(fileunit)
    return

  end subroutine WriteRBCtoBinaryFile
    !  #]  
  !---------------------------------
  subroutine Fill_RBC_Headers(RBC, & ! the data struct to be filled
       file_class, &                                      ! FH inputs
       validity_start_datetime, validity_stop_datetime, & ! FH inputs
       file_version, system, &                            ! FH inputs
       proc_stage, proc_center, product_err, &            ! MPH inputs
       Num_P, Num_T, Num_F, Num_FP, Num_fd, Num_RR, &     ! SPH inputs
       P_min, P_max, T_min, T_max, &                      ! SPH inputs
       Laser_Freq_Offset_Start, Laser_Freq_Offset_Stop,&  ! Ditto
       Total_Num_of_Observations,&
       Total_Num_of_Measurements,&
       Total_Num_of_Reference_Pulses,&
       Total_Num_of_Corrupt_Mie_Meas, Total_Num_of_Corrupt_Ray_Meas,&
       Total_Num_of_Corrupt_Mie_RefP, Total_Num_of_Corrupt_Ray_RefP,&
       Average_Error_FP_Response_A,   Average_Error_FP_Response_B,&
       error_flag) ! result
    !  #[
    ! the datastructure to be filled
    type(RBC_DataType), intent(inout):: RBC
    ! inputs needed to fill the FH
    character(len=*),   intent(in)   :: file_class
    TYPE(DateTimeType), intent(in)   :: validity_start_datetime
    TYPE(DateTimeType), intent(in)   :: validity_stop_datetime
    integer,            intent(in)   :: file_version
    character(len=*),   intent(in)   :: system
    ! inputs needed to fill the MPH
    character(len=1),   intent(in)   :: proc_stage
    character(len=*),   intent(in)   :: proc_center    
    logical,            intent(in)   :: product_err
    ! inputs needed to fill the SPH
    integer(IntAs),     intent(in)   :: Num_P  ! =  23
    integer(IntAs),     intent(in)   :: Num_T  ! = 161
    integer(IntAs),     intent(in)   :: Num_F  ! = 937
    integer(IntAs),     intent(in)   :: Num_FP ! = 877
    integer(IntAs),     intent(in)   :: Num_fd ! =  61
    Integer(IntAs),     intent(in)   :: Num_RR ! = 101
    real(r8_),          intent(in)   :: P_min  ! in [Pa]
    real(r8_),          intent(in)   :: P_max  ! in [Pa]
    real(r8_),          intent(in)   :: T_min  ! in [K]
    real(r8_),          intent(in)   :: T_max  ! in [K]
    !
    real(FAdoxy),       intent(in)  :: Laser_Freq_Offset_Start
    real(FAdoxy),       intent(in)  :: Laser_Freq_Offset_Stop
    integer(IntAl),     intent(in)  :: Total_Num_of_Observations
    integer(IntAl),     intent(in)  :: Total_Num_of_Measurements
    integer(IntAl),     intent(in)  :: Total_Num_of_Reference_Pulses
    integer(IntAl),     intent(in)  :: Total_Num_of_Corrupt_Mie_Meas
    integer(IntAl),     intent(in)  :: Total_Num_of_Corrupt_Ray_Meas
    integer(IntAl),     intent(in)  :: Total_Num_of_Corrupt_Mie_RefP
    integer(IntAl),     intent(in)  :: Total_Num_of_Corrupt_Ray_RefP
    real(FAdoxy),       intent(in)  :: Average_Error_FP_Response_A
    real(FAdoxy),       intent(in)  :: Average_Error_FP_Response_B

    ! success or fail ?
    integer,            intent(out)  :: error_flag
    
    ! local variables
    integer                                :: AllocStatus
    
    ! used to fill the FH (filled with constants below)
    character(len=maxlen_file_type)        :: file_type
    character(len=maxlen_file_description) :: file_description
    character(len=maxlen_notes)            :: notes
    character(len=maxlen_creator)          :: creator
    character(len=maxlen_creator_version)  :: creator_version
    
    ! used to fill the MPH
    character(len=maxlen_Product)      :: product
    character(len=maxlen_Ref_Doc)      :: ref_doc
    character(len=maxlen_Software_Ver) :: software_version
    integer(IntAd) :: tot_size
    integer(IntAl) :: sph_size, num_DSD, num_data_sets
    
    ! used to fill the SPH
    character(len=maxlen_SPH_Descriptor) :: SPH_Descriptor
    character(len=maxlen_Ref_RBC_Suite)  :: Ref_RBC_Suite
    integer(IntAul) :: iP_min ! in [Pa]
    integer(IntAul) :: iP_max ! in [Pa]
    integer(IntAus) :: iT_min ! in [10^-2 K]
    integer(IntAus) :: iT_max ! in [10^-2 K]
    
    ! used to fill the DSD
    character(len=maxlen_DS_Name)       :: DS_Name
    character(len=maxlen_DS_Type)       :: DS_Type
    character(len=maxlen_DSD_Filename)  :: Filename
    integer(IntAd)                      :: Ds_Offset
    integer(IntAl)                      :: Num_Dsr
    integer(IntAl)                      :: Dsr_Size
    
    error_flag = no_error
    
    !*** set the Fixed Header ***
    allocate(RBC%FH,STAT=AllocStatus)
    IF (AllocStatus.ne. 0) THEN
       print *,"Allocation problem in Fill_RBC_Headers"
       error_flag = error_allocate
       return
    END IF
    
    call Init_FH(RBC%FH, error_flag)
    IF (error_flag .ne. no_error) return
    
    ! these settings are constant, independent of the location
    ! where the code is executed, to full them here
    file_description = "RBC look-up table file"
    notes            = "Room for some"//newline//"additional remarks"
    file_type        = file_type_AuxRBC
    creator          = "L2BP"
    creator_version  = software_version_L2BP

    call Fill_FH_items(RBC%FH, &
         file_description, notes, file_class, file_type, &
         validity_start_datetime, validity_stop_datetime, &
         file_version, system, creator, creator_version, error_flag)
    IF (error_flag .ne. no_error) return
    
    print *,'Handle_RBC_Headers: Filled FH'
    
    !*** set the Main Product Header ***
    allocate(RBC%MPH,STAT=AllocStatus)
    IF (AllocStatus .ne. 0) THEN
       print *,"Allocation problem in Fill_RBC_Headers"
       error_flag = error_allocate
       return
    END IF
    
    call Init_MPH(RBC%MPH, error_flag)
    IF (error_flag .ne. no_error) return
    
    product = RBC%FH%File_Name
    !proc_stage       = defined in the calling program
    ref_doc          = Ref_Doc_AuxRBC
    !proc_center      = defined in the calling program
    software_version = software_version_L2BP
    !product_err      = defined in the calling program

    ! these are constants for the RBC filetype
    ! see L2B IODD V1.0, sec. 5.2
    tot_size      = GetByteSize_RBC_DataStruct_KVT(RBC) ! = 33774184
    sph_size      = GetByteSize_RBC_SPH_KVT() ! = 947
    num_DSD       = 2_IntAl
    num_data_sets = 1_IntAl
    
    call Fill_MPH_items(RBC%MPH, product, proc_stage, ref_doc, proc_center, &
                        software_version, product_err, tot_size, &
                        sph_size, num_dsd, num_data_sets, &
                        validity_start_datetime, validity_stop_datetime, &
                        error_flag)
    IF (error_flag .ne. no_error) return
    
    print *,'Handle_RBC_Headers: Filled MPH'
    
    !*** set the Specific Product Header ***
    allocate(RBC%SPH,STAT=AllocStatus)
    IF (AllocStatus .ne. 0) THEN
       print *,"Allocation problem in Fill_RBC_Headers"
       error_flag = error_allocate
       return
    END IF
    
    call Init_RBC_SPH(RBC%SPH, error_flag)
    IF (error_flag .ne. no_error) return
    
    SPH_Descriptor = "RBC look-up table file"
    Ref_RBC_Suite  = "Test RBC suite"
    iP_min         = FAdoxy_to_IntAul(P_min) ! in [Pa]
    iP_max         = FAdoxy_to_IntAul(P_max) ! in [Pa]
    iT_min         = FAdoxy_to_IntAus(T_min,scale=100._r8_) ! in [10^-2 K]
    iT_max         = FAdoxy_to_IntAus(T_max,scale=100._r8_) ! in [10^-2 K]
    
    call Fill_RBC_SPH_items(RBC%SPH, error_flag, &
         SPH_Descriptor,Ref_RBC_Suite, Num_P, Num_T, Num_F, Num_FP, Num_fd,&
         Num_RR, iP_min, iP_max, iT_min, iT_max,&
         Laser_Freq_Offset_Start, Laser_Freq_Offset_Stop,&
         Total_Num_of_Observations,&
         Total_Num_of_Measurements,&
         Total_Num_of_Reference_Pulses,&
         Total_Num_of_Corrupt_Mie_Meas, Total_Num_of_Corrupt_Ray_Meas,&
         Total_Num_of_Corrupt_Mie_RefP, Total_Num_of_Corrupt_Ray_RefP,&
         Average_Error_FP_Response_A,   Average_Error_FP_Response_B)
    IF (error_flag .ne. no_error) return
    
    print *,'Handle_RBC_Headers: Filled SPH'
    
    !*** set the RBC Data Set Descriptor ***
    allocate(RBC%SPH%RBC_DSD,STAT=AllocStatus)
    IF (AllocStatus .ne. 0) THEN
       print *,"Allocation problem in Fill_RBC_Headers"
       error_flag = error_allocate
       return
    END IF
    
    call Init_DSD(RBC%SPH%RBC_DSD, error_flag)
    IF (error_flag .ne. no_error) return
    
    DS_Name    = RBC_DS_Name_RBC_ADS ! = "Rayleigh_Brillouin_ADS      "
    ! WARNING: in the L2B datafile it is "R" but here it is "A" !
    DS_Type    = RBC_DS_Type_RBC_ADS ! "A" 
    Ds_Offset  = GetByteSize_MPH_KVT() + &
                 GetByteSize_RBC_SPH_KVT() ! = 1865
    Num_Dsr    = 1_IntAl
    Dsr_Size   = GetByteSize_RBC_DS(RBC%RBC_DS) ! = 33771990
    
    call Fill_DSD_items(RBC%SPH%RBC_DSD, &
         DS_Name, DS_Type, DS_Offset, Num_Dsr, Dsr_Size, error_flag)
    
    !*** set the FP Data Set Descriptor ***
    allocate(RBC%SPH%FP_DSD,STAT=AllocStatus)
    IF (AllocStatus .ne. 0) THEN
       print *,"Allocation problem in Fill_RBC_Headers"
       error_flag = error_allocate
       return
    END IF
    
    call Init_DSD(RBC%SPH%FP_DSD, error_flag)
    IF (error_flag .ne. no_error) return
    
    DS_Name    = RBC_DS_Name_FP_MDS ! = "Fabry_Perot_MDS              "
    DS_Type    = RBC_DS_Type_FP_MDS ! = "R"
    Filename(:) = 'ISR_or_IAT_file'
    Ds_Offset  = Ds_Offset + Dsr_Size
    Num_Dsr    = 1_IntAl
    Dsr_Size   = 0
    
    call Fill_DSD_items(RBC%SPH%FP_DSD, &
         DS_Name, DS_Type, DS_Offset, Num_Dsr, Dsr_Size, error_flag, Filename)
    
    print *,'Handle_RBC_Headers: Filled DSD'
    
  end subroutine Fill_RBC_Headers
  !  #]
  !---------------------------------
end module WriteRBCdata
