program GenerateRBCdata
  !  #[ Documentation
  !     a program to read the RBC data in (old) binary format
  !     and write data in EE-CFI format.
  !     Based on TestWriteRBCdata written by J. de Kloe.
  !     written by: P. Poli
  !     created:    11-04-2006
  !
  !     Modifications:
  !     D. Tan,     03-May-2006 Rename T1_FP to TA_FP, T2_FP to TB_FP.
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     07-Aug-2006  D. Tan       Switch Model_Tenti to select i/o filenames
  !     30-Oct-2006  J. de Kloe   added r8_ to some constants 
  !                               and added the use_read_grid switch
  !     30-Jan-2007  P. Poli      modified text outputs to reflect value 
  !                               of the Model_Tenti switch
  !     30-Jul-2007  J. de Kloe   adapt to changed interface of Fill_FH_Items
  !     16-Oct-2007  P. Poli      change RBC default dates
  !     14-Nov-2007  J. de Kloe   decrease validity_period with 1 ms
  !     07-Dec-2007  J. de Kloe   adapted to changed FH/MPH interface
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_
  !     25-Feb-2008  D. Tan       New FixedHeader.System and MPH.Proc_Center
  !
  !  #]
  !  #[ Modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE WriteRBCdata, only: InitWriteRBCfileModule,WriteRBCfile,Fill_RBC_Headers
  USE RBC_DataStructure, only: RBC_DataType, Init_RBC_DataStructure, &
       Delete_RBC_DataStructure, Init_RBC_DS
  USE FixedHeader,  only: systemstr_LOCAL_MET_INST, &
       maxlen_file_class, maxlen_system
  USE MainProductHeader, only: maxlen_proc_stage, maxlen_proc_center

  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting
  USE Compiler_Features, only: newline
  USE ErrorHandler, only: no_error, program_abort, error_allocate, &
                          error_programming, request_program_to_stop
  USE Numerics, only: r8_
  USE EE_CFI_Datatypes, only: IntAs, PrintDateTime, SetDateTime, &
                              IntAl, IntAul, IntAus, IntAd, &
                              FAdoxy,  maxlen_DateTime_KVT, &
                              DateTimeType, Construct_KVT_DateTime, &
                              IncrementDateTime
  USE TentiSpectrum, only: read_tentispectrum, read_trans, read_grid
  USE LunManager, only: get_lun, free_lun
  USE Logging, only: logmsg,log_error
  USE GenerateRBC_settings, only : Model_Tenti, &
                                   set_default_GenRBC_settings, &
                                   get_cmdline_options_GenRBC, &
                                   set_derived_GenRBC_settings, &
                                   print_GenRBC_settings
  !  #]
  !  #[ local variables and parameters
  implicit none

  ! switches the gfortran compatibility reading code on
!  logical, parameter :: use_read_grid = .false.
  logical, parameter :: use_read_grid = .true.

  ! used to fill the FH
  character(len=maxlen_file_class) :: file_class
  TYPE(DateTimeType)               :: validity_start_datetime
  TYPE(DateTimeType)               :: validity_stop_datetime
  real(r8_)                        :: validity_period 
  integer                          :: file_version
  character(len=maxlen_system)     :: system
  ! used to fill the MPH
  character(len=maxlen_proc_stage)  :: proc_stage
  character(len=maxlen_proc_center) :: proc_center
  logical                           :: product_err
  ! used to fill the SPH
  integer(IntAs)   :: Num_P  ! =  23
  integer(IntAs)   :: Num_T  ! = 161
  integer(IntAs)   :: Num_F  ! = 937
  integer(IntAs)   :: Num_FP ! = 877
  integer(IntAs)   :: Num_fd ! =  61
  Integer(IntAs)   :: Num_RR ! = 101
  real(r8_)        :: P_min  ! in [Pa]
  real(r8_)        :: P_max  ! in [Pa]
  real(r8_)        :: T_min  ! in [K]
  real(r8_)        :: T_max  ! in [K]
  !
  real(FAdoxy)     :: Laser_Freq_Offset_Start
  real(FAdoxy)     :: Laser_Freq_Offset_Stop
  integer(IntAl)   :: Total_Num_of_Observations
  integer(IntAl)   :: Total_Num_of_Measurements
  integer(IntAl)   :: Total_Num_of_Reference_Pulses
  integer(IntAl)   :: Total_Num_of_Corrupt_Mie_Meas
  integer(IntAl)   :: Total_Num_of_Corrupt_Ray_Meas
  integer(IntAl)   :: Total_Num_of_Corrupt_Mie_RefP
  integer(IntAl)   :: Total_Num_of_Corrupt_Ray_RefP
  real(FAdoxy)     :: Average_Error_FP_Response_A
  real(FAdoxy)     :: Average_Error_FP_Response_B

  integer            :: error_flag, AllocStatus
  TYPE(RBC_DataType) :: RBC
  character(len=maxlen_DateTime_KVT) :: DateTimeString

  ! needed for loading RBC tables in old format
  integer :: nfreq,nP,nT,nfr,nRR,nfd,iP,iT,ifr,iRR,ifd,ifreq,iunit
  real(r8_), dimension(:), allocatable:: freq,TA,TB, RR0,fd,Fint_R
  real(r8_), dimension(:), allocatable:: Pgrid,Tgrid, freqtmp
  real(r8_), dimension(:,:,:), allocatable:: spec,Fcalib,Na_F,Nb_F
  character(len=100) :: filename_transi,filename_spec,filename_grid

  !  #]
  !  #[ program code
  print *,"starting GenerateRBCdata"

  call Set_Default_GenRBC_Settings

  call Get_Cmdline_Options_GenRBC(error_flag)
  IF (error_flag .ne.  no_error) THEN
     IF  (error_flag .ne. request_program_to_stop) &
          call logmsg(log_error,"aborted Get_Cmdline_Options_GenRBC")
     call program_abort(error_flag,"Get_Cmdline_Options_GenRBC")
  END IF

  call Set_Derived_GenRBC_Settings
  IF (error_flag .ne. no_error) THEN
     call logmsg(log_error,"aborted Set_Derived_GenRBC_Settings")
     call program_abort(error_flag,"Set_Derived_GenRBC_Settings")
  END IF

  ! print the settings defined in Set_GenRBC_Settings
  ! and any changes added to them by the command line options
  call Print_GenRBC_Settings

  !*** initialisations ***
  error_flag = no_error

!  call InitWriteRBCfileModule(error_flag)
  call InitWriteRBCfileModule(error_flag, ChoosenVerbosity=VerbosityReport)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitWriteRBCfileModule()")

  call Init_RBC_DataStructure(RBC,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DataStructure()")

  allocate(RBC%RBC_DS,STAT=AllocStatus)
  IF (AllocStatus .ne. 0) &
       call program_abort(error_allocate,"GenerateRBCdata")

  ! specify the dimensions of the look-up table
  Num_P  =  23
  Num_T  = 161
  Num_F  = 937
  Num_FP = 877
  Num_fd =  61
  Num_RR = 101
  ! allocate and initialise the whole structure
  call Init_RBC_DS(RBC%RBC_DS, Num_P, Num_T, Num_F, Num_FP, Num_fd, Num_RR, &
                   error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_RBC_DS")

  ! fill the datastructure with data from old format
       filename_transi(:)=' '
       if(Model_Tenti)then
          filename_transi='TransFPinterp.dat'
       else
          filename_transi='TransFPinterp_Gauss.dat'
       endif
       ! first read the size of this data
       call read_trans(nfreq,.true.,error_flag,filename=filename_transi)
       if (error_flag .ne. no_error) &
            call program_abort(error_flag,"read_trans")
       IF (nfreq .ne. Num_FP) then
          print *,'GenerateRBC: expected size ',Num_FP,' but found size ',nfreq
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       ! allocate arrays to hold this data
       allocate (freq(nfreq),TA(nfreq),TB(nfreq),stat=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"Allocation failure in subroutine GenerateRBCdata"
          error_flag = error_allocate
          call program_abort(error_flag,"GenerateRBC")
       END IF
       ! read the data into the allocated memory
       call read_trans(nfreq,.false.,error_flag,&
            TA=TA,TB=TB,f=freq,filename=filename_transi)
       if (error_flag .ne. no_error) &
            call program_abort(error_flag,"read_trans")
       DO ifreq=1,nfreq
         RBC%RBC_DS%TA_FP(ifreq)=TA(ifreq)
         RBC%RBC_DS%TB_FP(ifreq)=TB(ifreq)
         RBC%RBC_DS%F_FP (ifreq)=INT(freq(ifreq),IntAd)
       ENDDO
       deallocate(freq,TA,TB)

       print *,'GenerateRBCdata: loaded '//trim(filename_transi)

       filename_spec(:)=' '
       if(Model_Tenti)then
          filename_spec='TENTI_SPECTRUM.dat'
       else
          filename_spec='GAUSS_SPECTRUM.dat'
       endif
       ! first read the size of this data
       call read_TentiSpectrum(nP,nT,nfr,.true.,error_flag,filename=filename_spec)
       if (error_flag .ne. no_error) &
            call program_abort(error_flag,"read_TentiSpectrum")
       IF (nfr.ne. Num_F) then
          print *,'GenerateRBC: expected size ',Num_F,' but found size ',nfr
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nT.ne. Num_T) then
          print *,'GenerateRBC: expected size ',Num_T,' but found size ',nT
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nP.ne. Num_P) then
          print *,'GenerateRBC: expected size ',Num_P,' but found size ',nP
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       ! allocate arrays to hold this data
       allocate(Pgrid(nP),Tgrid(nT),freqtmp(nfr),spec(nP,nT,nfr), &
                stat=AllocStatus)
       IF (AllocStatus .ne. 0) THEN
          print *,"Allocation failure in subroutine GenerateRBCdata"
          error_flag = error_allocate
          call program_abort(error_flag,"GenerateRBC")
       END IF
       ! read the data into the allocated memory
       call read_TentiSpectrum(nP,nT,nfr,.false., error_flag,&
            Pgrid=Pgrid,Tgrid=Tgrid,freqtmp=freqtmp,spec=spec, &
            filename=filename_spec)
       if (error_flag .ne. no_error) &
            call program_abort(error_flag,"read_TentiSpectrum")
       DO iP=1,nP
         RBC%RBC_DS%P_Grid(iP)=INT(Pgrid(iP)*1.e2,IntAul)
       ENDDO
       DO iT=1,nT
         RBC%RBC_DS%T_Grid(iT)=INT(Tgrid(iT)*1.e2,IntAus)
       ENDDO
       DO ifr=1,nfr
         RBC%RBC_DS%F_Gridtmp(ifr)=INT(freqtmp(ifr),IntAd)
       ENDDO
       DO ip=1,nP
         DO iT=1,nT
           DO ifr=1,nfr
             RBC%RBC_DS%Spec_Grid_PTF(iP)%Spec_Grid_TF(iT)%Spec_Grid_F(ifr)= spec(iP,iT,ifr)
           ENDDO
         ENDDO
       ENDDO
       deallocate(freqtmp,spec)
       
       print *,'GenerateRBCdata: loaded '//trim(filename_spec)
       
       filename_grid(:)=' '
       if(Model_Tenti)then
          filename_grid='CALIB_GRID.dat'
       else
          filename_grid='CALIB_GRID_Gauss.dat'
       endif
       ! first read the size of this data
       nRR=101
       nfd=61
       IF (use_read_grid) THEN
          ! Poli : could not get this call to work properly 
          !        -- messes up memory.
          ! Paul, this works fine on my workstation with gfortran,
          ! could you please try again? Possibly the move of this program
          ! from RBC_FileHandling to RayleighBrillouinProcessing
          ! solved your problem?
          call read_grid(nP,nT,nRR,nfd,nfreq,.true.,error_flag,&
               filename=filename_grid)
          if (error_flag .ne. no_error) &
               call program_abort(error_flag,"read_grid")
       END IF

       !  #[ dimension checks and allocation
       IF (nfd .ne. Num_fd) then
          print *,'GenerateRBC: expected size ',Num_fd,' but found size ',nfd
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nRR .ne. Num_RR) then
          print *,'GenerateRBC: expected size ',Num_RR,' but found size ',nRR
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nfreq .ne. Num_FP) then
          print *,'GenerateRBC: expected size ',Num_FP,' but found size ',nfreq
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nT.ne. Num_T) then
          print *,'GenerateRBC: expected size ',Num_T,' but found size ',nT
          error_flag=error_programming
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       IF (nP.ne. Num_P) then
          print *,'GenerateRBC: expected size ',Num_P,' but found size ',nP
          error_flag=error_programming 
          call program_abort(error_flag,"GenerateRBC")
       ENDIF
       ! allocate arrays to hold this data
       allocate (fd(nfd),RR0(nRR),freq(nfreq),Fint_R(nRR),stat=AllocStatus)
       IF (AllocStatus .ne. no_error) THEN
          print *,"Allocation failure in subroutine GenerateRBCdata"
          error_flag = error_allocate
          call program_abort(error_flag,"GenerateRBC")
       END IF
       allocate (Fcalib(nP,nT,nRR),Na_F(nP,nT,nfd),Nb_F(nP,nT,nfd), &
            stat=AllocStatus)
       IF (AllocStatus .ne. no_error) THEN
          print *,"Allocation failure in subroutine GenerateRBCdata"
          error_flag = error_allocate
          call program_abort(error_flag,"GenerateRBC")
       END IF
       !  #]

       ! read the data into the allocated memory
       IF (use_read_grid) THEN
          ! Poli : could not get this call to work properly 
          !        -- messes up memory.
          call read_grid(nP,nT,nRR,nfd,nfreq,.false., error_flag,&
               Pgrid=Pgrid,Tgrid=Tgrid,RR0=RR0,fd=fd,freq=freq,Fcalib=Fcalib, &
               Na_F=Na_F, Nb_F=Nb_F, Fint_R=Fint_R,filename=filename_grid)
          if (error_flag .ne. no_error) &
               call program_abort(error_flag,"GenerateRBC")
       ELSE
          iunit = get_lun()
          open (iunit,FILE=trim(filename_grid),STATUS='old',FORM='UNFORMATTED')
          read (iunit) nP,nT,nRR,nfd,nfreq
          !print *,"nP,nT,nRR,nfd,nfreq = ",nP,nT,nRR,nfd,nfreq
          read (iunit) Pgrid
          read (iunit) Tgrid
          read (iunit) RR0
          read (iunit) fd
          read (iunit) freq
          read (iunit) Fcalib
          read (iunit) Na_F
          read (iunit) Nb_F
          read (iunit) Fint_R
          close (iunit)
          call free_lun(iunit)
       END IF

       DO iRR=1,nRR
         RBC%RBC_DS%RR(iRR)=RR0(iRR)
         RBC%RBC_DS%Fint_R(iRR)=INT(Fint_R(iRR),IntAd)
       ENDDO
       DO ifd=1,nfd
         RBC%RBC_DS%fd(ifd)=INT(fd(ifd),IntAd)
       ENDDO
       DO ip=1,nP
         DO iT=1,nT
           DO iRR=1,nRR
             RBC%RBC_DS%Fcalib_PTR(iP)%Fcalib_TR(iT)%Fcalib_R(iRR)=Fcalib(iP,iT,iRR)
           ENDDO
           DO ifd=1,nfd
             RBC%RBC_DS%Nab_PTFd(iP)%Nab_TFd(iT)%Na_Fd(ifd)= &
              & INT(Na_F(iP,iT,ifd),IntAul)
             RBC%RBC_DS%Nab_PTFd(iP)%Nab_TFd(iT)%Nb_Fd(ifd)= &
              & INT(Nb_F(iP,iT,ifd),IntAul)
           ENDDO
         ENDDO
       ENDDO
       deallocate(Pgrid,Tgrid,RR0,fd,Fcalib,freq)
      
       print *,'GenerateRBCdata: loaded '//trim(filename_grid)

  ! set some test values for FH input
  file_class = "TEST"
  file_version = 1
  system = systemstr_LOCAL_MET_INST
  if(Model_Tenti)then
    call SetDateTime(validity_start_datetime,2007,10,03,12,34,56.0_r8_)
  else
    call SetDateTime(validity_start_datetime,2007,10,26,12,34,56.0_r8_)
  endif
  validity_period = 1.0_r8_*365*24*60*60 - 0.001_r8_ ! 1 year in seconds
  ! NOTE: subtract 1 millisecond to make sure the ceil() rounds
  ! in the intended way. The ceil() used to get
  ! FH%Validity_Period%Validity_Stop might return with 1 second more
  ! due to machine inaccuracy ! (And this makes the result not portable!)

  validity_stop_datetime = validity_start_datetime
  call IncrementDateTime(validity_stop_datetime,validity_period)

  ! set some test values for MPH input
  ! proc_stage might be: N (near-real-time), R (reprocessing) or T (test)
  proc_stage  = "T"     ! indicating this is a only testfile
  proc_center = system(1:maxlen_Proc_Center)
                        ! the location where this file was produced
                        ! Copy (possibly truncated) of FixedHeader system
  product_err = .false. ! no errors in this test product

  ! set some test values for SPH input
  P_min          = minval(REAL(RBC%RBC_DS%P_Grid)) ! in [Pa]
  P_max          = maxval(REAL(RBC%RBC_DS%P_Grid)) ! in [Pa]
  T_min          = minval(REAL(RBC%RBC_DS%T_Grid)*1.e-2_r8_) ! in [K]
  T_max          = maxval(REAL(RBC%RBC_DS%T_Grid)*1.e-2_r8_) ! in [K]
  !
  Laser_Freq_Offset_Start       = minval(REAL(RBC%RBC_DS%F_FP,FAdoxy))*1.e-6 ! in [Mhz]
  Laser_Freq_Offset_Stop        = maxval(REAL(RBC%RBC_DS%F_FP,FAdoxy))*1.e-6 ! in [Mhz]
  Total_Num_of_Observations     = 0
  Total_Num_of_Measurements     = 0
  Total_Num_of_Reference_Pulses = 0
  Total_Num_of_Corrupt_Mie_Meas = 0
  Total_Num_of_Corrupt_Ray_Meas = 0
  Total_Num_of_Corrupt_Mie_RefP = 0
  Total_Num_of_Corrupt_Ray_RefP = 0
  Average_Error_FP_Response_A   = 0.
  Average_Error_FP_Response_B   = 0.

  ! some test prints
  call PrintDateTime(validity_start_datetime)
  DateTimeString = Construct_KVT_DateTime(validity_start_datetime)
  print *,"validity_start_datetime:KVT formatted = "//trim(DateTimeString)
       

  call Fill_RBC_Headers(RBC, &
       file_class, &                                      ! FH inputs
       validity_start_datetime, validity_stop_datetime, & ! FH inputs
       file_version, system, &                            ! FH inputs
       proc_stage, proc_center, product_err, &            ! MPH inputs
       Num_P, Num_T, Num_F, Num_FP, Num_fd, Num_RR, &     ! SPH inputs
       P_min, P_max, T_min, T_max, &                      ! SPH inputs
       Laser_Freq_Offset_Start, Laser_Freq_Offset_Stop,&  ! Ditto
       Total_Num_of_Observations,&
       Total_Num_of_Measurements,&
       Total_Num_of_Reference_Pulses,&
       Total_Num_of_Corrupt_Mie_Meas, Total_Num_of_Corrupt_Ray_Meas,&
       Total_Num_of_Corrupt_Mie_RefP, Total_Num_of_Corrupt_Ray_RefP,&
       Average_Error_FP_Response_A,   Average_Error_FP_Response_B,&
       error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Fill_RBC_Headers()")

  !call Init_L2B_PCD_ADS(L2B(iBRC)%PCD_ADS, &
  !        int(N_Meas,IntAs), int(N_Obs_Max_Rayleigh,IntAs), &
  !        int(N_Range_Gates,IntAs), error_flag)

  print *,'GenerateRBCdata: calling WriteRBCfile'
  call WriteRBCfile(RBC, error_flag)
  IF (error_flag .ne. no_error) call program_abort(error_flag,"WriteRBCfile")

  call Delete_RBC_DataStructure(RBC)

  print *,"done writing RBC test datafile"
  !  #]
end program GenerateRBCdata
