program Test_Rayl_Br_Proc
  !  #[ documentation
  !
  ! This program estimates the wind velocity according to the values
  ! of pressures Pmod, temperatures Tmod, measured Rayleigh channel
  ! responses RRmes, and scattering ratios rho.
  !
  !
  ! SUBROUTINES CALLED: 1
  ! subroutine Rayl_Brill_Proc()
  !
  ! History
  !     13-Jun-2005  Paul Poli METEO FRANCE   Original code
  !     28-Sep-2005  Paul Poli    Added dwind/dT, dwind/dP
  !     04-Nov-2005  J. de Kloe   simplified by adding all subroutines together
  !                               in a single module named TentiSpectrum
  !     09-Nov-2005  J. de Kloe   simplified the interface
  !     23-Dec-2005  D. Tan       Add dwind/dR
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     30-Jul-2007  P. Poli Make RBC dataflow explicit
  !     18-Oct-2007  J. de Kloe   added a nullify statement
  !     19-Dec-2007  P. Poli      added a nullify statement
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE RBC_DataStructure, only: RBC_DataType
  USE RayleighBrillouinProcessing, only: Rayl_Brill_Proc,&
                                         Unload_Rayl_Brill_Tables

  USE Numerics,      only: r8_,missing_indicator_real_r8
  USE ErrorHandler,  only: no_error, program_abort
  !  #]
  !  #[ variables
  implicit none

  ! variables needed to test the subroutine correction
  integer                          :: i,n
  real(r8_), dimension(:), pointer :: Pmod,Tmod,RRmes,rho,&
                                      corr0,corr0dR,corr0dT,&
                                      corr0dP,corr0drho
  integer                          :: error_flag
  TYPE(RBC_DataType)               :: RBC         ! input/output
  !  #]
  !  #[ program code
  ! init 
  nullify(Pmod,Tmod,RRmes,rho,corr0,corr0dR,corr0dT,corr0dP,corr0drho)
  nullify(RBC%RBC_DS)

  ! setup input profiles
  n=3
  allocate(Pmod(n),Tmod(n),RRmes(n),rho(n))
  allocate(corr0(n),corr0dR(n),corr0dT(n),corr0dP(n),corr0drho(n))
  Pmod=(/10._r8_,510._r8_,1010._r8_/)
  Tmod=(/200._r8_,250._r8_,300._r8_/)
  RRmes=(/-0.3_r8_,0._r8_,0.4_r8_/)
  rho=(/10._r8_,10._r8_,10._r8_/)
  print *,'Test_correction: ',n,' points in input for function correction'
  print *,' Pmod Tmod   RRmes  rho'
  print *,"REALACC(6)"
  do i=1,n
    print '(F6.1,1X,F5.1,1X,F6.3,1X,F7.3)',Pmod(i),Tmod(i),RRmes(i),rho(i)
  enddo
  print *,"ENDREALACC"

  ! call the subroutine Rayl_Brill_Proc whichn loads all tables at the first
  ! call and then calls correction()
  error_flag = no_error
  call Rayl_Brill_Proc(RBC,Pmod,Tmod,RRmes,rho,corr0,corr0dR,&
                       corr0dT,corr0dP,corr0drho,error_flag)
  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"correction")

  print *,"REALACC(6)"
  print '(A,3G20.12)',"corr0    (1:3)=",corr0(1:3)
  print '(A,3G20.12)',"corr0dR  (1:3)=",corr0dR(1:3)
  print '(A,3G20.12)',"corr0dT  (1:3)=",corr0dT(1:3)
  print '(A,3G20.12)',"corr0dP  (1:3)=",corr0dP(1:3)
  print '(A,3G20.12)',"corr0drho(1:3)=",corr0drho(1:3)
  print *,"ENDREALACC"

  ! free up memory
  deallocate(Pmod,Tmod,RRmes,rho,corr0,corr0dR,corr0dT,corr0dP,corr0drho)
  call Unload_Rayl_Brill_Tables(RBC)
  !  #]
end program Test_Rayl_Br_Proc

