PROGRAM CALIB_TENTI

  !
  ! This program calculates the Tenti spectrum for various values of 
  ! pressures, temperatures, and frequencies (freqtmp).
  ! The result is a 3D matrix of size (dim_P,dim_T,dim_freqtmp), stored in 
  ! the binary file TENTI_SPECTRUM.dat
  !
  ! This program needs to be run only once, 
  ! at the very beginning of the mission.
  !
  ! SUBROUTINES CALLED: 4
  ! function FSPEC_TENTI4grid
  ! subroutine write_TentiSpectrum  -> outputs 'TENTI_SPECTRUM.dat'
  ! function fcalcn
  ! function farray1
  !
  ! 13-Jun-2005 Paul Poli, METEO FRANCE    Original code from a Matlab
  !                                        code by M.-L. Denneulin
  ! 12-May-2006 D. Tan     Null use of ParKind1 module
  ! 26-Jul-2006 P. Poli    Changed call to FSPEC_TENTI4grid into
  !                        call to FSPEC_GAUSS4grid
  ! 04-Aug-2006 D. Tan     Switch Model_Tenti to select fspec_*4grid and
  !                        filename_spec (output filename)
  ! 16-Jan-2008 J. de Kloe phase out integer kind i_
  !

USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
USE Numerics, only: r8_
USE ErrorHandler, only: no_error, program_abort
USE Arraytools, only: fcalcn,farray1
USE AeolusConstants, only: FSR,USR,df,dP,Pmin,Pmax,dT,Tmin,Tmax
USE TentiSpectrum, only: FSPEC_TENTI4grid, WRITE_TENTISPECTRUM, FSPEC_GAUSS4grid

implicit none

integer :: nP,nT,nfr,error_flag
real(r8_), dimension(:),     allocatable :: Pgrid,Tgrid,freqtmp
real(r8_), dimension(:,:,:), allocatable :: spec
character(len=100)                       :: filename_spec 
                                          !,filename_trans ! never used
logical :: Model_Tenti ! Switch to select Tenti or Gauss model

print *,'calib_tenti: starting ...'

Model_Tenti = .true. ! .true. for Tenti, .false. for Gauss
                             ! TODO: accept as command line option?

nT =fcalcn(Tmin           , dT,  Tmax          +0.9_r8_*dT )
nP =fcalcn(Pmin           , dP,  Pmax          +0.9_r8_*dP )
nfr=fcalcn(-FSR-USR/2._r8_, df,  FSR+USR/2._r8_+0.9_r8_*df )
allocate(Pgrid(nP),Tgrid(nT),freqtmp(nfr))
Tgrid  =farray1(Tmin           ,dT,nT )
Pgrid  =farray1(Pmin           ,dP,nP )
freqtmp=farray1(-FSR-USR/2._r8_,df,nfr)

print *,'calib_tenti: nP,nT,nfr',nP,nT,nfr

allocate(spec(nP,nT,nfr))
!print *,'calib_tenti: calling fspec_tenti4grid'
! spec=FSPEC_TENTI4grid(nP,nT,nfr,freqtmp,Pgrid,Tgrid)
! replaced by:
if(Model_Tenti)then
   print *,'calib_tenti: calling fspec_tenti4grid'
   call FSPEC_TENTI4grid(nP,nT,nfr,freqtmp,Pgrid,Tgrid,spec,error_flag);
   if (error_flag .ne. no_error) &
      call program_abort(error_flag,"FSPEC_TENTI4grid")
else
   print *,'calib_tenti: calling fspec_gauss4grid'
   call FSPEC_GAUSS4grid(nP,nT,nfr,freqtmp,Pgrid,Tgrid,spec,error_flag);
   if (error_flag .ne. no_error) &
     call program_abort(error_flag,"FSPEC_GAUSS4grid")
endif

filename_spec=''
if(Model_Tenti)then
   filename_spec='TENTI_SPECTRUM.dat'
else
   filename_spec='GAUSS_SPECTRUM.dat'
endif
!
call write_TentiSpectrum(nP,nT,nfr,Pgrid,Tgrid,freqtmp,spec,&
                         error_flag,filename=filename_spec)
if (error_flag .ne. no_error) &
     call program_abort(error_flag,"write_TentiSpectrum")

deallocate(Pgrid,Tgrid,freqtmp,spec)

print *,'calib_tenti: over.'

END PROGRAM CALIB_TENTI

