module GenerateRBC_Settings
  !  #[ Documentation
  !
  ! this module defines the program settings as used by the 
  ! GenerateRBCdata program.
  !
  ! Written by David Tan (ECMWF)
  ! Based on l2b_proc_settings.F90 by J. de Kloe (KNMI)
  ! Created:     07-Aug-2006
  !
  ! Modifications:
  !    07-Aug-2007 D. Tan     Original version
  !    16-Jan-2008 J. de Kloe phase out integer kind i_
  !
  !  #] 
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE ErrorHandler, only: no_error, error_cmdline_options, &
                          request_program_to_stop
  !  #]
  !  #[ Variable and Parameter definitions
  implicit none
  !-------------------------------------------------------------
  ! list of choices/options for GenerateRBCdata
  !-------------------------------------------------------------
  character(len=256) :: RBCModelName
  !-------------------------------------------------------------
  ! derived switches
  !-------------------------------------------------------------
  ! switch controlling actions to be performed by the main program
  logical, save :: Model_Tenti
  !-------------------------------------------------------------
  !  #]
contains
  !-----------------------
  subroutine Set_Default_GenRBC_Settings
    !  #[

    ! these settings can be changed by setting the different
    ! commandline options
    ! or later by using the thin-layer xml files

    RBCModelName   = "Tenti"
    Model_Tenti    = .true.

  end subroutine Set_Default_GenRBC_Settings
    !  #]
  subroutine Get_Cmdline_Options_GenRBC(error_flag)
    !  #[
    USE Compiler_Features, only: iargc_aeolus,getarg_aeolus

    integer, intent(out) :: error_flag

    ! local variables
    integer :: nr_of_arguments, current_argument_nr
    character(len=256) :: current_argument

    error_flag = no_error
    nr_of_arguments = iargc_aeolus()

    IF (nr_of_arguments .lt. 1) THEN
       CALL Print_Cmdline_Options_GenRBC
       error_flag = request_program_to_stop
       return
    ENDIF

    current_argument_nr = 0  
    
    argloop: DO WHILE (current_argument_nr .lt. nr_of_arguments)
       !  #[ process the given commandline arguments

       current_argument_nr = current_argument_nr+1
       call getarg_aeolus(current_argument_nr,current_argument)

       SELECT CASE (trim(current_argument))
       CASE("-RBCModel")
          !  #[
          current_argument_nr = current_argument_nr+1
          IF (current_argument_nr .gt. nr_of_arguments) THEN
             print *,"RBCModelName is missing after -RBCModel option !!!"
             CALL Print_Cmdline_Options_GenRBC;
             error_flag = error_cmdline_options
             return
          END IF
          call getarg_aeolus(current_argument_nr,RBCModelName)
          select case (RBCModelName)
          case("Tenti")
             Model_Tenti = .true.
          case("Gauss")
             Model_Tenti = .false.
          case default
             print *,"unrecognised RBCModel: ",&
                  trim(RBCModelName)
             CALL Print_Cmdline_Options_GenRBC;
             error_flag = error_cmdline_options
             return
          end select
          print *,'using RBCModel: ',trim(RBCModelName)
          !  #]
       CASE DEFAULT
          !  #[
          CALL Print_Cmdline_Options_GenRBC;
          print *,"============================================"
          print *,"unrecognised commandline option: ",trim(current_argument)
          IF (current_argument(1:1) .ne. '-') THEN
             print *,"This argument does not start with: -"
             print *,"Maybe you forgot the - character to indicate that this"
             print *,"is an option, or maybe you supplied the wrong number"
             print *,"of items to an option."
          END IF
          print *,"============================================"
          error_flag = error_cmdline_options
          return
          !  #]
       END SELECT

       !  #]
    END DO argloop

  end subroutine Get_Cmdline_Options_GenRBC
    !  #]
  subroutine Set_Derived_GenRBC_Settings
    !  #[

    IF (RBCModelName.eq."Gauss")THEN
       Model_Tenti    = .false.
    ENDIF

  end subroutine Set_Derived_GenRBC_Settings
    !  #]
  subroutine Print_GenRBC_Settings
    !  #[

    ! print the Model_Tenti switch
    print *,"Model_Tenti = ", Model_Tenti

  end subroutine Print_GenRBC_Settings
    !  #]
  subroutine Print_Cmdline_Options_GenRBC
    !  #[
    print *,""
    print *,"Possible commandline options for the GenerateRBCData main program:"
    print *,""
    print *,"GenerateRBCData"
    print *,"   -RBCModel         [Tenti|Gauss]"
    print *,""
  end subroutine Print_Cmdline_Options_GenRBC
  !  #]
  !-----------------------
end module GenerateRBC_Settings
