#!/usr/bin/env python
#  #[ documentation
#
# a little script to check if a consistent version numbering has been
# applied. It should detect problems if for example a version number
# in the TaskTable.AE_L1B_L2B_WIND.xml file has not been updated
# after an increase in version number due to a new software release...
#
# written by: J. de Kloe, KNMI
#
# modifications:
#   09-May-2007 J. de Kloe  original version
#   08-Jun-2007 J. de Kloe  added checks for all JobOrder files in the L2BP
#   20-Feb-2008 J. de Kloe  adapted to changed L2B_ReportGenerator.py script
#   25-Feb-2008 J. de Kloe  added checks for the IODD version mentioned in our
#                           AuxPar files (MPH%Ref_Doc field) and for software
#                           release version used in the datapack name
#  #]
#  #[ imported modules
import os
from StringIO import StringIO

try:
    from xml.dom import minidom # xml document object model functions
    from xml import xpath
except:
    print "ERROR could not import python module: xml.dom or xml.xpath"    
    print "These module are needed for processing xml files."
    print "The CheckVersionNumbers.py script cannot be used without them."
    print "Please install the xml or PyXml python package."
    os._exit(1)
#  #]
#  #[ settings
# location of this script compared to the L2BP source root
# Note the we expect this script to be run from within the
# directory Test/Scripts
RootDir = "../../"

# files to be checked:
TestFile1 = "ThinLayer/TaskTable.AE_L1B_L2B_WIND.xml"
TestFile2 = "ThinLayer/WorkstationConfigurationFile.xml"
TestFile3 = "ThinLayer/order.5.xml"
TestFile4 = "support/aeolusconstants.F90"
TestFile5 = "Scripts/L2B_ReportGenerator.py"
TestFile6 = "install_binary_datapack"
#  #]
#  #[ check version numbers in file 1 (TaskTable)
File = os.path.join(RootDir,TestFile1)
print "testing file: "+File
try:
    ParsedFile = minidom.parse(StringIO(open(File).read()))
except:
    print "ERROR: could not open file: "+File
    print "This script is intended only for use in the automatic L2B test"
    print "suite and should always be run from within the directory:"
    print "$BUILD/Test/Scripts"
    print "are you sure your installation is correct?"
    print "(or did you try to execute this script from $BUILD/Scripts...)"
    os._exit(1)

# the is the processor version, which should match the version in the
# WorkStationConfiguration file, otherwise the ThinLayer will not work
Xpath = "/Task_Table/Version"
nodes = xpath.Evaluate(Xpath,ParsedFile)
Version_Processor_AE_L1B_L2B_WIND_TT = nodes[0].firstChild.nodeValue

# the version of the Cfg file is probably not very important(?)
Xpath = "/Task_Table/Private_Config/List_of_Cfg_Files/Cfg_File/Version"
nodes = xpath.Evaluate(Xpath,ParsedFile)
Version_Cfg_File = nodes[0].firstChild.nodeValue

# this is the version of the L2B_processor executable
# it should match the one mentioned in the fortran code
Xpath = "/Task_Table/List_of_Pools/Pool[1]/List_of_Tasks/Task/Version"
nodes = xpath.Evaluate(Xpath,ParsedFile)
Version_Task_IPF2B_L1B_L2B_TT = nodes[0].firstChild.nodeValue

# this is the version of the L2B_ReportGenerator
# it should match the one mentioned inside this generator
Xpath = "/Task_Table/List_of_Pools/Pool[2]/List_of_Tasks/Task/Version"
nodes = xpath.Evaluate(Xpath,ParsedFile)
Version_Task_IPF2B_REP_GEN_TT = nodes[0].firstChild.nodeValue

print "TaskTable results:"
print "Version_Processor_AE_L1B_L2B_WIND_TT  = ["+\
      Version_Processor_AE_L1B_L2B_WIND_TT+"]"
print "Version_Cfg_File                      = ["+\
      Version_Cfg_File+"]"
print "Version_Task_IPF2B_L1B_L2B_TT         = ["+\
      Version_Task_IPF2B_L1B_L2B_TT+"]"
print "Version_Task_IPF2B_REP_GEN_TT         = ["+\
      Version_Task_IPF2B_REP_GEN_TT+"]"
#  #]
#  #[ check version numbers in file 2 (WSconfig)
File = os.path.join(RootDir,TestFile2)
print "testing file: "+File
ParsedFile = minidom.parse(StringIO(open(File).read()))

Xpath = "/Proc_Table/List_of_Processors/Processor/Version"
nodes = xpath.Evaluate(Xpath,ParsedFile)

Version_Processor_AE_L1B_L2B_WIND_WSC = nodes[0].firstChild.nodeValue

print "WorkstationConfigurationFile results:"
print "Version_Processor_AE_L1B_L2B_WIND_WSC = ["+\
      Version_Processor_AE_L1B_L2B_WIND_WSC+"]"

#  #]
#  #[ check version numbers in file 3 (order)
File = os.path.join(RootDir,TestFile3)
print "testing file: "+File
ParsedFile = minidom.parse(StringIO(open(File).read()))

Xpath = "/Root/dbOrder/_processorVersion"
nodes = xpath.Evaluate(Xpath,ParsedFile)

Version_Processor_AE_L1B_L2B_WIND_ORD = nodes[0].firstChild.nodeValue

print "order file results:"
print "Version_Processor_AE_L1B_L2B_WIND_ORD = ["+\
      Version_Processor_AE_L1B_L2B_WIND_ORD+"]"

#  #]
#  #[ check version numbers in file 4 (aeolusconstants)

File = os.path.join(RootDir,TestFile4)
print "testing file: "+File

# retrieve the software version number
Command = "grep software_version "+File+\
          " | grep character | cut -d\'\"\' -f 2 > version.txt"
os.system(Command)

Version_Task_IPF2B_L1B_L2B_F90 = open("version.txt").read().replace("\n","")

Command = "\\rm -rf version.txt"
os.system(Command)

# retrieve the L2B IODD version number
Command = "grep -i 'iodd_version_L2B_N   =' "+File+\
          " | cut -d\'\"\' -f 2 > version.txt"
os.system(Command)
Version_IODD_L2BC = open("version.txt").read().replace("\n","")

Command = "\\rm -rf version.txt"
os.system(Command)

print "Fortran90 results:"
print "Version_Task_IPF2B_L1B_L2B_F90        = ["+\
      Version_Task_IPF2B_L1B_L2B_F90+"]"
print "iodd_version_L2B_N                    = ["+\
      Version_IODD_L2BC+"]"

#  #]
#  #[ check version numbers in file 5 (L2B_ReportGenerator)
File = os.path.join(RootDir,TestFile5)
print "testing file: "+File

Command = "grep ScriptVersion "+File+\
          " | grep only | cut -d\'\"\' -f 2 > version.txt"
os.system(Command)

Version_Task_IPF2B_REP_GEN_PY = open("version.txt").read().replace("\n","")

Command = "\\rm -rf version.txt"
os.system(Command)

print "Python results:"
print "Version_Task_IPF2B_REP_GEN_PY         = ["+\
      Version_Task_IPF2B_REP_GEN_PY+"]"
#  #]
#  #[ check version numbers in file 6 (install_binary_datapack)
File = os.path.join(RootDir,TestFile6)
print "testing file: "+File

for line in open(File).readlines():
    if (line[:3] == "set"):
        #print "line:"+line
        parts = line.split("Release")
        #print "parts: ",parts
        SecondPart = parts[1].replace("\n","")
        Version_DataPack =  "0"+SecondPart[:4]

print "install_binary_datapack, L2B release version = ["+\
      Version_DataPack+"]"
#  #]
#  #[ cross-checking the results
ProblemsFound = False

if (Version_Processor_AE_L1B_L2B_WIND_TT  !=
    Version_Processor_AE_L1B_L2B_WIND_WSC    ):
    print "ERROR in CheckVersionNumbers.py:"
    print "version numbering for Processor AE_L1B_L2B_WIND is inconsistent!"
    print "The TaskTable uses:      ["+\
          Version_Processor_AE_L1B_L2B_WIND_TT+"]"
    print "While the WSconfig uses: ["+\
          Version_Processor_AE_L1B_L2B_WIND_WSC+"]"
    ProblemsFound = True

if (Version_Processor_AE_L1B_L2B_WIND_TT  !=
    Version_Processor_AE_L1B_L2B_WIND_ORD    ):
    print "ERROR in CheckVersionNumbers.py:"
    print "version numbering for Processor AE_L1B_L2B_WIND is inconsistent!"
    print "The TaskTable uses:      ["+\
          Version_Processor_AE_L1B_L2B_WIND_TT+"]"
    print "While the order file uses: ["+\
          Version_Processor_AE_L1B_L2B_WIND_ORD+"]"
    ProblemsFound = True

if (Version_Task_IPF2B_L1B_L2B_TT !=
    Version_Task_IPF2B_L1B_L2B_F90  ):
    print "ERROR in CheckVersionNumbers.py:"
    print "version numbering for Task IPF2B_L1B_L2B is inconsistent!"
    print "The TaskTable uses:                  ["+\
          Version_Task_IPF2B_L1B_L2B_TT+"]"
    print "While the aeolusconstants file uses: ["+\
          Version_Task_IPF2B_L1B_L2B_F90+"]"
    ProblemsFound = True
    
if (Version_Task_IPF2B_REP_GEN_TT !=
    Version_Task_IPF2B_REP_GEN_PY   ):
    print "ERROR in CheckVersionNumbers.py:"
    print "version numbering for Task IPF2B_REP_GEN is inconsistent!"
    print "The TaskTable uses:                 ["+\
          Version_Task_IPF2B_REP_GEN_TT+"]"
    print "While the L2B_ReportGenerator uses: ["+\
          Version_Task_IPF2B_REP_GEN_PY+"]"
    ProblemsFound = True

if (Version_DataPack != Version_Task_IPF2B_L1B_L2B_F90):
    print "ERROR in CheckVersionNumbers.py:"
    print "version numbering for Task IPF2B_L1B_L2B is inconsistent!"
    print "The TaskTable uses:                          ["+\
          Version_Task_IPF2B_L1B_L2B_TT+"]"
    print "While the install_binary_datapack file uses: ["+\
          Version_DataPack+"]"
    ProblemsFound = True
#  #]
#  #[ check version numbers in all JobOrders in the L2BP
print "Checking all JobOrder files in this package ..."
Command = "find "+RootDir+\
          " | grep JobOrder | grep -i xml > list_of_JobOrderFiles.txt"
os.system(Command)

FileList = open("list_of_JobOrderFiles.txt").readlines()

for File in FileList:
    # remove the tailing newline character
    FileName = File.replace("\n","")
    ParsedFile = minidom.parse(StringIO(open(FileName).read()))

    Xpath = "/Ipf_Job_Order/Ipf_Conf/Processor_Name"
    nodes = xpath.Evaluate(Xpath,ParsedFile)
    Proc_Name = nodes[0].firstChild.nodeValue
    Xpath = "/Ipf_Job_Order/Ipf_Conf/Version"
    nodes = xpath.Evaluate(Xpath,ParsedFile)
    Proc_Version = nodes[0].firstChild.nodeValue
    # this check for Proc_Name excludes the L1B JobOrder
    # Matlab_Tools/CREATIONDATA/JobOrderRef.xml
    # which would not pass this test (and doesn't need to pass it)
    if (Proc_Name == "AE_L1B_L2B_WIND"):
        if (Proc_Version != Version_Processor_AE_L1B_L2B_WIND_TT):
            print "Error in file: "+FileName
            print "Version of Processor AE_L1B_L2B_WIND"
            print "should be:       "+Version_Processor_AE_L1B_L2B_WIND_TT
            print "but seems to be: "+Proc_Version
            ProblemsFound = True
        
    Xpath = "/Ipf_Job_Order/List_of_Ipf_Procs[1]/Ipf_Proc"
    nodes = xpath.Evaluate(Xpath,ParsedFile)

    for Node in nodes:
        Xpath = "Task_Name"
        nodes = xpath.Evaluate(Xpath,Node)
        Task_Name = nodes[0].firstChild.nodeValue
        Xpath = "Task_Version"
        nodes = xpath.Evaluate(Xpath,Node)
        Task_Version = nodes[0].firstChild.nodeValue

        if (Task_Name == "IPF2B_L1B_L2B"):
            if (Task_Version != Version_Task_IPF2B_L1B_L2B_TT):
                print "Error in file: "+FileName
                print "Version of Task_Name: "+Task_Name
                print "should be:       "+Version_Task_IPF2B_L1B_L2B_TT
                print "but seems to be: "+Task_Version
                ProblemsFound = True
        elif ( (Task_Name == "IPF2B_REP_GEN") or
               (Task_Name == "IPF2_REP_GEN")   ):
            if (Task_Version != Version_Task_IPF2B_REP_GEN_PY):
                print "Error in file: "+FileName
                print "Version of Task_Name: "+Task_Name
                print "should be:       "+Version_Task_IPF2B_REP_GEN_PY
                print "but seems to be: "+Task_Version
                ProblemsFound = True
        elif ( (Task_Name == "IPF1_RAW_L0") or
               (Task_Name == "IPF1_L0_L1A") or
               (Task_Name == "IPF1_L1A_L1B") or
               (Task_Name == "IPF1_REP_GEN") or
               (Task_Name == "IPF2_L1B_L2A") ):
            # pass: these are L1B and L2A software version numbers, so
            #       checking their consistency is not our job
            pass
        else:
            print "testing file: "+FileName
            print "Task_Name = "+Task_Name
            print "Task_Version = "+Task_Version
            print "No check implemented for this case"
            ProblemsFound = True
                
Command = "\\rm -rf list_of_JobOrderFiles.txt"
os.system(Command)
#  #]
#  #[ check version numbers in all AuxPar files in the L2BP
print "Checking all AuxPar files in this package ..."
Command = "find "+RootDir+\
          " | grep AUX_PAR | grep -i eef > list_of_AuxParFiles.txt"
os.system(Command)

FileList = open("list_of_AuxParFiles.txt").readlines()

for File in FileList:
    # remove the tailing newline character
    FileName = File.replace("\n","")
    ParsedFile = minidom.parse(StringIO(open(FileName).read()))

    Xpath = "/Earth_Explorer_File/Earth_Explorer_Header/Variable_Header/"+\
            "Main_Product_Header/Ref_Doc"
    nodes = xpath.Evaluate(Xpath,ParsedFile)
    Ref_Doc = nodes[0].firstChild.nodeValue
    AuxPar_IODD_Version = Ref_Doc[-5:]
    if (AuxPar_IODD_Version != Version_IODD_L2BC):
        print "Error in file: "+File
        print "Ref_Doc in this AuxPar file should refer to the current IODD"
        print "version used by the Processor:"
        print "it should be:    "+Version_IODD_L2BC
        print "but seems to be: "+AuxPar_IODD_Version
        print "Ref_Doc = "+Ref_Doc
        ProblemsFound = True
                
Command = "\\rm -rf list_of_AuxParFiles.txt"
os.system(Command)
#  #]
#  #[ print a message about success or failure
if (ProblemsFound):
    print "Please fix these inconsistencies before delivering the software ..."
else:
    print "All version numbers seem consistent"
#  #]
#  #[ some old code using minidom nodes
#print "result:",tree.find("List_of_Pools/Pool#0/List_of_Tasks/Version").text

# this would be the way using minidom nodes directly ...
#e = ParsedFile.documentElement
#print "root tag of this document is: ",e.localName
#ProcNameNode = e.getElementsByTagName("Processor_Name")
#VersionNode  = e.getElementsByTagName("Version")
#print "len(VersionNode) = ",len(VersionNode)
#print "Version = ",VersionNode[0].firstChild.nodeValue
#  #]
