#!/bin/csh

echo "checking python installation"

# first test for presence of the /usr/bin/env utility
/usr/bin/env >& /dev/null
if ( $status != 0) then
  echo "ERROR: the /usr/bin/env utility seems not present on this machine"
  echo "The L2B_ReportGenerator.py script cannot be used directly in "
  echo "this case."
  echo "If you are sure python is installed, edit the L2B_ReportGenerator.py"
  echo "and indicate the path of the python interpreter in the first line"
  echo "of this file."
  exit
endif

# remark: removing it explicitely is needed in case the user has
# set the noclobber variable, which prohibits overwriting existing
# files by redirected output.
\rm -f python_version_file.txt

# version information seems to be written to stderr
# so using var = `python -V` seems not to work ...
/usr/bin/env python -V >& python_version_file.txt

if ( $status != 0) then
  echo "ERROR: no python installation found on this machine"
  echo "The L2B_ReportGenerator.py script cannot be used in this case."
  exit
endif

echo "checking python version"
set python_version = `cat python_version_file.txt`
set bare_version   = `echo $python_version | cut -d ' ' -f 2- | tr -d ' '`
set major_version_number = `echo $bare_version | cut -d '.' -f 1`

#echo "python_version: $python_version"
#echo "bare_version: $bare_version"
#echo "major_version_number: $major_version_number"

if ( $major_version_number != '2' ) then
  echo "WARNING: python version $bare_version is too old"
  echo "The L2B_ReportGenerator.py script may not function properly"
  echo "in this case."
else
  echo "python version okay"
endif

./Check_Python_Modules.py

echo "python check finished"
