#!/usr/bin/env python
#  #[ documentation
#
# experimental script to:
# -merge a copy of a ClearCase branch to a cvs branch or vice versa
# -adapted to handle cvs, perforce as well
# -adapted to merge two branches not in version control as well
#
# Written by: Jos de Kloe (KNMI)
# Created:     03-Oct-2005
# last change: 23-Apr-2007
#  #]
#  #[ imported modules
import sys                  # system facilities
import os                   # operating system
import stat                 # operating system
#  #]
#  #[ settings and constants

# pathname settings CC=>tarfile=>CVS
#FROM_BRANCH_DIR = "/nobackup/users/kloedej/temp_ectrans/clearcasedir"
#FROM_BRANCH_DIR = "/nobackup/users/kloedej/temp_ectrans/"+\
#                  "nl8_CY30R1_reorg_makefiles/aeolus"
#TO_BRANCH_DIR = "/nobackup/users/kloedej/versiebeheer/adm/L2B_processor"

# pathname settings CVS=>tarfile=>CC
#FROM_BRANCH_DIR  = "/scratch/ms/nl/nl8/export/adm/L2B_processor"
#TO_BRANCH_DIR = "/ccvobs/aeolus"

# pathname settings tarfile=>perforce
#FROM_BRANCH_DIR = "/scratch/ms/nl/nl8/"+\
#                  "dat_CY30R1_sensref_plus_extinction_clean/aeolus"
#TO_BRANCH_DIR = "/scratch/ms/nl/nl8/p4w/nl8_CY30R1_NewExtCalc/aeolus"

# pathname settings tarfile=>tarfile
#FROM_BRANCH_DIR = "/nobackup/users/kloedej/temp_ectrans/"+\
#                  "dat_CY31R1_Dec07_g95bufr_clean/aeolus"
#TO_BRANCH_DIR   = "/nobackup/users/kloedej/temp_ectrans/"+\
#                  "dat_CY31R1_Jan25_IncludeBufr_clean/aeolus"
FROM_BRANCH_DIR = "/home/jos/werk/"+\
                  "nl8_CY31R1_Mar16_clean/aeolus"
TO_BRANCH_DIR   = "/home/jos/werk/"+\
                  "dat_CY31R1_Mar16_Expected/aeolus"

# to add: a switch to enable merging *.expected* files
#         and to set startdir to Test

StartDir = ""
#StartDir = "Test"

# some constants used by the code
FileType    = 1
DirType     = 2
NotFound    = 3
SymLinkType = 4

BRANCH_TYPE_CC  = 1 # clearcase
BRANCH_TYPE_CVS = 2 # cvs
BRANCH_TYPE_PER = 3 # perforce
BRANCH_TYPE_NVC = 4 # not version controlled

ReportFile = "MergeReport.txt"

DiffCommand  = "diff"
#XDiffCommand = "xdiff"
# on my laptop this should be:
#XDiffCommand = "xxdiff"
# on ecgate this may be xdiff (you get the clearcase xdiff)
# or it may be tkdiff (my preferred one)
#XDiffCommand = "/usr/local/apps/tkcvs/current/bin/tkdiff"
# on my linux workstation: xdiff and xxdiff are aliasses:
XDiffCommand = "xxdiff"

quote = '"'

Orig_Extension  = ".orig"
Merge_Extension = ".merge"
#  #]
#  #[ inspect branch types

CVS_DIR = os.path.join(FROM_BRANCH_DIR,"CVS")
if (os.path.exists(CVS_DIR)):
    FROM_BRANCH_TYPE = BRANCH_TYPE_CVS
else:
    if (FROM_BRANCH_DIR[:8] == "/ccvobs/"):
        FROM_BRANCH_TYPE = BRANCH_TYPE_CC
    else:
        if (FROM_BRANCH_DIR[:23] == "/scratch/ms/nl/nl8/p4w/"):
            FROM_BRANCH_TYPE = BRANCH_TYPE_PER
	else:
            FROM_BRANCH_TYPE = BRANCH_TYPE_NVC

CVS_DIR = os.path.join(TO_BRANCH_DIR,"CVS")
if (os.path.exists(CVS_DIR)):
    TO_BRANCH_TYPE = BRANCH_TYPE_CVS
else:
    if (TO_BRANCH_DIR[:8] == "/ccvobs/"):
        TO_BRANCH_TYPE = BRANCH_TYPE_CC
    else:
        if (TO_BRANCH_DIR[:23] == "/scratch/ms/nl/nl8/p4w/"):
            TO_BRANCH_TYPE = BRANCH_TYPE_PER
	else:
            TO_BRANCH_TYPE = BRANCH_TYPE_NVC

#print "FROM_BRANCH_TYPE = ",FROM_BRANCH_TYPE
#print "TO_BRANCH_TYPE = ",TO_BRANCH_TYPE
#print "len=",len("/scratch/ms/nl/nl8/p4w/")
#print "test="+FROM_BRANCH_DIR[:23]
#print "test="+TO_BRANCH_DIR[:23]
#os._exit(1)

#  #]
#  #[ sanity checks
if (FROM_BRANCH_TYPE != BRANCH_TYPE_NVC):
    print "merging branches usually starts from an unpacked tarball"
    print "not from within a cvs/cc/perforce repository....."
    print "FROM_BRANCH_TYPE = ",FROM_BRANCH_TYPE
    print "TO_BRANCH_TYPE = ",TO_BRANCH_TYPE    
    os._exit(1)

if (not os.path.exists(FROM_BRANCH_DIR)):
    print "Error: FROM_BRANCH_DIR does not exists or not accessible..."
    print "FROM_BRANCH_DIR="+FROM_BRANCH_DIR
    print "please check your path settings ..."
    os._exit(1)
    
if (not os.path.exists(TO_BRANCH_DIR)):
    print "Error: TO_BRANCH_DIR does not exists or not accessible..."
    print "TO_BRANCH_DIR="+TO_BRANCH_DIR
    print "please check your path settings ..."
    os._exit(1)

if (TO_BRANCH_TYPE == BRANCH_TYPE_CC | 
    TO_BRANCH_TYPE == BRANCH_TYPE_PER):
    if (os.getcwd() != TO_BRANCH_DIR):
        #  #[ print some help
        print "Current directory should be: "+TO_BRANCH_DIR
        print "before entering this script."
        print "First enter the clearcase or perforce branch of your choice"
        print "by entering the following commands on the commandline:"
        print ""
        print "[for clearcase]"
        print ""
        print "   newgrp ifs"
        print "   use rdcc"
        print "   selview -s aeolus"
        print "   [now select a view from the list]"
        print "   tcsh"
        print "   cd ../aeolus"
        print "   ~/copy_branch.py"
        print ""
        print "[for perforce]"
        print "   q2 select_client"
        print "   [now select a view from the list]"
        print "   tcsh"
        print "   ~/copy_branch.py"
        print ""
        os._exit(1)
        #  #]
#  #]
#  #[ discarded files and directories

# list of discared files/directories (full names)
DiscardedFiles = [ "clear_case_howto_personal_notes",
                   "CombineAllObjects.py",
                   "convert_GENSCAT_to_AEOLUS2B_in_Set_Makeoptions",
                   "doen_voor_elke_clearcase_checkin.txt",
                   "ee_cfi_software",
                   "ee_cfi_software_3.2",
                   "ee_cfi_software_3.3",
                   "ee_cfi_software_3.4",
                   "input_rpt/run",
                   "input_rpt/plot_backscatterdata.pro",
                   "input_rpt/xxx",
                   "L1B_file_handling/l1bpDraftProduct",
                   "L1B_file_handling/l1bDraftProduct_051013_dlr",
                   "L1B_file_handling/l1bdraftproduct_051013_dlr.tar",
                   "L1B_file_handling/l1bdraftproduct_051013_dlr.tar.gz",
                   "laptop_setenv_for_g95",
                   "lost+found",
                   "main/CALIB_GRID.dat",
                   "main/TENTI_SPECTRUM.dat",
                   "main/TransFPinterp.dat",
                   "Makeoptions",
                   "MergeReport.txt",
                   "MolScat/plot_testprofile.pro",
                   "MolScat/run",
                   "MolScat/xxx",
                   "RayleighBrillouinProcessing/CALIB_GRID.dat",
                   "RayleighBrillouinProcessing/CALIB_GRID.dat.gz",
                   "RayleighBrillouinProcessing/plot.pro",
                   "RayleighBrillouinProcessing/REMARK.TXT",
                   "RayleighBrillouinProcessing/run",
                   "RayleighBrillouinProcessing/TENTI_SPECTRUM.dat",
                   "RayleighBrillouinProcessing/TENTI_SPECTRUM.dat.gz",
                   "RayleighBrillouinProcessing/TransFPinterp.dat",
                   "RayleighBrillouinProcessing/xxx",
                   "Scripts/convert_GENSCAT_to_AEOLUS2B_in_Set_Makeoptions",
                   "Scripts/do_filecount.sc",
                   "Scripts/test_all_compilers",
                   "Scripts/Makeoptions.g95.extra_debug_options1",
                   "Scripts/Makeoptions.g95.extra_debug_options2",
                   "support/parkind1.F90",
                   "Systemsettings",
                   "Test/testresults.accumulated.g95.ee_cfi",
                   "Test/testresults.accumulated.g95.simple_xml",
                   "Test/testresults.accumulated.gfortran.simple_xml",
                   "Test/testresults.accumulated.ifort.simple_xml",
                   "Test/testresults.accumulated.pgf90.simple_xml",
                   "Test/testresults.accumulated.pgf90.ee_cfi",
                   "Test/RayleighBrillouinProcessing/CALIB_GRID.dat",
                   "Test/RayleighBrillouinProcessing/TENTI_SPECTRUM.dat",
                   "Test/RayleighBrillouinProcessing/TransFPinterp.dat",
                   "xml_module" ] 

# list of discared file extensions
DiscardedExtensions = [ ".contrib",
                        ".contrib.1",
                        ".contrib.2",
                        ".contrib.3",
                        ".genscat",
#                        ".expected",
                        ".expected.LINUX_pgf90",
                        ".expected.LINUX_g95",
                        ".expected.LINUX_f95",
                        ".expected.AIX_f90",
                        "CVS",
                        "Makefile",
                        ".mod",
                        ".merge",
                        ".orig",
                        "~",
                        ".orig",
                        ".a",
                        ".o" ]
#  #]
#------------------------------------
# subroutine definitions
#------------------------------------
def AddDir(Full_TO_Path,Dir,TO_BRANCH_TYPE):
    #  #[
    if (TO_BRANCH_TYPE == BRANCH_TYPE_CC):
        CcAddDir(Full_TO_Path,Dir)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_CVS):
        CvsAddDir(Full_TO_Path,Dir)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_PER):
        PerAddDir(Full_TO_Path,Dir)
    else:
        GenAddDir(Full_TO_Path,Dir)        
    #  #]
def ReplaceFile(Full_TO_Path,pathname_FROM,pathname_TO,File):
    #  #[
    if (TO_BRANCH_TYPE == BRANCH_TYPE_CC):
        CcReplaceFile(Full_TO_Path,pathname_FROM,
                      pathname_TO,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_CVS):
        CvsReplaceFile(Full_TO_Path,pathname_FROM,
                       pathname_TO,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_PER):
        PerReplaceFile(Full_TO_Path,pathname_FROM,
                       pathname_TO,File)
    else:
        GenReplaceFile(Full_TO_Path,pathname_FROM,
                       pathname_TO,File)
    #  #]
def AddFile(Full_TO_Path,pathname_FROM,pathname_TO,File):
    #  #[
    if (TO_BRANCH_TYPE == BRANCH_TYPE_CC):
        CcAddFile(Full_TO_Path,pathname_FROM,pathname_TO,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_CVS):
        CvsAddFile(Full_TO_Path,pathname_FROM,pathname_TO,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_PER):
        PerAddFile(Full_TO_Path,pathname_FROM,pathname_TO,File)
    else:
        GenAddFile(Full_TO_Path,pathname_FROM,pathname_TO,File)
    #  #]
def RmFile(Full_TO_Path,File):
    #  #[
    if (TO_BRANCH_TYPE == BRANCH_TYPE_CC):
        CcRmFile(Full_TO_Path,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_CVS):
        CvsRmFile(Full_TO_Path,File)
    elif (TO_BRANCH_TYPE == BRANCH_TYPE_PER):
        PerRmFile(Full_TO_Path,File)
    else:
        GenRmFile(Full_TO_Path,File)
    #  #]        
#------------------------------------
def GenAddDir(Full_Path,Dir):
    #  #[
    Cmd1 = "cd "+Full_Path+";"
    Cmd2 = "mkdir "+Dir+";"
    Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def GenReplaceFile(Full_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_Path+";"
    Cmd2 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def GenAddFile(Full_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_Path+";"
    Cmd2 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)    
    #  #]
def GenRmFile(Full_Path,File):
    #  #[
    Cmd1 = "cd "+Full_Path+";"
    Cmd2 = "rm -f "+File+";"
    Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
#------------------------------------
def PerAddDir(Full_PER_Path,Dir):
    #  #[
    Cmd1 = "cd "+Full_PER_Path+";"
    Cmd2 = "mkdir "+Dir+";"
    Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def PerReplaceFile(Full_PER_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_PER_Path+";"
    Cmd2 = "p4 edit "+File+";"
    Cmd3 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd = Cmd1+Cmd2+Cmd3
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def PerAddFile(Full_PER_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_PER_Path+";"
    Cmd2 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd3 = "p4 add "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3
    print "Executing command: "+Cmd
    os.system(Cmd)    
    #  #]
def PerRmFile(Full_PER_Path,File):
    #  #[
    Cmd1 = "cd "+Full_PER_Path+";"
    Cmd2 = "p4 delete "+File+";"
    Cmd3 = "rm -f "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
#------------------------------------
def CcAddDir(Full_CC_Path,Dir):
    #  #[
    Comment = raw_input("Give a comment describing this change:")
    Cmd1 = "cd "+Full_CC_Path+";"
    Cmd2 = "cleartool co -nc .;"
    Cmd3 = "cleartool mkdir -c "+quote+Comment+quote+" ./"+Dir+";"
    Cmd4 = "cleartool protect -chmod 770 "+Dir+";"
    Cmd5 = "cleartool ci -nc .;"
    Cmd = Cmd1+Cmd2+Cmd3+Cmd4+Cmd5
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def CcReplaceFile(Full_CC_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Comment = raw_input("Give a comment describing this change:")
    Cmd1 = "cd "+Full_CC_Path+";"
    Cmd2 = "cleartool co -nc "+File+";"
    Cmd3 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd4 = "cleartool ci -c "+quote+Comment+quote+" "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3+Cmd4
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def CcAddFile(Full_CC_Path,pathname_FROM,pathname_TO,File):
    #  #[
    # adding comments seems not possible with the addfile script
    #Comment = raw_input("Give a comment describing this new file:")
    Cmd1 = "cd "+Full_CC_Path+";"
    Cmd2 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd3 = "addfile -f "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3
    print "Executing command: "+Cmd
    os.system(Cmd)    
    #  #]
def CcRmFile(Full_CC_Path,File):
    #  #[
    Comment = raw_input("Give a comment describing why this file is removed:")
    Cmd1 = "cd "+Full_CC_Path+";"
    Cmd2 = "cleartool co -nc .;"
    Cmd3 = "cleartool rmname "+File+";"
    Cmd4 = "cleartool ci -c "+quote+Comment+quote+" . ;"
    Cmd5 = "rm -f "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3+Cmd4+Cmd5
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
#------------------------------------
def CvsKnown(Full_CVS_Path,File):
    #  #[
    pathname = os.path.join(Full_CVS_Path, File)
    IsKnown = False
    if (os.path.exists(pathname)):
        mode = os.stat(pathname)[stat.ST_MODE]
        if (stat.S_ISREG(mode)):
            # this seems a regular file
            if (not os.access(pathname,os.W_OK)):
                # the file has no write permission, which is what we expect
                # unless the file is already checked out
                # (should we check that too?)
                IsKnown = True
    return IsKnown
    #  #]
def CvsAddDir(Full_CVS_Path,Dir):
    #  #[
    # first create the Dir
    Cmd1 = "cd "+Full_CVS_Path+";"
    Cmd2 = "mkdir "+Dir
    Cmd  = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    # then make a dummy file in it, and check it in
    Cmd1 = "cd "+Full_CVS_Path+"/"+Dir+";"
    Cmd2 = "touch dummy ;"
    Cmd3 = "vbci dummy "+"'"+"dummy comment"+"'"
    Cmd  = Cmd1+Cmd2+Cmd3
    print "Executing command: "+Cmd
    os.system(Cmd)
    # then remove the dummy file again
    Cmd1 = "cd "+Full_CVS_Path+";"
    Cmd2 = "vbrm dummy "+"'"+"dummy comment"+"'"
    Cmd  = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def CvsReplaceFile(Full_CVS_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_CVS_Path+";"
    Cmd2 = "vbco "+File+";"
    Cmd3 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd4 = "vbci "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3+Cmd4
    print "Executing command: "+Cmd
    os.system(Cmd)
    #  #]
def CvsAddFile(Full_CVS_Path,pathname_FROM,pathname_TO,File):
    #  #[
    Cmd1 = "cd "+Full_CVS_Path+";"
    Cmd2 = "cp -f "+pathname_FROM+" "+pathname_TO+";"
    Cmd3 = "chmod u+w "+File+";"
    Cmd4 = "vbci "+File+";"
    Cmd = Cmd1+Cmd2+Cmd3+Cmd4
    print "Executing command: "+Cmd
    os.system(Cmd)    
    #  #]
def CvsRmFile(Full_CVS_Path,File):
    #  #[

    if (CvsKnown(Full_CVS_Path,File)):
        print "removing a cvs controlled file using our own vbrm script"
        Cmd1 = "cd "+Full_CVS_Path+";"
        Cmd2 = "vbrm "+File+";"
        Cmd = Cmd1+Cmd2
    else:
        print "removing a normal, not cvs controlled file"
        Cmd1 = "cd "+Full_CVS_Path+";"
        Cmd2 = "rm -f "+File+";"
        Cmd = Cmd1+Cmd2
    print "Executing command: "+Cmd
    os.system(Cmd)

    #  #]
#------------------------------------
def GetObjectType(Object):
    #  #[
    if (os.path.exists(Object)):
        mode_object = os.lstat(Object)[stat.ST_MODE]
        if (stat.S_ISLNK(mode_object)):
            ObjectType = SymLinkType
        else:
            if (stat.S_ISDIR(mode_object)):
                ObjectType = DirType
            else:
                ObjectType = FileType
    else:
        ObjectType = NotFound
    return ObjectType
    #  #]
#------------------------------------
def DescendDirs(Dir,fd):
    #  #[ process this directory, and descend into its subdirs

    #  #[ construct a list with all files to consider
    #
    # define both directory paths
    Full_FROM_Path = os.path.join(FROM_BRANCH_DIR,Dir)
    Full_TO_Path   = os.path.join(TO_BRANCH_DIR,  Dir)
    #
    # get the filelists from both branches
    if (os.path.exists(Full_FROM_Path)):
        FileList_FROM  = os.listdir(Full_FROM_Path)
    else:
        FileList_FROM  = []
    #
    if (os.path.exists(Full_TO_Path)):
        FileList_TO   = os.listdir(Full_TO_Path)
    else:
        FileList_TO   = []
    #
    # construct a list with all files in both branches
    FileList = FileList_FROM
    for File in FileList_TO:
        if (FileList.count(File) == 0):
            FileList.append(File)
    #  #]    
    for File in FileList:
        #  #[ exceptions

        DiscardFile = False
        for ext in DiscardedExtensions:
            if (File[-len(ext):] == ext):
                DiscardFile = True

        if (DiscardFile):
            #print "Discarding file due to extension: "+File
            continue # dont process this file
        
        RelativePath  = os.path.join(Dir,File)
        if (RelativePath in DiscardedFiles):
            #print "Discarding File: "+RelativePath
            #fd.write("Discarding File: "+RelativePath+"\n")
            continue # dont process this file

        #  #]
        #  #[ is it a file or a dir?
        pathname_FROM = os.path.join(Full_FROM_Path,File)
        pathname_TO   = os.path.join(Full_TO_Path,  File)
        Item_FROM = GetObjectType(pathname_FROM)
        Item_TO   = GetObjectType(pathname_TO)

        if (Item_FROM == SymLinkType):
            continue # dont process this symlink

        if (Item_TO == SymLinkType):
            continue # dont process this symlink

        #  #]
        if (Item_FROM == DirType):
            #  #[

            if (Item_TO == DirType):
                print "FROM Dir: "+pathname_FROM
                print "TO   Dir: "+pathname_TO
                DescendDirs(os.path.join(Dir,File),fd)
            elif (Item_TO == FileType):
                print "Error: conflicting names"
                print "Dir name in FROM tree seems identical to "+\
                      "File name in TO tree"
                print "FROM Dir: "+pathname_FROM
                print "TO File: "+pathname_TO
                print "Merge failed"
                Answer = raw_input("Continue with the rest of this tree?[y/n]")
                if (Answer[0] == 'n'):
                    os._exit(1)
            else: # Item_TO == NotFound
                print "This Dir is not present in the TO tree"
                print "FROM Dir: "+pathname_FROM
                Answer = raw_input("Should I add it? [y/n]")
                if (Answer[0] == 'y'):
                    print "Adding it..."
                    AddDir(Full_TO_Path,File,TO_BRANCH_TYPE)
                    fd.write("Adding Dir: "+pathname_TO+"\n")
                    DescendDirs(os.path.join(Dir,File),fd)
                else: # Answer[0] == 'n'
                    fd.write("Not added Dir: "+pathname_TO+"\n")

            #  #]
        elif (Item_FROM == FileType):
            #  #[
            if (Item_TO == FileType):
                #print "File: "+File    
                Cmd = DiffCommand+" "+\
                      pathname_FROM+" "+pathname_TO+" > /dev/null"
                DiffResult = os.system(Cmd)
                if (DiffResult == 0):
                    #print "Files seem identical, nothing to do"
                    pass
                else:
                    print "Files differ: "+File
                    Cmd = XDiffCommand+" "+\
                          pathname_FROM+" "+pathname_TO
                    os.system(Cmd)
                    Answer = raw_input("Should I replace the TO file "+\
                                       "with the FROM file? [y/n/m]")
                    if (Answer[0] == 'y'):
                        print "Replacing it..."
                        ReplaceFile(Full_TO_Path,pathname_FROM,
                                    pathname_TO,File)
                        fd.write("Replaced File: "+pathname_TO+"\n")
                    elif (Answer[0] == 'm'):
                        print "Replacing it with the merged file ..."
                        Cmd1 = "cd "+Full_FROM_Path+";"
                        Cmd2 = "\mv "+File+" "+File+Orig_Extension+";"
                        Cmd3 = "\mv "+File+Merge_Extension+" "+File
                        Cmd = Cmd1+Cmd2+Cmd3
                        print "Executing command: "+Cmd
                        os.system(Cmd)
                        ReplaceFile(Full_TO_Path,pathname_FROM,
                                    pathname_TO,File)
                        fd.write("Merged File: "+pathname_TO+"\n")
                    else:
                        # Answer[0] == 'n'
                        fd.write("Not merged File: "+pathname_FROM+"\n")
            elif (Item_TO == DirType):
                print "Error: conflicting names"
                print "File name in FROM tree seems identical to "+\
                      "Dir name in TO tree"
                print "FROM File: "+pathname_FROM
                print "TO   Dir:  "+pathname_TO
                print "Merge failed"
                Answer = raw_input("Continue with the rest of this tree?[y/n]")
                if (Answer[0] == 'n'):
                    os._exit(1)
            else: # Item_TO == NotFound
                print "This File is not present in the TO tree"
                print "FROM File: "+pathname_FROM
                Answer = raw_input("Should I add it? [y/n]")
                if (Answer[0] == 'y'):
                    print "Adding it ..."
                    AddFile(Full_TO_Path,pathname_FROM,pathname_TO,File)
                    fd.write("Added File: "+pathname_TO+"\n")
                else: # Answer[0] == 'n'
                    fd.write("Not added File: "+pathname_TO+"\n")
            #  #]
        else: # Item_FROM == NotFound
            #  #[
            if (Item_TO == FileType):
                print "This File is not present in the FROM tree"
                print "TO File:  "+pathname_TO
                Answer = raw_input("should it be removed? [y/n]")
                if (Answer[0] == 'y'):
                    print "Removing it ..."
                    RmFile(Full_TO_Path,File)
                    fd.write("Removed File: "+pathname_TO+"\n")
                else: # Answer[0] == 'n'
                    fd.write("Not Removed File: "+pathname_TO+"\n")
            elif (Item_TO == DirType):
                print "This Dir is not present in the FROM tree"
                print "TO Dir:  "+pathname_TO
                Answer = raw_input("should it be removed? [y/n]")
                if (Answer[0] == 'y'):
                    print "Not yet implemented, do this manually,"
                    print "and then continue this script ...."
                    fd.write("Removed Dir: "+pathname_TO+"\n")
                    Answer = raw_input("removed by hand? [y/n]")
                    if (Answer[0] == 'n'):
                        os._exit(1)                        
            else: # Item_TO == NotFound
                print "This condition should never be in the FileList..."
                print "pathname_FROM = ",pathname_FROM
                print "pathname_TO   = ",pathname_TO                
                print "however, this seems to occur anyway if the FROM branch"
                print "was used for testing, and contains broken symlinks"
                print "which are missing in the TO branch ....."
                #os._exit(1)
            #  #]
#  #]
#------------------------------------
# start the main program
#------------------------------------
#  #[ open reportfile
if (os.path.exists(ReportFile)):
    print "ReportFile: "+ReportFile+" already exists"
    Answer = raw_input("Should I append to it? [y/n]")
    if (Answer[0] == 'y'):
        print "Appending to ReportFile: "+ReportFile+"..."
        fd = open(ReportFile,'a')
    else:
        print "Replacing ReportFile: "+ReportFile+"..."
        print "(saving copy as: "+ReportFile+".bak)"
        os.system("\cp -f "+ReportFile+" "+ReportFile+".bak")
        fd = open(ReportFile,'w')
else:
    print "Creating Reportfile: "+ReportFile+"..."
    fd = open(ReportFile,'w')
#  #]
DescendDirs(StartDir,fd)
fd.close()
print "done"
#------------------------------------
