#!/bin/csh

#################################################################
# a little wrapper script intended to:
# ==>run the L2B_processor main program
# ==>allow the stderr output to the usual channel, which will be
#    picked up by the ThinLayer HMI
# ==>pipe the stdout output to a filtering script to remove all
#    messages that do not conform to the standard ThinLayer log format
#
# usage:
# ./run_L2BP.sc JobOrder.xml
#
# this script assumes that:
# ==>the L2B_processor processor executable,
#    or a symlink to it, are available in the current directory.
# ==>
#
#################################################################

#################################################################
# settings
#################################################################

# the processor to run
set EXECUTABLE = ./L2B_processor
# the pattern toi use to filter on valid log messages
set PATTERN = "....-..-..T..:..:..\....... .* L2BP ..\... \[..........\]: "
# a copy of the nargs variable (to have better readable code)
set N_ARGS      = $#argv
# a copy of the actual name of this script (including a leading path)
set SCRIPTNAME = $0
# the directory in which this script is placed
set RUNDIR = "${SCRIPTNAME:h}"

# bugfix
# remark: if there is no leading path in the $0 variable, then the RUNDIR
# variable will end up having the name of the script itself as value 
# (don't know why this is true, but it is)
# so fix this with a special test
if ( ${SCRIPTNAME} == ${RUNDIR} ) then
  set RUNDIR = "./"
endif

# test prints
#echo "EXECUTABLE = ${EXECUTABLE}"
#echo "PATTERN    = ${PATTERN}"
#echo "SCRIPTNAME = ${SCRIPTNAME}"
#echo "RUNDIR     = ${RUNDIR}"
#echo "N_ARGS     = ${N_ARGS}"
#echo "JobOrder   = $1"

#################################################################
# check for valid input
#################################################################

if ( "${N_ARGS}" != "1" ) then
  echo "ERROR: nr. of arguments incorrect \!\!\!"
  goto usage
endif

if (!(-x ${RUNDIR}/${EXECUTABLE})) then
  echo "ERROR: file ${EXECUTABLE} is not executable,"
  echo "does not exist, or is not accessible ..."
  goto usage
endif

if (!(-e $1)) then
  echo "ERROR: could not find or access JobOrderfile: $1"
  goto usage
endif

${RUNDIR}/${EXECUTABLE} $1 | grep "${PATTERN}"
#${RUNDIR}/${EXECUTABLE} $1 -LogFile InternalLogfile.log | grep "${PATTERN}"

# end of this script in case everything is ok
exit

##############################################################################
# usage text to be printed in case of problems
##############################################################################
usage:
  echo "usage:"
  echo "run_L2BP.sc [JobOrder]"
  echo "aborting script"
  exit 1
