program TestAuxiliaryModule
  !  #[ Description:
  !     This little program tests the AuxiliaryModule
  !     which is:
  !     -a temporary replacement for the module at ECMWF
  !      which retrieves the real NWP and auxiliary data
  !  #]
  !  #[ Modifications:
  !     01-Apr-2005 J. de Kloe   Original version.
  !     26-Aug-2005 D. Tan       Write 19 profiles to EE files with
  !                              profile number used as version number
  !     01-Sep-2005 J. de Kloe   moved some datastructs to FixedHeader
  !     12-May-2006 D. Tan       Null use of ParKind1 module
  !     17-Aug-2006 P. Poli      Removed obsolete variables
  !     17-Apr-2007 D. Tan       Remove call to writeauxmetdata - phased out.
  !     08-Aug-2007 D. Tan       New interface to GetAuxiliaryData which
  !                              returns multiple profiles
  !     18-Oct-2007 J. de Kloe   added a nullify statement
  !     16-Jan-2008 J. de Kloe   phase out integer kind i_
  !
  !  #]
  !  #[ Modules used:
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  ! define the default integer and real to be used by the module
  USE Numerics, only: r8_
  ! define some parameters for verbosity control
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, VerbosityReport
  ! define a module to handle errors and program aborts
  USE ErrorHandler, only: no_error, program_abort

  ! define profile type and a function to fill it with data
  USE AuxiliaryModule, only: profile_type, GetAuxiliaryData, &
       TestPrintAuxiliaryData, DeallocAuxiliaryData, &
       InitAuxiliaryModule

  !  #]
  !  #[ Parameter/Variable/Type declarations
  implicit none

  integer        :: num_model_layers      ! nr of layers in each profile
                                          ! TODO: deduce from file
                                          !       or commandline arg?
  integer        :: num_profiles_in_file  ! nr of profiles in_file
                                          ! TODO: deduce from file
                                          !       or commandline arg?
  integer        :: nprofiles             ! output from GetAUxiliaryData
                                          !  = copy of nr of profiles in_file
  integer        :: error_flag ! to be able to report errors in subroutines
  TYPE(profile_type), dimension(:), pointer :: profiles
  character(len=256) :: in_file ! filename to read aux ODB data from
  !  #]
  !  #[ The program code

  nullify(profiles)

  call InitAuxiliaryModule(error_flag,ChoosenVerbosity=VerbosityError)
  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"InitAuxiliaryModule")

  ! define some dummy testdata
  in_file = "./infile.dat"
!  knmi_file="./KNMI_19940910_200010_02_112_000012.asc"

num_model_layers     = 60
num_profiles_in_file = 38 ! 19

  ! call the get_aux routine to request the NWP data
  call GetAuxiliaryData( &
                         in_file,             & ! input
                         num_model_layers,    & ! input
                         num_profiles_in_file,& ! input
                         nprofiles,profiles,  & ! output
                         error_flag) !,          & ! output

  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"get_auxiliary_data")

  ! print the result
  ! print *,"test result:"
  ! print *,"nprofiles = ",nprofiles
  ! do i = 1,nprofiles
  !    print *,"Contents of profile ", i
  !    call TestPrintAuxiliaryData(profiles(i))
  ! end do

  ! free allocated memory when done

  print *,"first deallocation call"
  call DeallocAuxiliaryData(profiles)

  print *,"second deallocation call"
  ! call it again to test the robustness
  call DeallocAuxiliaryData(profiles)

  !  #]
end program TestAuxiliaryModule
