program TestDummyAuxiliaryModule
  !  #[ Description:
  !     This little program tests the DummyAuxilliaryModule
  !     which is:
  !     -a temporary replacement for the module at ECMWF
  !      which retrieves the real NWP and auxilliary data
  !     OR
  !     -a special local module defined by any MET-center
  !      that wishes to run the L2B processor using their
  !      own model data.
  !  #]
  !  #[ Modifications
  !       Written by: J. de Kloe, 01-04-2005
  !
  !     Modified:
  !     11-Jan-2006 J. de Kloe   added the new DateTime type
  !     12-May-2006 D. Tan       Null use of ParKind1 module
  !     17-Oct-2007 J. de Kloe   added a nullify statement
  !     16-Jan-2008 J. de Kloe   phase out integer kind i_ and real kind r_
  !
  !  #]
  !  #[ Modules used:
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  ! define the datatypes needed for the ee_cfi fileformat
  USE EE_CFI_Datatypes, only: DateTimeType,SetDateTime
  ! define the default integer and real to be used by the module
  USE Numerics, only: r8_

  ! define profile type and a function to fill it with data
  USE DummyAuxiliaryModule, only: profile_type, &
                                  GetAuxiliaryData, InitAuxiliaryModule, &
                                  TestPrintAuxiliaryData, DeallocAuxiliaryData
  ! define a module to handle errors, and program aborts
  USE ErrorHandler, only: no_error, program_abort
  USE VerbosityControl, only: VerbosityError, VerbosityWarn, &
                              VerbosityReport, print_verbosity_setting

  !  #]
  !  #[ Parameter/Variable/Type declarations
  implicit none

  real(r8_)          :: lat1, lon1 ! in degrees
  TYPE(DateTimeType) :: datetime   ! struct
  real(r8_)          :: lat2, lon2 ! in degrees
  real(r8_)          :: timestep   ! in seconds

  integer            :: nprofiles 
  integer            :: error_flag
  TYPE(profile_type), dimension(:), pointer :: profiles
  !  #]
  !  #[ The program code

  nullify(profiles)
  error_flag = no_error

  ! define some dummy testdata
  lat1 = 55.0_r8_
  lat2 = lat1 + 1._r8_
  lon1 = 5.0_r8_
  lon2 = lon1 + 1._r8_
  call SetDateTime(datetime,2005,01,31,12,59,59.99_r8_)
  timestep = 1._r8_

  call InitAuxiliaryModule(ChoosenVerbosity=VerbosityError)

  ! call the get_aux routine to request the NWP data
  call GetAuxiliaryData( lat1,lon1,datetime,  & ! input
                         lat2,lon2,timestep,  & ! input
                         nprofiles,profiles,  & ! output
                         error_flag)            ! output

  if (error_flag .ne. no_error) &
       call program_abort(error_flag,"get_auxiliary_data")

  ! print the result
  print *,"nprofiles = ",nprofiles
  call TestPrintAuxiliaryData(profiles)

  ! free allocated memory when done
  nprofiles = nprofiles + 1 ! change it to test the check on nprofiles
  call DeallocAuxiliaryData(profiles)

  ! call it again to test the robustness
  call DeallocAuxiliaryData(profiles)
  !  #]
end program TestDummyAuxiliaryModule
