/* #[ Description

  **************************************************************************
    a collection of wrappers to interface the ee_cfi library to Fortran90

    created by:   J. de Kloe, may 2005

    Modifications:
    21-Jul-2006 J. de Kloe  adapted to ee_cfi 3.5
    07-May-2007 J. de Kloe  added functions xf_tree_go_to_path_node_
                            and get_xf_cfi_bad_input_argument_
    11-Jun-2007 J. de Kloe  added function get_xf_header_format_hdr_
    20-Jul-2007 J. de Kloe  added wrappers for schema handling
    24-Jul-2007 J. de Kloe  disable call to xf_set_schema() and use of 
                            XF_CHANGE_SCHEMA_AND_VERSION
                            to allow compilation with ee_cfi v3.4A again
    05-Nov-2007 J. de Kloe  added xf_tree_get_curr_element_name_

  ***************************************************************************/

/* #] */

/* to interface with fortran we need wrappers which add an underscore
   to the function/subroutine names */
/* WARNING: all names below need to be LOWERCASE
   otherwise you can expect linking problems with some compilers !!!! */

/* make the c-version of the ee_cfi xml-library available*/
#include <explorer_file_handling.h>
/* make the global variable xmlSaveNoEmptyTags available */
#include <libxml/globals.h>

void force_savenoemptytags_(void)
{ xmlSaveNoEmptyTags=1; };

long xf_tree_create_(long *error)
{ return xf_tree_create( error ); }; 

void xf_tree_rewind_(long *id, long *error)
{ xf_tree_rewind(id, error); };

void xf_tree_create_root_(long *id, char *name, long *error)
{ xf_tree_create_root(id, name, error); };

void xf_tree_write_(long *id, char *name, long *error )
{ xf_tree_write(id, name, error ); };

void xf_set_schema_(char *filename, char *schemafilename,
                    long *action, long *error )
{};
/*{ xf_set_schema(filename, schemafilename, action, error ); };*/

/* no longer available since ee_cfi version 3.5 !!! */
/* void xf_error_msg_(char *error_message)
   { xf_error_msg(error_message); }; */

void xf_basic_error_msg_(long error_code, char *error_message)
{ xf_basic_error_msg(error_code,error_message); };

void xf_tree_cleanup_all_parser_(void)
{ xf_tree_cleanup_all_parser(); };

void xf_tree_cleanup_parser_(long *fd, long *error)
{ xf_tree_cleanup_parser(fd, error); };

void xf_tree_add_child_(long *id, char *parent, char *name, long *error)
{ xf_tree_add_child(id, parent, name, error ); };

void xf_tree_set_string_node_value_(long *id, char *name,
        char *value, char *format, long *error )
{ xf_tree_set_string_node_value(id, name, value, format, error); };

void xf_tree_add_attribute_(long *id, char *current, char *name, long *error)
{ xf_tree_add_attribute(id, current, name, error); };

void xf_tree_create_header_(long *fd, long *file_extension_type, long *error)
{ xf_tree_create_header(fd, file_extension_type, error); };

void xf_tree_set_fixed_header_items_( long *fd, char *file_name, 
              char *file_description, char *notes,
              char *mission, char *file_class, char *file_type, 
	      char *validity_start, char *validity_stop, long *file_version, 
              char *system, char *creator, char *creator_version,
              char *creation_date, long *error)
{ xf_tree_set_fixed_header_items(fd, file_name, file_description, 
	      notes, mission, file_class, file_type, validity_start, 
	      validity_stop, file_version, system, creator, creator_version,
              creation_date, error); };

void xf_tree_get_fixed_header_items_( long *fd, char *file_name, 
              char *file_description, char *notes,
              char *mission, char *file_class, char *file_type, 
	      char *validity_start, char *validity_stop, long *file_version, 
              char *system, char *creator, char *creator_version,
              char *creation_date, long *error)
{ xf_tree_get_fixed_header_items(fd, file_name, file_description, 
	      notes, mission, file_class, file_type, validity_start, 
	      validity_stop, file_version, system, creator, creator_version,
              creation_date, error); };

long xf_tree_init_parser_(char *file, long *error)
{ xf_tree_init_parser(file, error); };

/*
    ! name used to be:
    !  xf_tree_path_read_string_node_value_()
    ! but this exceeds the 31 char limit for fortran
    ! so I had to shorter it a bit
    call xf_tp_read_string_node_value_( &
          xml_tree, path, c_string_value, error)
*/
void xf_tp_read_string_node_value_( 
        long *fd, char *path, char *value, long *error )
{ xf_tree_path_read_string_node_value(fd, path, value, error); };

void xf_tree_go_to_path_node_(long *fd, char *path, long *error )
{ xf_tree_go_to_path_node(fd, path, error); };

void xf_tree_read_string_attribute_(
        long *fd, char *path, char *attr_name, char *attr_value, long *error)
{ xf_tree_read_string_attribute(fd, path, attr_name, attr_value, error); };

void  xf_tree_get_curr_element_name_(long *fd, char *name_value, long *error)
{ xf_tree_get_current_element_name(fd, name_value, error); }

/* wrappers to get some constants defined in the c-code */
long get_xf_header_format_eef_(void)
{ return XF_HEADER_FORMAT_EEF; }

long get_xf_header_format_hdr_(void)
{ return XF_HEADER_FORMAT_HDR; }

long get_xf_max_error_msg_length_(void)
{ return XF_MAX_ERROR_MSG_LENGTH; }

long get_xf_max_value_length_(void)
{ return XF_MAX_VALUE_LENGTH; }

long get_xf_max_path_length_(void)
{ return XF_MAX_PATH_LENGTH; }

long get_xf_cfi_ok_(void)
{ return XF_CFI_OK; }

long get_xf_cfi_empty_element_(void)
{ return XF_CFI_EMPTY_ELEMENT; }

long get_xf_cfi_bad_input_argument_(void)
{ return XF_CFI_BAD_INPUT_ARGUMENT; }

long get_xf_change_schema_and_vers_(void)
{ return 0; }
/*{ return XF_CHANGE_SCHEMA_AND_VERSION; }*/

