/* #[ Description
  *********************************************************************
    a simple test program to test usage if the ee_cfi software
    for xml file handling.

    created by:   J. de Kloe, may 2005

    Modifications:
    21-Jul-2006 J. de Kloe  adapted to ee_cfi 3.5
  *********************************************************************/
/* #] */
/* #[ include files */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <explorer_file_handling.h>
/* #] */

void closeup ( long error_code )
{  
/* #[ Auxiliary function for closing in case of error */
   char 	error_message [ XF_MAX_ERROR_MSG_LENGTH ];
   
   /* upto ee_cfi 3.4: xf_error_msg ( error_message ); */
   /* replaced since ee_cfi 3.5 by */
   xf_basic_error_msg ( error_code, error_message );
   printf ( "%s\n", error_message );

   /* Cleaning */
   xf_tree_cleanup_all_parser ();
   exit ( EXIT_FAILURE );
/* #] */
}

int main(int argc, char **argv)
{
/* #[ Main program */

   /* General purpose variables */
   long		fd_1;
   static char 	xmlFile [] = "simple_testdata.xml";
   char 	error_message [ XF_MAX_ERROR_MSG_LENGTH ];
   int  	i;
   long 	error, array_length, duration;
   clock_t 	start, stop;
	
   /* String reading variables */
   char	string_value [ XF_MAX_VALUE_LENGTH ], 
        string_array [ XF_MAX_ARRAY_SIZE ][ XF_MAX_VALUE_LENGTH ], 
	element_node [ XF_MAX_XML_NODE_NAME_LENGTH ], 
        attr_name [ XF_MAX_XML_NODE_NAME_LENGTH ], 
        attr_value [ XF_MAX_VALUE_LENGTH ];
   static char 	string_element[]  = "TAI_of_ANX";
   
   /* Integer reading variables */
   long 	integer_value, 
                integer_array [ XF_MAX_ARRAY_SIZE ], 
		integer_attr;
   static char 	integer_element[]  = "Cycle_Length";

   /* Double reading variables */
   double 	double_value, 
                double_array [ XF_MAX_ARRAY_SIZE ], 
		double_attr;
   static char 	double_element[]  = "ANX_Longitude";

   /* Path reading variables */
   char 	path [ XF_MAX_PATH_LENGTH ];
   
   /* Search function variables */
   char found_path[XF_MAX_PATH_LENGTH],
        search_path[XF_MAX_PATH_LENGTH],
        search_value[XF_MAX_VALUE_LENGTH];

   static char 	system_all [XF_MAX_VALUE_LENGTH] 	  = "system"; 
   static char 	creator_all [XF_MAX_VALUE_LENGTH] 	  = "creator"; 
   static char 	creator_version_all [XF_MAX_VALUE_LENGTH] = "creator_version";
   static char 	creation_date_all [XF_MAX_VALUE_LENGTH]   = "creation_date";
   
   /* Initialization */
   /* -------------- */
   printf ( "\n*** Loading the file %s ***\n\n", xmlFile );
   
   start = clock();
   fd_1 = xf_tree_init_parser ( xmlFile, &error );
   stop = clock();
   
   if ( error < XF_CFI_OK )
   {
      printf("\nError parsing file %s\n", xmlFile);
      closeup (error);
   }
	
   duration = ((long)stop-(long)start)/1000;
   printf("\nFile %s loaded OK: %d ms\n", xmlFile, duration);
   
   /* Reading first element as string  */
   /* -------------------------------- */
   printf ( "\n*** Reading element as string ***" );
   printf ( "\n*** this reads the first element of this type in the file" );

   xf_tree_read_string_element_value( &fd_1, 
				      string_element, string_value, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError reading element as string\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %s\n", string_element, string_value );
   

   /* Reading all elements as strings */
   /* ------------------------------- */
   printf ( "\n*** Reading all the %s elements as strings ***\n", 
	    string_element );
   
   xf_tree_read_string_element_array ( &fd_1, 
                                       string_element, 
				       string_array, 
				       &array_length, 
				       &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError reading all the %s elements as strings\n", 
	     string_element );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Element: %s\n", string_element );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %s\n", i, string_array [ i ] );
      }
   }

   /* Reading element as integer  */
   /* --------------------------- */
   printf ( "\n*** Reading element as integer ***\n\n" );
   
   /*Rewind to the top of the file */
   xf_tree_rewind ( &fd_1, &error );
   if ( error < XF_CFI_OK )
   {
      printf("\nError rewinding\n" );
      closeup (error);
   }

   xf_tree_read_integer_element_value ( &fd_1, 
                                        integer_element, 
					&integer_value, 
					&error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading element as integer\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %d\n", integer_element, integer_value );

   /* Reading all elements as integers */
   /* -------------------------------- */
   printf ( "\n*** Reading all the %s elements as integers ***\n\n", 
	    integer_element );
   
   xf_tree_read_integer_element_array ( &fd_1, 
                                        integer_element, 
					integer_array, 
					&array_length, 
					&error );
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading all the %s elements as integers\n", 
	     integer_element );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Element: %s\n", integer_element );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %d\n", i, integer_array [ i ] );
      }
   }
	
   /* Reading element as double  */
   /* -------------------------- */
   printf ( "\n*** Reading element as double ***\n\n" );
   
   /*Rewind to the top of the file */
   xf_tree_rewind ( &fd_1, &error );
   if ( error < XF_CFI_OK )
   {
      printf("\nError rewinding\n" );
      closeup (error);
   }

   xf_tree_read_real_element_value ( &fd_1, 
                                     double_element, 
				     &double_value, 
				     &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading element as double\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %f\n", double_element, double_value );

   /* Reading all elements as doubles */
   /* ------------------------------- */
   printf ( "\n*** Reading all the %s elements as doubles ***\n\n", 
	    double_element );
   
   xf_tree_read_real_element_array ( &fd_1, 
                                     double_element, 
				     double_array, 
				     &array_length, 
				     &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError reading all the %s elements as doubles\n", double_element );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Element: %s\n", double_element );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %f\n", i, double_array [ i ] );
      }
   }
	
   /* Reading an element as string given a path  */
   /* ------------------------------------------ */
   printf ( "\n*** Reading an element as string given a path ***\n\n" );
   
   strcpy ( path, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[1]/Time_of_ANX/UTC_of_ANX" );
   xf_tree_path_read_string_node_value ( &fd_1, 
                                         path, 
					 string_value, 
					 &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an element as string given a path\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %s\n", path, string_value );

   /* Reading an element as integer given a path  */
   /* ---------------------------------------------------- */
   printf ( "\n*** Reading an element as integer given a path ***\n\n" );
   
   strcpy ( path, "../../../Orbit_Change[2]/Orbit/Absolute_Orbit" );
   xf_tree_path_read_integer_node_value ( &fd_1, 
                                          path, 
					  &integer_value, 
					  &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an element as integer given a path\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %ld\n", path, integer_value );

   /* Reading a double element given a path  */
   /* -------------------------------------- */
   printf ( "\n*** Reading a double element given a path ***\n\n" );
   
   strcpy ( path, "../../../Orbit_Change[2]/Cycle/ANX_Longitude" );
   xf_tree_path_read_real_node_value ( &fd_1, 
                                       path, 
				       &double_value, 
				       &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading a double element given a path\n" );
      closeup (error);
   }
   else
      printf ( "Element: %s *** Value: %g\n", path, double_value );

   /* Reading the attribute of an element given a path  */
   /* ------------------------------------------------- */
   printf ( "\n*** Reading the attribute of an element given a path ***\n\n" );
   
   strcpy ( path, "../Repeat_Cycle/@unit" );
   xf_tree_path_read_string_node_value ( &fd_1, 
                                         path, 
					 string_value, 
					 &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an attribute given a path\n" );
      closeup (error);
   }
   else
      printf ( "Attribute: %s *** Value: %s\n", path, string_value );

   /* Reading an array of strings given a path  */
   /* ----------------------------------------- */
   printf ( "\n*** Reading an array of strings given a path ***\n\n" );
   
   strcpy ( path, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[*]/Time_of_ANX/UTC_of_ANX" );
   xf_tree_path_read_string_node_array ( &fd_1, 
                                         path, 
				         string_array, 
				         &array_length, 
				         &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an array of strings given the path: \n", path );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Path: %s\n", path );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %s\n", i, string_array [ i ] );
      }
   }

   /* Reading an array of integers given a path  */
   /* ------------------------------------------ */
   printf ( "\n*** Reading an array of integers given a path ***\n\n" );
   
   strcpy ( path, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[*]/Orbit/Absolute_Orbit" );
   xf_tree_path_read_integer_node_array ( &fd_1, 
                                          path, 
				          integer_array, 
				          &array_length, 
				          &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an array of integers given the path: \n", path );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Path: %s\n", path );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %ld\n", i, integer_array [ i ] );
      }
   }

   /* Reading an array of doubles given a path  */
   /* ----------------------------------------- */
   printf ( "\n*** Reading an array of doubles given a path ***\n\n" );
   
   strcpy ( path, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[*]/Cycle/MLST_Drift" );
   xf_tree_path_read_real_node_array ( &fd_1, 
                                       path, 
				       double_array, 
				       &array_length, 
				       &error );
		
   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an array of doubles given the path: \n", path );
      closeup (error);
   }
   else
   {
      /* Look for warnings */
      if ( error > XF_CFI_OK )
      {
         /* xf_error_msg ( error_message ); */
         xf_basic_error_msg ( error, error_message );
         printf ( "WARNING: %s\n", error_message );
      }
      printf ( "Path: %s\n", path );
      for ( i =  0; i < array_length; i++ )
      {
         printf ( " Value [ %d ]: %lf\n", i, double_array [ i ] );
      }
   }

   /* Rewinding  */
   /* ---------- */
   printf ( "\n*** Rewinding ***\n\n" );
   
   xf_tree_rewind ( &fd_1, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError rewinding\n" );
      closeup (error);
   }

   /* Moving to an element  */
   /* --------------------- */
   strcpy ( element_node, "List_of_Orbit_Changes" );
   printf ( "\n*** Moving to the element %s ***\n\n", element_node );
   
   xf_tree_go_to_element_node ( &fd_1, (char *)element_node, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError navigating\n" );
      closeup (error);
   }

   /* Reading an integer attribute from the current element  */
   /* ------------------------------------------------------ */
   strcpy ( attr_name, "count" );
   printf ( "\n*** Reading the integer attribute %s from the current element %s ***\n\n", attr_name, element_node );
   
   xf_tree_read_integer_attribute ( &fd_1, NULL, (char *)attr_name, &integer_attr, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError reading an integer attribute\n" );
      closeup (error);
   }
   else
      printf ( "Attribute: %s *** Value: %ld\n", attr_name, integer_attr );

   /* Moving to an element  */
   /* --------------------- */
   strcpy ( element_node, "Repeat_Cycle" );
   printf ( "\n*** Moving to the element %s ***\n\n", element_node );
   
   xf_tree_go_to_element_node ( &fd_1, (char *)element_node, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError navigating\n" );
      closeup (error);
   }

   /* Reading a string attribute from the current element  */
   /* ---------------------------------------------------- */
   strcpy ( attr_name, "unit" );
   printf ( "\n*** Reading the string attribute %s from the current element %s ***\n\n", attr_name, element_node );
   
   xf_tree_read_string_attribute ( &fd_1, NULL, (char *)attr_name, 
				   attr_value, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError reading a string attribute\n" );
      closeup (error);
   }
   else
      printf ( "Attribute: %s *** Value: %s\n", attr_name, attr_value );

   /* Moving to a path node  */
   /* ---------------------- */
   strcpy ( path, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[1]/Cycle/MLST_Drift" );
   printf ( "\n*** Moving to the node %s ***\n\n", path );
   
   xf_tree_go_to_path_node ( &fd_1, path, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError moving to path node\n" );
      closeup (error);
   }

   /* Getting the current node path and comparing  */
   /* ------------------------------------------- */
   printf ( "\n*** Getting the current node path %s ***\n\n", path );
   
   xf_tree_get_path ( &fd_1, path, &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError getting path node\n" );
      closeup (error);
   }
   else
      printf ( "Path: %s\n", path );

   /* SEARCH FUNCTIONS */
   /* ---------------- */
   
   printf("\n*** Search Functions ***\n\n");
   xf_tree_rewind(&fd_1, &error);
   
   printf("\t*** Search using the element name:");
   strcpy(search_path, "Absolute_Orbit");
   strcpy(search_value, "2");

   xf_tree_find_string_value_element(&fd_1, search_path, search_value, found_path, &error);

   printf("\n\tSearch element %s with value = %s\n", search_path, search_value);
   printf("\n\tFound path = %s\n\n", found_path);



   printf("\t*** Search using an XPath expression:");
   strcpy(search_path,
          "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[*]/Orbit/Absolute_Orbit");
   strcpy(search_value, "1");

   xf_tree_find_string_value_path(&fd_1, search_path, search_value, found_path, &error);

   printf("\n\tSearch path %s with value = %s\n", search_path, search_value);
   printf("\n\tFound path = %s\n", found_path);

   /* Cleaning */
   xf_tree_cleanup_all_parser ();
   
   return XF_CFI_OK;
/* #] */
}
