/* #[ Description

  **************************************************************************
    a simple test program to test usage if the ee_cfi software
    for xml file handling.

    created by:   J. de Kloe, may 2005

    Modifications:
    21-Jul-2006 J. de Kloe  adapted to ee_cfi 3.5
    20-Jul-2007 J. de Kloe  added tests for xf_set_schema()
  ***************************************************************************/

/* #] */
/* #[ include files 
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <explorer_file_handling.h>
#include <libxml/globals.h>

/* see: /usr/include/libxml2/libxml/globals.h 
    or: ../ee_cfi_software/aux_tools/libxml/include/libxml2/libxml/globals.h */

/* #] */

void closeup ( long error_code )
{
/* #[ Auxiliary function for closing in case of error */
   char 	error_message [ XF_MAX_ERROR_MSG_LENGTH ];
   
   /* upto ee_cfi 3.4: xf_error_msg ( error_message ); */
   /* replaced since ee_cfi 3.5 by */
   xf_basic_error_msg ( error_code, error_message );
   printf ( "%s\n", error_message );

   /* Cleaning */
   xf_tree_cleanup_all_parser ();
   exit ( EXIT_FAILURE );
/* #] */
}

int main(int argc, char **argv)
{
/* #[ Main program */

   /* General purpose variables */
   char 	error_message [ XF_MAX_ERROR_MSG_LENGTH ];
   int  	i;
   long 	error, array_length, duration, action;
	
   /* String reading variables */
   char 	string_value [ XF_MAX_VALUE_LENGTH ], 
         string_array [ XF_MAX_ARRAY_SIZE ][ XF_MAX_VALUE_LENGTH ], 
	      element_node [ XF_MAX_XML_NODE_NAME_LENGTH ], 
	      attr_name [ XF_MAX_XML_NODE_NAME_LENGTH ], 
	      attr_value [ XF_MAX_VALUE_LENGTH ];
   static char 	string_element[]  = "TAI_of_ANX";
   
   /* Integer reading variables */
   long 	integer_value, 
                integer_array [ XF_MAX_ARRAY_SIZE ], 
		integer_attr;
   static char 	integer_element[]  = "Cycle_Length";

   /* Double reading variables */
   double 	double_value, 
                double_array [ XF_MAX_ARRAY_SIZE ], 
		double_attr;
   static char 	double_element[]  = "ANX_Longitude";

   /* Path reading variables */
   char 	path [ XF_MAX_PATH_LENGTH ];
   
   /* Search function variables */
   char found_path[XF_MAX_PATH_LENGTH],
        search_path[XF_MAX_PATH_LENGTH],
        search_value[XF_MAX_VALUE_LENGTH];

   /* Writing XML file variables */
   long		fd_3;
   static char	writtenXmlFile  [] = "simple_testdata_written.xml";
   static char	writtenXmlFile2 [] = "simple_testdata_written2.xml";
   static char	DummySchema     [] = "Dummy_Schema.xsd";
   
   /* Reading Fixed Header values variables */
   static char 	item_id [XF_MAX_VALUE_LENGTH]              = "Notes";
   static char 	item_value [XF_MAX_VALUE_LENGTH]           = "This is a example of adding value to a Header element";
   long         header_type;   
   static char 	file_name_all [XF_MAX_VALUE_LENGTH]        = "file_name_value";
   static char 	file_description_all [XF_MAX_VALUE_LENGTH] = "file_description";
   static char 	notes_all [XF_MAX_VALUE_LENGTH]            = "notes";
   static char 	mission_all [XF_MAX_VALUE_LENGTH]          = "mission";
   static char 	file_class_all [XF_MAX_VALUE_LENGTH]       = "file_class";
   static char 	file_type_all [XF_MAX_VALUE_LENGTH]        = "file_type";
   static char 	validity_start_all [XF_MAX_VALUE_LENGTH]   = "validity_start";
   static char 	validity_stop_all [XF_MAX_VALUE_LENGTH]    = "validity_stop";
   long 	*file_version_all;
   static char 	system_all [XF_MAX_VALUE_LENGTH]           = "system"; 
   static char 	creator_all [XF_MAX_VALUE_LENGTH]          = "creator"; 
   static char 	creator_version_all [XF_MAX_VALUE_LENGTH]  = "creator_version";    
   static char 	creation_date_all [XF_MAX_VALUE_LENGTH]    = "creation_date";      
   
   /* write filename variables */
   static char 	filename [XF_MAX_FILENAME_LENGTH];
   long         satellite_id                      = XF_SAT_CRYOSAT;
   static char 	file_class [5] 	                  = "CCCC";
   static char 	file_type [XF_MAX_VALUE_LENGTH]   = "TTTTTTTTTT";
   static char 	instance_id [XF_MAX_VALUE_LENGTH] = "yyyymmddThhmmss_YYYYMMDDTHHMMSS_vvvv";   
   XF_File_extension_type_enum  fet               = XF_HEADER_FORMAT_DBL; 
   
   /* read filename variables */
   long        satellite_id_r;
   static char file_class_r   [5];
   static char file_type_r   [11];
   static char instance_id_r [43];  

   long file_extension_type,
        file_version;

   /* Initialization */
   /* -------------- */
   file_version=0;
   file_version_all= &file_version;

   /* Creating an XML file */
   printf ( "*** Creating the XML file %s ***\n", writtenXmlFile );

   fd_3 = xf_tree_create( &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError creating XML document\n" );
      closeup (error);
   }

   xf_tree_create_root ( &fd_3, "Earth_Explorer_File", &error );

   if ( error < XF_CFI_OK )
   {
      printf("\nError creating root in XML document\n" );
      closeup (error);
   }
   /* In the next block, errors are not checked for the sake of simplicity.
      However, the user must do it in order to have a robust code */

   /* Creating a child after another */
   xf_tree_add_child ( &fd_3, ".", "Data_Block", &error );
   xf_tree_add_child ( &fd_3, ".", "List_of_Orbit_Changes", &error );
   xf_tree_add_child ( &fd_3, ".", "Orbit_Change", &error );
   xf_tree_add_child ( &fd_3, ".", "Orbit", &error );
   xf_tree_add_child ( &fd_3, ".", "Absolute_Orbit", &error );
   
   /* Adding a value to the current node */
   xf_tree_set_string_node_value ( &fd_3, ".", "10", "%s", &error );
   
   /* Adding a sibling next to the current node, and then its value */
   xf_tree_add_next_sibling ( &fd_3, ".", "Relative_Orbit", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "15", "%s", &error );
   
   xf_tree_add_child ( &fd_3, "..", "Phase_Number", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "1", "%s", &error );
   
   /* Adding a sibling before the current node, and then its value */
   xf_tree_add_previous_sibling ( &fd_3, ".", "Cycle_Number", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "1", "%s", &error );
   
   /* Another child, witl a different path */
   xf_tree_add_child ( &fd_3, "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/Orbit_Change[1]",
               "Cycle", &error );
	       
   xf_tree_add_child ( &fd_3, ".", "Repeat_Cycle", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "3", "%s", &error );
   xf_tree_add_attribute ( &fd_3, ".", "units", &error );
   xf_tree_set_string_node_value ( &fd_3, "@units", "day", "%s", &error );
   xf_tree_add_next_sibling ( &fd_3, ".", "Cycle_Length", &error );
   
   /* Adding an attribute to the current node, and then its value */
   xf_tree_add_attribute ( &fd_3, ".", "unit", &error );
   xf_tree_set_string_node_value ( &fd_3, "@unit", "orbit", "%s", &error );
   
   xf_tree_set_string_node_value ( &fd_3, ".", "43", "%s", &error );
   xf_tree_add_next_sibling ( &fd_3, ".", "ANX_Longitude", &error );
   /* Adding a value as double. Please, pay attention to the format */
   double_value = 0.1335;
   xf_tree_set_real_node_value ( &fd_3, ".", &double_value, "%06.4lf",&error );
   
   xf_tree_add_attribute ( &fd_3, ".", "unit", &error );
   xf_tree_set_string_node_value ( &fd_3, "@unit", "deg", "%s", &error );
   
   /* Removing an attribute, and then the current node */
   xf_tree_remove_node ( &fd_3, "@unit", &error );
   xf_tree_remove_node ( &fd_3, "../Repeat_Cycle", &error );

 
   xf_tree_add_previous_sibling(&fd_3,"Cycle_Length", "Repeat_Cycle", &error );

   /* Adding a value as long. Please, pay attention to the format */
   integer_value = 5L;
   xf_tree_set_integer_node_value (&fd_3,".", &integer_value, "%ld", &error );
   
   xf_tree_add_attribute ( &fd_3, ".", "unit", &error );
   xf_tree_set_string_node_value ( &fd_3, "@unit", "day", "%s", &error );
   xf_tree_set_string_node_value ( &fd_3, "../Cycle_Length", "57", "%s", &error );
   xf_tree_add_child ( &fd_3, "..", "MLST", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "22:00:00.45545", "%s", &error );
   xf_tree_add_next_sibling ( &fd_3, ".", "MLST_Drift", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "011.010101", "%s", &error );
   xf_tree_add_child ( &fd_3, "../..", "Time_of_ANX", &error );
   xf_tree_add_child ( &fd_3, ".", "TAI_of_ANX", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "TAI=2002-03-01T03:01:09.01345", "%s", &error );
   xf_tree_add_next_sibling ( &fd_3, ".", "UTC_of_ANX", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "UTC=2002-03-01T03:01:10.01345", "%s", &error );
   xf_tree_add_next_sibling ( &fd_3, ".", "UT1_of_ANX", &error );
   xf_tree_set_string_node_value ( &fd_3, ".", "UT1=2002-03-01T03:01:11.01345", "%s", &error );

   /*********************************************************** 
   *	Adding Header and values for the Header elements 
   ************************************************************/

   /* adding Header */
   header_type =  XF_HEADER_FORMAT_EEF;

   printf ( "	... Creating Header ...\n\n"); 
   xf_tree_create_header ( &fd_3, &header_type, &error );   
   if ( error < XF_CFI_OK ) {
     printf("\n	... Error creating header ...\n\n" );      	      	
     closeup (error);
   } 		
   
   /* adding value for Header elemnts */
   printf ( "... Setting fixed-header element value ...\n"); 
   xf_tree_set_fixed_header_item ( &fd_3, item_id, item_value, &error ); 
   if ( error < XF_CFI_OK ) {
     printf("\n	... Error setting fixed-header item ...\n\n" );
     closeup (error);
   }

   fet = XF_HEADER_FORMAT_NONE;

   /* generating filename */ 
   printf ( "... Writing filename ...\n");
   xf_create_filename ( (long * )&satellite_id, file_class, file_type, 
			instance_id, (long * )&fet, filename, &error); 
   if ( error < XF_CFI_OK ) {
      	printf("\nError writting filename\n" );      	      	
      	closeup (error);
   } else {
   	printf("... filename ---> %s\n", &filename );	
   }
   xf_tree_set_fixed_header_item ( &fd_3, "File_Name", filename, &error ); 
   if ( error < XF_CFI_OK ) {
     printf("\n	... Error setting fixed-header item ...\n\n" );      	      	
     closeup (error);
   }
   
   /* reading a Header value */  
   printf("... Reading an item in the Fixed header ...\n", item_value);
   strcpy(item_id,"Notes");
   strcpy(item_value,"dummy");
   xf_tree_get_fixed_header_item ( &fd_3, item_id, item_value, &error );
   if ( error < XF_CFI_OK ) {
      	printf("\nError reading fixed-header item\n" );      	      	
      	closeup (error);
   } else {
   	printf("... value is: ---> %s\n", item_value);	
   }
   
   /* reading all Header values */
   printf("... Reading the complete Fixed header ...\n", item_value);
   xf_tree_get_fixed_header_items ( &fd_3, 
   				    file_name_all, 
   				    file_description_all, 
   				    notes_all, 
   				    mission_all, 
   				    file_class_all, 
   				    file_type_all,
				    validity_start_all, 
				    validity_stop_all, 
				    file_version_all, 
				    system_all, 
				    creator_all, 
				    creator_version_all,
				    creation_date_all, 
				    &error);   
   if ( error < XF_CFI_OK && error != XF_CFI_EMPTY_ELEMENT ) {
      	printf("\nError reading fixed-header item\n" );      	      	
      	closeup (error);
   } else {
   	printf("... File_Name: ---> %s\n", file_name_all);	
   	printf("... Notes    : ---> %s\n", notes_all);	
   }
      
   
   /* reading filename */  
   printf ( "... Reading filename ...\n\n");
   xf_read_filename_items ( file_name_all, &satellite_id_r, file_class_r, 
			    file_type_r, instance_id_r, &error); 
   if ( error < XF_CFI_OK ) {
      	printf("\nError reading filename\n" );      	      	
      	closeup (error);
   } else {
   	printf("... satellite_id ---> %ld\n" , satellite_id_r );	
   	printf("... file_class   ---> %s\n"  , &file_class_r );
   	printf("... file_type    ---> %s\n"  , &file_type_r );
   	printf("... instance_id  ---> %s\n"  , &instance_id_r );
   }    

   xf_tree_add_child ( &fd_3, "/Earth_Explorer_File/Data_Block", 
		       "TestEmptyTag", &error );
   if ( error < XF_CFI_OK ) {
      	printf("\nError creating TestEmptyTag\n" );      	      	
      	closeup (error);
   }
   /* fill the test tag with an empty string */
   xf_tree_set_string_node_value ( &fd_3, ".","","", &error );
   if ( error < XF_CFI_OK ) {
      	printf("\nError setting TestEmptyTag\n" );      	      	
      	closeup (error);
   }

   /* WARNING: this tag behaves VERY STRANGE on my system.
               When I don't fill it with an empty string it seems to
               contain a single linebreak (looks like a not-initialised
               piece of memory to me).
               When I initialise it with an empty string it writes the 
               form <TestEmptyTag></TestEmptyTag>
               regardless of the setting of xmlSaveNoEmptyTags below !!!
               The empty elements within the different headers do follow
	       the xmlSaveNoEmptyTags setting, however, which seems to
	       me a programming error in the ee_cfi xml library handling.
	       Since we only write headers in xml form we can live with
	       this problem for now........ JK
   */
   

   /*********************************************************** 
   *	/Adding Header and values for the Header elements 
   ************************************************************/   
   
   printf("*** Test SaveNoEmptyTags switch ***\n");

   /* print the global switch */
   printf("global setting for this switch:\n");
   printf("xmlSaveNoEmptyTags = %i\n",xmlSaveNoEmptyTags);

   /* set the switch to 0 and print again */
   xmlSaveNoEmptyTags=0;
   printf("current setting:\n");
   printf("xmlSaveNoEmptyTags = %i\n",xmlSaveNoEmptyTags);

   /* now write the xml tags to file, using the default setting,
      which is using empty tags for my platform. JK */

   /* Writing to disk. If the errors are XF_CFI_CREATING_NODE_ERROR or
      XF_CFI_ADDING_SIBLING_ERROR, then an indentation error has ocurred,
      which is not very important and can be ignored */

   xf_tree_write ( &fd_3, writtenXmlFile, &error );

   if ( error < XF_CFI_OK && 
	error != XF_CFI_CREATING_NODE_ERROR && 
	error != XF_CFI_ADDING_SIBLING_ERROR )
   {
      printf("\nWriting error\n" );
      closeup (error);
   }

   /* test the new xf_set_schema routine added in ee_cfi v3.5 */
   /*
   action = XF_CHANGE_SCHEMA_AND_VERSION;
   xf_set_schema (writtenXmlFile,  DummySchema, &action, &error);

   if ( error < XF_CFI_OK )
   {
     printf("\nError setting schema reference for file: %s\n",writtenXmlFile);
     closeup (error);
   }
   */
   
   /* set the switch to 1 and print again */
   xmlSaveNoEmptyTags=1;
   printf("current setting:\n");
   printf("xmlSaveNoEmptyTags = %i\n",xmlSaveNoEmptyTags);

   /* now write the xml tags to file, using a setting
      which disables the use of empty tags */

   /* write to stdout */
   /* xf_tree_write ( &fd_3, "-", &error ); */

   /* Writing to disk. If the errors are XF_CFI_CREATING_NODE_ERROR or
      XF_CFI_ADDING_SIBLING_ERROR, then an indentation error has ocurred,
      which is not very important and can be ignored */
   xf_tree_write ( &fd_3, writtenXmlFile2, &error );

   if ( error < XF_CFI_OK && 
	error != XF_CFI_CREATING_NODE_ERROR && 
	error != XF_CFI_ADDING_SIBLING_ERROR )
   {
      printf("\nWriting error\n" );
      closeup (error);
   }

   /* test the new xf_set_schema routine added in ee_cfi v3.5 */
   /*
   action = XF_CHANGE_SCHEMA_AND_VERSION;
   xf_set_schema (writtenXmlFile2, DummySchema, &action, &error);

   if ( error < XF_CFI_OK )
   {
     printf("\nError setting schema reference for file: %s\n",writtenXmlFile2);
     closeup (error);
   }
   */

   printf("*** End of Test for SaveNoEmptyTags switch ***\n");

   printf("File written succesfully\n");

   return XF_CFI_OK;

/* #] */
}
