program test_ee_xml
  !  #[ Description:
  !     A module containing interface routines with the
  !     explorer_file_handling library from the 
  !     ee_cfi_software package.
  !
  !     created by:  J. de Kloe
  !
  !     Modifications:
  !     12-May-2006  D. Tan       Null use of ParKind1 module
  !     25-Jul-2006  J. de Kloe   added call to xml_tree_print()
  !     11-Jun-2007  J. de Kloe   adapted to changed xml_interface
  !     20-Jul-2007  J. de Kloe   added call to xml_tree_set_schema()
  !     19-Oct-2007  J. de Kloe   added a nullify statement
  !     05-Nov-2007  J. de Kloe   added xml_tree_set_namespace
  !     16-Jan-2008  J. de Kloe   phase out integer kind i_ 
  !
  !  #]
  !  #[ Modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE XML_module
  USE ErrorHandler, only: program_abort, no_error
  !  #]
  !  #[ Parameter/Variable/Type declarations
  implicit none

  ! general xml items
  TYPE(xml_node_type), pointer :: xml_tree
  integer :: error_flag
  character(len=max_str_len) :: path
  character(len=max_str_len) :: text
  character(len=max_str_len) :: attr_name
  character(len=max_str_len) :: attr_value
!  character(len=max_str_len) :: xml_schema_ref
  character(len=max_str_len) :: xml_namespace_uri

  ! header items
  character(len=max_str_len) :: file_name        = "file_name_value"
  character(len=max_str_len) :: file_description = "file_description_value"
  character(len=max_str_len) :: notes            = "notes_value"
  character(len=max_str_len) :: mission          = "mission_value"
  character(len=max_str_len) :: file_class       = "file_class_value"
  character(len=max_str_len) :: file_type        = "file_type_value"
  character(len=max_str_len) :: validity_start   = "validity_start_value"
  character(len=max_str_len) :: validity_stop    = "validity_stop_value"
  integer                    :: file_version     = 1
  character(len=max_str_len) :: system           = "system_value"
  character(len=max_str_len) :: creator          = "creator_value"
  character(len=max_str_len) :: creator_version  = "creator_version_value"
  character(len=max_str_len) :: creation_date    = "creation_date_value"
  !  #]
  !  #[ Initialisation
  call Init_XML(error_flag) ! (ChoosenVerbosity=VerbosityWarn)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"Init_XML")
  !  #]
  !  #[ Program code for generating and writing xml

  ! define the xml-tree
  nullify(xml_tree)
  call create_xml_tree(xml_tree, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"create_xml_tree")

  print *,"... Creating xml root ..."
  call xml_tree_create_root(xml_tree,ee_root_name,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_create_root")

  print *,"... Setting namespace ..."
  xml_namespace_uri="http://www.esa.int/schemas/ae/AUX_MET_12_01.01" 
  call xml_tree_set_namespace(xml_tree, xml_namespace_uri, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_set_namespace")

  print *,"... Setting second namespace ..."
  xml_namespace_uri="http://www.esa.int/schemas/ae/AUX_MET_12_01.01" 
  call xml_tree_set_namespace(xml_tree, xml_namespace_uri, error_flag,&
                              xml_scope_name="xsi")
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_set_namespace")

!  print *,"... Setting Schema ..."
!  xml_schema_ref = "Dummy_Schema.xsd"
!  call xml_tree_set_schema(xml_tree,xml_schema_ref,error_flag)

  print *,"... Creating Header ..."
  call xml_tree_create_header(xml_tree,ee_root_name,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_create_header")

  call xml_set_fixed_header(xml_tree, ee_root_name,&
       file_name, file_description, &
       notes, mission, file_class, file_type, validity_start, validity_stop,&
       file_version, system, creator, creator_version, creation_date, &
       error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_set_fixed_header")

  print *,"... Adding data ..."
  path = "/"//ee_root_name
!  path = "."
  call xml_tree_add_child(xml_tree,path,"test_1",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  call xml_tree_add_child(xml_tree,path,"test_2",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  call xml_tree_add_child(xml_tree,path,"test_3",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  path = trim(path)//"/test_2"
  call xml_tree_add_child(xml_tree,path,"test_2a",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  call xml_tree_add_child(xml_tree,path,"test_2b",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  call xml_tree_add_child(xml_tree,path,"test_2c",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_child")
  path = trim(path)//"/test_2b"
  call xml_tree_add_attribute(xml_tree,path,"count",error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_add_attribute")

  ! fill the xml tree with data
  path = "/Earth_Explorer_File/test_2/test_2b"
  text = "this is a simple xml test"
  call xml_tree_set_string_node_value(xml_tree,path,text,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_set_string_node_value")
  
  path = trim(path)//"/@count"
  text = "12345"
  call xml_tree_set_string_node_value(xml_tree,path,text,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_set_string_node_value")

  ! test the new xml_tree_set_string_atrribute routine
  path = "/Earth_Explorer_File/test_2/test_2b"
  attr_name = "version"
  attr_value = "1.2"
  call xml_tree_add_and_set_attribute(xml_tree,path,attr_name,&
                                      attr_value,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_set_string_attribute")

  ! write the xml-tree to file
  print *,"... Writing xml to file ..."
  call xml_tree_write(xml_tree,"test.xml", error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_write")

  ! free no longer needed memory
  print *,"... Cleaning up memory ..."
  call delete_xml_tree(xml_tree, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"delete_xml_tree")
  !  #]
  !  #[ Program code for reading xml
  file_name = "simple_testdata.xml"
  print *,"... loading file ",trim(file_name)," ..."
  call xml_load_file(file_name, xml_tree, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_load_file")
  print *,"file successfully loaded"

  ! added this test because many testresults in our testsuite
  ! depend on this print function, but I tend to forget testing
  ! with the ee_cfi xml handling switched on.
  ! This would have catched the ee_cfi problem in the L2BtoTXT2
  ! branch and saved us a lot of trouble.
  print *,"... printing parsed file ",trim(file_name)," ..."
  call xml_tree_print(xml_tree,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_print")
  print *,"file successfully printed"

  ! get all the elements in the fixed header
  path = "/"//ee_h_name
  call xml_get_fixed_header(xml_tree, path, file_name, &
       file_description, &
       notes, mission, file_class, file_type, validity_start, validity_stop,&
       file_version, system, creator, creator_version, creation_date, &
       error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_get_fixed_header")

  print *,"file_name        = ",trim(file_name)
  print *,"file_description = ",trim(file_description)
  print *,"notes            = ",trim(notes)
  print *,"mission          = ",trim(mission)
  print *,"file_class       = ",trim(file_class)
  print *,"file_type        = ",trim(file_type)
  print *,"validity_start   = ",trim(validity_start)
  print *,"validity_stop    = ",trim(validity_stop)
  print *,"file_version     = ",file_version
  print *,"system           = ",trim(system)
  print *,"creator          = ",trim(creator)
  print *,"creator_version  = ",trim(creator_version)
  print *,"creation_date    = ",trim(creation_date)

  ! there are 2 orbit changes in this file, so the count may be 1 or 2
  ! without specifying the count, just the first occurrence will be taken
  ! so the value in the 1st orbit change
  path = "/Earth_Explorer_File/Data_Block/List_of_Orbit_Changes/"//&
       "Orbit_Change/Time_of_ANX/TAI_of_ANX"
!       "Orbit_Change[1]/Time_of_ANX/TAI_of_ANX"

  call xml_node_get_string(xml_tree, path, text, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_node_get_string")
  print "(4a)"," node: ",trim(path)," has value: ",trim(text)

  ! test retrieving an attribute value
  attr_name = "unit"
  path = "/Earth_Explorer_File/Data_Block/"//&
         "List_of_Orbit_Changes/Orbit_Change[1]/Cycle/MLST_Drift"
  call xml_tree_read_string_attribute(xml_tree,&
       path,attr_name,attr_value,error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"xml_tree_read_string_attribute")
  print *,"result of xml_tree_read_string_attribute() test:"
  print *,"path       = ",trim(path)
  print *,"attr_name  = ",trim(attr_name)
  print *,"attr_value = ",trim(attr_value)

  ! free no longer needed memory
  print *,"... Cleaning up memory ..."
  call delete_xml_tree(xml_tree, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"delete_xml_tree")
  
  !  #]
end program test_ee_xml

