program L2B_processor
  !  #[ Description
  !        a first version of the L2B processor
  !        to be developed into the full L2B processor
  !     
  !     created 08-04-2005 by: J. de Kloe, KNMI, and David Tan, ECMWF
  !        
  !     Modifications:
  !        03-May-2005 D. Tan,     introduce preprocessed met data
  !        30-may-2005 J. de Kloe, split data in L1B, L2B and
  !                                included preprocessed met data in L2B
  !        03-jun-2005 J. de Kloe, added loop on BRC's
  !        26-jul-2005 J. de Kloe, split into primary/preproc/main parts
  !        04-aug-2005 D. Tan,     pass NWPfilename to primary
  !        06-oct-2005 D. Tan,     pass L1Bfilename to primary
  !        16-nov-2005 J. de Kloe, added use of L2B_Proc_Settings module and
  !                                moved commandline option handling to it.
  !        04-Apr-2006 J. de Kloe  added Logging module
  !        11-Apr-2006 P. Poli     added use of RBC file in EE-CFI format
  !        24-Apr-2006 J. de Kloe  added Read_L2B_AuxPar and Read_JobOrderFile
  !        28-Apr-2006 J. de Kloe  added use of some JobOrder settings
  !        12-May-2006 D. Tan      Null use of ParKind1 module
  !        23-May-2006 D. Tan,     Include L2B_AuxPar argument to L2BP_primary
  !        20-Jul-2006 J. de Kloe  big reshuffle of code to create a clear 
  !                                interface to the L2BP_primary subroutine by
  !                                moving all file handling into this main 
  !                                program. (this is also needed to interface
  !                                with the IFS/ODB system)
  !        21-Jul-2006 D. Tan      Pass L1B+L2BAuxPar filenames to WriteL2Bfile
  !        09-Aug-2006 J. de Kloe  changed so it requires use of the JobOrder
  !                                AuxPar files for input now.
  !        31-Aug-2006 J. de Kloe  added log messages in case reading of 
  !                                JobOrder of AuxPar files failed
  !        07-Sep-2006 J. de Kloe  Activated workaround in ReadJobOrderFile()
  !        08-Sep-2006 J. de Kloe  Activated L2Bresultdir setting, and removed
  !                                unwanted exceptions that caused the 
  !                                program to continue in case no JobOrder 
  !                                file is present
  !        25-Oct-2006 J. de Kloe  added call to WriteProductList
  !        10-Nov-2006 J. de Kloe  adapted to some moved code blocks
  !        15-Feb-2007 J. de Kloe  adapted to allow applying a timewindow
  !                                in the JobOrder.
  !        01-Mar-2007 J. de Kloe  added a test call to trigger a log_debug msg
  !        01-Mar-2007 J. de Kloe  removed use of SummaryVars
  !        21-Mar-2007 J. de Kloe  added call to inspect_luns_at_program_end()
  !        27-Mar-2007 J. de Kloe  rename Fill_L2B_SPH to Fill_L2BC_SPH, &
  !                                Fill_L2B_FH to Fill_L2BC_FH, and &
  !                                Fill_L2B_MPH to Fill_L2BC_MPH
  !        13-Jun-2007 J. de Kloe  adapted to changed interface of the
  !                                WriteL2BCfile routine
  !        18-Jul-2007 D. Tan      Move code to new 
  !                                routines aeolus_l2bp_setup and
  !                                aeolus_l2bp_unsetup in l2bp_module.F90
  !        26-Jul-2007 J. de Kloe  adapted to handle multiple AMD input files
  !                                and to use the corresponding timeranges
  !        30-Jul-2007 J. de Kloe  moved Fill_L2BC_FH outside the BRC loop and
  !                                added some code to save and retrieve the
  !                                datetime-range for the processed L1B data
  !        30-Jul-2007 J. de Kloe  adapted the MPH filling code to enable use
  !                                of the filename based on the DateTimeRange
  !                                which can only be composed after all
  !                                L1B processing is done.
  !        30-Jul-2007 P. Poli     Make RBC dataflow explicit
  !        22-Aug-2007 P. Poli     Move matching and screening from
  !                                L2BP_primary to aeolus_l2bp_prepare_primary
  !        03-Sep-2007 P. Poli     Introduced L1B_Headers and L2B_Headers
  !                                change L1B_DataType to L1B_BRC_DataType 
  !                                and L1B_HDR_DataType, change L2BC_DataType 
  !                                to L2BC_BRC_DataType and L2BC_HDR_DataType
  !        10-Oct-2007 J. de Kloe  adapted aeolus_l2bp_fill_L2B_Headers to 
  !                                take into account the default header fields.
  !        11-Oct-2007 J. de Kloe  added some counters to count how many BRC's
  !                                were and were not processed, for each AMD
  !                                input file, and in total.
  !        19-Oct-2007  P. Poli    split AMD_DataType in two parts:
  !                                AMD_HDR_DataType and AMD_BRC_DataType
  !        07-Dec-2007  D. Tan     Adapt to new FH/MPH interface
  !        16-Jan-2008  J. de Kloe phase out integer kind i_
  !        17-Jan-2008  J. de Kloe removed a last few 64bit incompatibilities
  !
  !  #]
  !  #[ modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE L2BP_module, only: L2BP_preprocess, L2BP_primary, &
                         aeolus_l2bp_setup, &
                         aeolus_l2bp_import_L1B_Headers, &
                         aeolus_l2bp_import_one_L1B_BRC, &
                         aeolus_l2bp_import_AMD_Headers, &
                         aeolus_l2bp_import_one_AMD_BRC, &
                         aeolus_l2bp_prepare_primary, &
                         aeolus_l2bp_export_one_L1B_BRC, &
                         aeolus_l2bp_fill_L2B_Headers, &
                         aeolus_l2bp_export_L1B_Headers, &
                         aeolus_l2bp_export_L2B_Headers, &
                         aeolus_l2bp_export_one_L2B_BRC, &
                         aeolus_l2bp_export_AMD_Headers, &
                         aeolus_l2bp_export_one_AMD_BRC, &
                         aeolus_l2bp_unsetup, &
                         filename_LiteTestData
  USE L2B_Proc_Settings, only: L2BP_Settings_type, &
                               do_preprocessing_step, &
                               do_primary_processing, &
                               use_lite_test_data, &
                               NWPmetadataFilename
  USE ReadAMDdata, only: Load_All_AMD_Profiles
  USE JobOrder_DataStructure, only: JobOrderData_type
  USE ReadL1Bdata, only: ReadL1Bfile
  USE L1B_DataStructure, only: L1B_HDR_DataType, L1B_BRC_DataType, &
                     InitLatLonDateTimeRanges, DetermineLatLonDateTimeRanges,&
                     GetLatLonRanges, GetDateTimeRange
  USE L2BC_DataStructure, only: L2BC_HDR_DataType, L2BC_BRC_DataType, &
                                Init_L2BC_BRC_DataStructure
  USE WriteL2BCdata, only: WriteL2BCfile, WriteProductList
  USE AMD_DataStructure, only: AMD_HDR_DataType, &
                               AMD_BRC_DataType
  USE RBC_DataStructure, only: RBC_DataType
  USE Working_DataStructure, only: WorkingDataType
  USE LiteDataModule, only: Get_Nr_available_BRCs
  USE Classification, only: CloseClassification
  USE EE_CFI_DataTypes, only: DateTime_1_after_or_eq_2, IntAuc, IntAus, &
                              IntAl, GetIntAus, GetIntAuc, CheckWithinTimeRange, &
                              DateTimeType
  ! support modules
  USE Logging, only: logmsg,log_debug,log_info,log_progress,log_warn,log_error
  USE ErrorHandler, only: no_error, program_abort,&
                          error_allocate, error_reading_file
  USE Numerics, only: r8_
  USE LunManager, only: inspect_luns_at_program_end,&
                        fileunit_stdout ! for testing only

  !  #]
  !  #[ Variables

  implicit none 

  ! local variables
  integer :: error_flag, iBRC, Num_BRC, Num_AMD_BRC, AMD_iBRC, AMD_file
  integer :: Num_BRC_processed
  integer :: Num_BRC_not_processed
  integer :: Num_BRC_processed_this_AMD_file
  integer :: Num_BRC_not_proc_this_AMD_file
  integer :: nr_Lite_BRCs_available, AllocStatus
  integer :: L2B_iBRC
  logical     :: ProcessBRC,IsWithinTimeRange
  integer(IntAus) :: Num_L2B_BRCs
  integer(IntAl)  :: lat_start, lat_stop, lon_start, lon_stop
  integer(IntAuc)    :: M_Meas0, N_Meas_to_write
  integer(IntAuc)    :: M_Mie0, N_Obs_Mie_to_write
  integer(IntAuc)    :: M_Rayleigh0, N_Obs_Rayleigh_to_write

  ! a structure to hold the JobOrder settings
  TYPE(JobOrderData_type) :: JobOrderData
  character(len=256)      :: filename_JobOrder
  character(len=6)        :: iBRC_txt
  character(len=6)        :: NumBRC_txt
  character(len=6)        :: L2B_iBRC_txt
  type(DateTimeType)      :: DateTimeStart, DateTimeStop

  !character(len=256)      :: txt ! for debugging

  ! a structure to hold all processor settings
  TYPE(L2BP_Settings_type) :: L2BP_Settings
  ! a struct to hold the necessary input data for the processor
  TYPE(L1B_BRC_DataType)      :: L1B
  ! a struct to hold the necessary input data for the processor
  TYPE(L1B_HDR_DataType)      :: L1B_Headers
  ! a struct to hold all results from the processor
  TYPE(L2BC_BRC_DataType), dimension(:), pointer :: L2B
  ! a struct to hold all header results from the processor
  TYPE(L2BC_HDR_DataType)     :: L2B_Headers
  ! a struct to hold all NWP data
  TYPE(AMD_BRC_DataType), dimension(:), pointer :: AMD
  TYPE(AMD_HDR_DataType)      :: AMD_Headers
  ! a struct to hold all RBC data
  TYPE(RBC_DataType) :: RBC
  ! a struct to hold all WorkingData
  TYPE(WorkingDataType) :: WorkingData

  logical :: open_fileunit_found
  character(len=256) :: lun_message

  !  #]
  !  #[ Main Program code

  ! TODO (JK 17-05-2006):
  ! let the processor check that all needed files are available, before we 
  ! start doing  some real work (like loading the RBC tables). This would 
  ! prevent waiting a long time while testing, before getting an error 
  ! that a certain file (say AMD or L2B) is missing ... 
  ! At the moment the RBC file is loaded first, which takes a full 2 minutes 
  ! at my system, and only then you notice if you made a typing error in one
  ! of the other filenames ...

  ! 1.1 init
  error_flag = no_error
  nullify(L2B)
  nullify(AMD)

  call logmsg(log_progress,"Starting L2B-Processor")

  call aeolus_l2bp_setup(filename_JobOrder, &
                         JobOrderData, L2BP_settings, &
                         RBC, &
                         error_flag, standalone_version=.true.)
  IF (error_flag .ne.  no_error) THEN
     call logmsg(log_error,"aborted aeolus_l2bp_setup")
     call program_abort(error_flag,"aeolus_l2bp_setup")
  END IF

  IF (do_preprocessing_step) THEN
     !  #[

     ! this routine reads the L1B datafile, and produces a small text file
     ! describing the measurement geometry (date,time,lat,lon)
     ! needed to retrieve NWP data 
     call logmsg(log_progress,"Starting preprocessing")
     call L2BP_preprocess(L2BP_Settings%L1Bfile%name,&
                          NWPmetadataFilename,error_flag)
     IF (error_flag .ne. no_error) THEN
        call logmsg(log_error,"aborted L2BP_preprocess")
        call program_abort(error_flag,"L2BP_preprocess")
     END IF

     !  #]
  END IF

  ! at this point it is assumed that the needed NWP datafile
  ! is available. So in the IFS the Simulobs2odb routine should be called
  ! between the L2BP_preprocess and the L2BP_primary routine.

  ! Alternatively, a MetCenter may 
  ! -call this main program to do L2BP_preprocess first 
  !  (using the --preprocess or -1 option), 
  ! -then execute some program to get NWP data,
  ! -and then call this main program to do L2BP_primary
  !  (using the --primary or -2 option)
  ! All 3 steps could be packed into a little script for convenience

  IF (do_primary_processing) THEN
     !  #[

     ! this routine does the L2B processing. It reads the L1B datafile,
     ! performs classification, produces the L2B products
     ! and writes them to a L2B datafile
     call logmsg(log_progress,"Starting primary L2B processing")

     Num_BRC_processed     = 0
     Num_BRC_not_processed = 0

     ! 2.0 loop on all the AMD files
     ! we agreed to repeat the complete primary processing in a do-loop
     ! when multiple NWP input files are present.
     ! Note that for each NWP file a L2B product file will be generated.
     ! We currently have no other choice, because we have only room to
     ! record one AMD file in the L2B product file.

     AMD_file_loop: do AMD_file=1,L2BP_Settings%NumNWPfiles

        ! 2.1 prepare processing for AMD file number AMD_file: get L1B_Headers, 
        ! allocate L2B array, and read AMD data

        Num_BRC_processed_this_AMD_file = 0
        Num_BRC_not_proc_this_AMD_file  = 0

        ! initialize lat-lon and date-time ranges. 
        ! This needs to be done again for each L2B product generated, 
        ! so cannot be done in aeolus_l2bp_setup() called above outside 
        ! the AMD_file_loop
        call InitLatLonDateTimeRanges()

        ! import L1B header information
        call aeolus_l2bp_import_L1B_Headers(L1B_Headers, Num_BRC, error_flag, &
             L1B_filename=L2BP_Settings%L1Bfile%name)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_import_L1B_Headers")
           call program_abort(error_flag,"aeolus_l2bp_import_L1B_Headers")
        END IF

        call logmsg(log_progress,"Headers from L1B file: "//&
                   trim(L2BP_Settings%L1Bfile%name)//" loaded")

        IF (use_lite_test_data) THEN
           call Get_Nr_available_BRCs(filename_LiteTestData,&
                                   nr_Lite_BRCs_available,error_flag)
           IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted Get_Nr_available_BRCs")
              call program_abort(error_flag,"Get_Nr_available_BRCs")
           END IF
           ! In case of using LiteData, set the number of BRCs. 
           ! Take the minimum of the nr of BRC's
           ! available in the L1B file and the nr available in the Lite 
           ! datafile to make sure that we don't run out of data when 
           ! combining both sources
           Num_BRC = min(nr_Lite_BRCs_available,&
                         int(L1B_Headers%SPH%Geolocation_DSD%nr_of_dataset_records))
        END IF

        ! allocate L2B structure array
        allocate(L2B(Num_BRC),stat=AllocStatus)
        IF (AllocStatus .ne. 0) THEN
           error_flag = error_allocate
           call logmsg(log_error,"L2B_processor: Allocation problem"//&
                  " could not get memory to allocate: L2B(Num_BRC)")
           call program_abort(error_flag,"L2B_processor")
        END IF

        ! eventhough some BRCs may not be processed, the L2B struct must
        ! be initialised otherwise the call to Delete_L2BC_DataStructure()
        ! below will crash with a segmentation fault. This next call just 
        ! nullifies all pointers in this struct.
        ! (recall that for processed BRC's this initialisation and allocation
        !  of the datasets is done inside L2BP_primary)
        DO iBRC=1,Num_BRC
              call Init_L2BC_BRC_DataStructure(L2B(iBRC), &
                                     0, 0_IntAuc, 0_IntAuc, 0, error_flag)
        END DO

        ! import AMD header information
        call aeolus_l2bp_import_AMD_Headers (AMD_Headers, &
           Num_AMD_BRC, error_flag, &
           AMD_filename=L2BP_Settings%NWPfile(AMD_file)%name)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_import_AMD_Headers")
           call program_abort(error_flag,"aeolus_l2bp_import_AMD_Headers")
        END IF

        ! allocate AMD structure array
        allocate (AMD(Num_AMD_BRC),stat=AllocStatus)
        IF (AllocStatus .ne. 0) THEN
           error_flag = error_allocate
           call logmsg(log_error,"L2B_processor: Allocation problem"//&
                " could not get memory to allocate: AMD(Num_AMD_BRC)")
           call program_abort(error_flag,"L2B_processor")
        END IF

        ! import AMD BRC information
        DO AMD_iBRC = 1, Num_AMD_BRC

           call aeolus_l2bp_import_one_AMD_BRC (AMD_Headers, &
              AMD(AMD_iBRC), &
              AMD_iBRC, error_flag, &
              AMD_filename = L2BP_Settings%NWPfile(AMD_file)%name)
        IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted aeolus_l2bp_import_one_AMD_BRC")
              call program_abort(error_flag,"aeolus_l2bp_import_one_AMD_BRC")
        END IF

        ENDDO

        call logmsg(log_progress,"AMD file: "//&
                    trim(L2BP_Settings%NWPfile(AMD_file)%name)//" loaded")

        ! 2.2 start loop processing for all BRCs corresponding to AMD file number AMD_file
        ! loop over all BRCs in the L1B file
        L2B_iBRC = 0
        brc_loop: DO iBRC=1,Num_BRC
        !  #[
           write(iBRC_txt,"(i6)",err=999) iBRC
           call logmsg(log_progress,"processing BRC nr.: "//&
                trim(adjustl(iBRC_txt)))

           ! 2.2.1 get the instrument data for that particular BRC, and verify time matching
           call aeolus_l2bp_import_one_L1B_BRC (L1B_Headers, L1B, iBRC, &
                error_flag, L1B_filename=L2BP_Settings%L1Bfile%name)
           IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted aeolus_l2bp_import_one_L1B_BRC")
              call program_abort(error_flag,"aeolus_l2bp_import_one_L1B_BRC")
           END IF

           call logmsg(log_progress," loaded dsr "//trim(adjustl(iBRC_txt))//&
                " from L1B file: "//trim(L2BP_Settings%L1Bfile%name))
           
           ProcessBRC = .true.

           ! to enable using a timerange in the JobOrder to select
           ! only part of the L1B BRC's to be processed, the following
           ! checks have been added.

           !  #[ check the timerange used on the current L1B file
           call CheckWithinTimeRange(L1B%Geoloc_ADS%start_of_obs_datetime, &
                        L2BP_Settings%L1Bfile%start, L2BP_Settings%L1Bfile%stop, &
                        IsWithinTimeRange, error_flag)
           IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted L2BP_primary")
              call program_abort(error_flag,"L2B_processor_primary")
           END IF

           IF (.not. IsWithinTimeRange) THEN
              ProcessBRC = .false.
              call logmsg(log_progress,"Skipping L1B BRC nr.: "//&
                   trim(adjustl(iBRC_txt))//&
                   " because it is outside the timewindow"//&
                   " requested on L1B file: "//trim(L2BP_Settings%L1Bfile%name))
           END IF
           !  #]

           !  #[ check the timerange used on the current AMD file
           call CheckWithinTimeRange(L1B%Geoloc_ADS%start_of_obs_datetime, &
                        L2BP_Settings%NWPfile(AMD_file)%start, &
                        L2BP_Settings%NWPfile(AMD_file)%stop, &
                        IsWithinTimeRange, error_flag)
           IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted L2BP_primary")
              call program_abort(error_flag,"L2B_processor_primary")
           END IF

           IF (.not. IsWithinTimeRange) THEN
              ProcessBRC = .false.
              call logmsg(log_progress,"Skipping L1B BRC nr.: "//&
                   trim(adjustl(iBRC_txt))//&
                   " because it is outside the timewindow"//&
                   " requested on AMD file: "//&
                   trim(L2BP_Settings%NWPfile(AMD_file)%name))
           END IF
           !  #]

           ! TODO:
           ! add similar timerange checks on the other input files:
           ! -AuxPar, AuxClm, AuxCal

           ! 2.2.2 do actual processing
           process_brc: IF (ProcessBRC) THEN

              ! 2.2.2.1 start L2B processing, including finding a matching profile for the BRC 
              ! in the meteorological data

              ! increment the counter of L2B BRCs actually processed
              L2B_iBRC = L2B_iBRC + 1
              write(L2B_iBRC_txt,"(i6)",err=999) L2B_iBRC

              call logmsg(log_progress,"starting primary processing for "//&
                   "L2B BRC nr.: "//trim(adjustl(L2B_iBRC_txt)))

              ! Prepare the processing: screening, AMD collocation
              call aeolus_l2bp_prepare_primary(L1B,L2B(L2B_iBRC),AMD,iBRC, &
                   L2BP_Settings%L2B_AuxPar, &
                   WorkingData,error_flag)
              IF (error_flag .ne. no_error) THEN
                 call logmsg(log_error,"aborted aeolus_l2bp_prepare_primary")
                 call program_abort(error_flag,"L2B_processor_primary")
              END IF

              ! Do the actual L2B processing
              call L2BP_primary(L1B,L2B(L2B_iBRC),WorkingData,RBC,&
                                L2BP_Settings%L2B_Auxpar,error_flag)
              IF (error_flag .ne. no_error) THEN
                 call logmsg(log_error,"aborted L2BP_primary")
                 call program_abort(error_flag,"L2B_processor_primary")
              END IF

              call logmsg(log_progress,"finished primary processing for "//&
                   "L2B BRC nr.: "//trim(adjustl(L2B_iBRC_txt)))

              ! 2.2.2.2 update information regarding the date/time ranges
              ! determination of intersection lat-lon ranges 
              ! used to fill some SPH elements, and associated DateTime range
              ! used to fill the FH validity elements. It stores values for
              ! lat/lon_start at the first call, and lat/lon_stop for each
              ! call, so in the end the full lat/lon range is available
              ! and may be requested using subroutine GetLatLonRanges()
              ! It also stores min/max values for DateTime of each used L1B BRC
              ! which may be requested using subroutine GetDateTimeRange()
              call DetermineLatLonDateTimeRanges(L1B, error_flag)
              IF (error_flag .ne. no_error) THEN
                 call logmsg(log_error,"aborted DetermineLatLonDateTimeRanges")
                 call program_abort(error_flag,"DetermineLatLonDateTimeRanges")
              END IF

              Num_BRC_processed = Num_BRC_processed + 1
              Num_BRC_processed_this_AMD_file = &
                   Num_BRC_processed_this_AMD_file + 1
           ELSE
              Num_BRC_not_proc_this_AMD_file = &
                   Num_BRC_not_proc_this_AMD_file + 1
           END IF process_brc

           ! 2.2.3 clear instrument data from memory
           
           ! clean up to enable handling the next BRC
        
           ! possibly export L1B, and free up memory
           call aeolus_l2bp_export_one_L1B_BRC (L1B, error_flag)
           IF (error_flag .ne. no_error) THEN
              call logmsg(log_error,"aborted aeolus_l2bp_export_one_L1B_BRC")
              call program_abort(error_flag,"aeolus_l2bp_export_one_L1B_BRC")
           END IF

        !  #]
        END DO brc_loop

        ! 2.3 gather statistics on the processed BRCs and fill L2B_Headers

        Num_L2B_BRCs = int(L2B_iBRC,IntAus)

        ! request lat-lon ranges of all L1B data in the L1B file 
        ! this uses some variables, saved inside the l1b_datastructure,
        ! in which the min/max values have been stored
        call GetLatLonRanges(lat_start,lat_stop, &
                          lon_start,lon_stop,error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted GetLatLonRanges")
           call program_abort(error_flag,"GetLatLonRanges")
        END IF

        call GetDateTimeRange(DateTimeStart,DateTimeStop,error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted GetDateTimeRange")
           call program_abort(error_flag,"GetDateTimeRange")
        END IF

        ! fill the L2B_Headers% FH,MPH,SPH
        call aeolus_l2bp_fill_L2B_Headers(L1B_Headers, L2B, L2B_Headers, &
             L2BP_settings, &
             Num_L2B_BRCs, DateTimeStart, DateTimeStop, &
                        lat_start, lat_stop, lon_start, lon_stop, &
             M_Meas0, M_Mie0, M_Rayleigh0, error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_fill_L2B_Headers")
           call program_abort(error_flag,"aeolus_l2bp_fill_L2B_Headers")
        END IF

        N_Meas_to_write         = M_Meas0
        N_Obs_Mie_to_write      = M_Mie0
        N_Obs_Rayleigh_to_write = M_Rayleigh0

        ! 2.4 write the L2B data to file
        
        ! Write out file
        call logmsg(log_info,"L2BP_primary: Writing output")
     
        call WriteL2BCfile(L2B_iBRC, L2B_Headers, L2B, N_Meas_to_write, &
                        N_Obs_Mie_to_write, N_Obs_Rayleigh_to_write, &
                        L2BCresultdir = L2BP_Settings%L2Bresultdir,&
                      L1Bfilename   = L2BP_Settings%L1Bfile%name, &
                      NWPfilename   = L2BP_Settings%NWPfile(AMD_file)%name, &
                        RBCfilename   = L2BP_Settings%RBCfilename, &
                      CLMfilename   = "[Undefined]", & ! TODO
                      CALfilename   = "[Undefined]", & ! TODO
                      ParamFilename = L2BP_Settings%L2BAuxPar_filename, &
                      error_flag    = error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted WriteL2BCfile")
           call program_abort(error_flag,"WriteL2BCfile")
        END IF

        call WriteProductList(L2B_Headers,filename_JobOrder,error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted WriteProductList")
           call program_abort(error_flag,"WriteProductList")
        END IF

        call logmsg(log_progress,"Done writing L2B file")

        IF (L2BP_Settings%NumNWPfiles .gt. 1) THEN
           write(NumBRC_txt,"(i6)",err=995) Num_BRC_processed_this_AMD_file
           call logmsg(log_info,"Number of BRC's processed for this "//&
                "AMD file: "//trim(adjustl(NumBRC_txt)))
           write(NumBRC_txt,"(i6)",err=996) Num_BRC_not_proc_this_AMD_file
           call logmsg(log_info,"Number of BRC's not processed for this "//&
                "AMD file: "//trim(adjustl(NumBRC_txt)))
        END IF

        ! 2.5 clear L1B_Headers, L2B_Headers, and all AMD data from memory

        ! possibly export L1B_Headers, and free up memory
        call aeolus_l2bp_export_L1B_Headers (L1B_Headers, error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_export_L1B_Headers")
           call program_abort(error_flag,"aeolus_l2bp_export_L1B_Headers")
        END IF
        
        ! possibly export L2B_Headers, and free up memory
        call aeolus_l2bp_export_L2B_Headers (L2B_Headers, error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_export_L2B_Headers")
           call program_abort(error_flag,"aeolus_l2bp_export_L2B_Headers")
        END IF

        IF (associated(L2B)) THEN
           DO iBRC = 1, size(L2B) ! Num_BRC
              ! possibly export L2B, and free up memory
              call aeolus_l2bp_export_one_L2B_BRC (L2B(iBRC), error_flag)
              IF (error_flag .ne. no_error) THEN
                 call logmsg(log_error,"aborted aeolus_l2bp_export_one_L2B_BRC")
                 call program_abort(error_flag,"aeolus_l2bp_export_one_L2B_BRC")
              END IF
           ENDDO
        END IF

        deallocate(L2B)

        IF (associated(AMD)) THEN
           DO AMD_iBRC = 1, size(AMD) ! Num_BRC
              ! possibly export AMD, and free up memory
              call aeolus_l2bp_export_one_AMD_BRC (AMD(AMD_iBRC), error_flag)
              IF (error_flag .ne. no_error) THEN
                 call logmsg(log_error,"aborted aeolus_l2bp_export_one_AMD_BRC")
                 call program_abort(error_flag,"aeolus_l2bp_export_one_AMD_BRC")
              END IF
           ENDDO
        END IF

        deallocate(AMD)
        
        ! possibly export AMD_Headers, and free up memory
        call aeolus_l2bp_export_AMD_Headers (AMD_Headers, error_flag)
        IF (error_flag .ne. no_error) THEN
           call logmsg(log_error,"aborted aeolus_l2bp_export_AMD_Headers")
           call program_abort(error_flag,"aeolus_l2bp_export_AMD_Headers")
        END IF

     end do AMD_file_loop

     call logmsg(log_progress,"Primary L2B processing finished")

     Num_BRC_not_processed = Num_BRC - Num_BRC_processed

     write(NumBRC_txt,"(i6)",err=997) Num_BRC_processed
     call logmsg(log_info,"Total number of BRC's processed: "//&
          trim(adjustl(NumBRC_txt)))
     write(NumBRC_txt,"(i6)",err=998) Num_BRC_not_processed
     call logmsg(log_info,"Number of BRC's not processed: "//&
          trim(adjustl(NumBRC_txt)))
     !  #]
  END IF ! do_primary_processing

  ! 3.0 clean-up remaining items from memory

  ! clean up, free allocated memory
  call aeolus_l2bp_unsetup(JobOrderData, L2BP_settings, RBC)
  !IF (error_flag .ne. no_error) THEN
  !   call logmsg(log_error,"aborted aeolus_l2bp_unsetup")
  !   call program_abort(error_flag,"aeolus_l2bp_unsetup")
  !END IF

  !  #]
  !  #[ Error handlers
  goto 111

995 call logmsg(log_error,"Error in L2B_processor: "//&
                "Could not convert Num_BRC_processed_this_AMD_file "//&
                "counter to a 6 digit string")
  call program_abort(error_reading_file,"L2B_processor")

996 call logmsg(log_error,"Error in L2B_processor: "//&
                "Could not convert Num_BRC_not_proc_this_AMD_file "//&
                "counter to a 6 digit string")
  call program_abort(error_reading_file,"L2B_processor")

997 call logmsg(log_error,"Error in L2B_processor: "//&
                "Could not convert Num_BRC_processed counter "//&
                "to a 6 digit string")
  call program_abort(error_reading_file,"L2B_processor")

998 call logmsg(log_error,"Error in L2B_processor: "//&
                "Could not convert Num_BRC_not_processed counter "//&
                "to a 6 digit string")
  call program_abort(error_reading_file,"L2B_processor")

999 call logmsg(log_error,"Error in L2B_processor: "//&
                "Could not convert BRC counter to a 6 digit string")
  call program_abort(error_reading_file,"L2B_processor")

111 continue
  !  #]
  !  #[ Closing
  call inspect_luns_at_program_end(open_fileunit_found,lun_message)
  IF (open_fileunit_found) &
       call logmsg(log_warn,"Warning at end of L2B_Processor: "//&
               trim(lun_message)//". This looks like a programming mistake...")

  call logmsg(log_progress,"Finished L2B-Processor")
  !  #]
end program L2B_processor
