module L2BP_module
  !  #[ Description

  !        this module contains the
  !        subroutines performing the main tasks of the L2B processor
  !     
  !     created 26-07-2005 by: J. de Kloe, KNMI, and David Tan, ECMWF
  !     mostly copied from the L2B_processor.F90 file
  !
  !     Modifications:
  !        26-jul-2005 J. de Kloe  initial version
  !        04-aug-2005 D. Tan,     Read raw AMD in EE-format
  !        26-aug-2005 D. Tan,     Specify skip of zero in interface to
  !                                reading of AMD profiles in ODB ascii format
  !        29-sep-2005 J. de Kloe, adapted to change in datastruct definition
  !        06-oct-2005 D. Tan,     Use L1B file(EE-format)in primary processing
  !        26-oct-2005 J. de Kloe, removed memory leaks for multiple BRC's
  !        04-Nov-2005 J. de Kloe, added use of Rayleigh MDS
  !        15-Nov-2005 J. de Kloe, added different classification possibilities
  !                                and change the architecture to allow
  !                                choosing between them
  !        28-Nov-2005 P. Poli     added call to WriteL2B and adequate wrapper
  !        30-Nov-2005 P. Poli     changed L2B variable into an array so that
  !                                it can store several BRCs (important for
  !                                writing out!)
  !        19-Dec-2005 J. de Kloe  small correction on allocatable L2B array
  !        02-Jan-2006 J. de Kloe, added code to allow using the actual 
  !                                numbers from the L1B inputfile
  !        11-Jan-2006 J. de Kloe  added use of new DateTime type
  !        12-Jan-2006 J. de Kloe  added use of new OpticalProperties module
  !        12-Jan-2006 J. de Kloe  added new methods switch for backscat calc.
  !        11-Jan-2006 J. de Kloe  added use of the new DateTime type
  !        16-Jan-2006 J. de Kloe  corrected determining of Num_BRC 
  !        18-Jan-2006 J. de Kloe  corrected a bug in the use of btest()
  !        22-Feb-2006 P. Poli     upgrade to DLR IODD L1B-IODD.231105
  !        22-Feb-2006 J. de Kloe  Precision changes, r_/r8_.
  !        04-Apr-2006 J. de Kloe  added Logging module, 
  !                                removed temporary ascii outputfile
  !        11-Apr-2006 P. Poli     added use of RBC file in EE-CFI format
  !        12-Apr-2006 D. Tan      Switches use_ECF_AX and use_ECF_truncated.
  !        21-Apr-2006 D. Tan      Interface to AUX_MET EE-format
  !        28-Apr-2006 P. Poli     Pass RBCfilename to WriteL2Bfile
  !        02-May-2006 J. de Kloe  added Copy_JobOrderData_Items 
  !        12-May-2006 D. Tan      Null use of ParKind1 module
  !        23-May-2006 D. Tan,     Include L2B_AuxPar argument to subroutines
  !        22-Jun-2006 D. Tan      Prepare for matchup of L2B BRC and AMD DSR.
  !        03-Jul-2006 D. Tan      Bugfix to indexing of Raw_Profiles
  !        20-Jul-2006 J. de Kloe  big reshuffle of code to create a clear 
  !                                interface to the L2BP_primary subroutine 
  !                                and enable creating a dummy 
  !                                Find_Matching_AMD_Profile() routine
  !        09-Aug-2006 J. de Kloe  added more use of JobOrder and AuxPar items
  !                                and reorganised classification calls
  !        24-Jul-2006 D. Tan      Change workaround in Load_All_AMD_Profiles
  !        17-Aug-2006 P. Poli     Removed obsolete variables
  !        31-Aug-2006 D. Tan      Use HLOS_retrieval module
  !        08-Sep-2006 J. de Kloe  disabled a test print
  !        15-Sep-2006 J. de Kloe  moved copy routines for extinction and
  !                                backscatter into the Opt. Properties module,
  !                                added Mie hlos profile printing
  !        05-Oct-2006 J. de Kloe  added use of the WorkingDataStructure
  !        23-Oct-2006 D. Tan      Use of WorkingDataStructure for old
  !                                L2B-working data elements
  !        01-Nov-2006 J. de Kloe  implemented Find_Matching_AMD_Profile()
  !        10-Nov_2006 J. de Kloe  moved the Load_All_AMD_Profiles routine 
  !                                to readamddata.F90, and moved 
  !                                Copy_L1B_fields2WorkingArrays to 
  !                                working_datastructure.F90, and moved 
  !                                Copy_JobOrderData_Items to l2b_proc_settings
  !        10-Nov_2006 J. de Kloe  moved the Find_Matching_AMD_Profile()
  !                                routine into its own module in a new
  !                                directory named Match_AMD
  !        13-Nov-2006 J. de Kloe  renamed Format_L2B_Output to 
  !                                Format_and_Copy_WD_to_L2B_Outp and copying
  !                                now from the working data struct
  !        19-Feb-2007 J. de Kloe  added some lines to copy N_Obs_*_Actual
  !        17-Apr-2007 D. Tan      Remove obsolete routine from Use statement
  !        18-Apr-2007 D. Tan      Geoid in Copy_RawNWP_Prof_to_WorkingData
  !        24-Apr-2007 D. Tan      Remove obsolete use of AuxPreProc_ecmwf_module.
  !        24-Apr-2007 J. de Kloe  phase out the SummaryVars code
  !        03-May-2007 D. Tan      Simplification: Copy_AMD_to_WorkingData()
  !        29-May-2007 D. Tan      Move Copy_AMD_to_WorkingData() and
  !                                   subsidiary routines to auxiliary module.
  !        06-Jun-2007 P. Poli     Suggest to Phase out WorkingData in Screen_L1B_Data
  !                                Add L2B%PCD_ADS as input of Classify
  !        18-Jul-2007 D. Tan      Import aeolus_l2bp_setup and
  !                                aeolus_l2bp_unsetup from L2B_processor.F90
  !        27-Jul-2007 J. de Kloe  moved call InitLatLonRanges() from the
  !                                aeolus_l2bp_setup routine to the main loop
  !                                in L2B_processor.F90
  !                                Added Init_Headers=.false. in call to
  !                                Init_L2BC_DataStructure.
  !        30-Jul-2007 P. Poli     Make RBC dataflow explicit
  !        22-Aug-2007 P. Poli     Move matching and screening from
  !                                L2BP_primary to aeolus_l2bp_prepare_primary
  !        28-Aug-2007 D. Tan      Remove references to obsolete routines.
  !        03-Sep-2007 P. Poli     Introduced L1B_Headers and L2B_Headers,
  !                                change L1B_DataType to L1B_BRC_DataType and
  !                                L1B_HDR_DataType, change L2BC_DataType to 
  !                                L2BC_BRC_DataType and L2BC_HDR_DataType
  !        02-Oct-2007 J. de Kloe  added RBC parameter to InitOpticalProperties
  !        05-Oct-2007 J. de Kloe  added use of the new Meas_Selection_Weighting 
  !                                module. And some cleaning up: changed a number
  !                                of print statements into logmsg() calls
  !        10-Oct-2007 J. de Kloe  adapted Fill_L2BC_FH to take into account the 
  !                                FH_Default_Fields
  !        18-Oct-2007 J. de Kloe  adapted Fill_Remaining_L2BC_MPH_Items to take
  !                                into account the MPH_Default_Fields
  !        19-Oct-2007 P. Poli     Split AMD_DataType in two parts:
  !                                AMD_HDR_DataType and AMD_BRC_DataType
  !                                Add import/export routines for AMD and
  !                                export routine for L2B BRC data
  !        06-Dec-2007 J. de Kloe  changed the interface for 
  !                                Fill_Remaining_L2BC_MPH_Items to allow use
  !                                of proper values in stead, needed in case 
  !                                only a selection of BRC's has been processed
  !        07-Dec-2007 J. de Kloe  changed interface for subroutine 
  !                                aeolus_l2bp_fill_L2B_Headers to allow passing
  !                                JobOrder items to the filling routines for FH/MPH
  !                                and changed Fill_Remaining_L2BC_MPH_Items to have 
  !                                L2BP_Settings as input
  !        21-Dec-2007 P. Poli     added aeolus_l2bp_load_data_to_vdas and
  !                                aeolus_l2bp_clear_vdas
  !        03-Jan-2008 J. de Kloe  removed several memory leaks from the virtual das
  !        16-Jan-2008 J. de Kloe  phase out integer kind i_
  !        17-Jan-2008 J. de Kloe  removed a last few 64bit incompatibilities
  !        21-Feb-2008 P. Poli     add optional flag_init_only in routines
  !                                aeolus_l2bp_import_one_[AMD/L1B]_BRC
  !
  !  #]
  !  #[ modules used

  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.

  ! modules defining L2B algoritms

  ! a module to read the L1B xml/kvt/binary files
  USE ReadL1Bdata, only: InitReadL1BfileModule, ReadL1Bfile

  ! define types and functions for auxiliary data
  ! rename the profile_type, to prevent confusion with the other profile_type 
  ! defined in profileinterpolate.F90
  USE AuxiliaryModule, only: &
                             InitAuxiliaryModule, &
                             Copy_AMD_to_WorkingData

  USE AMD_DataStructure, only: AMD_HDR_DataType, &
                               AMD_BRC_DataType, &
                               Init_AMD_HDR_DataStructure, &
                               Init_AMD_BRC_DataStructure, &
                               Delete_AMD_HDR_DataStructure, &
                               Delete_AMD_BRC_DataStructure
  USE ReadAMDdata, only: ReadAMDfile, InitReadAMDfileModule
  USE RBC_DataStructure, only: RBC_DataType

  ! routines to classify the atmosphere into cloud/no_cloud etc.
  USE LiteDataModule, only: Fill_WorkingData_with_Lite_Data, &
                            AddDummyLatLonData
  USE OpticalProperties, only: InitOpticalProperties, &
                               CloseOpticalProperties, &
                               Match_Mie_And_Rayleigh_Bins, &
                               Calc_BackscatterRatio, &
                               Calc_Extinction
  ! routines to classify the atmosphere into cloud/no_cloud etc.
  USE Classification, only: InitClassification, CloseClassification, &
                            Classify
  USE Select_And_Weigh_Measurements, only: Select_And_Weigh_Rayleigh_Meas,&
                                           Select_And_Weigh_Mie_Meas
  USE Construct_Obs, only:  Print_Constructed_Obs, &
                            Print_Constr_Obs_Matched, &
                            Construct_Rayleigh_Obs, Construct_Mie_Obs
  USE HLOS_retrieval, only: determine_vcog, &
                            Process_Mie_Observations, &
                            Process_Rayleigh_Observations

  ! a routine to define the globally used L1B data struct
  USE L1B_DataStructure, only: L1B_HDR_DataType, &
                               L1B_BRC_DataType, &
                               Init_L1B_HDR_DataStructure, &
                               Init_L1B_BRC_DataStructure, &
                               Delete_L1B_HDR_DataStructure, &
                               Delete_L1B_BRC_DataStructure, &
                               Empty_L1B_BinaryDatasets

  USE L2B_AuxPar_DataStructure, only: L2B_AuxPar_Type, &
                            Init_L2B_AuxPar, Delete_L2B_AuxPar, &
                            AMD_Matchup_Params_type, &
                            Matchup_Dummy, Matchup_Nearest_Neighbour

  ! a routine to define the globally used L2B data struct
  USE L2BC_DataStructure, only: L2BC_HDR_DataType, &
                                L2BC_BRC_DataType, &
                                Init_L2BC_HDR_DataStructure, &
                                Init_L2BC_BRC_DataStructure, &
                                Delete_L2BC_HDR_DataStructure, &
                                Delete_L2BC_BRC_DataStructure, &
                                Fill_L2BC_FH, &
                                Fill_Remaining_L2BC_MPH_Items, &
                                Fill_L2BC_SPH, &
                                GetStatisticsL2BMieDS, &
                                GetStatisticsL2BRayleighDS, &
                                L2B_ObsRangeGateType, &
                                L2B_MieRangeGateType, &
                                L2B_RayleighRangeGateType
  USE L2B_Rayleigh_MDS, only: Print_L2B_Rayleigh_hlos_profile
  USE L2B_Mie_MDS, only: Print_L2B_Mie_hlos_profile
  USE L2B_PCD_ADS, only: Init_L2B_PCD_ADS, L2B_AMD_Collocations_Type, &
                         L2B_AMD_Coll_OK, L2B_AMD_Coll_Failed
  USE L2B_Geoloc_ADS, only: Geolocation_Bin_type
  USE L2B_Proc_Settings, only: L2BP_Settings_type, &
                               Init_L2B_Proc_Settings, &
                               Delete_L2B_Proc_Settings, &
                               Set_Derived_L2B_Proc_Settings, &
                               Print_L2B_Proc_Settings, &
                               Get_JobOrderName_from_CmdLine, &
                               Get_Additional_Cmdline_Options, &
                               do_primary_processing, &
                               Print_L2B_Proc_Settings, &
                               Copy_JobOrderData_Items, &
                               Extinction_Needed, &
                               MolExt_Method, PartExt_Method, &
                               use_dummy_lat_lon, use_lite_test_data
                               !BackscatterRatio_Needed
  USE Working_DataStructure, only: WorkingDataType, &
                      InitWorkingData, DeleteWorkingData, &
                      WD_MieObsRangeGateType  => MieObsRangeGateType, &
                      WD_RaylObsRangeGateType => RayleighObsRangeGateType, &
                      Copy_L1B_fields2WorkingArrays
  USE EE_CFI_Datatypes, only: IntAs, IntAuc, IntAus, IntAl, FAdoxy_to_IntAl, &
                              GetIntAl, &
                              SetFAdoFull_or_Missing, DateTime2String, &
                              get_averaged_datetime, get_time_difference, &
                              DateTimeType
  USE Meas_Map, only: Print_Meas_Map
  USE Match_AMD_Module, only: Find_Matching_AMD_Profile
  USE virtual_das, only: DAS_L1B_Headers, DAS_L1B_BRCs, &
                         DAS_AMD_Headers, DAS_AMD_BRCs, &
                         virtual_das_initialised

  ! L2B output
  USE WriteL2BCdata, only: InitWriteL2BCfileModule
  USE ErrorHandler, only: no_error, error_programming, &
       error_writing_file, error_reading_file, &
       error_opening_file, error_allocate
  USE Screening_L1B_Data, only: Screen_L1B_Data

  ! Set up datastructures etc
  USE JobOrder_DataStructure, only: JobOrderData_type, Print_JobOrderData, &
                                    Init_JobOrderData, Delete_JobOrderData
  USE ReadJobOrderData, only: ReadJobOrderFile
  USE Read_L2B_AuxPar_data, only: InitRead_L2B_AuxPar_fileModule, &
                                  Read_L2B_AuxPar_file
  USE RayleighBrillouinProcessing, only: Load_Rayl_Brill_Tables, &
                                         Unload_Rayl_Brill_Tables

  ! routines to fill header structures
  USE MainProductHeader, only: Fill_L2B_MPH_From_L1B_MPH

  ! support modules

  ! define the default integer and real to be used by the program
  USE Numerics, only: r8_, &
                      missing_indicator_real, missing_real
  ! define error parameters
  USE ErrorHandler, only: no_error, error_programming, error_allocate, &
                          error_writing_file, error_opening_file
  USE LunManager, only: get_lun, free_lun
  USE LatLon_Handling, only: get_midpoint, get_distance
  USE Logging, only: Init_Logging, logmsg, &
                     log_debug,log_info,log_progress,log_warn,log_error
  USE AeolusConstants, only: N_range_gates, N_Obs_Max_Mie, N_Obs_Max_Rayleigh

  !  #]
  !  #[ Variables

  implicit none

  ! this is no longer a parameter, but the actual value, read from the L1B file
  integer :: N_meas

  ! temporary, te be removed when the Lite tests are removed
  character(len=*), parameter :: filename_LiteTestData = "backscatterdata.dat"
  !  #]
contains
  !---------------------------------------------------------
  subroutine Init_Modules(L2BP_Settings,error_flag)
    !  #[ initialise the modules used by the program

    TYPE(L2BP_Settings_type), intent(in)  :: L2BP_Settings
    integer,                  intent(out) :: error_flag

    error_flag = no_error

    IF (trim(L2BP_Settings%LogFilename) .eq. "[undefined]") THEN
       call Init_Logging(L2BP_Settings%Stdout_Log_Level,&
                         L2BP_Settings%Stderr_Log_Level,error_flag)
       if (error_flag .ne. no_error) return
    ELSE
       call Init_Logging(L2BP_Settings%Stdout_Log_Level,&
                         L2BP_Settings%Stderr_Log_Level,error_flag,&
                         OptionalLogFile_in = L2BP_Settings%LogFilename)
       if (error_flag .ne. no_error) return
    END IF

    call InitAuxiliaryModule(error_flag)
    IF (error_flag .ne. no_error) return
    
    call InitReadAMDfileModule(error_flag)
    IF (error_flag .ne. no_error) return

    call InitClassification(&
              L2BP_Settings%L2B_AuxPar%Data%WVM_Params%Classification_Params, &
              error_flag)
    IF (error_flag .ne. no_error) return

    ! Initialize reading of L1B file (EE-format)
    call InitReadL1BfileModule(error_flag)
    IF (error_flag .ne. no_error) return

    call InitWriteL2BCfileModule(error_flag)
    IF (error_flag .ne. no_error) return
    
  end subroutine Init_Modules
    !  #]
  subroutine Close_Modules()
    !  #[ close the modules used by the program

    !  free the memory allocated by this module
    call CloseClassification()

    call CloseOpticalProperties()
    
  end subroutine Close_Modules
    !  #]
  subroutine aeolus_l2bp_setup(filename_JobOrder, &
                               JobOrderData, L2BP_settings, &
                               RBC, &
                               error_flag, standalone_version)
    !  #[ set up of the following datastructures

    !       - JobOrder
    !       - L2BP_Settings, including L2B_AuxPar
    !       - RBC
    !       - CLM ! TODO
    !       - CAL ! TODO

    character(len=256),       intent(out) :: filename_JobOrder
    TYPE(JobOrderData_type),  intent(out) :: JobOrderData
    TYPE(L2BP_Settings_type), intent(out) :: L2BP_Settings
    TYPE(RBC_DataType),       intent(out) :: RBC
    integer,                  intent(out) :: error_flag
    logical, optional,        intent(in)  :: standalone_version

    ! local variables
    logical :: is_standalone_version
    integer :: Task_Number

    ! init
    error_flag = no_error

    is_standalone_version = .false.
    IF (present(standalone_version)) is_standalone_version = standalone_version

    !  #[ Set the default L2B Processor Settings

    call Init_L2B_Proc_Settings(L2BP_Settings,error_flag)
    IF (error_flag .ne.  no_error) return

    !  #]

    !  #[ read the JobOrder file

    filename_JobOrder='JobOrder.AeolusL2BP.xml' ! Default for subroutine version
    IF (is_standalone_version) THEN
       call Get_JobOrderName_from_CmdLine(filename_JobOrder,error_flag)
       IF (error_flag .ne.  no_error) return
    END IF

    ! initialise the JobOrder datastructure
    call Init_JobOrderData(JobOrderData)

    ! load the JobOrder file
    call logmsg(log_info,"Reading Joborder file: "//&
         trim(filename_JobOrder))
    call ReadJobOrderFile(JobOrderData,filename_JobOrder,&
                          Task_Number,error_flag,workaround=.true.)
    IF (error_flag .ne. no_error) return
    call logmsg(log_progress,"Joborder file: "//&
         trim(filename_JobOrder)//" loaded")

    ! do a test print of these settings
    call Print_JobOrderData(JobOrderData,Task_Number,&
         "JobOrderData",error_flag)
    IF (error_flag .ne. no_error) return

    ! copy items from the JobOrderData to the variables in the
    ! L2B_Proc_Settings module, so the processor will use them
    call Copy_JobOrderData_Items(JobOrderData,L2BP_Settings,error_flag)
    IF (error_flag .ne. no_error) return

    !  #]

    !  #[ read the AuxPar file

    ! not needed. It is already called from within Init_L2B_Proc_Settings()
    !call Init_L2B_AuxPar(L2BP_Settings%L2B_AuxPar,error_flag)
    !IF (error_flag .ne. no_error) return

    ! load the L2BAuxPar input file
    call InitRead_L2B_AuxPar_fileModule(error_flag)
    IF (error_flag .ne. no_error) return

    call Read_L2B_AuxPar_file(L2BP_Settings%L2B_AuxPar, &
                              L2BP_Settings%L2BAuxPar_filename,error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_progress,"L2B_AuxPar file: "//&
                trim(L2BP_Settings%L2BAuxPar_filename)//" loaded")
    !  #]

    !  #[ cmdline options and derived settings
    IF (is_standalone_version) THEN
      call logmsg(log_progress,"Reading commandline options")
      ! remark: the commandline switches can be used to override
      !         the settings from the JobOrder and AuxPar files
      !         that have been read above.
      call Get_Additional_Cmdline_Options(L2BP_Settings,error_flag)
      IF (error_flag .ne.  no_error) return
    END IF

    call Set_Derived_L2B_Proc_Settings(L2BP_Settings,error_flag)
    IF (error_flag .ne. no_error) return

    !  #]

    ! print the settings defined in Set_L2B_Proc_Settings
    ! and any changed added to them by the command line options
    call Print_L2B_Proc_Settings(L2BP_Settings,"L2BP_Settings",error_flag)
    IF (error_flag .ne. no_error) return

    ! init all modules used by primary or preprocessing
    ! remind that this initialisation uses Stdout_Log_Level, Stderr_Log_Level
    ! and PartExt_Method as input so it must be called after determining 
    ! all settings.
    call Init_Modules(L2BP_Settings,error_flag)
    IF (error_flag .ne. no_error) return
  
    ! REMARK: only after the above call to Init_Modules the settings 
    !         Stdout_Log_Level, Stderr_Log_Level from the JobOrder file
    !         have been activated.
    !         Before this call the default values are used for any
    !         message passed to the logging system !
    !         (this is the reason that the log_debug demonstration can
    !          only be done after this point)

    ! a debug log message, intended to demonstrate that the L2BP 
    ! properly reacts to the log level defined in the JobOrder file
    ! We need at least one debug message for the FAT testing
    call logmsg(log_debug,"JobOrder loaded and initialisation was successfull")

    IF (do_primary_processing) THEN
       !  #[ load RBC, CLM, CAL data files

       ! this routine loads the auxiliary data needed for primary processing
       ! i.e. the RBC, CLM and CAL data.

       call logmsg(log_progress,"Loading EE-auxiliary files")

       ! Load RBC tables in memory (EE-format)
       call Load_Rayl_Brill_Tables(RBC,L2BP_Settings%RBCfilename,error_flag)
       IF (error_flag .ne. no_error) return

       call logmsg(log_progress,"RBC file: "//&
                   trim(L2BP_Settings%RBCfilename)//" loaded")

       ! TODO: load CLM AND CAL data

       !  #]
    END IF ! do_primary_processing

    ! GJM
    call InitOpticalProperties(PartExt_Method, error_flag, &
                               RBC=RBC)
    IF (error_flag .ne. no_error) return

  end subroutine aeolus_l2bp_setup

    !  #]
  subroutine aeolus_l2bp_unsetup( &
                               JobOrderData, L2BP_settings, RBC)
    !  #[ un-setup, in reverse order, of the following datastructures

    !       - JobOrder
    !       - L2BP_Settings%L2B_AuxPar
    !       - RBC
    !       - CLM ! TODO
    !       - CAL ! TODO

    TYPE(JobOrderData_type),  intent(out) :: JobOrderData
    TYPE(L2BP_Settings_type), intent(out) :: L2BP_Settings
    TYPE(RBC_DataType),       intent(inout) :: RBC

    call Close_Modules()
    IF (do_primary_processing) THEN
       !  #[

       ! this routine unloads the auxiliary data needed for primary processing
       ! i.e. the RBC, CLM and CAL data.

       call logmsg(log_progress,"Unloading EE-auxiliary files")

       ! TODO: unload CAL data

       ! TODO: unload CLM data

       ! Unload RBC tables in memory
       call Unload_Rayl_Brill_Tables(RBC)

       !  #]
    END IF ! do_primary_processing

    call Delete_L2B_Proc_Settings(L2BP_Settings)
    call Delete_JobOrderData(JobOrderData)

  end subroutine aeolus_l2bp_unsetup
    !  #]

  subroutine L2BP_preprocess(L1Bfilename,NWPmetadataFilename,error_flag)
    !  #[ 

    character(len=*), intent(in)  :: L1Bfilename
    character(len=*), intent(in)  :: NWPmetadataFilename
    integer,          intent(out) :: error_flag

    ! local variables
    integer :: i
    integer :: fileunit_hdr
    integer :: fileunit_body
    character(len=19)  :: datetime_string
    character(len=256) :: NWPmetadataFilename_hdr
    character(len=256) :: NWPmetadataFilename_body
    character(len=6)   :: iBRC_txt

    ! a struct to hold the necessary input data for the processor
    TYPE(L1B_BRC_DataType) :: L1B
    TYPE(L1B_HDR_DataType) :: L1B_Headers

    ! init
    call logmsg(log_progress,"start of L2BP_preprocess ...")
    error_flag = no_error

    ! initialise the L1B datastructure, but do not yet allocate its 
    ! substructures, that is only possible after reading the L1B_SPH
    ! note that for preprocessing we don't need (yet) the L1B working-arrays
    ! so they are not allocated in this subroutine
    call Init_L1B_HDR_DataStructure(L1B_Headers, error_flag)
    call Init_L1B_BRC_DataStructure(L1B, error_flag, &
                                use_binary_datasets = .false.,&
                                use_Geoloc_dataset  = .true.)
    IF (error_flag .ne. no_error) return

    call ReadL1Bfile(L1B_Headers, L1Bfilename, error_flag, L1B=L1B, &
         ReadHeaders=.true.)
    IF (error_flag .ne. no_error) return

    ! extra safety check
    IF (trim(NWPmetadataFilename) .eq. "[undefined]") THEN
       error_flag = error_opening_file
       call logmsg(log_error,"L2BP_preprocess: "//&
            "no output filename was defined for NWP metadata")
       return
    END IF

    ! create the NWP metadata files
    NWPmetadataFilename_hdr  = trim(NWPmetadataFilename)//".hdr"
    NWPmetadataFilename_body = trim(NWPmetadataFilename)//".body"

    fileunit_hdr = get_lun()
    IF (fileunit_hdr .lt. 0) THEN
       error_flag = error_writing_file
       call logmsg(log_error,"L2BP_preprocess: "//&
            "no free fileunits seem available when trying to open"//&
            "outfile: "//trim(NWPmetadataFilename_hdr))
       return
    END IF 

    open(unit=fileunit_hdr,file=NWPmetadataFilename_hdr,status="replace",&
         action="write",form="FORMATTED",ERR=996)

    write(fileunit_hdr,'(a)',err=998) " #hdr"
    write(fileunit_hdr,'(a)',err=998) " "
    write(fileunit_hdr,'(a)',err=998) ' statid = "AEOLUS"'
    write(fileunit_hdr,'(a)',err=998) " stalt = NULL"
    write(fileunit_hdr,'(a)',err=998) " obstype = 6"
    write(fileunit_hdr,'(a)',err=998) " codetype = 187"
    write(fileunit_hdr,'(a)',err=998) " sensor = 666"
    write(fileunit_hdr,'(a)',err=998) " areatype = 2"
    write(fileunit_hdr,'(a)',err=998) " sat.len = 1"
    write(fileunit_hdr,'(a)',err=998) " "
    write(fileunit_hdr,'(a)',err=998) &
         "   date   time        lat           lon    body.len   seqno"

    fileunit_body = get_lun()
    IF (fileunit_body .lt. 0) THEN
       error_flag = error_writing_file
       call logmsg(log_error,"L2BP_preprocess: "//&
            "no free fileunits seem available when trying to open"//&
            "outfile: "//trim(NWPmetadataFilename_body))
       return
    END IF

    open(unit=fileunit_body,file=NWPmetadataFilename_body,status="replace",&
         action="write",form="FORMATTED",ERR=997)
  
    write(fileunit_body,'(a)',err=999) " #body"
    write(fileunit_body,'(a)',err=999) " "
    write(fileunit_body,'(a)',err=999) " an_depar = NULL"
    write(fileunit_body,'(a)',err=999) " fg_depar = NULL"
    write(fileunit_body,'(a)',err=999) " fg_check_1 = NULL"
    write(fileunit_body,'(a)',err=999) " fg_check_2 = NULL"
    write(fileunit_body,'(a)',err=999) " "
    write(fileunit_body,'(a)',err=999) &
         " varno   obsvalue      aux_1      press   entryno"

    ! extra safety check
    IF (.not. associated(L1B_Headers%SPH%Geolocation_DSD)) THEN
       error_flag = error_programming
       call logmsg(log_error,"L2BP_preprocess: "//&
            "L1B%Geolocation_DSD seems not associated")
       return
    END IF

    ! loop over the BRC's in this L1B file
    call logmsg(log_info,"starting BRC loop")
    DO i = 1,L1B_Headers%SPH%Geolocation_DSD%nr_of_dataset_records
       write(iBRC_txt,"(i6)",err=995) i
       call logmsg(log_info,"Reading Geolocation for BRC: "//&
                   trim(adjustl(iBRC_txt)))

       ! read only the binary Geolocation data for this BRC
       call ReadL1Bfile(L1B_Headers, L1Bfilename, error_flag, &
                        L1B=L1B, Which_DSR=i, ReadGeoloc=.true.)
       IF (error_flag .ne. no_error) return

       ! write data to the NWP metadata file
       datetime_string = &
            DateTime2String(L1B%Geoloc_ADS%Obs_AOCS%obs_centroid_datetime)
       !print *,"datetime: ",datetime_string
       !stop 1

       !print *,&
       !     "lat = ",&
       !     1.e-6*L1B%Geoloc_ADS%Obs_Geoloc%lat_of_DEM_intersection,&
       !     " lon = ",&
       !     1.e-6*L1B%Geoloc_ADS%Obs_Geoloc%lon_of_DEM_intersection
       write(fileunit_hdr,'(a8,1x,a6,1x,2(f11.6,1x),i6,i6)',err=999) &
            datetime_string(1:8),datetime_string(10:15),&
            1.e-6*L1B%Geoloc_ADS%Obs_Geoloc%lat_of_DEM_intersection, &
            1.e-6*L1B%Geoloc_ADS%Obs_Geoloc%lon_of_DEM_intersection, &
            1,i
       write(fileunit_body,'(i6,1x,3(f10.1,1x),i6)',err=999) &
            187, 1., 90., real(i), i

       ! empty the datasets that were defined inside the call
       ! to ReadL1Bfile(), in this case the Geoloc_ADS only,
       ! to trigger the next call to ReadL1Bfile() to define it again
       ! This might be needed if for example the nr. of measurements
       ! varies in a file between the different BRC's (is this possible?)
       call Empty_L1B_BinaryDatasets(L1B)
    END DO

    ! close the NWP metadata files
    close(unit=fileunit_hdr)
    call free_lun(fileunit_hdr)
    close(unit=fileunit_body)
    call free_lun(fileunit_body)

    ! free allocated memory when done with the complete input file
    call Delete_L1B_BRC_DataStructure(L1B)
    call Delete_L1B_HDR_DataStructure(L1B_Headers)

    call logmsg(log_progress,"end of L2BP_preprocess ...")

    return

995 error_flag = error_programming
    call logmsg(log_error,"Error in L2BP_preprocess: "//&
                "Could not convert BRC counter to a 6 digit string")
    return

996 error_flag = error_opening_file
    call logmsg(log_error,"L2BP_preprocess: "//&
         "could not open outputfile: "//trim(NWPmetadataFilename_hdr))
    return

997 error_flag = error_opening_file
    call logmsg(log_error,"L2BP_preprocess: "//&
         "could not open outputfile: "//trim(NWPmetadataFilename_body))
    return

998 error_flag = error_writing_file
    call logmsg(log_error,"L2BP_preprocess: "//&
         "could not write to outputfile: "//trim(NWPmetadataFilename_hdr))
    return

999 error_flag = error_writing_file
    call logmsg(log_error,"L2BP_preprocess: "//&
         "could not write to outputfile: "//trim(NWPmetadataFilename_body))
    return

  end subroutine L2BP_preprocess

    !  #]

  subroutine aeolus_l2bp_import_L1B_Headers &
    (L1B_Headers, Num_BRC, error_flag, L1B_filename)
    !  #[
    TYPE(L1B_HDR_DataType),     intent(out) :: L1B_Headers
    integer,                    intent(out) :: Num_BRC
    integer,                    intent(out) :: error_flag
    character(len=*), optional, intent(in)  :: L1B_filename ! without extension

    ! no local variables

    error_flag = no_error

    IF (present(L1B_filename)) THEN

       ! initialise the L1B HDR datastructure (FH, MPH, SPH)
       call Init_L1B_HDR_DataStructure(L1B_Headers, error_flag)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted Init_L1B_HDR_DataStructure")
          return
       END IF

       ! read the headers of the L1B file.
       call ReadL1Bfile(L1B_Headers, L1B_filename, error_flag, &
                        Which_DSR =  1, ReadHeaders=.true.)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted ReadL1Bfile")
          return
       END IF

    ELSE
       ! import the L1B headers from memory
       IF (.not.associated(DAS_L1B_Headers)) THEN
          call logmsg(log_error,"aeolus_l2bp_import_L1B_Headers: ERROR!! "//&
               "L1B_Headers data are not available in the virtual DAS")
          error_flag = error_programming
          return
       ELSE
          ! JK: DO NOT initialise this struct again for now,
          ! since this introduces memory leaks in the system.
          ! It will allocate the FH/MPH/SPH blocks, after which the pointers
          ! to them are lost, because they are redirected to the
          ! virtual das below !

          ! initialise the L1B HDR datastructure (FH, MPH, SPH)
          !call Init_L1B_HDR_DataStructure(L1B_Headers, error_flag)
          !IF (error_flag .ne. no_error) THEN
          !   call logmsg(log_error,"aborted Init_L1B_HDR_DataStructure")
          !   return
          !END IF

          ! WARNING: no data is copied by this statement
          ! only the 4 logical switches in the L1B_HDR_DataType are copied, 
          ! and the 3 pointers are cloned.
          ! Paul, is this what you intended?
          ! If the data should be copied, then a Copy_L1B_HDR() routine must be 
          ! added to the l1b_datastructure.F90 module (and the '=' operator may be 
          ! overloaded to let the following assignment use it automatically).
          
          !L1B_Headers = DAS_L1B_Headers
          ! make explicit what happens here:
          L1B_Headers%headers_are_available = DAS_L1B_Headers%headers_are_available
          L1B_Headers%FH_is_available       = DAS_L1B_Headers%FH_is_available
          L1B_Headers%MPH_is_available      = DAS_L1B_Headers%MPH_is_available
          L1B_Headers%SPH_is_available      = DAS_L1B_Headers%SPH_is_available
          L1B_Headers%FH  => DAS_L1B_Headers%FH
          L1B_Headers%MPH => DAS_L1B_Headers%MPH
          L1B_Headers%SPH => DAS_L1B_Headers%SPH
       ENDIF
    ENDIF

    ! Set the number of BRCs
    Num_BRC = L1B_Headers%SPH%Geolocation_DSD%nr_of_dataset_records
    ! remind that the JobOrder system allows defining a timewindow,
    ! which enables selecting only part of the BRC's available in the
    ! L1B file, so the number of processed BRC's may be less than Num_BRC

  end subroutine aeolus_l2bp_import_L1B_Headers
    !  #]

  subroutine aeolus_l2bp_import_one_L1B_BRC &
    (L1B_Headers, L1B, iBRC, error_flag, L1B_filename, flag_init_only)
    !  #[

    TYPE(L1B_HDR_DataType),     intent(inout) :: L1B_Headers
    TYPE(L1B_BRC_DataType),     intent(out)   :: L1B
    integer,                    intent(in)    :: iBRC
    integer,                    intent(out)   :: error_flag
    character(len=*), optional, intent(in)    :: L1B_filename ! without extension
    logical,          optional, intent(in)    :: flag_init_only ! optional

    ! local variables

    logical :: do_init_only

    error_flag = no_error

    do_init_only = .FALSE.
    IF (present(flag_init_only)) do_init_only=flag_init_only

    IF (do_init_only.or.present(L1B_filename)) THEN
       ! initialise the L1B BRC datastructure (no headers, datasets only)
       call Init_L1B_BRC_DataStructure (L1B, error_flag)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted Init_L1B_BRC_DataStructure")
          return
       END IF
    END IF

    IF (.not.(do_init_only)) THEN
       IF (present(L1B_filename)) THEN
          ! read the L1B file
          call ReadL1Bfile(L1B_Headers, L1B_filename, error_flag, &
                 L1B = L1B, Which_DSR =  iBRC, &
                 ReadHeaders=.false., ReadGeoloc=.true., ReadPCD=.true., & 
                 ReadGWD=.true., ReadMeas=.true., ReadUS=.true., &
                 ReadWV=.true., ReadCal=.true.)
          ! TODO:
          !     ReadAll=.true., ReadHeaders=.false.) 
          ! this does not yet work because ReadAll overwrites the ReadHeaders
          ! setting. To fix this the order of the optional argument handling
          ! in ReadL1Bfile needs to be changed.
          IF (error_flag .ne. no_error) THEN
             call logmsg(log_error,"aborted ReadL1Bfile")
          return
          END IF

       ELSE
          !  import from memory
          IF (.not.associated(DAS_L1B_BRCs)) THEN
             call logmsg(log_error,"aeolus_l2bp_import_one_L1B_BRC: ERROR!! "//&
                  "L1B data are not available in the virtual DAS")
             error_flag = error_programming
             return
          ELSE
             IF (iBRC > size(DAS_L1B_BRCs)) THEN
                call logmsg(log_error,"aeolus_l2bp_import_one_L1B_BRC: ERROR!! "//&
                     "requested BRC is beyond available range")
                error_flag = error_programming
                return
             ELSE
                ! JK: DO NOT initialise this struct again for now,
                ! since this introduces memory leaks in the system.
                ! It will allocate the binary datasets, after which the pointers
                ! to them are lost, because they are redirected to the
                ! virtual das below !
   
                ! initialise the L1B BRC datastructure (no headers, datasets only)
                !call Init_L1B_BRC_DataStructure (L1B, error_flag)
                !IF (error_flag .ne. no_error) THEN
                !   call logmsg(log_error,"aborted Init_L1B_BRC_DataStructure")
                !   return
                !END IF
                
                !L1B = DAS_L1B_BRCs(iBRC)
                ! make explicit what happens here:
                ! (see remark in aeolus_l2bp_import_L1B_Headers for details)
                L1B%Geoloc_ADS => DAS_L1B_BRCs(iBRC)%Geoloc_ADS
                L1B%PCD_ADS    => DAS_L1B_BRCs(iBRC)%PCD_ADS
                L1B%GWD_ADS    => DAS_L1B_BRCs(iBRC)%GWD_ADS
                L1B%Meas_ADS   => DAS_L1B_BRCs(iBRC)%Meas_ADS
                L1B%Cal_ADS    => DAS_L1B_BRCs(iBRC)%Cal_ADS
                L1B%US_MDS     => DAS_L1B_BRCs(iBRC)%US_MDS
                L1B%WV_MDS     => DAS_L1B_BRCs(iBRC)%WV_MDS
             ENDIF
          ENDIF
       ENDIF
    ENDIF

  end subroutine aeolus_l2bp_import_one_L1B_BRC
    !  #]

  subroutine aeolus_l2bp_import_AMD_Headers &
    (AMD_Headers, Num_AMD_BRC, error_flag, AMD_filename, iAMD_file)
    !  #[
    TYPE(AMD_HDR_DataType),     intent(out) :: AMD_Headers
    integer,                    intent(out) :: Num_AMD_BRC
    integer,                    intent(out) :: error_flag
    character(len=*), optional, intent(in)  :: AMD_filename ! without extension
    integer,          optional, intent(in)  :: iAMD_file

    ! local variables
    integer(IntAl) :: Temp_Num_AMD_BRC

    error_flag = no_error

    IF (present(AMD_filename)) THEN
       call logmsg(log_progress,"aeolus_l2bp_import_AMD_Headers: "//&
            "reading aux met data from EE(.hdr, .dbl): "//trim(AMD_filename))
        
       ! initialise the AMD HDR datastructure (FH, MPH, SPH)
       call Init_AMD_HDR_DataStructure(AMD_Headers, error_flag)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted Init_AMD_HDR_DataStructure")
          return
       END IF

       ! read the headers of the AMD file.
       call ReadAMDfile(AMD_Headers, AMD_filename, error_flag, &
                        Which_DSR =  1, ReadHeaders=.true.)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted ReadAMDfile")
          return
       END IF

    ELSE
       !  import the AMD headers from memory
       IF (.not.associated(DAS_AMD_Headers)) THEN
          call logmsg(log_error,"aeolus_l2bp_import_AMD_Headers: ERROR!! "//&
               "AMD_Headers data are not available in the virtual DAS")
          error_flag = error_programming
          return
       ELSE IF (.not.present(iAMD_file)) THEN
             call logmsg(log_error,"aeolus_l2bp_import_AMD_Headers: ERROR!! "//&
                  "iAMD_file argument is missing")
             error_flag = error_programming
             return
       ELSE
          IF (iAMD_file > size(DAS_L1B_BRCs)) THEN
             call logmsg(log_error,"aeolus_l2bp_import_AMD_Headers: ERROR!! "//&
                  "requested iAMD_file is beyond available range")
             error_flag = error_programming
             return
          ELSE
             ! JK: DO NOT initialise this struct again for now,
             ! since this introduces memory leaks in the system.
             ! It will allocate the FH/MPH/SPH blocks, after which the pointers
             ! to them are lost, because they are redirected to the
             ! virtual das below !

             ! initialise the AMD HDR datastructure (FH, MPH, SPH)
             !call Init_AMD_HDR_DataStructure(AMD_Headers, error_flag)
             !IF (error_flag .ne. no_error) THEN
             !   call logmsg(log_error,"aborted Init_AMD_HDR_DataStructure")
             !   return
             !END IF

             !AMD_Headers = DAS_AMD_Headers(iAMD_file)
             ! make explicit what happens here:
             ! (see remark in aeolus_l2bp_import_L1B_Headers for details)
             AMD_Headers%FH_is_available  = DAS_AMD_Headers(iAMD_file)%FH_is_available
             AMD_Headers%MPH_is_available = DAS_AMD_Headers(iAMD_file)%MPH_is_available
             AMD_Headers%SPH_is_available = DAS_AMD_Headers(iAMD_file)%SPH_is_available
             AMD_Headers%FH  => DAS_AMD_Headers(iAMD_file)%FH
             AMD_Headers%MPH => DAS_AMD_Headers(iAMD_file)%MPH
             AMD_Headers%SPH => DAS_AMD_Headers(iAMD_file)%SPH
          ENDIF
       ENDIF
    ENDIF

    ! Set the number of AMD BRCs
    call GetIntAl(AMD_Headers%SPH%GeoADS1_DSD%num_dsr, &
                  Temp_Num_AMD_BRC, error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"conversion error for "//&
            "AMD_Headers%SPH%GeoADS1_DSD%num_dsr; "//&
            "Aborted aeolus_l2bp_import_AMD_Headers")
       return
    END IF
    Num_AMD_BRC = int(Temp_Num_AMD_BRC)

  end subroutine aeolus_l2bp_import_AMD_Headers
    !  #]

  subroutine aeolus_l2bp_import_one_AMD_BRC &
    (AMD_Headers, AMD, AMD_iBRC, error_flag, AMD_filename, iAMD_file, flag_init_only)
    !  #[

    TYPE(AMD_HDR_DataType),     intent(inout) :: AMD_Headers
    TYPE(AMD_BRC_DataType),     intent(out)   :: AMD
    integer,                    intent(in)    :: AMD_iBRC
    integer,                    intent(out)   :: error_flag
    character(len=*), optional, intent(in)    :: AMD_filename ! without extension
    integer,          optional, intent(in)    :: iAMD_file
    logical,          optional, intent(in)    :: flag_init_only ! optional

    ! local variables

    logical :: do_init_only

    error_flag = no_error

    do_init_only = .FALSE.
    IF (present(flag_init_only)) do_init_only=flag_init_only

    IF (do_init_only.or.present(AMD_filename)) THEN
       ! initialise the AMD BRC datastructure (no headers, datasets only)
       call Init_AMD_BRC_DataStructure (AMD, 0, error_flag)
       IF (error_flag .ne. no_error) THEN
          call logmsg(log_error,"aborted Init_AMD_BRC_DataStructure")
          return
       END IF
    ENDIF

    IF (.not.(do_init_only)) THEN

       IF (present(AMD_filename)) THEN
          ! read the AMD file
          call ReadAMDfile(AMD_Headers, AMD_filename, error_flag, &
                 AMD = AMD, Which_DSR =  AMD_iBRC, &
                 ReadHeaders=.false., &
                 ReadGeoloc_offnadir=.true., &
                 ReadGeoloc_nadir=.true., & 
                 ReadMet_offnadir=.true., &
                 ReadMet_nadir=.true.)
          ! TODO:
          !     ReadAll=.true., ReadHeaders=.false.) 
          ! this does not yet work because ReadAll overwrites the ReadHeaders
          ! setting. To fix this the order of the optional argument handling
          ! in ReadL1Bfile needs to be changed.
          IF (error_flag .ne. no_error) THEN
             call logmsg(log_error,"aborted ReadAMDfile")
             return
          END IF

       ELSE
          !  import from memory
          IF (.not.associated(DAS_AMD_BRCs)) THEN
             call logmsg(log_error,"aeolus_l2bp_import_one_AMD_BRC: ERROR!! "//&
                  "AMD_BRCs data are not available in the virtual DAS")
             error_flag = error_programming
             return
          ELSE IF (.not.present(iAMD_file)) THEN
                call logmsg(log_error,"aeolus_l2bp_import_one_AMD_BRC: ERROR!! "//&
                     "iAMD_file argument is missing")
                error_flag = error_programming
                return
          ELSE
             IF (iAMD_file > size(DAS_AMD_BRCs,1)) THEN
                call logmsg(log_error,"aeolus_l2bp_import_one_AMD_BRC: ERROR!! "//&
                     "requested iAMD_file is beyond available range")
                error_flag = error_programming
                return
             ELSE
                IF (AMD_iBRC > size(DAS_AMD_BRCs,2)) THEN
                   call logmsg(log_error,"aeolus_l2bp_import_one_AMD_BRC: ERROR!! "//&
                        "requested AMD_iBRC is beyond available range")
                   error_flag = error_programming
                   return
                ELSE
                   ! JK: DO NOT initialise this struct again for now,
                   ! since this introduces memory leaks in the system.
                   ! It will allocate the binary blocks, after which the pointers
                   ! to them are lost, because they are redirected to the
                   ! virtual das below !
   
                   ! initialise the AMD BRC datastructure (no headers, datasets only)
                   !call Init_AMD_BRC_DataStructure (AMD, 0, error_flag)
                   !IF (error_flag .ne. no_error) THEN
                   !   call logmsg(log_error,"aborted Init_AMD_BRC_DataStructure")
                   !   return
                   !END IF

                   !AMD = DAS_AMD_BRCs (iAMD_file,AMD_iBRC)
                   ! make explicit what happens here:
                   ! (see remark in aeolus_l2bp_import_L1B_Headers for details)
                   AMD%Geoloc_ADS_off_nadir => DAS_AMD_BRCs (iAMD_file,AMD_iBRC)%Geoloc_ADS_off_nadir
                   AMD%Geoloc_ADS_nadir     => DAS_AMD_BRCs (iAMD_file,AMD_iBRC)%Geoloc_ADS_nadir
                   AMD%Met_MDS_off_nadir    => DAS_AMD_BRCs (iAMD_file,AMD_iBRC)%Met_MDS_off_nadir
                   AMD%Met_MDS_nadir        => DAS_AMD_BRCs (iAMD_file,AMD_iBRC)%Met_MDS_nadir
                ENDIF
             ENDIF
          ENDIF
       ENDIF
    ENDIF

  end subroutine aeolus_l2bp_import_one_AMD_BRC
    !  #]

  subroutine aeolus_l2bp_load_data_to_vdas &
      (L2BP_Settings, error_flag)
    !  #[

    TYPE(L2BP_Settings_type), intent(in)  :: L2BP_Settings
    integer,                  intent(out) :: error_flag

    ! local variables
    integer                  :: AMD_file, AllocStatus, AMD_iBRC, iBRC
    ! intermediate data necessary to load the virtual data assimilation system
!    TYPE(L1B_HDR_DataType)   :: preDAS_L1B_Headers
!    TYPE(L1B_BRC_DataType)   :: preDAS_L1B
!    TYPE(AMD_HDR_DataType)   :: preDAS_AMD_Headers
!    TYPE(AMD_BRC_DataType)   :: preDAS_AMD
    integer                  :: DAS_Num_BRC
    integer, allocatable, dimension(:) :: DAS_Num_AMD_BRC

    error_flag = no_error

    ! read the L1B Headers and transfer to virtual DAS
    allocate(DAS_L1B_Headers,stat=AllocStatus)
    IF (AllocStatus.ne.0) error_flag=error_allocate

    call aeolus_l2bp_import_L1B_Headers (DAS_L1B_Headers, &
               DAS_Num_BRC, error_flag, &
               L1B_filename= L2BP_Settings%L1Bfile%name)
    IF (error_flag.ne.no_error) return
    call logmsg(log_progress,"finished aeolus_l2bp_import_L1B_Headers")

!    DAS_L1B_Headers = preDAS_L1B_Headers

! this logmsg is located in a strange place now,
! but I'll leave it here for now to not break any tests
! TODO: move in upwards to the allocate statement
    call logmsg(log_progress,"finished allocate DAS_L1B_Headers")

    ! read the L1B BRCs and transfer to virtual DAS
    allocate(DAS_L1B_BRCs(DAS_Num_BRC),stat=AllocStatus)
    IF (AllocStatus.ne.0) error_flag=error_allocate
    IF (error_flag.ne.no_error) return
    call logmsg(log_progress,"finished allocate DAS_L1B_BRCs(preDAS_Num_BRC)")
! TODO: replace with
!    call logmsg(log_progress,"finished allocate DAS_L1B_BRCs(DAS_Num_BRC)")

    DO iBRC = 1, DAS_Num_BRC
          call aeolus_l2bp_import_one_L1B_BRC (DAS_L1B_Headers, DAS_L1B_BRCs(iBRC), iBRC, &
                       error_flag, L1B_filename=L2BP_Settings%L1Bfile%name)
          IF (error_flag.ne.no_error) return
          call logmsg(log_progress,"finished aeolus_l2bp_import_one_L1B_BRC")
!          DAS_L1B_BRCs(iBRC) = preDAS_L1B
    ENDDO

    ! read the AMD Headers and transfer to virtual DAS
    allocate(DAS_AMD_Headers(L2BP_Settings%NumNWPfiles),stat=AllocStatus)
    IF (AllocStatus.ne.0) error_flag=error_allocate
    IF (error_flag.ne.no_error) return
    call logmsg(log_progress,"finished allocate DAS_AMD_Headers(L2BP_Settings%NumNWPfiles)")

    allocate(DAS_Num_AMD_BRC(L2BP_Settings%NumNWPfiles),stat=AllocStatus)
    IF (AllocStatus.ne.0) error_flag=error_allocate
    IF (error_flag.ne.no_error) return
    call logmsg(log_progress,"finished allocate preDAS_Num_AMD_BRC"//&
         & "(L2BP_Settings%NumNWPfiles)")
! TODO: replace with
!    call logmsg(log_progress,"finished allocate DAS_Num_AMD_BRC"//&
!         & "(L2BP_Settings%NumNWPfiles)")

    DO AMD_file = 1, L2BP_Settings%NumNWPfiles
          call aeolus_l2bp_import_AMD_Headers (DAS_AMD_Headers(AMD_file), &
                       DAS_Num_AMD_BRC(AMD_file), error_flag, &
                       AMD_filename=L2BP_Settings%NWPfile(AMD_file)%name)
          IF (error_flag.ne.no_error) return
          call logmsg(log_progress,"finished aeolus_l2bp_import_AMD_Headers")
!          DAS_AMD_Headers(AMD_file) = preDAS_AMD_Headers
    ENDDO

    ! read the AMD BRCs and transfer to virtual DAS
    allocate(DAS_AMD_BRCs(L2BP_Settings%NumNWPfiles, &
        maxval(DAS_Num_AMD_BRC)),stat=AllocStatus)
    IF (AllocStatus.ne.0) error_flag=error_allocate
    IF (error_flag.ne.no_error) return
    call logmsg(log_progress,"finished allocate DAS_AMD_BRCs" //&
         & "(L2BP_Settings%NumNWPfiles,maxval(preDAS_Num_AMD_BRC))")
! TODO: replace with
!    call logmsg(log_progress,"finished allocate DAS_AMD_BRCs" //&
!         & "(L2BP_Settings%NumNWPfiles,maxval(DAS_Num_AMD_BRC))")

    DO AMD_file = 1, L2BP_Settings%NumNWPfiles
          DO AMD_iBRC = 1, DAS_Num_AMD_BRC(AMD_file)
!                  preDAS_AMD_Headers = DAS_AMD_Headers(AMD_file)
                  call aeolus_l2bp_import_one_AMD_BRC (DAS_AMD_Headers(AMD_file),  &
                               DAS_AMD_BRCs(AMD_file,AMD_iBRC),  &
                               AMD_iBRC, error_flag,  &
                               AMD_filename= L2BP_Settings%NWPfile(AMD_file)%name)
                  IF (error_flag.ne.no_error) return
                  call logmsg(log_progress,"finished aeolus_l2bp_import_one_AMD_BRC")
!                  DAS_AMD_BRCs (AMD_file,AMD_iBRC) = preDAS_AMD
          ENDDO
    ENDDO

    virtual_das_initialised = .true.

  end subroutine aeolus_l2bp_load_data_to_vdas

    !  #]

  subroutine aeolus_l2bp_clear_vdas  (error_flag)
    !  #[
    integer, intent(out)    :: error_flag

    ! local variables
    integer :: iBRC, iAMD_file, iAMD_BRC
    integer :: Num_BRCs, Num_AMD_files, Num_AMD_BRCs

    error_flag = no_error

    
    virtual_das_initialised = .false.

    if (associated(DAS_L1B_Headers)) then
       call Delete_L1B_HDR_DataStructure(DAS_L1B_Headers)
       deallocate(DAS_L1B_Headers)
    end if

    if (associated(DAS_L1B_BRCs)) then
       Num_BRCs = size(DAS_L1B_BRCs)
       DO iBRC=1,Num_BRCs
          call Delete_L1B_BRC_DataStructure(DAS_L1B_BRCs(iBRC))
       END DO
       deallocate(DAS_L1B_BRCs)
    end if

    if (associated(DAS_AMD_BRCs)) then
       Num_AMD_files = size(DAS_AMD_BRCs(:,1))
       Num_AMD_BRCs  = size(DAS_AMD_BRCs(1,:))
       DO iAMD_file=1,Num_AMD_files
          DO iAMD_BRC=1,Num_AMD_BRCs
             call Delete_AMD_BRC_DataStructure(DAS_AMD_BRCs(iAMD_file,iAMD_BRC))
          END DO
       END DO
       deallocate(DAS_AMD_BRCs)
    end if

    if (associated(DAS_AMD_Headers)) then
       Num_AMD_files = size(DAS_AMD_Headers)
       DO iAMD_file=1,Num_AMD_files
          call Delete_AMD_HDR_DataStructure(DAS_AMD_Headers(iAMD_file))
       END DO
       deallocate(DAS_AMD_Headers)
    end if

  end subroutine aeolus_l2bp_clear_vdas
    !  #]

  subroutine aeolus_l2bp_prepare_primary(L1B,L2B,AMD,iBRC, &
        L2B_Auxpar,WorkingData,error_flag)
    !  #[

    TYPE(L1B_BRC_DataType),           intent(inout)  :: L1B
    TYPE(L2BC_BRC_DataType),          intent(out)    :: L2B
    TYPE(AMD_BRC_DataType), dimension(:), pointer    :: AMD
    integer,                          intent(in)     :: iBRC
    TYPE(L2B_AuxPar_Type),            intent(in)     :: L2B_AuxPar
    TYPE(WorkingDataType),            intent(inout)  :: WorkingData
    integer,                          intent(out)    :: error_flag

    ! remark: L1B should become intent(in), but for the moment it holds
    ! also some working arrays, which need to be allocated/filled in here,
    ! so it is intent(inout) for the moment

    ! local variables
  
    integer            :: nr_Lite_BRCs_available
    integer            :: which_AMD_DSR
    character(len=6)   :: txt_N_meas
    character(len=6)   :: txt_which_AMD_DSR
    
    ! Variables for handling of AMD_DataStructure
    logical     :: from_off_nadir_flag

    ! init
    call logmsg(log_progress,"start of aeolus_l2bp_prepare_primary ...")

    error_flag = no_error

    ! Poli: comment out:
    ! N_meas = L1B%SPH%N_Meas_Max
    ! and replace by:
    ! (because L1B_Headers are not available at this level)
    N_meas = size(L1B%Geoloc_ADS%Meas_AOCS)

    call logmsg(log_info,"initialising L2B data structure")
    call Init_L2BC_BRC_DataStructure(L2B,N_meas, &
         N_Obs_Max_Mie, N_Obs_Max_Rayleigh, int(N_range_gates), &
         error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_info,"initialising and filling working arrays...")
    write(txt_N_meas,"(i6)",err=999) N_meas
    call logmsg(log_info,"N_Meas = "//trim(adjustl(txt_N_meas)))

    call InitWorkingData(WorkingData, int(N_range_gates,IntAs), &
         int(N_Meas,IntAs), int(N_Obs_Max_Mie,IntAs), &
         int(N_Obs_Max_Rayleigh,IntAs), error_flag)
    IF (error_flag .ne. no_error) return
    
    IF (use_lite_test_data) THEN
       ! Temporary read of Lite data to be able to do classification.       
       ! Fills in the working datastructs the rangegate definitions
       ! (alt_start, alt_end) and the obs_signal_level with Lite data
       ! also reads expected molecular backscatter which was produced
       ! by using a parametrised (climatological) profile
       call Fill_WorkingData_with_Lite_Data(WorkingData, &
            filename_LiteTestData, iBRC, nr_Lite_BRCs_available, error_flag)
       IF (error_flag .ne. no_error) return
    ELSE
       call Copy_L1B_fields2WorkingArrays(L1B,WorkingData)
    END IF

    IF (use_dummy_lat_lon) THEN
       ! overwrite in the working datastructs the lat-lon values 
       ! (to generate lat-lon values, in case they are not yet 
       !  available in the L1B-file, i.e. when using Lite data)
       call AddDummyLatLonData(WorkingData)
       ! remind that Copy_L1B_fields2WorkingArrays() also fills 
       ! these lat-lon fields
    END IF

    ! screen the L1B input data and write the results to the
    ! L2B%PCD_ADS datastructure in case any problems are found.
    ! also sets missing_indicators in the L1B datastructure
    ! for those elements that we use, and are missing according
    ! to the L1B%PCD_ADS
    call Screen_L1B_Data(L1B, WorkingData, L2B%PCD_ADS%L1B_Input_Screening, &
         L2B_AuxPar%Data%Screening_Params%L1B_Screening_Params,error_flag)
    IF (error_flag .ne. no_error) return

    ! this sets the vcog altitudes halfway in the range-bins.
    ! which is used to determine the altitudes for the NWP preprocessing
    call determine_vcog(WorkingData, error_flag)
    IF (error_flag .ne. no_error) return
       
    ! do_matching to select the proper AMD struct
    ! remark: the result may be: which_AMD_DSR=-1 
    !         when no suitable profile was found !
    ! TODO: return logical parameter from_off_nadir_flag
    call Find_Matching_AMD_Profile(AMD,WorkingData,&
         L2B_AuxPar%Data%WVM_Params%AMD_Matchup_Params,&
         L2B%PCD_ADS%L2B_AMD_Screening%L2B_AMD_Collocations, &
         which_AMD_DSR, iBRC,error_flag)
    IF (error_flag .ne. no_error) return

    write(txt_which_AMD_DSR,"(i6)",err=998) which_AMD_DSR
    call logmsg(log_info,"Selected AMD profile nr.: "//&
                trim(adjustl(txt_which_AMD_DSR)))

    if (which_AMD_DSR .gt. 0) then
       !  #[ Transfer from AMD to WorkingData - via interpolation

       from_off_nadir_flag = .true. ! TODO: get from Matchup output
       call Copy_AMD_to_WorkingData(from_off_nadir_flag, &
                                    AMD(which_AMD_DSR),&
                                    L1B,&
                                    WorkingData,error_flag)
       IF (error_flag .ne. no_error) return

       !  #]
    end if

    call logmsg(log_progress,"end of aeolus_l2bp_prepare_primary ...")

    return

998 error_flag = error_programming
    call logmsg(log_error,"Error in aeolus_l2bp_prepare_primary: "//&
                "Could not convert the integer which_AMD_DSR to "//&
                "a 6 digit string")
    return
       
999 error_flag = error_programming
    call logmsg(log_error,"Error in aeolus_l2bp_prepare_primary: "//&
                "Could not convert N_meas integer to a 6 digit string")
          return
       
  end subroutine aeolus_l2bp_prepare_primary
    !  #]
  subroutine L2BP_primary(L1B,L2B,WorkingData,RBC,L2B_Auxpar,error_flag)
    !  #[
       
    TYPE(L1B_BRC_DataType),           intent(inout)  :: L1B
    TYPE(L2BC_BRC_DataType),          intent(inout)  :: L2B
    TYPE(WorkingDataType),            intent(inout)  :: WorkingData
    TYPE(RBC_DataType),               intent(inout)  :: RBC
    TYPE(L2B_AuxPar_Type),            intent(in)     :: L2B_AuxPar ! input
    integer,                          intent(out)    :: error_flag
       
    ! remark: L1B should become intent(in), but for the moment it holds
    ! also some working arrays, which need to be allocated/filled in here,
    ! so it is intent(inout) for the moment
       
    ! local variables

    integer :: o
       
    ! init
    call logmsg(log_progress,"start of L2BP_primary ...")

    error_flag = no_error

    ! here a new routine to do vertical interpolation for all range
    ! gates of all measurements can be added
       
    call Match_Mie_And_Rayleigh_Bins(WorkingData, error_flag)
    IF (error_flag .ne. no_error) return

    ! new, to add: copy_Meas_Geoloc_L1B_to_L2B
    ! (filling L2B Obs Geoloc. is done inside ConstructObservations)

    ! to add: calc_exp_molecular_backscatter from NWP or climatological data

    ! compare Rayleigh result to NWP/Climatological profiles
    ! to obtain an estimate of the extinction
    IF (Extinction_Needed) THEN
       call Calc_Extinction(L1B, WorkingData, &
                            MolExt_Method, PartExt_Method,&
                            error_flag, use_lite_test_data,&
                            calibr_on_upper_bin = (.not.use_lite_test_data))
       IF (error_flag .ne. no_error) return
    END IF

    ! NOTE (JK, 06-Dec-2007)
    ! We allways need backscatter_ratio for Rayleigh processing,
    ! so not calculating it is not an option.
    ! TODO: Therefore the BackscatterRatio_Needed switch may be either
    ! phased out, or we need to split it, one switch for calculating
    ! backscatter_ratio for calibration (that one might be switched off), 
    ! and another switch for calculating backscatter_ratio for Rayleigh 
    ! processing (has to be done always). But then they may come
    ! from different sources, and must be stored in different variables,
    ! the first one on measurement level, the second one on observation level.
    ! That's too much change for now. If everyone agrees this may be
    ! added in alter version.

    ! derive the backscatter ratio
    !IF (BackscatterRatio_Needed) THEN
    call Calc_BackscatterRatio(L1B, WorkingData, error_flag,             &
            Method         = L2B_AuxPar%Data%WVM_Params%&
                                Optical_Properties_Params%ScatRatio_Method, &
            FallbackMethod = L2B_AuxPar%Data%WVM_Params%&
                                Optical_Properties_Params%ScatRatio_Method2  )
    IF (error_flag .ne. no_error) return
    !END IF

    ! to add: Apply_Extinction_Correction() to Mie

    call Classify(WorkingData, &
                  L2B_AuxPar%Data%WVM_Params%Classification_Params, &
                  error_flag)
    IF (error_flag .ne. no_error) return
      
    ! General remark:
    ! the routines above, to calculate extinction, backscatterratio
    ! and to classify measurements may be replaced by the routines
    ! from the L2A algorithm when they become available
    ! Note from JK:
    ! At the current state of development I think we should not replace
    ! any algorithms, but just add new ones when they become available
    ! and present the user/researcher the choice to run the old or the
    ! new algorithm.

    call logmsg(log_info,"calling: Select_And_Weigh_Rayleigh_Meas")
    call Select_And_Weigh_Rayleigh_Meas(WorkingData, L2B, error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_info,"calling: Select_And_Weigh_Mie_Meas")
    call Select_And_Weigh_Mie_Meas(WorkingData, L2B, error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_info,"calling: Construct_Rayleigh_Obs")
    call Construct_Rayleigh_Obs(WorkingData, L2B, error_flag)
    IF (error_flag .ne. no_error) return

    call logmsg(log_info,"calling: Construct_Mie_Obs")
    call Construct_Mie_Obs(WorkingData, L2B, error_flag)
    IF (error_flag .ne. no_error) return

    ! TODO: add calculation of backscatter ratio on the observation level
    ! to be used as input for the Illiad scheme.
    ! -based on weighted average of signal levels?
    ! -or on weigted average of backscatter ratio's on measurement level ?

    call Process_Rayleigh_Observations(L1B, WorkingData, RBC, L2B, error_flag)
    IF (error_flag .ne. no_error) return
    call Process_Mie_Observations(L1B, WorkingData, L2B_AuxPar, &
                                  L2B, error_flag)
    IF (error_flag .ne. no_error) return
       
    ! Copy the processing results from the WorkingData to the L2B output
    ! structure, and apply unit conversions like m->km, Pa->hPa, etc.
    call Format_and_Copy_WD_to_L2B_Outp(WorkingData,L2B,error_flag)
    IF (error_flag .ne. no_error) return

    ! copy N_Obs_Actual from Mie/Rayleigh datasets to other datasets

    L2B%PCD_ADS%N_Obs_Mie_Actual         = L2B%Mie_MDS%N_Obs_Mie_Actual
    L2B%Geoloc_ADS%N_Obs_Mie_Actual      = L2B%Mie_MDS%N_Obs_Mie_Actual
    L2B%PCD_ADS%N_Obs_Rayleigh_Actual    = L2B%Rayleigh_MDS%N_Obs_Rayleigh_Actual
    L2B%Geoloc_ADS%N_Obs_Rayleigh_Actual = L2B%Rayleigh_MDS%N_Obs_Rayleigh_Actual

! TODO: remove these test prints
!       or convert it to use logmsg in stead of directly print/write

    ! print classification counts for this BRC
    print *,"----"
    call Print_Constructed_Obs(L2B)
    ! improved version printing the same numbers
    print *,"----"
    call Print_Constr_Obs_Matched(WorkingData,L2B)
    print *,"----"
    
! TODO: remove these test prints
!       or convert it to use logmsg in stead of directly print/write

    ! print the Map describing which measurement contributed
    ! to which observation for the Rayleigh channel
    print *,"Map of Rayleigh Observations:"
    call Print_Meas_Map(L2B%Rayleigh_MDS%Map_of_L1B_Measurements_Used)!,&
    !        prefix="DummyPrefix",PrintAsMap=.true.,PrintAll=.false.,&
    !        error_flag=error_flag)
    !IF (error_flag .ne. no_error) return

    ! print the Map describing which measurement contributed
    ! to which observation for the Rayleigh channel
    print *,"Map of Mie Observations:"
    call Print_Meas_Map(L2B%Mie_MDS%Map_of_L1B_Measurements_Used)!,&
    !        prefix="DummyPrefix",PrintAsMap=.true.,PrintAll=.false.,&
    !        error_flag=error_flag)
    !IF (error_flag .ne. no_error) return
       
    ! at this point a second and better(?) vcog determination
    ! can be done, using the extinction from the L2A code
    ! (extinction is now only available after calling the classify 
    !  code below)
    !
    ! then with these new vcog values
    ! the new routine to do vertical interpolation for all range
    ! gates of all measurements can be called again
    
! TODO: remove these test prints
!       or convert it to use logmsg in stead of directly print/write
    
    !print *,"=========== printing L2B%Rayleigh_MDS: =============="
    !call Print_L2B_Rayleigh_MDS(L2B%Rayleigh_MDS,error_flag,PrintAll=.false.)
    print *,"=========== printing hlos_profiles:    =============="
    do o=1,L2B%Rayleigh_MDS%N_Obs_Rayleigh_Actual
       print *,"hlos profile for rayleigh observation nr. ",o
       call Print_L2B_Rayleigh_hlos_profile(L2B%Rayleigh_MDS,&
                                            error_flag,Obs_Nr=o)
    end do
    do o=1,L2B%Mie_MDS%N_Obs_Mie_Actual
       print *,"hlos profile for mie observation nr. ",o
       call Print_L2B_Mie_hlos_profile(L2B%Mie_MDS,&
                                       error_flag,Obs_Nr=o)
    end do

    !------------------------------------------------------------
    ! done with the L2BP algorithm for this BRC
    ! clean up to enable handling the next BRC
    !------------------------------------------------------------

    call DeleteWorkingData(WorkingData)

    call logmsg(log_progress,"end of L2BP_primary ...")

    return

  end subroutine L2BP_primary
    !  #]

  subroutine aeolus_l2bp_export_one_L1B_BRC (L1B, error_flag)
    !  #[
    TYPE(L1B_BRC_DataType), intent(inout) :: L1B
    integer,                intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error
    
    ! this would be the proper place to export to memory, if needed

    if (virtual_das_initialised) then
       ! the pointers inside L1B currently point to the virtual DAS
       ! so all we have to do is nullify them.
       ! The actual deallocation is done in subroutine aeolus_l2bp_clear_vdas
       nullify(L1B%Geoloc_ADS)
       nullify(L1B%PCD_ADS)
       nullify(L1B%GWD_ADS)
       nullify(L1B%Meas_ADS)
       nullify(L1B%Cal_ADS)
       nullify(L1B%US_MDS)
       nullify(L1B%WV_MDS)
    else
! not needed, is called from within Delete_L1B_BRC_DataStructure()
!       call Empty_L1B_BinaryDatasets (L1B)

       call Delete_L1B_BRC_DataStructure(L1B)
    end if

  end subroutine aeolus_l2bp_export_one_L1B_BRC
    !  #]

  subroutine aeolus_l2bp_fill_L2B_Headers (L1B_Headers, L2B, L2B_Headers, &
       L2BP_Settings, &
       Num_L2B_BRCs, DateTimeStart, DateTimeStop, &
       lat_start, lat_stop, lon_start, lon_stop, &
       M_Meas0, M_Mie0, M_Rayleigh0, error_flag)
    !  #[
    TYPE(L1B_HDR_DataType),        intent(in)  :: L1B_Headers   ! input
    TYPE(L2BC_BRC_DataType), dimension(:), pointer :: L2B       ! input
    TYPE(L2BC_HDR_DataType),       intent(out) :: L2B_Headers ! output
    TYPE(L2BP_Settings_type),      intent(in)  :: L2BP_Settings ! input
    integer(IntAus),               intent(in)  :: Num_L2B_BRCs  ! input
    type(DateTimeType),            intent(in)  :: DateTimeStart ! input
    type(DateTimeType),            intent(in)  :: DateTimeStop  ! input
    integer(IntAl),                intent(in)  :: lat_start     ! input
    integer(IntAl),                intent(in)  :: lat_stop      ! input
    integer(IntAl),                intent(in)  :: lon_start     ! input
    integer(IntAl),                intent(in)  :: lon_stop      ! input
    integer(IntAuc),               intent(out) :: M_Meas0     ! output
    integer(IntAuc),               intent(out) :: M_Mie0      ! output
    integer(IntAuc),               intent(out) :: M_Rayleigh0 ! output
    integer,                       intent(out) :: error_flag  ! output

    ! local variables

    real(r8_)       :: Sat_Track
    integer(IntAus) :: Num_Valid_Mie_Profiles, Num_Valid_Rayleigh_Profiles
    integer(IntAus) :: Num_Clear_Mie_Profiles, Num_Clear_Rayleigh_Profiles
    integer(IntAus) :: Num_Cloud_Mie_Profiles, Num_Cloud_Rayleigh_Profiles
    integer(IntAus) :: Num_Mie_Prof_Warm_Pulses, Num_Rayleigh_Prof_Warm_Pulses
    integer(IntAus) :: Num_Profiles_Surface_Mie,      Num_Profiles_Surface_Ray
    integer(IntAus) :: Num_Valid_Obs_Prof_L1B_Mie,    Num_Valid_Obs_Prof_L1B_Ray
    integer(IntAus) :: Num_Valid_Meas_Prof_L1B_Mie,   Num_Valid_Meas_Prof_L1B_Ray
    integer(IntAus) :: Num_Valid_Obs_Bins_L1B_Mie,    Num_Valid_Obs_Bins_L1B_Ray
    integer(IntAus) :: Num_Valid_Meas_Bins_L1B_Mie,   Num_Valid_Meas_Bins_L1B_Ray
    integer(IntAus) :: Num_Invalid_Meas_Prof_L1B_Mie, Num_Invalid_Meas_Prof_L1B_Ray
    integer(IntAus) :: Num_Invalid_Meas_Prof_L2B_Mie, Num_Invalid_Meas_Prof_L2B_Ray
    integer(IntAus) :: Num_Invalid_Meas_Bins_L1B_Mie, Num_Invalid_Meas_Bins_L1B_Ray
    integer(IntAus) :: Num_Invalid_Meas_Bins_L2B_Mie, Num_Invalid_Meas_Bins_L2B_Ray
    integer(IntAus) :: Num_Valid_Obs_L2B_Mie,         Num_Valid_Obs_L2B_Ray

    error_flag = no_error

    ! initialise the L2B_Headers datastructure (FH,MPH,SPH)
    call Init_L2BC_HDR_DataStructure (L2B_Headers, error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted Init_L2BC_BRC_DataStructure")
       return
    END IF

    ! fill the L2B FH
    ! TODO: passing L2BP_Settings%L2B_AuxPar%Data%FH_Default_Fields is no longer
    !       needed here, so may be phased out
    call Fill_L2BC_FH(L2B_Headers, L2BP_Settings%L2B_AuxPar%Data%FH_Default_Fields, &
                      L2BP_Settings,DateTimeStart, DateTimeStop, error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted Fill_L2BC_FH")
       return
    END IF

    ! fill the L2B MPH  
    call Fill_L2B_MPH_From_L1B_MPH(L1B_Headers%MPH,L2B_Headers%MPH,error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted Fill_L2B_MPH_From_L1B_MPH")
       return
    END IF

    ! this routine fills the MPH items that have not yet been copied
    ! from the L1B MPH (by subroutine Fill_L2B_MPH_From_L1B_MPH)
    ! Note the L2BC_Headers%FH is an input for Fill_Remaining_L2BC_MPH_Items
    ! so it MUST be called after Fill_L2BC_FH
    call Fill_Remaining_L2BC_MPH_Items(L2B_Headers%MPH,L2B_Headers%FH,&
                         L2BP_Settings,&
                         L2BP_Settings%L2B_AuxPar%Data%MPH_Default_Fields, &
                         DateTimeStart, DateTimeStop, &
                         error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted Fill_L2BC_MPH")
       return
    END IF

    ! gather statistics for the L2B SPH
    call GetStatisticsL2BMieDS(L2B,                  &
                      int(Num_L2B_BRCs),             &
                      M_Mie0,                        &
                      Num_Valid_Mie_Profiles,        &
                      Num_Clear_Mie_Profiles,        &
                      Num_Cloud_Mie_Profiles,        &
                      Num_Mie_Prof_Warm_Pulses,      &
                      Num_Profiles_Surface_Mie,      &
                      Num_Invalid_Meas_Prof_L2B_Mie, &
                      Num_Invalid_Meas_Bins_L2B_Mie, &
                      Num_Valid_Obs_L2B_Mie,         &
                      error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted GetStatisticsL2BMieDS")
       return
    END IF

    call GetStatisticsL2BRayleighDS(L2B,             &
                      int(Num_L2B_BRCs),&
                      M_Rayleigh0,                   &
                      Num_Valid_Rayleigh_Profiles,   &
                      Num_Clear_Rayleigh_Profiles,   &
                      Num_Cloud_Rayleigh_Profiles,   &
                      Num_Rayleigh_Prof_Warm_Pulses, &
                      Num_Profiles_Surface_Ray,      &
                      Num_Invalid_Meas_Prof_L2B_Ray, &
                      Num_Invalid_Meas_Bins_L2B_Ray, &
                      Num_Valid_Obs_L2B_Ray,         &
                      error_flag)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted GetStatisticsL2BRayleighDS")
       return
    END IF

    ! TODO: the following are dummy values for now:
    Num_Valid_Obs_Prof_L1B_Mie    = 0_IntAuc
    Num_Valid_Obs_Prof_L1B_Ray    = 0_IntAuc
    Num_Valid_Meas_Prof_L1B_Mie   = 0_IntAuc
    Num_Valid_Meas_Prof_L1B_Ray   = 0_IntAuc
    Num_Valid_Obs_Bins_L1B_Mie    = 0_IntAuc
    Num_Valid_Obs_Bins_L1B_Ray    = 0_IntAuc
    Num_Valid_Meas_Bins_L1B_Mie   = 0_IntAuc
    Num_Valid_Meas_Bins_L1B_Ray   = 0_IntAuc
    Num_Invalid_Meas_Prof_L1B_Mie = 0_IntAuc
    Num_Invalid_Meas_Prof_L1B_Ray = 0_IntAuc
    Num_Invalid_Meas_Bins_L1B_Mie = 0_IntAuc
    Num_Invalid_Meas_Bins_L1B_Ray = 0_IntAuc

    ! grab info from the L1B_Headers%SPH
    M_Meas0 = int(L1B_Headers%SPH%N_Meas_Max,IntAuc)
    read (L1B_Headers%SPH%Sat_Track,*) Sat_Track

    ! fill the L2B SPH
    call Fill_L2BC_SPH(L2B_Headers, int(Num_L2B_BRCs), &
           lat_start, lat_stop, lon_start, lon_stop, &
           Sat_Track, M_Mie0, M_Rayleigh0, M_Meas0,  &
           Num_Valid_Mie_Profiles,        Num_Valid_Rayleigh_Profiles,   &
           Num_Clear_Mie_Profiles,        Num_Clear_Rayleigh_Profiles,   &
           Num_Cloud_Mie_Profiles,        Num_Cloud_Rayleigh_Profiles,   &
           Num_Mie_Prof_Warm_Pulses,      Num_Rayleigh_Prof_Warm_Pulses, &
           Num_Profiles_Surface_Mie,      Num_Profiles_Surface_Ray,      &
           Num_Valid_Obs_Prof_L1B_Mie,    Num_Valid_Obs_Prof_L1B_Ray,    &
           Num_Valid_Meas_Prof_L1B_Mie,   Num_Valid_Meas_Prof_L1B_Ray,   &
           Num_Valid_Obs_Bins_L1B_Mie,    Num_Valid_Obs_Bins_L1B_Ray,    &
           Num_Valid_Meas_Bins_L1B_Mie,   Num_Valid_Meas_Bins_L1B_Ray,   &
           Num_Invalid_Meas_Prof_L1B_Mie, Num_Invalid_Meas_Prof_L1B_Ray, &
           Num_Invalid_Meas_Prof_L2B_Mie, Num_Invalid_Meas_Prof_L2B_Ray, &
           Num_Invalid_Meas_Bins_L1B_Mie, Num_Invalid_Meas_Bins_L1B_Ray, &
           Num_Invalid_Meas_Bins_L2B_Mie, Num_Invalid_Meas_Bins_L2B_Ray, &
           Num_Valid_Obs_L2B_Mie,         Num_Valid_Obs_L2B_Ray,         &
           error_flag, &
           L2C_flag=.false.)
    IF (error_flag .ne. no_error) THEN
       call logmsg(log_error,"aborted Fill_L2BC_SPH")
       return
    END IF

  end subroutine aeolus_l2bp_fill_L2B_Headers
    !  #]

  subroutine aeolus_l2bp_export_L1B_Headers (L1B_Headers, error_flag)
    !  #[
    TYPE(L1B_HDR_DataType), intent(inout) :: L1B_Headers
    integer,                intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error

    ! this would be the proper place to export to memory, if needed

    if (virtual_das_initialised) then
       ! the pointers inside L1B_Headers currently point to the virtual DAS
       ! so all we have to do is nullify them.
       ! The actual deallocation is done in subroutine aeolus_l2bp_clear_vdas
       nullify(L1B_Headers%FH)
       nullify(L1B_Headers%MPH)
       nullify(L1B_Headers%SPH)
    else
       call Delete_L1B_HDR_DataStructure(L1B_Headers)
    end if

  end subroutine aeolus_l2bp_export_L1B_Headers
    !  #]

  subroutine aeolus_l2bp_export_L2B_Headers (L2B_Headers, error_flag)
    !  #[
    TYPE(L2BC_HDR_DataType), intent(inout) :: L2B_Headers
    integer,                 intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error

    ! this would be the proper place to export to memory, if needed

    call Delete_L2BC_HDR_DataStructure (L2B_Headers)

  end subroutine aeolus_l2bp_export_L2B_Headers
    !  #]

  subroutine aeolus_l2bp_export_one_L2B_BRC (L2B, error_flag)
    !  #[
    TYPE(L2BC_BRC_DataType), intent(inout) :: L2B
    integer,                 intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error
    
    ! this would be the proper place to export to memory, if needed

    call Delete_L2BC_BRC_DataStructure(L2B)

  end subroutine aeolus_l2bp_export_one_L2B_BRC
    !  #]

  subroutine aeolus_l2bp_export_AMD_Headers (AMD_Headers, error_flag)
    !  #[
    TYPE(AMD_HDR_DataType), intent(inout) :: AMD_Headers
    integer,                intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error
    
    ! this would be the proper place to export to memory, if needed

    if (virtual_das_initialised) then
       ! the pointers inside AMD_Headers currently point to the virtual DAS
       ! so all we have to do is nullify them.
       ! The actual deallocation is done in subroutine aeolus_l2bp_clear_vdas
       nullify(AMD_Headers%FH)
       nullify(AMD_Headers%MPH)
       nullify(AMD_Headers%SPH)
    else
       call Delete_AMD_HDR_DataStructure (AMD_Headers)
    end if

  end subroutine aeolus_l2bp_export_AMD_Headers
    !  #]

  subroutine aeolus_l2bp_export_one_AMD_BRC (AMD, error_flag)
    !  #[
    TYPE(AMD_BRC_DataType), intent(inout) :: AMD
    integer,                intent(out)   :: error_flag

    ! no local variables

    error_flag = no_error
    
    ! this would be the proper place to export to memory, if needed

    if (virtual_das_initialised) then
       ! the pointers inside AMD currently point to the virtual DAS
       ! so all we have to do is nullify them.
       ! The actual deallocation is done in subroutine aeolus_l2bp_clear_vdas
       nullify(AMD%Geoloc_ADS_off_nadir)
       nullify(AMD%Geoloc_ADS_nadir)
       nullify(AMD%Met_MDS_off_nadir)
       nullify(AMD%Met_MDS_nadir)
    else
       call Delete_AMD_BRC_DataStructure(AMD)
    end if

  end subroutine aeolus_l2bp_export_one_AMD_BRC
    !  #]

  subroutine Format_and_Copy_WD_to_L2B_Outp(WorkingData,L2B,error_flag)
    !  #[
    TYPE(WorkingDataType),  intent(in)    :: WorkingData ! input 
    TYPE(L2BC_BRc_DataType),intent(inout) :: L2B         ! some items filled
    integer,                intent(out)   :: error_flag  ! output

    ! local variable
    TYPE(WD_RaylObsRangeGateType), pointer :: WD_RaylObsRangeGate 
    TYPE(WD_MieObsRangeGateType),  pointer :: WD_MieObsRangeGate 
    TYPE(Geolocation_Bin_type),    pointer :: L2B_Geoloc_Bin
    integer :: o,r,Nr_Obs

    error_flag = no_error

    Nr_Obs = L2B%Rayleigh_MDS%N_Obs_Rayleigh_Actual
    obs_type_loop1: DO o=1,Nr_Obs
       rangegateloop1: DO r=1, N_range_gates
          !  #[
          ! point to this ObsRangeGate for shorter notation
          WD_RaylObsRangeGate => WorkingData%RayleighObs(o)%RangeGate(r)
       
          ! copy all obtained geolocation results to the Geoloc_ADS datastruct
          L2B_Geoloc_Bin=>L2B%Geoloc_ADS%Rayleigh_Geolocation(o)%Height_Bin(r)

          L2B_Geoloc_Bin%Latitude_Start   = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lat_start,scale=1.e6_r8_)
          L2B_Geoloc_Bin%Latitude_Stop    = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lat_stop, scale=1.e6_r8_)
          L2B_Geoloc_Bin%Latitude_COG     = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lat_cog,  scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_Start  = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lon_start,scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_Stop   = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lon_stop, scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_COG    = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%lon_cog,  scale=1.e6_r8_)
          L2B_Geoloc_Bin%Altitude_Bottom  = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%alt_start )
          L2B_Geoloc_Bin%Altitude_Top     = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%alt_end   )
          L2B_Geoloc_Bin%Altitude_COG     = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_RaylObsRangeGate%alt_cog   )
          !  #]
       end DO rangegateloop1
    end DO obs_type_loop1

    Nr_Obs = L2B%Mie_MDS%N_Obs_Mie_Actual
    obs_type_loop2: DO o=1,Nr_Obs
       rangegateloop2: DO r=1, N_range_gates
          !  #[
          ! point to this ObsRangeGate for shorter notation
          WD_MieObsRangeGate => WorkingData%MieObs(o)%RangeGate(r)
       
          ! copy all obtained geolocation results to the Geoloc_ADS datastruct
          L2B_Geoloc_Bin=>L2B%Geoloc_ADS%Mie_Geolocation(o)%Height_Bin(r)

          L2B_Geoloc_Bin%Latitude_Start   = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lat_start,scale=1.e6_r8_)
          L2B_Geoloc_Bin%Latitude_Stop    = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lat_stop, scale=1.e6_r8_)
          L2B_Geoloc_Bin%Latitude_COG     = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lat_cog,  scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_Start  = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lon_start,scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_Stop   = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lon_stop, scale=1.e6_r8_)
          L2B_Geoloc_Bin%Longitude_COG    = & ! [10^-6 degrees]
               FAdoxy_to_IntAl( WD_MieObsRangeGate%lon_cog,  scale=1.e6_r8_)
          L2B_Geoloc_Bin%Altitude_Bottom  = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_MieObsRangeGate%alt_start )
          L2B_Geoloc_Bin%Altitude_Top     = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_MieObsRangeGate%alt_end   )
          L2B_Geoloc_Bin%Altitude_COG     = & ! [m] above geoid
               FAdoxy_to_IntAl( WD_MieObsRangeGate%alt_cog   )
          !  #]
       end DO rangegateloop2
    end DO obs_type_loop2
  
  end subroutine Format_and_Copy_WD_to_L2B_Outp
    !  #]
  !---------------------------------------------------------
end module L2BP_module

  
