#!/usr/bin/env python
#  #[ documentation
"""
unit tests for Validate.py and EE_DataTypes.xsd

Written by: Jos de Kloe, KNMI

Modifications:
J. de Kloe  26-Apr-2007 initial version
J. de Kloe  27-Apr-2007 added L2B/L2C_SPH, FH, MPH tests
J. de Kloe  29-Apr-2007 added L2B_HDR test
J. de Kloe  14-May-2007 added L2B_AUX_PAR test
J. de Kloe  16-May-2007 added L2B_HDR_SplittedSchema.xml test
J. de Kloe  23-Jan-2008 removed test for splitted schema
"""
#  #]
#  #[ modules used
from Validate import *  # imports the Validate* functions
from minixsv import *   # imports XsvalError
import unittest
#  #]

#  #[ separate tests for each Earth Explorer datatype
class CheckDateTime(unittest.TestCase):
    #  #[ 2 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
# empty DateTime's are now allowed, and are used as missing indicator
#    def testDateTimeEmpty(self):
#        XmlTxt = "<DateTimeTest "+self.XmlNs+"></DateTimeTest>"
#        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testDateTimeInvalid(self):
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "UTC=2007-Apr-26T19:41:37</DateTimeTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "UTC=2007-04-26T19:41:37.4</DateTimeTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "UTC=2007=Apr-26T19:41:37</DateTimeTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "2007-04-26T19:41:37</DateTimeTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "UTC=2007-04-26T19:41:37</DateTimeTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testDateTimeOK(self):
        XmlTxt = "<DateTimeTest "+self.XmlNs+">"+\
                 "UTC=2007-04-26T19:41:37.000000</DateTimeTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckString(unittest.TestCase):
    #  #[ 2 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testStringEmpty(self):
        XmlTxt = "<StringTest "+self.XmlNs+"></StringTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testStringOk(self):
        XmlTxt = "<StringTest "+self.XmlNs+">"+\
                 "This is a text string</StringTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<StringTest "+self.XmlNs+""">This is a text string
        containing multiple lines</StringTest>"""
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckStringOrEmpty(unittest.TestCase):
    #  #[ 1 test
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testStringOrEmpty(self):
        XmlTxt = "<StringOrEmptyTest "+self.XmlNs+"></StringOrEmptyTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<StringOrEmptyTest "+self.XmlNs+">"+\
                 "This is a text string</StringOrEmptyTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<StringOrEmptyTest "+self.XmlNs+""">This is a text string
        containing multiple lines</StringOrEmptyTest>"""
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckEnum(unittest.TestCase):
    #  #[ 2 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testEnumEmpty(self):
        XmlTxt = "<EnumTest "+self.XmlNs+"></EnumTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testEnumOk(self):
        XmlTxt = "<EnumTest "+self.XmlNs+">N</EnumTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckBoolean(unittest.TestCase):
    #  #[ 4 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testBooleanEmpty(self):
        XmlTxt = "<BooleanTest "+self.XmlNs+"></BooleanTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testBooleanInvalid(self):
        XmlTxt = "<BooleanTest "+self.XmlNs+">"+\
                 "its simply not True</BooleanTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testBooleanWrongCase(self):
        XmlTxt = "<BooleanTest "+self.XmlNs+">false</BooleanTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<BooleanTest "+self.XmlNs+">falsE</BooleanTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
        XmlTxt = "<BooleanTest "+self.XmlNs+">FALSE</BooleanTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testBooleanOk(self):
        XmlTxt = "<BooleanTest "+self.XmlNs+">False</BooleanTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<BooleanTest "+self.XmlNs+">True</BooleanTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAuc(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAucTooSmall(self):
        XmlTxt = "<IntAucTest "+self.XmlNs+">-1</IntAucTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAucTooLarge(self):
        XmlTxt = "<IntAucTest "+self.XmlNs+">256</IntAucTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAucNaN(self):
        XmlTxt = "<IntAucTest "+self.XmlNs+">abc</IntAucTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAucEmpty(self):
        XmlTxt = "<IntAucTest "+self.XmlNs+"></IntAucTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAucOk(self):
        XmlTxt = "<IntAucTest "+self.XmlNs+">0</IntAucTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAucTest "+self.XmlNs+">255</IntAucTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAc(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAcTooSmall(self):
        XmlTxt = "<IntAcTest "+self.XmlNs+">-129</IntAcTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAcTooLarge(self):
        XmlTxt = "<IntAcTest "+self.XmlNs+">128</IntAcTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAcNaN(self):
        XmlTxt = "<IntAcTest "+self.XmlNs+">abc</IntAcTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAcEmpty(self):
        XmlTxt = "<IntAcTest "+self.XmlNs+"></IntAcTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAcOk(self):
        XmlTxt = "<IntAcTest "+self.XmlNs+">-128</IntAcTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAcTest "+self.XmlNs+">+127</IntAcTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAus(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAusTooSmall(self):
        XmlTxt = "<IntAusTest "+self.XmlNs+">-1</IntAusTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAusTooLarge(self):
        XmlTxt = "<IntAusTest "+self.XmlNs+">65536</IntAusTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAusNaN(self):
        XmlTxt = "<IntAusTest "+self.XmlNs+">abc</IntAusTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAusEmpty(self):
        XmlTxt = "<IntAusTest "+self.XmlNs+"></IntAusTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAusOk(self):
        XmlTxt = "<IntAusTest "+self.XmlNs+">+0</IntAusTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAusTest "+self.XmlNs+">+65535</IntAusTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAs(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAsTooSmall(self):
        XmlTxt = "<IntAsTest "+self.XmlNs+">-32769</IntAsTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAsTooLarge(self):
        XmlTxt = "<IntAsTest "+self.XmlNs+">32768</IntAsTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAsNaN(self):
        XmlTxt = "<IntAsTest "+self.XmlNs+">abc</IntAsTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAsEmpty(self):
        XmlTxt = "<IntAsTest "+self.XmlNs+"></IntAsTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAsOk(self):
        XmlTxt = "<IntAsTest "+self.XmlNs+">-32768</IntAsTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAsTest "+self.XmlNs+">+32767</IntAsTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAul(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAulTooSmall(self):
        XmlTxt = "<IntAulTest "+self.XmlNs+">-1</IntAulTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAulTooLarge(self):
        XmlTxt = "<IntAulTest "+self.XmlNs+">4294967296</IntAulTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAulNaN(self):
        XmlTxt = "<IntAulTest "+self.XmlNs+">abc</IntAulTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAulEmpty(self):
        XmlTxt = "<IntAulTest "+self.XmlNs+"></IntAulTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAulOk(self):
        XmlTxt = "<IntAulTest "+self.XmlNs+">0</IntAulTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAulTest "+self.XmlNs+">+4294967295</IntAulTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAl(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAlTooSmall(self):
        XmlTxt = "<IntAlTest "+self.XmlNs+">-2147483649</IntAlTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAlTooLarge(self):
        XmlTxt = "<IntAlTest "+self.XmlNs+">+2147483648</IntAlTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAlNaN(self):
        XmlTxt = "<IntAlTest "+self.XmlNs+">a1b2c3</IntAlTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAlEmpty(self):
        XmlTxt = "<IntAlTest "+self.XmlNs+"></IntAlTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAlOk(self):
        XmlTxt = "<IntAlTest "+self.XmlNs+">-2147483648</IntAlTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAlTest "+self.XmlNs+">+2147483647</IntAlTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckIntAd(unittest.TestCase):
    #  #[ 5 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testIntAdTooSmall(self):
        XmlTxt = "<IntAdTest "+self.XmlNs+">-9223372036854775809</IntAdTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAdTooLarge(self):
        XmlTxt = "<IntAdTest "+self.XmlNs+">+9223372036854775808</IntAdTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAdNaN(self):
        XmlTxt = "<IntAdTest "+self.XmlNs+">a1b2c3</IntAdTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAdEmpty(self):
        XmlTxt = "<IntAdTest "+self.XmlNs+"></IntAdTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAdOk(self):
        XmlTxt = "<IntAdTest "+self.XmlNs+">-9223372036854775808</IntAdTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<IntAdTest "+self.XmlNs+">+9223372036854775807</IntAdTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckFAdoxy(unittest.TestCase):
    #  #[ 3 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    #
    # remark:
    # for fortran for 8 byte reals the limits are:
    #  r8: huge        1.7976931348623157E+308
    #  r8: tiny        2.2250738585072014E-308
    # (symmetric around zero)

    # the range check for reals seems not yet implemented in pyxsval
    #def testFAdoxyTooSmall(self):
    #    # this is probably just rounded to zero, and I don't know
    #    # any way to test for this, so this raises no error
    #    #XmlTxt = "<FAdoxyTest>1.0e-309</FAdoxyTest>"
    #    #XmlTxt = "<FAdoxyTest>-1.0e+309</FAdoxyTest>"
    #    XmlTxt = "<FAdoxyTest>-1.0e+409</FAdoxyTest>"
    #    self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)

    # the range check for reals seems not yet implemented in pyxsval
    #def testFAdoxyTooLarge(self):
    #    #XmlTxt = "<FAdoxyTest>2.0e+309</FAdoxyTest>"
    #    XmlTxt = "<FAdoxyTest>2.0e+409</FAdoxyTest>"
    #    self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    
    # this check is not possible, because the fractionDigits property
    # seems not allowd for double or float types??? I dont have a clue why ....
    #def testFAdoxyTooManyDigits(self):
    #   XmlTxt = "<FAdoxyTest>1.123456789012345678901234567890e-5</FAdoxyTest>"
    #   self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)

    def testFAdoxyNaN(self):
        XmlTxt = "<FAdoxyTest "+self.XmlNs+">1.234ea34</FAdoxyTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testFAdoxyEmpty(self):
        XmlTxt = "<FAdoxyTest "+self.XmlNs+"></FAdoxyTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testFAdoxyOk(self):
        XmlTxt = "<FAdoxyTest "+self.XmlNs+">2.0e-308</FAdoxyTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<FAdoxyTest "+self.XmlNs+">1.7e+308</FAdoxyTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
class CheckSpare(unittest.TestCase):
    #  #[ 3 tests
    SchemaFile = "Test_EE_DataTypes.xsd"    
    XmlNs = "xmlns=\"http://www.esa.int/schemas/ae/TestEEdataTypes\""
    def testSpareEmpty(self):
        XmlTxt = "<SpareTest "+self.XmlNs+"></SpareTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testSpareHasText(self):
        XmlTxt = "<SpareTest "+self.XmlNs+">some text</SpareTest>"
        self.assertRaises(XsvalError,ValidateXmlString,XmlTxt,self.SchemaFile)
    def testIntAucOk(self):
        XmlTxt = "<SpareTest "+self.XmlNs+">              </SpareTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<SpareTest "+self.XmlNs+">"+\
                 "            \n       </SpareTest>"
        ValidateXmlString(XmlTxt,self.SchemaFile)
        XmlTxt = "<SpareTest "+self.XmlNs+""">
                    </SpareTest>"""
        ValidateXmlString(XmlTxt,self.SchemaFile)
        # this should NOT raise any errors
    #  #]
#  #]

# tests for xsd files
class CheckXsdFiles(unittest.TestCase):
    #  #[ 10 tests
    # note that the name MUST start with the 4 characters "test"
    # otherwise the unittest system will not recognise the test
    # note also that the docstring provided after each def is used
    # as name for the test when the result is printed to tstdout
    def testValidateEE_DataTypes(self):
        """testing ValidateFile('EE_DataTypes.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "EE_DataTypes.xsd"
        ValidateFile(TestXsdFile)
    def testValidateTest_EE_DataTypes(self):
        """testing ValidateFile('Test_EE_DataTypes.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "Test_EE_DataTypes.xsd"
        ValidateFile(TestXsdFile)
    def testValidateFixedHeader(self):
        """testing ValidateFile('FixedHeader.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "FixedHeader.xsd"
        ValidateFile(TestXsdFile)
    def testValidateMainProductHeader(self):
        """testing ValidateFile('MainProductHeader.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "MainProductHeader.xsd"
        ValidateFile(TestXsdFile)
    def testValidateDataSetDescriptor(self):
        """testing ValidateFile('DataSetDescriptor.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "DataSetDescriptor.xsd"
        ValidateFile(TestXsdFile)
    def testValidateL2B_SpecificProductHeader(self):
        """testing ValidateFile('L2B_SpecificProductHeader.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "L2B_SpecificProductHeader.xsd"
        ValidateFile(TestXsdFile)
    def testValidateL2C_SpecificProductHeader(self):
        """testing ValidateFile('L2C_SpecificProductHeader.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "L2C_SpecificProductHeader.xsd"
        ValidateFile(TestXsdFile)
    def testValidateL2B_HDR(self):
        """testing ValidateFile('L2B_HDR.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "L2B_HDR.xsd"
        ValidateFile(TestXsdFile)
    def testValidateL2C_HDR(self):
        """testing ValidateFile('L2C_HDR.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "L2C_HDR.xsd"
        ValidateFile(TestXsdFile)
    def testValidateL2B_AUX_PAR(self):
        """testing ValidateFile('L2B_AUX_PAR.xsd') """
        # no errors should occur when validating this file
        TestXsdFile = "L2B_AUX_PAR.xsd"
        ValidateFile(TestXsdFile)
    #  #]

# tests for xml files
class CheckXmlFiles(unittest.TestCase):
    #  #[ 19 tests
    def testValidateTest_EE_DataTypes(self):
        """testing ValidateFile('Test_EE_DataTypes.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "Test_EE_DataTypes.xml"
        ValidateFile(TestXmlFile)
    def testValidateFixedHeader(self):
        """testing ValidateFile('FixedHeader.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "FixedHeader.xml"
        ValidateFile(TestXmlFile)
    def testValidateFixedHeader_invalid(self):
        """testing ValidateFile('FixedHeader_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "FixedHeader_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateFixedHeader_WrongSchemaRef(self):
        """testing ValidateFile('FixedHeader_WrongSchemaReference.xml') """
        # this file should raise an IOError
        TestXmlFile = "FixedHeader_WrongSchemaReference.xml"
        self.assertRaises(IOError,ValidateFile,TestXmlFile)
    def testValidateMainProductHeader(self):
        """testing ValidateFile('MainProductHeader.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "MainProductHeader.xml"
        ValidateFile(TestXmlFile)
    def testValidateMainProductHeader_invalid(self):
        """testing ValidateFile('MainProductHeader_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "MainProductHeader_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateDataSetDescriptor(self):
        """testing ValidateFile('DataSetDescriptor.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "DataSetDescriptor.xml"
        ValidateFile(TestXmlFile)
    def testValidateDataSetDescriptor_invalid(self):
        """testing ValidateFile('DataSetDescriptor_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "DataSetDescriptor_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateL2B_SpecificProductHeader(self):
        """testing ValidateFile('L2B_SpecificProductHeader.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "L2B_SpecificProductHeader.xml"
        ValidateFile(TestXmlFile)
    def testValidateL2B_SpecificProductHeader_invalid(self):
        """testing ValidateFile('L2B_SpecificProductHeader_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "L2B_SpecificProductHeader_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateL2C_SpecificProductHeader(self):
        """testing ValidateFile('L2C_SpecificProductHeader.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "L2C_SpecificProductHeader.xml"
        ValidateFile(TestXmlFile)
    def testValidateL2C_SpecificProductHeader_invalid(self):
        """testing ValidateFile('L2C_SpecificProductHeader_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "L2C_SpecificProductHeader_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateL2B_HDR(self):
        """testing ValidateFile('L2B_HDR.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "L2B_HDR.xml"
        ValidateFile(TestXmlFile)
    def testValidateL2B_HDR_invalid(self):
        """testing ValidateFile('L2B_HDR_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "L2B_HDR_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
#    def testValidateL2B_HDR_Splitted(self):
#        """testing ValidateFile('L2B_HDR_SplittedSchema.xml') """
#        # no errors should occur when validating this file
#        TestXmlFile = "L2B_HDR_SplittedSchema.xml"
#        ValidateFile(TestXmlFile)
    def testValidateL2C_HDR(self):
        """testing ValidateFile('L2C_HDR.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "L2C_HDR.xml"
        ValidateFile(TestXmlFile)
    def testValidateL2C_HDR_invalid(self):
        """testing ValidateFile('L2C_HDR_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "L2C_HDR_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    def testValidateL2B_AUX_PAR(self):
        """testing ValidateFile('L2B_AUX_PAR.xml') """
        # no errors should occur when validating this file
        TestXmlFile = "L2B_HDR.xml"
        ValidateFile(TestXmlFile)
    def testValidateL2B_AUX_PAR_invalid(self):
        """testing ValidateFile('L2B_AUX_PAR_invalid.xml') """
        # this file should raise several errors
        TestXmlFile = "L2B_AUX_PAR_invalid.xml"
        self.assertRaises(XsvalError,ValidateFile,TestXmlFile)
    #  #]

if __name__ == "__main__":
    # this just runs all tests
    print "Running unit tests:"
    # 82 defined at the moment, running them
    # takes 321 seconds running time on my laptop
    # and   163 seconds running time on my KNMI workstation
    unittest.main()

#
# This is how to run some of the tests separately:
#
#import TestValidate, unittest
#suite = unittest.TestLoader().loadTestsFromTestCase(TestValidate.CheckBoolean)
#unittest.TextTestRunner(verbosity=2).run(suite)


#.....................................................XsdFile:           DataSetDe
#12345678901234567890123456789012345678901234567890123
#0        1         2         3         4         5
