#!/usr/bin/env python
#  #[ documentation
#  a little python report generation script
#
# Written by: J. de Kloe, KNMI
#
# Modifications:
#   26-Apr-2007 J. de Kloe   Initial version
#   16-May-2007 J. de Kloe   added possibility to define schemas for other
#                            tags than the root tag of the document
#   24-Jul-2007 J. de Kloe   added a workaround to allow validating xml
#                            using schema references added by ee_cfi v3.6
#  #]
#  #[ imported modules
import os, sys, string
try:
    from xml.parsers.expat import ExpatError
except:
    print "Error: this script needs the module: xml.parsers.expeat"
    print "Please install this module and try again"

try:
    from minixsv import pyxsval, \
                        XSI_NAMESPACE, XSD_NAMESPACE, \
                        XMLNS_NAMESPACE, XML_NAMESPACE, EMPTY_NAMESPACE
    
except:
    print "Error: this script needs the module: minixsv"
    print "Please install this module and try again"

try:
    from minixsv.xsvalBase import XsValBase
except:
    print "Error: this script needs the module: minixsv.xsvalBase"
    print "Please install this module and try again"
#  #]
def GetXsdFiles(XsdDir):
    #  #[

    XsdFileList = []
    FileList = os.listdir(XsdDir)
    for File in FileList:
        Ext = string.lower(os.path.splitext(File)[1])
        if (Ext == ".xsd"):
            TargetNameSpace = "[undefined]"
            # get an instance of the validator class
            validator = pyxsval.XsValidator()
            inputTreeWrapper = validator.parse(File)
            root = inputTreeWrapper.getRootNode()
            if (root.getLocalName() != "schema"):
                print "ERROR in GetXsdFiles:"
                print "an xsd file should always have the schema tag as"
                print "root tag, but this file seems to have:"
                print "rootname:  ["+root.getLocalName()+"]"
                print "File:      ["+File+"]"
            for attrName, attrValue in  root.getAttributeDict().items():
                if (attrName == (None,"targetNamespace")):
                    #print "==>attrName[0]: ["+str(attrName[0])+"]"
                    #print "==>attrName[1]: ["+str(attrName[1])+"]"
                    #print "==>attrValue:   ["+attrValue+"]"
                    TargetNameSpace = attrValue
            XsdFileList.append((File,TargetNameSpace))

    return XsdFileList

    #  #]
def NodeToXml(Node):
    #  #[ an adapted version of printNode from xmlifbase.py
    #     because that one inserts to much newlines and indentations
      patternXmlTagShort = "<%(tagName)s%(attributeString)s/>"
      patternXmlTagLong  = "<%(tagName)s%(attributeString)s>"+\
                           "%(subTreeString)s"+\
                           "</%(tagName)s>"
      patternXmlTagText  = "<%(tagName)s%(attributeString)s>"+\
                           "%(elementValueString)s"+\
                           "</%(tagName)s>"

      subTreeStringList = []
      for childNode in Node.getChildren():
          subTreeStringList.append(NodeToXml(childNode))
      subTreeString = string.join (subTreeStringList, "\n")

      # from: /usr/people/kloedej/bin/pythonlib/minixsv/__init__.py
      #EMPTY_NAMESPACE = None
      #XML_NAMESPACE   = "http://www.w3.org/XML/1998/namespace"
      #XMLNS_NAMESPACE = "http://www.w3.org/2000/xmlns/"
      #XSD_NAMESPACE   = "http://www.w3.org/2001/XMLSchema"
      #XSI_NAMESPACE   = "http://www.w3.org/2001/XMLSchema-instance"

      attributeStringList = []
      for attrName, attrValue in Node.getAttributeDict().items():
          #print "==>attrName[0]: ["+str(attrName[0])+"]"
          #print "==>attrName[1]: ["+str(attrName[1])+"]"
          #print "==>attrValue:   ["+attrValue+"]"
          if (attrName[0] == XSI_NAMESPACE):
              #print "Appending attribute: "+\
              #      'xsi:%s="%s"' % (attrName[1], attrValue) 
              attributeStringList.append('xsi:%s="%s"' %
                                    (attrName[1], attrValue) )
          elif (attrName[0] == XMLNS_NAMESPACE):
              #print "Appending attribute: "+\
              #      'xmlns:%s="%s"' % (attrName[1], attrValue) 
              attributeStringList.append('xmlns:%s="%s"' %
                                    (attrName[1], attrValue) )
          elif (attrName[0] == EMPTY_NAMESPACE):
              #print "Appending attribute: "+\
              #      '%s="%s"' % (attrName[1], attrValue) 
              attributeStringList.append('%s="%s"' % (attrName[1], attrValue) )
          else:
              validator = pyxsval.XsValidator()
              validator.errorHandler.raiseError(\
                    "ERROR: unhandled attribute type")

      attributeString = ' '.join(attributeStringList)
      if (attributeString != ''):
          attributeString = ' '+attributeString

      tagName = Node.getTagName()

      NumChildren = len(Node.getChildren())

      elementValueString = ""
      if NumChildren == 0:
          elementValueString = Node.getElementValue()

      if subTreeString == "" and elementValueString == "":
          # this is an empty tag, possibly with some attributes in it
          printPattern = patternXmlTagShort
      elif subTreeString == "" and elementValueString != "":
          # this is a text tag
          printPattern = patternXmlTagText
      else:
          # this is a tag containing child tags
          printPattern = patternXmlTagLong

      return printPattern %vars()
    #  #]
def ValidateXmlNode(Node,verbosity=0):
    #  #[ recursive search for schemas 
    # the current node (might be the root node) seems not to have
    # a schema definition, so check whether its children have such
    # a definition. If they all have one, consider the schema validation
    # passed anyway.
    Validates = True
    if (len(Node.getChildren()) == 0 ):
        # no children to be validates, so the validation fails
        Validates = False
    
    for child in Node.getChildren():
        if (verbosity > 0):
            print "childname:  ["+child.getLocalName()+"]"
        xsdFile = ""
        # print "value: ["+child.getElementValue()+"]"
        for attrName, attrValue in child.getAttributeDict().items():
            #print "==>attrName[0]: ["+str(attrName[0])+"]"
            #print "==>attrName[1]: ["+str(attrName[1])+"]"
            #print "==>attrValue:   ["+attrValue+"]"
            if attrName == (XSI_NAMESPACE, "schemaLocation"):
                xsdFile = attrValue
            elif attrName == (XSI_NAMESPACE, "noNamespaceSchemaLocation"):
                xsdFile = attrValue
        if (xsdFile == ""):
            Result = ValidateXmlNode(child,verbosity)
            if (Result == False):
                print "ERROR in Validate.py:"
                print "this child tag seems to have no schema defined,"
                print "and also not ALL (grand)children of it have a"
                print "schema reference ..."
                print "Unable to validate this node against a schema..."
                print "No schema file specified for childtag: "+\
                      child.getLocalName()
            Validates=Result
        else:
            if (verbosity > 0):
                print "schema file: "+xsdFile
            #XmlString      = Node.toxml()
            #XmlString      = child.printNode(indent="",deep=1)
            XmlString      = NodeToXml(child)
            validateSchema = 0
            # note: switch off verbosity here, unless you are debugging
            # otherwise you get a lot of output ...
            ValidateXmlString(XmlString,xsdFile,verbosity=0)
            if (verbosity > 0):
                print "all seems okay for node: "+\
                      child.getLocalName()
            
    return Validates
    #  #]
def ValidateFile(InputFile,XsdDir="./",verbosity=0):    
    #  #[ check all xsd files in the current directory
    XsdFileList = GetXsdFiles(XsdDir)
    if (verbosity > 0):
        for XsdFile in XsdFileList:
            print "%s %30s %s %s" % \
                  ("XsdFile: ",XsdFile[0]," defines ",XsdFile[1])
    #  #]
    #  #[ checking the type of the input file
    if (verbosity > 0):
        print "Verbosity level = ",verbosity
    file,extension = os.path.splitext(InputFile)
    XmlInput = False
    XsdInput = False
    if ( (extension.upper() == ".XML") or
         (extension.upper() == ".HDR") or
         (extension.upper() == ".EEF")   ):
        if (verbosity > 0):
            print "input file is xml"
        XmlInput = True
    if (extension.upper() == ".XSD"):
        if (verbosity > 0):
            print "input file is xsd"
        XsdInput = True

    if ( (not XmlInput) and (not XsdInput) ):
        validator = pyxsval.XsValidator()
        validator.errorHandler.raiseError(\
                    "Sorry, filetype not recognised from extension: "+\
                    extension+" aborting valitation...")
    #  #]
    #  #[ the actual work
    # get an instance of the validator class
    validator = pyxsval.XsValidator(verbose=verbosity)
    
    if (verbosity > 0):
        print "Validating file: "+InputFile
    inputTreeWrapper = validator.parse(InputFile)

    if (XmlInput):
        #  #[
        root = inputTreeWrapper.getRootNode()
        xsdFile = ""
        if (verbosity > 0):
            print "rootname:  ["+root.getLocalName()+"]"
        for attrName, attrValue in  root.getAttributeDict().items():
            if (verbosity > 0):
                print "==>attrName[0]: ["+str(attrName[0])+"]"
                print "==>attrName[1]: ["+str(attrName[1])+"]"
                print "==>attrValue:   ["+attrValue+"]"
            if attrName == (XSI_NAMESPACE, "schemaLocation"):
                # I have no files with this setting, so this type
                # has not yet been tested
                if (verbosity > 0):
                    print "test: schemaLocation=",attrValue
                xsdFile = attrValue
            if attrName == (XSI_NAMESPACE, "noNamespaceSchemaLocation"):
                if (verbosity > 0):
                    print "test: noNamespaceSchemaLocation",attrValue
                xsdFile = attrValue
            if attrName == (XMLNS_NAMESPACE,"xmlns"):
                if (verbosity > 0):
                    print "test: xmlns",attrValue
                # search the list of xsd files to find one with an appropriate
                # namespace to be used for this xml file
                for (TmpXsdFile,TargetNameSpace) in XsdFileList:
                    if (TargetNameSpace == attrValue):
                        if (verbosity > 0):
                            print "Xsd file found !!!! : ",TmpXsdFile
                        xsdFile = TmpXsdFile
                
        if (xsdFile == ""):
            # no schema defined in root tag
            #if (verbosity > 0):

            #print "No schema defined in root tag of this xml document"
            #print "scanning higher level tags..."

            # disable this option for now
            validator.errorHandler.raiseError(\
                    "No xsd file found for namespace: "+attrValue+\
                    " or no xsd file specified")

            #Result = ValidateXmlNode(inputTreeWrapper.getRootNode(),verbosity)
            #if (Result == False):
            #    validator.errorHandler.raiseError(\
            #        "WARNING: not all parts of this xml file could be"+\
            #        "validated. Some parts have no schema reference!")
        else:
            # a schema is present in the root tag, use it for validation
            if (verbosity > 0):
                print "schema file: "+xsdFile
            XmlFile = InputFile
            validateSchema = 0
            # warning: the ee_cfi software seems to have a bug!
            # it references schemas with the schemaLocation attribute
            # but only gives the xsd filename, but not the URL!!!
            # If you don't want to give the URL you should use the
            # noNamespaceSchemaLocation attribute !!!
            # See http://www.edankert.com/grammars/xsd.referencing.html
            # for details.
            # This bug lets the Validate program crash.
            # 
            # Adding xmlns='http://www.example.esa.int'
            # xsi:schemaLocation='http://www.example.esa.int L2B_HDR.xsd'
            # to the root-tag of the xml is possible, but then all xsd
            # files need to have the
            # targetNamespace='http://www.example.esa.int'
            # attribute in the root xs:schema tag
            # I have tried to do this as test, but could not get all
            # dependant xsd files properly included.
            #
            # As a workaround, [ass the xsdFile explicitely to the validator 
            # for now.....
            # Note that this is a workaround, and probably
            # violates the xml/schema standards !
            #xsdFile = None            
            validator.validateXmlInput(XmlFile, inputTreeWrapper,
                                       xsdFile, validateSchema)

        #  #]
            
    if (XsdInput):
        #  #[
        xsdFile = InputFile
        validator.validateXmlSchema(xsdFile, inputTreeWrapper)
        #  #]

    if (verbosity > 0):
        print "all seems okay for file: "+InputFile
    #  #]
def ValidateXmlString(InputXmlString,SchemaFile,verbosity=0):
    #  #[ the actual work
    # get an instance of the validator class
    if (verbosity > 0):
        print "test: InputXmlString = "+InputXmlString
        print "test: SchemaFile = "+SchemaFile
    validator = pyxsval.XsValidator(verbose=verbosity)

    if (verbosity > 0):
        print "Validating XML String: "+InputXmlString
    inputTreeWrapper = validator.parseString(InputXmlString)
    xsdText = None
    validator.validateXmlInputString(inputTreeWrapper,file(SchemaFile).read())
        
    if (verbosity > 0):
        print "all seems okay"
    #  #]

#  #[ the main program
if __name__ == "__main__":
    # this just runs ValidateFile defined above on the file specified
    # on the commandline
    if ( len(sys.argv) != 2):
        print "ERROR: no input file specified, nothing to do..."
        print " "
        print "Usage:"
        print "ValidateFile <file.xml>"
        os._exit(1)
    InputFile = sys.argv[1]
    ValidateFile(InputFile,verbosity=1)

    # a short debug standalone test
    #SchemaFile = "Test_EE_DataTypes.xsd"
    #XmlTxt = "<SpareTest>some text</SpareTest>"
    #ValidateXmlString(XmlTxt,SchemaFile)
    
    print "done"

#  #]
