MODULE Compiler_Features
  !  #[ Description

  !---------------------------------------------------
  ! a module to manage all the compiler-dependent problems
  ! This one handles pgf90
  ! When executing the Set_MakeOptions, the file Compiler_Features creates
  ! a link to this module.
  !---------------------------------------------------
  !    Written by:  Paul Poli.
  !    created:     15-09-2005
  !
  ! Modifications:
  !    03-Apr-2006 J de Kloe  added flush_aeolus(fileunit)
  !    12-May-2006 D. Tan     Null use of ParKind1 module
  !    16-Jan-2008 J. de Kloe phase out integer kind i_ 
  !
  !---------------------------------------------------

  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Numerics, only: i4_
  !  #]
  !  #[ Variables and parameters
  IMPLICIT NONE

  character, parameter :: tabchar = '\t'
  character, parameter :: retchar = '\r'
  character, parameter :: newline = '\n'
  character, parameter :: bs = '\\'

  ! this little piece of test code was used inside SimpleXML.F90
  ! to test that the \t, \r and \r are the expected escape codes,
  ! and do return achar(9), achar(13) and achar(10)
  !print *,'xxx\txxx',ichar('\t'),'jjj',achar(9),'kkk'
  !print *,'xxx\rxxx',ichar('\r'),'jjj',achar(13),'kkk'
  !print *,'xxx\nxxx',ichar('\n'),'jjj',achar(10),'kkk'

  !  #]
CONTAINS 
  !----------------------------------------
  function iargc_aeolus() result(nargs)
    !  #[
    ! Points to the routine which returns the number of arguments
    integer :: nargs
    !  #[ Interfaces
    ! specify explicit interface for the fortran extension iargc()
    interface
      function iargc() result(result)  !  system fortran function
        integer :: result
      end function iargc
    end interface
    !  #]

    nargs = iargc()
    end function iargc_aeolus
    !  #]
  !--------------------------------
  subroutine getarg_aeolus(i_in,c_out)
    !  #[
    ! Returns the string of arguments in input
    integer,          intent(in)  :: i_in
    character(len=*), intent(out) :: c_out
    !  #[ Interfaces
    ! specify explicit interface for the fortran extension iargc()
    interface
      subroutine getarg(i, c) !  system fortran routine
        integer,          intent(in)  :: i
        character(len=*), intent(out) :: c
      end subroutine getarg
    end interface
    !  #]

    call getarg(i_in,c_out)
    end subroutine getarg_aeolus
    !  #]
  !--------------------------------
  subroutine flush_aeolus(fileunit)
    !  #[
    integer :: fileunit

    ! remarks:
    ! the thin-layer requires the processor to flush the text written
    ! to the logs after EVERY message, to be sure it is not delayed.
    ! However, Fortran90 has no flush() function specified, and the next
    ! call is an extention to the standard. Therefore it may be necessary
    ! for some compilers to deactivate it, and do nothing, and thus violate
    ! the thin-layer requirements.
    ! Note also that is is not usefull to call the flush function in the
    ! c-runtime library (does it exist there?) since fortran may/will use
    ! some buffering, independent of the underlying c-code.

    ! note also that the next call DOES work for the g95 compiler
    ! which uses this generic version of the COmpiler_Features module

    call flush(fileunit)

  end subroutine flush_aeolus
    !  #]
  !--------------------------------
END MODULE Compiler_Features
