MODULE Compiler_Features
  !  #[ Description

  !---------------------------------------------------
  ! a module to manage all the compiler-dependent problems
  ! This one handles gfortran
  ! When executing the Set_MakeOptions, the file Compiler_Features creates
  ! a link to this module.
  !---------------------------------------------------
  !    Written by:  J. de Kloe.
  !    created:     05-01-2006
  !
  ! Modifications:
  !    03-Apr-2006 J de Kloe  added flush_aeolus(fileunit)
  !    12-May-2006 D. Tan     Null use of ParKind1 module
  !    16-Jan-2008 J. de Kloe phase out integer kind i_ 
  !
  !---------------------------------------------------

  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE Numerics, only: i4_
  !  #]
  !  #[ Variables and parameters
  IMPLICIT NONE

  character, parameter :: tabchar = achar(9) !'\t'                              
  character, parameter :: retchar = achar(13) ! '\r'                            
  character, parameter :: newline = achar(10) ! '\n'                            
  character, parameter :: bs      = achar(92) ! '\\'                            

  !  #]
CONTAINS 
  !----------------------------------------
  function iargc_aeolus() result(nargs)
    !  #[
    ! Points to the routine which returns the number of arguments
    integer :: nargs
    ! WARNING: gfortran refuses to compile when you use
    !          an explicit interface for iargc() !!!
    nargs = iargc()
    end function iargc_aeolus
    !  #]
  !--------------------------------
  subroutine getarg_aeolus(i_in,c_out)
    !  #[
    ! Returns the string of arguments in input
    integer (i4_),    intent(in)  :: i_in
    character(len=*), intent(out) :: c_out
    ! WARNING: gfortran refuses to compile when you use
    !          an explicit interface for getarg() !!!
    call getarg(i_in,c_out)
    end subroutine getarg_aeolus
    !  #]
  !--------------------------------
  subroutine flush_aeolus(fileunit)
    !  #[
    integer :: fileunit

    ! remarks:
    ! the thin-layer requires the processor to flush the text written
    ! to the logs after EVERY message, to be sure it is not delayed.
    ! However, Fortran90 has no flush() function specified, and the next
    ! call is an extention to the standard. Therefore it may be necessary
    ! for some compilers to deactivate it, and do nothing, and thus violate
    ! the thin-layer requirements.
    ! Note also that is is not usefull to call the flush function in the
    ! c-runtime library (does it exist there?) since fortran may/will use
    ! some buffering, independent of the underlying c-code.

    call flush(fileunit)

  end subroutine flush_aeolus
    !  #]
  !--------------------------------
END MODULE Compiler_Features
