/* #[ Description

  **************************************************************************
    a collection of wrappers to interface the ee_cfi library to Fortran90

    created by:   J. de Kloe, may 2005

    Modifications:
    21-Jul-2006 J. de Kloe  adapted to ee_cfi 3.5
    22-Jul-2006 J. de Kloe  added error handling routines
  ***************************************************************************/

/* #] */

/* to interface with fortran we need wrappers which add an underscore
   to the function/subroutine names */
/* WARNING: all names below need to be LOWERCASE
   otherwise you can expect linking problems with some compilers !!!! */

/* make the c-version of the ee_cfi explorer_linbavailable */
#include <explorer_lib.h>

/* for the distance function, see p. 148, sec. 7.31 of LibSum.pdf */
void xl_geod_distance_(double *lon1, double *lat1, 
                       double *lon2, double *lat2, 
                       double *h, double *distance, 
                       double *az_1_to_2, double *az_2_to_1,
                       long *ext_status)
{ *ext_status = xl_geod_distance(lon1, lat1, lon2, lat2, h,          /* inputs */ 
                                 distance, az_1_to_2, az_2_to_1); }; /* outputs*/
/* REMARK: exstended status is an encoded integer holding information about
           which erors and/or warnings occurred. 
           If ext_status==0: no problems reported
           If ext_status<0: an error occurred
           If ext_status>0: an warning occurred
*/

/* XL_MAX_COD is defined in explorer_lib.h */

/* convert the extended status flag to usable error codes
   see p.39, sec. 8.7 of GenSum.pdf */
void xl_get_code_(long *func_id,        /* a user defined id number? */
                  long *ext_status,     /* the ext_status flag to be translated */
                  long *n,              /* nr of error codes returned */
                  long vec[XL_MAX_COD], /* a vector of XL_MAX_COD length with error codes*/
                  long *status)         /* the exit status of the get_code function*/
{
  *status = xl_get_code(func_id, ext_status, /* inputs */
                        n, vec);             /* outputs*/
}

/* convert the extended status flag to usable error message texts
   see p.41, sec. 8.8 of GenSum.pdf */
void xl_get_msg_(long *func_id,        /* a user defined id number? */
                 long *ext_status,     /* the ext_status flag to be translated */
                 long *n,              /* nr of error codes returned */
                 char msg[XL_MAX_COD][XL_MAX_STR], /* a string vector of XL_MAX_COD length */
                 long *status)         /* the exit status of the get_code function*/
{
  *status = xl_get_msg(func_id, ext_status, /* inputs */
                       n, msg);             /* outputs*/
}

/* the following errors and warnings might be
   returned by the xl_geod_distance function
   (see LibSum.pdf, sec. 7.31.5, p.150, for details)
XL_CFI_GEOD_DIST_SAT_ERR
XL_CFI_GEOD_DIST_ALTITUDE_ERR
XL_CFI_GEOD_DIST_GEO_CAR_ERR
XL_CFI_GEOD_DIST_DIR_RANGE_ERR
XL_CFI_GEOD_DIST_DIR_POINTING_ERR
XL_CFI_GEOD_DIST_ANTIPODAL_POINTS_WARN
XL_CFI_GEOD_DIST_DIR_POINTING_WARN
*/
void get_xl_geod_dist_errors_(long *e1, long *e2, long *e3, long *e4,
                              long *e5, long *w1, long *w2)
{ 
  *e1 = XL_CFI_GEOD_DIST_SAT_ERR;
  *e2 = XL_CFI_GEOD_DIST_ALTITUDE_ERR;
  *e3 = XL_CFI_GEOD_DIST_GEO_CAR_ERR;
  *e4 = XL_CFI_GEOD_DIST_DIR_RANGE_ERR;
  *e5 = XL_CFI_GEOD_DIST_DIR_POINTING_ERR;
  *w1 = XL_CFI_GEOD_DIST_ANTIPODAL_POINTS_WARN;
  *w2 = XL_CFI_GEOD_DIST_DIR_POINTING_WARN; 
  return;
}

/* some more definitions from explorer_lib.h 
   (look in the header file for their definitions):
XL_GEOD_TO_CART_ID
XL_CART_TO_GEOD_ID
XL_GEOD_DISTANCE_ID
XL_MAX_COD
XL_MAX_STR
*/
void get_some_defs_(long *GEOD_TO_CART_ID, 
                    long *CART_TO_GEOD_ID, 
                    long *GEOD_DISTANCE_ID, 
                    long *MAX_COD,
                    long *MAX_STR)
{ 
  *GEOD_TO_CART_ID  = XL_GEOD_TO_CART_ID;
  *CART_TO_GEOD_ID  = XL_CART_TO_GEOD_ID;
  *GEOD_DISTANCE_ID = XL_GEOD_DISTANCE_ID;
  *MAX_COD          = XL_MAX_COD;
  *MAX_STR          = XL_MAX_STR;
}
     
/* example: use this kind of function to wrap a c-function     
            onto a fortran90 function.
void xf_tree_create_root_(long *id, char *name, long *error)
{ xf_tree_create_root(id, name, error); };
*/

/* example: use this kind of function to retrieve a preprocessor constant
            when you need the value in your fortran90 code
long get_xf_cfi_empty_element_(void)
{ return XF_CFI_EMPTY_ELEMENT; }
*/
