/* c support functions for Genscat */

/* #[ Description:
   ---------------------------------------------
   this is a small collection of wrappers to the
   standard c library to get access to functions
    that are normally not available for fortran90

   Modifications:
     27-Oct-2005 J. de Kloe added this module to the L2BP (copy from genscat)
     03-Apr-2006 J. de Kloe added wrapper for gethostname and get_pid
     27-Jul-2006 J. de Kloe added wrapper for remove()
   ---------------------------------------------
*/
/* #] */

/* #[ include files */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

/* #] */

unsigned char get_size_of_long_(void)
/* #[  request the size of the default long type in c */
{ 
  
  long testinteger;
  unsigned char size_of_long;
  
  size_of_long = sizeof(testinteger);
  
  return size_of_long;
}

/* #] */

char *string_f90_to_c(char *fortchar)
/* #[ convert fortran to c string */
{ 

  char *name = fortchar;
  int   len  = 256;
  char *newp = (char *) malloc(len+1); /* 0:255+1*/
  
  if(!newp)
    {
      perror("malloc problem in string_f90_to_c");
      return((char*)0);
    }
  /* copy 256 chars, the last char with index 256 should already have been
     initialised to \0, but for security, do it again  */
  len=256;
  strncpy(newp,name,len); 
  newp[len] = '\0';

  len = 255; /* skip the final \0 in the next loop */
  while( (len > 0) && ( (newp[len] == ' ') || (newp[len] == '\0') ) )
    { len--; }
  /* when the first non-space character is found, add a \0 character
     which terminates a c string */
  newp[++len]='\0';

  /*
    printf("test van Jos inside string_f90_to_c():");
    printf("name = <%s>\n",newp);
  */

  return(newp);
}

/* #] */

/* warning: this function is probably not yet portable to the hpcd !! */
int get_file_size_c_(char *name, long len_name)
/* #[ wrapper to the fstat function */
{
/* Purpose: requests the filesize in bytes.
   FORTRAN90 call: size = get_file_size_c(filename)
 */

/* beware: this buffer should be at least one element larger
           then the fortran stringsize of the filename,
           which we force to be 256 chars                    */
#define NAMEBUFFLEN 257

  long size;
  char namebuff[NAMEBUFFLEN];
  char *fname;
  int stat_error_flag;   /* needed for getting the filesize */
  struct stat stbuf;     /* needed for getting the filesize */

  /* Put the character strings into buffers and ensure that there is a
     null terminator. The name string has 256 elements (0..255) so
     in case this name is full upto the right end with characters 
     we need a 257 th element (with index 256) to put the \0 in.   */
  strncpy( namebuff, name, NAMEBUFFLEN - 1);
  if (len_name >= NAMEBUFFLEN) { return -1; }
  namebuff[NAMEBUFFLEN-1] = '\0';

  /* convert fortran to c string : file name */
  fname = string_f90_to_c(namebuff);
  if(fname == (char*)0 ) 
    { 
      printf("string_f90_to_c returned with an error\n"); 
      return -1;
    }

  size = -1;
  stat_error_flag = stat(fname, &stbuf);
  if (stat_error_flag == 0) { size = (long) stbuf.st_size; }
  free(fname);

  return size;
}

/* #] */

int get_stat_result_c4_(char *name,         /* input: filename */
		        long *stat_result,  /* output: stat results */
		        long len_name)      /* hidden input: length filename */
/* #[ wrapper to the stat function */
{
/* Purpose: calls the c function stat() to get some file info
   FORTRAN90 call: err = get_stat_result_c(filename, stat_result) */

/* beware: this buffer should be at least one element larger
           then the fortran stringsize of the filename        */
#define NAMEBUFFLEN 257

  long size;
  char namebuff[NAMEBUFFLEN];
  char *fname;
  int stat_error_flag;   /* needed for getting the filesize */
  struct stat stbuf;     /* needed for getting the filesize */
  int i;

  /* Put the character strings into buffers and ensure that there is a
     null terminator. The name string has 256 elements (0..255) so
     in case this name is full upto the right end with characters 
     we need a 257 th element (with index 256) tp put the \0 in.   */
  strncpy( namebuff, name, NAMEBUFFLEN - 1);
  if (len_name >= NAMEBUFFLEN) { return -1; }
  namebuff[NAMEBUFFLEN-1] = '\0';

  /* convert fortran to c string : file name */
  fname = string_f90_to_c(namebuff);
  if(fname == (char*)0 ) 
    { 
      printf("string_f90_to_c return with an error\n"); 
      return -1; 
    }

  /* init the output array to -1 */
  for (i=0;i<13;i++){ stat_result[i] = -1;}

  stat_error_flag = stat(fname, &stbuf);
  if (stat_error_flag == 0) 
    {
      /* see the manpage of stat for a description of these elements */
      stat_result[ 0] = (long) stbuf.st_dev;   /* device */
      stat_result[ 1] = (long) stbuf.st_ino;   /* inode */
      stat_result[ 2] = (long) stbuf.st_mode;  /* protection */
      stat_result[ 3] = (long) stbuf.st_nlink; /* number of hard links */
      stat_result[ 4] = (long) stbuf.st_uid;   /* user ID of owner */
      stat_result[ 5] = (long) stbuf.st_gid;   /* group ID of owner */
      stat_result[ 6] = (long) stbuf.st_rdev;  /*device type(if inode device)*/
      stat_result[ 7] = (long) stbuf.st_size;   /* total size, in bytes */
      stat_result[ 8] = (long) stbuf.st_blksize;/*blcksize for filesystem I/O*/
      stat_result[ 9] = (long) stbuf.st_blocks; /* number of blocks allocated*/
      stat_result[10] = (long) stbuf.st_atime;  /* time of last access */
      stat_result[11] = (long) stbuf.st_mtime;  /* time of last modification */
      stat_result[12] = (long) stbuf.st_ctime;  /* time of last change */
    }
  else
    {
      /* printf("stat_error_flag = %i\n",stat_error_flag);*/
      return -1;
    }

  free(fname);

  return 0;
}

/* #] */

int get_stat_result_c8_(char *name,         /* input: filename */
		       long *stat_result,   /* output: stat results */
		       long len_name)       /* hidden input: length filename */
/* #[ wrapper to the stat function */
{
/* a one-liner calling the above routine, needed to be able to
   have two Fortran90 interfaces to basically the same piece of c-code,
   needed to cope with sizeof(long)=4 and sizeof(long)=8 */
  return get_stat_result_c4_(name, stat_result, len_name); 
}
/* #] */

int gethostname_c4_(char *hostname, long len_hostname)
/* #[ request the hostname */
{
  int result;
  result = gethostname(hostname, len_hostname); 
  return result;
}
/* #] */

int gethostname_c8_(char *hostname, long len_hostname)
/* #[ request the hostname */
{
  int result;
  result = gethostname(hostname, len_hostname); 
  return result;
}
/* #] */

int get_pid_c4_()
/* #[ request the process id */
{
  return getpid();
}
/* #] */

int get_pid_c8_()
/* #[ request the process id */
{
  return getpid();
}
/* #] */

/* see for details:
   http://www.opengroup.org/onlinepubs/009695399/functions/remove.html
*/
int remove_file_c4_(char *filename, long len_filename)
/* #[ wrapper to the remove function */
{
#define FILENAMEBUFFLEN 257
  char filenamebuff[FILENAMEBUFFLEN];
  char *fname;  

  /* Put the character strings into buffers and ensure that there is a
     null terminator. The filename string has 256 elements (0..255) so
     in case this name is full upto the right end with characters 
     we need a 257 th element (with index 256) to put the \0 in.   */
  strncpy( filenamebuff, filename, FILENAMEBUFFLEN - 1);
  if (len_filename >= FILENAMEBUFFLEN) { return -1; }
  filenamebuff[FILENAMEBUFFLEN-1] = '\0';

  /* convert fortran to c string : filename */
  fname = string_f90_to_c(filenamebuff);
  if(fname == (char*)0 ) 
    { 
      printf("string_f90_to_c returned with an error\n"); 
      return -1;
    }

  return remove(fname);
}
/* #] */

int remove_file_c8_(char *filename, long len_filename)
/* #[ wrapper to the remove function */
{
/* a one-liner calling the above routine, needed to be able to
   have two Fortran90 interfaces to basically the same piece of c-code,
   needed to cope with sizeof(long)=4 and sizeof(long)=8 */
  return remove_file_c4_(filename, len_filename);
}
/* #] */
