program test_c_support
  !  #[ documentation
  !  Written by J. de Kloe, KNMI
  !
  !  Modifications:
  !     16-Jan-2008  J. de Kloe phase out integer kind i_ 
  !
  !  #]
  !  #[ use statements
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  USE c_support, only: get_filesize, get_hostname, get_process_id, &
                       remove_file
  USE ErrorHandler, only: no_error, program_abort
  USE LunManager, only: get_lun, free_lun
  !  #]
  !  #[ variables
  implicit none
  integer :: size, fileunit, pid, error_flag
  character(len=256) :: filename
  character(len=256) :: hostname
  !  #]
  !  #[ program code
  filename = "c_support.F90"

  ! test get_filesize() by supplying the filename
  size = get_filesize(filename)
  print *,"file: ",trim(filename)," has a size of ",size," bytes"

  filename = "f90_c_support.c"
  fileunit = 21

  ! test get_filesize() by supplying the fileunit
  open(unit=fileunit,file=filename,status="old",&
       action="read",form="FORMATTED",ERR=99)
  size = get_filesize(fileunit)
  print *,"file: ",trim(filename)," with fileunit: ",fileunit
  print *,"has a size of ",size," bytes"
  close(unit=fileunit)

  ! added to circumvent the irritating "FORTRAN STOP"
  ! message issued by the pgf90 compiler at this point
  goto 100
  !stop

99 print *,"ERROR: unable to open input file: ",trim(filename)
  stop

100 continue

  hostname = get_hostname()
  print *,"hostname = [",trim(hostname),"]"

  pid = get_process_id()
  print *,"pid = ",pid

  filename = "dummyfile.txt"
  print *,"creating test file: ",trim(filename)
  fileunit = get_lun()
  open(unit=fileunit,file=filename,status="new",action="write",err=998)
  write(fileunit,"(a)",err=999) "some dummy text"
  close(unit=fileunit)
  call free_lun(fileunit)

  call remove_file(filename, error_flag)
  IF (error_flag .ne. no_error) &
       call program_abort(error_flag,"remove_file")

  print *,"successfully removed test file..."

  goto 1000

998 print *,"could not open outputfile ",trim(filename)
  goto 1000

999 print *,"could not write to outputfile ",trim(filename)
  goto 1000

1000 print *,"end of test program"
  !  #]
end program test_c_support
