module VerbosityControl
  !  #[ Description
  !     a little module to help with verbosity control
  !
  ! Modifications:
  !    12-May-2006 D. Tan     Null use of ParKind1 module
  !
  !  #]
  !  #[ modules used
  USE ParKind1, only:    ! For compatability with ECMWF-IFS build.
  !  #]
  !  #[ Public Parameters
  implicit none 

  integer, parameter :: VerbosityError  = 0 ! report error only
  integer, parameter :: VerbosityWarn   = 1 ! report errors and warnings
  integer, parameter :: VerbosityReport = 2 ! report much more
  !  #]
contains
  !-----------------------------
  subroutine print_verbosity_setting(Verbosity,ModuleName)
    !  #[
    integer,          intent(in) :: Verbosity
    character(len=*), intent(in) :: ModuleName

    ! local variable
    character(len=256) :: explanation

    print *,"Verbosity settings in module ",trim(Modulename)," is set to:"

    ! don't print it directly because you will get a runtime error
    ! on ecgate complaining about: I/O recursion detected.
    explanation = explain_verbosity_setting(Verbosity)
    print *,"==> "//trim(explanation)

  end subroutine print_verbosity_setting
    !  #]
  function explain_verbosity_setting(Verbosity) result(explanation)
    !  #[
    integer            :: Verbosity
    character(len=256) :: explanation

    select case (Verbosity)
    case(VerbosityError)
       explanation = "print errors only"
    case(VerbosityWarn)
       explanation = "print errors and warnings"
    case(VerbosityReport)
       explanation = "print everything"
    case default
       write(explanation,"(a,i6,a)") &
            "NON-FATAL ERROR: unknown verbosity setting: ",Verbosity,&
            " (this is probably due to a programming mistake !)"
    end select

  end function explain_verbosity_setting
    !  #]
  !-----------------------------
end module VerbosityControl
